#include "Target.h"
#include "Graphics.h"
#include "Box.h"
#include <random>

std::random_device rd2;
std::mt19937 rng2(rd2());
std::uniform_int_distribution<int> xDist2(0, 770);
std::uniform_int_distribution<int> yDist2(0, 570);

Target::Target()
{
	int xIndex = rand() % 8;
	int yIndex = rand() % 8;
	vx = arrayNum[xIndex];
	vy = arrayNum[yIndex];

	x = xDist2(rng2);
	y = yDist2(rng2);
}

void Target::Update(Box& box)
{

	const int boxright = box.GetX() + box.GetWidth();
	const int boxbottom = box.GetY() + box.GetHeight();
	const int targetright = x + width;
	const int targetbottom = y + height;

	if (
		boxright >= x &&
		box.GetX() <= targetright &&
		boxbottom >= y &&
		box.GetY() <= targetbottom)
	{
		int xIndex = rand() % 8;
		int yIndex = rand() % 8;
		vx = arrayNum[xIndex];
		vy = arrayNum[yIndex];
		x = xDist2(rng2);
		y = yDist2(rng2);
		collected += 1;
	}
	int PosVel[4] = {  2,  3, 4,  5};
	int NegVel[4] = { -2, -3,-4, -5};
	int xIndex = rand() % 4;
	int yIndex = rand() % 4;
	//update ball position by the set velocity
	x += vx;
	y += vy;
	// set the right side of the ball with the current position + the width that set in the 'Ball' class
	const int right = x + width;
	//check if ball is off the left side of the screen before we draw it so we dont break shit
	if (x < 0)
	{

		//if the ball has passed the left side, reverse velocity 
		vx = PosVel[xIndex];
		//and set it's position inside the screen so drawing won't break shit
		x = 0;
	}
	//if the above is false, then we check if the ball is off the right side of the screen
	else if (right >= Graphics::ScreenWidth)
	{

		//if the ball has passed the right side, reverse velocity
		vx = NegVel[xIndex];
		// and set its position inside of the screen
		x = (Graphics::ScreenWidth - 1) - width;
	}

	//and repeat for the y position
	const int bottom = y + height;
	if (y < 0)
	{

		vy = PosVel[yIndex];
		y = 0;
	}
	else if (bottom >= Graphics::ScreenHeight)
	{

		vy = NegVel[yIndex];
		y = (Graphics::ScreenHeight - 1) - height;
	}
	//if none of the above is true then the ball keeps current velocity
	//and draws on screen since its inside the screen
}

void Target::Blink()
{
	if (isIncreasing && BlinkTimer < 255)
	{
		g = BlinkTimer;
		b = BlinkTimer;
		BlinkTimer += BlinkSpeed;
	}
	else if (BlinkTimer >= 255)
	{
		BlinkTimer = 255;
		isIncreasing = false;
	}

	if (!isIncreasing && BlinkTimer > 0)
	{
		g = BlinkTimer;
		b = BlinkTimer;
		BlinkTimer -= BlinkSpeed;
	}
	else if (BlinkTimer <= 0)
	{
		BlinkTimer = 0;
		isIncreasing = true;
	}
}

void Target::Draw(Graphics & gfx)
{
	//line 1
	gfx.PutPixel(x + 4, y, r, g, b);
	gfx.PutPixel(x + 5, y, r, g, b);
	gfx.PutPixel(x + 6, y, r, g, b);
	gfx.PutPixel(x + 7, y, r, g, b);
	gfx.PutPixel(x + 8, y, r, g, b);
	gfx.PutPixel(x + 9, y, r, g, b);
	gfx.PutPixel(x + 10, y, r, g, b);

	//line 2
	gfx.PutPixel(x + 3, y + 1, r, g, b);
	gfx.PutPixel(x + 4, y + 1, r, g, b);
	gfx.PutPixel(x + 5, y + 1, r, g, b);
	gfx.PutPixel(x + 6, y + 1, r, g, b);
	gfx.PutPixel(x + 7, y + 1, r, g, b);
	gfx.PutPixel(x + 8, y + 1, r, g, b);
	gfx.PutPixel(x + 9, y + 1, r, g, b);
	gfx.PutPixel(x + 10, y + 1, r, g, b);
	gfx.PutPixel(x + 11, y + 1, r, g, b);

	//line 3
	gfx.PutPixel(x + 2, y + 2, r, g, b);
	gfx.PutPixel(x + 3, y + 2, r, g, b);
	gfx.PutPixel(x + 4, y + 2, r, g, b);
	gfx.PutPixel(x + 5, y + 2, r, g, b);
	gfx.PutPixel(x + 6, y + 2, r, g, b);
	gfx.PutPixel(x + 7, y + 2, r, g, b);
	gfx.PutPixel(x + 8, y + 2, r, g, b);
	gfx.PutPixel(x + 9, y + 2, r, g, b);
	gfx.PutPixel(x + 10, y + 2, r, g, b);
	gfx.PutPixel(x + 11, y + 2, r, g, b);
	gfx.PutPixel(x + 12, y + 2, r, g, b);

	//line 4
	gfx.PutPixel(x + 1, y + 3, r, g, b);
	gfx.PutPixel(x + 2, y + 3, r, g, b);
	gfx.PutPixel(x + 3, y + 3, r, g, b);
	gfx.PutPixel(x + 4, y + 3, r, g, b);
	gfx.PutPixel(x + 5, y + 3, r, g, b);
	gfx.PutPixel(x + 6, y + 3, r, g, b);
	gfx.PutPixel(x + 7, y + 3, r, g, b);
	gfx.PutPixel(x + 8, y + 3, r, g, b);
	gfx.PutPixel(x + 9, y + 3, r, g, b);
	gfx.PutPixel(x + 10, y + 3, r, g, b);
	gfx.PutPixel(x + 11, y + 3, r, g, b);
	gfx.PutPixel(x + 12, y + 3, r, g, b);
	gfx.PutPixel(x + 13, y + 3, r, g, b);

	//line 5
	gfx.PutPixel(x + 0, y + 4, r, g, b);
	gfx.PutPixel(x + 1, y + 4, r, g, b);
	gfx.PutPixel(x + 2, y + 4, r, g, b);
	gfx.PutPixel(x + 3, y + 4, r, g, b);
	gfx.PutPixel(x + 4, y + 4, r, g, b);
	gfx.PutPixel(x + 5, y + 4, r, g, b);
	gfx.PutPixel(x + 6, y + 4, r, g, b);
	gfx.PutPixel(x + 7, y + 4, r, g, b);
	gfx.PutPixel(x + 8, y + 4, r, g, b);
	gfx.PutPixel(x + 9, y + 4, r, g, b);
	gfx.PutPixel(x + 10, y + 4, r, g, b);
	gfx.PutPixel(x + 11, y + 4, r, g, b);
	gfx.PutPixel(x + 12, y + 4, r, g, b);
	gfx.PutPixel(x + 13, y + 4, r, g, b);
	gfx.PutPixel(x + 14, y + 4, r, g, b);

	//line 6
	gfx.PutPixel(x + 0, y + 5, r, g, b);
	gfx.PutPixel(x + 1, y + 5, r, g, b);
	gfx.PutPixel(x + 2, y + 5, r, g, b);
	gfx.PutPixel(x + 3, y + 5, r, g, b);
	gfx.PutPixel(x + 4, y + 5, r, g, b);
	gfx.PutPixel(x + 5, y + 5, r, g, b);
	gfx.PutPixel(x + 6, y + 5, r, g, b);
	gfx.PutPixel(x + 7, y + 5, r, g, b);
	gfx.PutPixel(x + 8, y + 5, r, g, b);
	gfx.PutPixel(x + 9, y + 5, r, g, b);
	gfx.PutPixel(x + 10, y + 5, r, g, b);
	gfx.PutPixel(x + 11, y + 5, r, g, b);
	gfx.PutPixel(x + 12, y + 5, r, g, b);
	gfx.PutPixel(x + 13, y + 5, r, g, b);
	gfx.PutPixel(x + 14, y + 5, r, g, b);

	//line 7
	gfx.PutPixel(x + 0, y + 6, r, g, b);
	gfx.PutPixel(x + 1, y + 6, r, g, b);
	gfx.PutPixel(x + 2, y + 6, r, g, b);
	gfx.PutPixel(x + 3, y + 6, r, g, b);
	gfx.PutPixel(x + 4, y + 6, r, g, b);
	gfx.PutPixel(x + 5, y + 6, r, g, b);
	gfx.PutPixel(x + 6, y + 6, r, g, b);
	gfx.PutPixel(x + 7, y + 6, r, g, b);
	gfx.PutPixel(x + 8, y + 6, r, g, b);
	gfx.PutPixel(x + 9, y + 6, r, g, b);
	gfx.PutPixel(x + 10, y + 6, r, g, b);
	gfx.PutPixel(x + 11, y + 6, r, g, b);
	gfx.PutPixel(x + 12, y + 6, r, g, b);
	gfx.PutPixel(x + 13, y + 6, r, g, b);
	gfx.PutPixel(x + 14, y + 6, r, g, b);

	//line 8
	gfx.PutPixel(x + 0, y + 7, r, g, b);
	gfx.PutPixel(x + 1, y + 7, r, g, b);
	gfx.PutPixel(x + 2, y + 7, r, g, b);
	gfx.PutPixel(x + 3, y + 7, r, g, b);
	gfx.PutPixel(x + 4, y + 7, r, g, b);
	gfx.PutPixel(x + 5, y + 7, r, g, b);
	gfx.PutPixel(x + 6, y + 7, r, g, b);
	gfx.PutPixel(x + 7, y + 7, r, g, b);
	gfx.PutPixel(x + 8, y + 7, r, g, b);
	gfx.PutPixel(x + 9, y + 7, r, g, b);
	gfx.PutPixel(x + 10, y + 7, r, g, b);
	gfx.PutPixel(x + 11, y + 7, r, g, b);
	gfx.PutPixel(x + 12, y + 7, r, g, b);
	gfx.PutPixel(x + 13, y + 7, r, g, b);
	gfx.PutPixel(x + 14, y + 7, r, g, b);

	//line 9
	gfx.PutPixel(x + 0, y + 8, r, g, b);
	gfx.PutPixel(x + 1, y + 8, r, g, b);
	gfx.PutPixel(x + 2, y + 8, r, g, b);
	gfx.PutPixel(x + 3, y + 8, r, g, b);
	gfx.PutPixel(x + 4, y + 8, r, g, b);
	gfx.PutPixel(x + 5, y + 8, r, g, b);
	gfx.PutPixel(x + 6, y + 8, r, g, b);
	gfx.PutPixel(x + 7, y + 8, r, g, b);
	gfx.PutPixel(x + 8, y + 8, r, g, b);
	gfx.PutPixel(x + 9, y + 8, r, g, b);
	gfx.PutPixel(x + 10, y + 8, r, g, b);
	gfx.PutPixel(x + 11, y + 8, r, g, b);
	gfx.PutPixel(x + 12, y + 8, r, g, b);
	gfx.PutPixel(x + 13, y + 8, r, g, b);
	gfx.PutPixel(x + 14, y + 8, r, g, b);

	//line 10
	gfx.PutPixel(x + 0, y + 9, r, g, b);
	gfx.PutPixel(x + 1, y + 9, r, g, b);
	gfx.PutPixel(x + 2, y + 9, r, g, b);
	gfx.PutPixel(x + 3, y + 9, r, g, b);
	gfx.PutPixel(x + 4, y + 9, r, g, b);
	gfx.PutPixel(x + 5, y + 9, r, g, b);
	gfx.PutPixel(x + 6, y + 9, r, g, b);
	gfx.PutPixel(x + 7, y + 9, r, g, b);
	gfx.PutPixel(x + 8, y + 9, r, g, b);
	gfx.PutPixel(x + 9, y + 9, r, g, b);
	gfx.PutPixel(x + 10, y + 9, r, g, b);
	gfx.PutPixel(x + 11, y + 9, r, g, b);
	gfx.PutPixel(x + 12, y + 9, r, g, b);
	gfx.PutPixel(x + 13, y + 9, r, g, b);
	gfx.PutPixel(x + 14, y + 9, r, g, b);

	//line 11
	gfx.PutPixel(x + 1, y + 10, r, g, b);
	gfx.PutPixel(x + 2, y + 10, r, g, b);
	gfx.PutPixel(x + 3, y + 10, r, g, b);
	gfx.PutPixel(x + 4, y + 10, r, g, b);
	gfx.PutPixel(x + 5, y + 10, r, g, b);
	gfx.PutPixel(x + 6, y + 10, r, g, b);
	gfx.PutPixel(x + 7, y + 10, r, g, b);
	gfx.PutPixel(x + 8, y + 10, r, g, b);
	gfx.PutPixel(x + 9, y + 10, r, g, b);
	gfx.PutPixel(x + 10, y + 10, r, g, b);
	gfx.PutPixel(x + 11, y + 10, r, g, b);
	gfx.PutPixel(x + 12, y + 10, r, g, b);
	gfx.PutPixel(x + 13, y + 10, r, g, b);

	//line 12
	gfx.PutPixel(x + 2, y + 11, r, g, b);
	gfx.PutPixel(x + 3, y + 11, r, g, b);
	gfx.PutPixel(x + 4, y + 11, r, g, b);
	gfx.PutPixel(x + 5, y + 11, r, g, b);
	gfx.PutPixel(x + 6, y + 11, r, g, b);
	gfx.PutPixel(x + 7, y + 11, r, g, b);
	gfx.PutPixel(x + 8, y + 11, r, g, b);
	gfx.PutPixel(x + 9, y + 11, r, g, b);
	gfx.PutPixel(x + 10, y + 11, r, g, b);
	gfx.PutPixel(x + 11, y + 11, r, g, b);
	gfx.PutPixel(x + 12, y + 11, r, g, b);

	//line 13
	gfx.PutPixel(x + 3, y + 12, r, g, b);
	gfx.PutPixel(x + 4, y + 12, r, g, b);
	gfx.PutPixel(x + 5, y + 12, r, g, b);
	gfx.PutPixel(x + 6, y + 12, r, g, b);
	gfx.PutPixel(x + 7, y + 12, r, g, b);
	gfx.PutPixel(x + 8, y + 12, r, g, b);
	gfx.PutPixel(x + 9, y + 12, r, g, b);
	gfx.PutPixel(x + 10, y + 12, r, g, b);
	gfx.PutPixel(x + 11, y + 12, r, g, b);

	// line 14
	gfx.PutPixel(x + 4, y + 13, r, g, b);
	gfx.PutPixel(x + 5, y + 13, r, g, b);
	gfx.PutPixel(x + 6, y + 13, r, g, b);
	gfx.PutPixel(x + 7, y + 13, r, g, b);
	gfx.PutPixel(x + 8, y + 13, r, g, b);
	gfx.PutPixel(x + 9, y + 13, r, g, b);
	gfx.PutPixel(x + 10, y + 13, r, g, b);
}

int Target::GetCollected()
{
	return collected;
}