#include "MainWindow.h"
#include "Game.h"
#include <stdlib.h>
#include <stdio.h>
#include <random>

Game::Game(MainWindow& wnd)
	:
	wnd(wnd),
	gfx(wnd)
{
}

void Game::Go()
{
	gfx.BeginFrame();
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::UpdateModel()
{
	if (!GameStarted)
	{
		if (wnd.kbd.KeyIsPressed(VK_RETURN))
		{
			GameStarted = true;
		}
	}

	if ( GameStarted && !GameOver)
	{
		box.Update(wnd);
		target.Update(box);
		target.Blink();
		progress.Update(target);
		for (int i = 0; i < nBalls; ++i)
		{
			//update each ball
			balls[i].Update();
			//check collision
			//the value returned is 'true' then end game
			if (balls[i].IsColliding(box))
			{
				GameOver = true;
				GameStarted = false;
			}
		}
		//check gamewin
		if (target.GetCollected() == progress.GetGoal())
		{
			GameOver = true;
			GameWin = true;
		}
	}
}

void Game::ComposeFrame()
{
	if (GameOver && !GameWin)
	{
		progress.Draw(progress.GetX(), progress.GetY(), gfx);
		DrawGameOver(100, 100);
	}

	else if (GameWin)
	{
		progress.Draw(progress.GetX(), progress.GetY(), gfx);
		DrawGameWin((gfx.ScreenWidth/2) - 210, (gfx.ScreenHeight/2) - 55);
	}

	else if (!GameStarted)
	{
		DrawStartScreen(192, 147);
	}

	else
	{
		if (target.GetCollected() == progress.GetGoal())
		{
			DrawGameWin((gfx.ScreenWidth / 2) - 210, (gfx.ScreenHeight / 2) - 55);
		}
		

		progress.Draw(progress.GetX(), progress.GetY(), gfx);
		box.Draw(gfx);
		target.Draw(gfx);
		for (int i = 0; i < nBalls; ++i)
		{
			balls[i].Draw(gfx);
		}
	}
}

void Game::DrawGameWin(int x, int y)
{
	gfx.PutPixel(x + 0, y + 0, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 1, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 2, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 3, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 4, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 5, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 6, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 7, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 9, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 10, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 11, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 12, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 13, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 14, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 15, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 16, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 17, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 18, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 19, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 20, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 21, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 22, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 23, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 24, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 25, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 26, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 27, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 28, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 29, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 30, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 31, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 32, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 33, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 101, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 102, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 103, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 104, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 105, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 106, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 107, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 108, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 109, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 110, 0, 0, 0);
	gfx.PutPixel(x + 0, y + 111, 0, 0, 0);
	gfx.PutPixel(x + 1, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 40, 7, 7, 7);
	gfx.PutPixel(x + 1, y + 41, 1, 1, 1);
	gfx.PutPixel(x + 1, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 1, y + 43, 4, 4, 4);
	gfx.PutPixel(x + 1, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 1, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 2, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 39, 6, 6, 6);
	gfx.PutPixel(x + 2, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 2, y + 41, 72, 72, 72);
	gfx.PutPixel(x + 2, y + 42, 85, 85, 85);
	gfx.PutPixel(x + 2, y + 43, 19, 19, 19);
	gfx.PutPixel(x + 2, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 2, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 2, y + 46, 6, 6, 6);
	gfx.PutPixel(x + 2, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 2, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 3, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 38, 6, 6, 6);
	gfx.PutPixel(x + 3, y + 39, 2, 2, 2);
	gfx.PutPixel(x + 3, y + 40, 133, 133, 133);
	gfx.PutPixel(x + 3, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 3, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 3, y + 43, 228, 228, 228);
	gfx.PutPixel(x + 3, y + 44, 152, 152, 152);
	gfx.PutPixel(x + 3, y + 45, 62, 62, 62);
	gfx.PutPixel(x + 3, y + 46, 5, 5, 5);
	gfx.PutPixel(x + 3, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 3, y + 48, 5, 5, 5);
	gfx.PutPixel(x + 3, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 3, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 4, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 4, y + 39, 135, 135, 135);
	gfx.PutPixel(x + 4, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 4, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 4, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 4, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 4, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 4, y + 45, 254, 254, 254);
	gfx.PutPixel(x + 4, y + 46, 200, 200, 200);
	gfx.PutPixel(x + 4, y + 47, 94, 94, 94);
	gfx.PutPixel(x + 4, y + 48, 12, 12, 12);
	gfx.PutPixel(x + 4, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 4, y + 50, 5, 5, 5);
	gfx.PutPixel(x + 4, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 4, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 5, y + 38, 75, 75, 75);
	gfx.PutPixel(x + 5, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 42, 251, 251, 251);
	gfx.PutPixel(x + 5, y + 43, 240, 240, 240);
	gfx.PutPixel(x + 5, y + 44, 239, 239, 239);
	gfx.PutPixel(x + 5, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 5, y + 48, 215, 215, 215);
	gfx.PutPixel(x + 5, y + 49, 98, 98, 98);
	gfx.PutPixel(x + 5, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 5, y + 52, 7, 7, 7);
	gfx.PutPixel(x + 5, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 5, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 6, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 6, y + 37, 5, 5, 5);
	gfx.PutPixel(x + 6, y + 38, 194, 194, 194);
	gfx.PutPixel(x + 6, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 40, 201, 201, 201);
	gfx.PutPixel(x + 6, y + 41, 82, 82, 82);
	gfx.PutPixel(x + 6, y + 42, 31, 31, 31);
	gfx.PutPixel(x + 6, y + 43, 51, 51, 51);
	gfx.PutPixel(x + 6, y + 44, 195, 195, 195);
	gfx.PutPixel(x + 6, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 6, y + 50, 203, 203, 203);
	gfx.PutPixel(x + 6, y + 51, 68, 68, 68);
	gfx.PutPixel(x + 6, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 6, y + 53, 3, 3, 3);
	gfx.PutPixel(x + 6, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 6, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 7, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 7, y + 37, 57, 57, 57);
	gfx.PutPixel(x + 7, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 39, 241, 241, 241);
	gfx.PutPixel(x + 7, y + 40, 29, 29, 29);
	gfx.PutPixel(x + 7, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 7, y + 42, 68, 68, 68);
	gfx.PutPixel(x + 7, y + 43, 212, 212, 212);
	gfx.PutPixel(x + 7, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 7, y + 51, 254, 254, 254);
	gfx.PutPixel(x + 7, y + 52, 154, 154, 154);
	gfx.PutPixel(x + 7, y + 53, 20, 20, 20);
	gfx.PutPixel(x + 7, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 7, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 7, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 8, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 8, y + 37, 120, 120, 120);
	gfx.PutPixel(x + 8, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 39, 234, 234, 234);
	gfx.PutPixel(x + 8, y + 40, 113, 113, 113);
	gfx.PutPixel(x + 8, y + 41, 185, 185, 185);
	gfx.PutPixel(x + 8, y + 42, 254, 254, 254);
	gfx.PutPixel(x + 8, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 8, y + 53, 219, 219, 219);
	gfx.PutPixel(x + 8, y + 54, 64, 64, 64);
	gfx.PutPixel(x + 8, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 8, y + 56, 6, 6, 6);
	gfx.PutPixel(x + 8, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 8, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 9, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 9, y + 37, 164, 164, 164);
	gfx.PutPixel(x + 9, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 9, y + 54, 249, 249, 249);
	gfx.PutPixel(x + 9, y + 55, 111, 111, 111);
	gfx.PutPixel(x + 9, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 9, y + 57, 5, 5, 5);
	gfx.PutPixel(x + 9, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 9, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 10, y + 36, 2, 2, 2);
	gfx.PutPixel(x + 10, y + 37, 191, 191, 191);
	gfx.PutPixel(x + 10, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 10, y + 56, 146, 146, 146);
	gfx.PutPixel(x + 10, y + 57, 4, 4, 4);
	gfx.PutPixel(x + 10, y + 58, 4, 4, 4);
	gfx.PutPixel(x + 10, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 10, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 35, 6, 6, 6);
	gfx.PutPixel(x + 11, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 11, y + 37, 202, 202, 202);
	gfx.PutPixel(x + 11, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 11, y + 57, 164, 164, 164);
	gfx.PutPixel(x + 11, y + 58, 6, 6, 6);
	gfx.PutPixel(x + 11, y + 59, 3, 3, 3);
	gfx.PutPixel(x + 11, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 87, 7, 7, 7);
	gfx.PutPixel(x + 11, y + 88, 5, 5, 5);
	gfx.PutPixel(x + 11, y + 89, 4, 4, 4);
	gfx.PutPixel(x + 11, y + 90, 4, 4, 4);
	gfx.PutPixel(x + 11, y + 91, 3, 3, 3);
	gfx.PutPixel(x + 11, y + 92, 2, 2, 2);
	gfx.PutPixel(x + 11, y + 93, 2, 2, 2);
	gfx.PutPixel(x + 11, y + 94, 3, 3, 3);
	gfx.PutPixel(x + 11, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 11, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 35, 6, 6, 6);
	gfx.PutPixel(x + 12, y + 36, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 37, 204, 204, 204);
	gfx.PutPixel(x + 12, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 12, y + 58, 165, 165, 165);
	gfx.PutPixel(x + 12, y + 59, 4, 4, 4);
	gfx.PutPixel(x + 12, y + 60, 4, 4, 4);
	gfx.PutPixel(x + 12, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 78, 7, 7, 7);
	gfx.PutPixel(x + 12, y + 79, 5, 5, 5);
	gfx.PutPixel(x + 12, y + 80, 3, 3, 3);
	gfx.PutPixel(x + 12, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 87, 4, 4, 4);
	gfx.PutPixel(x + 12, y + 88, 13, 13, 13);
	gfx.PutPixel(x + 12, y + 89, 19, 19, 19);
	gfx.PutPixel(x + 12, y + 90, 23, 23, 23);
	gfx.PutPixel(x + 12, y + 91, 27, 27, 27);
	gfx.PutPixel(x + 12, y + 92, 32, 32, 32);
	gfx.PutPixel(x + 12, y + 93, 33, 33, 33);
	gfx.PutPixel(x + 12, y + 94, 24, 24, 24);
	gfx.PutPixel(x + 12, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 12, y + 96, 2, 2, 2);
	gfx.PutPixel(x + 12, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 12, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 37, 163, 163, 163);
	gfx.PutPixel(x + 13, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 13, y + 59, 152, 152, 152);
	gfx.PutPixel(x + 13, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 13, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 71, 7, 7, 7);
	gfx.PutPixel(x + 13, y + 72, 5, 5, 5);
	gfx.PutPixel(x + 13, y + 73, 2, 2, 2);
	gfx.PutPixel(x + 13, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 79, 12, 12, 12);
	gfx.PutPixel(x + 13, y + 80, 27, 27, 27);
	gfx.PutPixel(x + 13, y + 81, 52, 52, 52);
	gfx.PutPixel(x + 13, y + 82, 72, 72, 72);
	gfx.PutPixel(x + 13, y + 83, 104, 104, 104);
	gfx.PutPixel(x + 13, y + 84, 128, 128, 128);
	gfx.PutPixel(x + 13, y + 85, 156, 156, 156);
	gfx.PutPixel(x + 13, y + 86, 178, 178, 178);
	gfx.PutPixel(x + 13, y + 87, 198, 198, 198);
	gfx.PutPixel(x + 13, y + 88, 215, 215, 215);
	gfx.PutPixel(x + 13, y + 89, 226, 226, 226);
	gfx.PutPixel(x + 13, y + 90, 234, 234, 234);
	gfx.PutPixel(x + 13, y + 91, 239, 239, 239);
	gfx.PutPixel(x + 13, y + 92, 241, 241, 241);
	gfx.PutPixel(x + 13, y + 93, 243, 243, 243);
	gfx.PutPixel(x + 13, y + 94, 235, 235, 235);
	gfx.PutPixel(x + 13, y + 95, 164, 164, 164);
	gfx.PutPixel(x + 13, y + 96, 40, 40, 40);
	gfx.PutPixel(x + 13, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 13, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 13, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 36, 4, 4, 4);
	gfx.PutPixel(x + 14, y + 37, 16, 16, 16);
	gfx.PutPixel(x + 14, y + 38, 185, 185, 185);
	gfx.PutPixel(x + 14, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 60, 122, 122, 122);
	gfx.PutPixel(x + 14, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 65, 7, 7, 7);
	gfx.PutPixel(x + 14, y + 66, 4, 4, 4);
	gfx.PutPixel(x + 14, y + 67, 1, 1, 1);
	gfx.PutPixel(x + 14, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 72, 12, 12, 12);
	gfx.PutPixel(x + 14, y + 73, 31, 31, 31);
	gfx.PutPixel(x + 14, y + 74, 57, 57, 57);
	gfx.PutPixel(x + 14, y + 75, 88, 88, 88);
	gfx.PutPixel(x + 14, y + 76, 123, 123, 123);
	gfx.PutPixel(x + 14, y + 77, 158, 158, 158);
	gfx.PutPixel(x + 14, y + 78, 187, 187, 187);
	gfx.PutPixel(x + 14, y + 79, 216, 216, 216);
	gfx.PutPixel(x + 14, y + 80, 237, 237, 237);
	gfx.PutPixel(x + 14, y + 81, 253, 253, 253);
	gfx.PutPixel(x + 14, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 14, y + 96, 237, 237, 237);
	gfx.PutPixel(x + 14, y + 97, 65, 65, 65);
	gfx.PutPixel(x + 14, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 14, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 14, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 15, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 37, 2, 2, 2);
	gfx.PutPixel(x + 15, y + 38, 10, 10, 10);
	gfx.PutPixel(x + 15, y + 39, 160, 160, 160);
	gfx.PutPixel(x + 15, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 60, 254, 254, 254);
	gfx.PutPixel(x + 15, y + 61, 80, 80, 80);
	gfx.PutPixel(x + 15, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 15, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 15, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 15, y + 65, 3, 3, 3);
	gfx.PutPixel(x + 15, y + 66, 19, 19, 19);
	gfx.PutPixel(x + 15, y + 67, 44, 44, 44);
	gfx.PutPixel(x + 15, y + 68, 74, 74, 74);
	gfx.PutPixel(x + 15, y + 69, 111, 111, 111);
	gfx.PutPixel(x + 15, y + 70, 149, 149, 149);
	gfx.PutPixel(x + 15, y + 71, 185, 185, 185);
	gfx.PutPixel(x + 15, y + 72, 216, 216, 216);
	gfx.PutPixel(x + 15, y + 73, 240, 240, 240);
	gfx.PutPixel(x + 15, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 15, y + 97, 227, 227, 227);
	gfx.PutPixel(x + 15, y + 98, 30, 30, 30);
	gfx.PutPixel(x + 15, y + 99, 2, 2, 2);
	gfx.PutPixel(x + 15, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 15, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 16, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 16, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 16, y + 40, 109, 109, 109);
	gfx.PutPixel(x + 16, y + 41, 245, 245, 245);
	gfx.PutPixel(x + 16, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 61, 219, 219, 219);
	gfx.PutPixel(x + 16, y + 62, 84, 84, 84);
	gfx.PutPixel(x + 16, y + 63, 117, 117, 117);
	gfx.PutPixel(x + 16, y + 64, 159, 159, 159);
	gfx.PutPixel(x + 16, y + 65, 195, 195, 195);
	gfx.PutPixel(x + 16, y + 66, 227, 227, 227);
	gfx.PutPixel(x + 16, y + 67, 249, 249, 249);
	gfx.PutPixel(x + 16, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 98, 138, 138, 138);
	gfx.PutPixel(x + 16, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 16, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 16, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 17, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 39, 6, 6, 6);
	gfx.PutPixel(x + 17, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 17, y + 41, 52, 52, 52);
	gfx.PutPixel(x + 17, y + 42, 203, 203, 203);
	gfx.PutPixel(x + 17, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 98, 218, 218, 218);
	gfx.PutPixel(x + 17, y + 99, 15, 15, 15);
	gfx.PutPixel(x + 17, y + 100, 5, 5, 5);
	gfx.PutPixel(x + 17, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 17, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 18, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 18, y + 42, 10, 10, 10);
	gfx.PutPixel(x + 18, y + 43, 128, 128, 128);
	gfx.PutPixel(x + 18, y + 44, 246, 246, 246);
	gfx.PutPixel(x + 18, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 98, 254, 254, 254);
	gfx.PutPixel(x + 18, y + 99, 53, 53, 53);
	gfx.PutPixel(x + 18, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 18, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 18, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 19, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 42, 5, 5, 5);
	gfx.PutPixel(x + 19, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 19, y + 44, 50, 50, 50);
	gfx.PutPixel(x + 19, y + 45, 190, 190, 190);
	gfx.PutPixel(x + 19, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 99, 87, 87, 87);
	gfx.PutPixel(x + 19, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 19, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 19, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 20, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 20, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 20, y + 46, 98, 98, 98);
	gfx.PutPixel(x + 20, y + 47, 227, 227, 227);
	gfx.PutPixel(x + 20, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 99, 101, 101, 101);
	gfx.PutPixel(x + 20, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 20, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 20, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 21, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 45, 6, 6, 6);
	gfx.PutPixel(x + 21, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 21, y + 47, 19, 19, 19);
	gfx.PutPixel(x + 21, y + 48, 170, 170, 170);
	gfx.PutPixel(x + 21, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 21, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 21, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 21, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 22, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 22, y + 48, 21, 21, 21);
	gfx.PutPixel(x + 22, y + 49, 230, 230, 230);
	gfx.PutPixel(x + 22, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 99, 70, 70, 70);
	gfx.PutPixel(x + 22, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 22, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 22, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 43, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 44, 1, 1, 1);
	gfx.PutPixel(x + 23, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 23, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 23, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 23, y + 48, 69, 69, 69);
	gfx.PutPixel(x + 23, y + 49, 244, 244, 244);
	gfx.PutPixel(x + 23, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 98, 162, 162, 162);
	gfx.PutPixel(x + 23, y + 99, 4, 4, 4);
	gfx.PutPixel(x + 23, y + 100, 7, 7, 7);
	gfx.PutPixel(x + 23, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 23, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 42, 3, 3, 3);
	gfx.PutPixel(x + 24, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 24, y + 44, 66, 66, 66);
	gfx.PutPixel(x + 24, y + 45, 126, 126, 126);
	gfx.PutPixel(x + 24, y + 46, 141, 141, 141);
	gfx.PutPixel(x + 24, y + 47, 187, 187, 187);
	gfx.PutPixel(x + 24, y + 48, 247, 247, 247);
	gfx.PutPixel(x + 24, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 96, 218, 218, 218);
	gfx.PutPixel(x + 24, y + 97, 111, 111, 111);
	gfx.PutPixel(x + 24, y + 98, 7, 7, 7);
	gfx.PutPixel(x + 24, y + 99, 5, 5, 5);
	gfx.PutPixel(x + 24, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 24, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 40, 7, 7, 7);
	gfx.PutPixel(x + 25, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 25, y + 42, 27, 27, 27);
	gfx.PutPixel(x + 25, y + 43, 159, 159, 159);
	gfx.PutPixel(x + 25, y + 44, 253, 253, 253);
	gfx.PutPixel(x + 25, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 64, 254, 254, 254);
	gfx.PutPixel(x + 25, y + 65, 232, 232, 232);
	gfx.PutPixel(x + 25, y + 66, 199, 199, 199);
	gfx.PutPixel(x + 25, y + 67, 172, 172, 172);
	gfx.PutPixel(x + 25, y + 68, 146, 146, 146);
	gfx.PutPixel(x + 25, y + 69, 125, 125, 125);
	gfx.PutPixel(x + 25, y + 70, 116, 116, 116);
	gfx.PutPixel(x + 25, y + 71, 109, 109, 109);
	gfx.PutPixel(x + 25, y + 72, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 73, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 74, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 75, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 76, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 77, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 78, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 79, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 80, 107, 107, 107);
	gfx.PutPixel(x + 25, y + 81, 113, 113, 113);
	gfx.PutPixel(x + 25, y + 82, 121, 121, 121);
	gfx.PutPixel(x + 25, y + 83, 128, 128, 128);
	gfx.PutPixel(x + 25, y + 84, 140, 140, 140);
	gfx.PutPixel(x + 25, y + 85, 154, 154, 154);
	gfx.PutPixel(x + 25, y + 86, 164, 164, 164);
	gfx.PutPixel(x + 25, y + 87, 173, 173, 173);
	gfx.PutPixel(x + 25, y + 88, 181, 181, 181);
	gfx.PutPixel(x + 25, y + 89, 188, 188, 188);
	gfx.PutPixel(x + 25, y + 90, 188, 188, 188);
	gfx.PutPixel(x + 25, y + 91, 182, 182, 182);
	gfx.PutPixel(x + 25, y + 92, 171, 171, 171);
	gfx.PutPixel(x + 25, y + 93, 144, 144, 144);
	gfx.PutPixel(x + 25, y + 94, 108, 108, 108);
	gfx.PutPixel(x + 25, y + 95, 59, 59, 59);
	gfx.PutPixel(x + 25, y + 96, 12, 12, 12);
	gfx.PutPixel(x + 25, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 25, y + 98, 6, 6, 6);
	gfx.PutPixel(x + 25, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 25, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 26, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 39, 6, 6, 6);
	gfx.PutPixel(x + 26, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 41, 82, 82, 82);
	gfx.PutPixel(x + 26, y + 42, 228, 228, 228);
	gfx.PutPixel(x + 26, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 63, 241, 241, 241);
	gfx.PutPixel(x + 26, y + 64, 93, 93, 93);
	gfx.PutPixel(x + 26, y + 65, 17, 17, 17);
	gfx.PutPixel(x + 26, y + 66, 4, 4, 4);
	gfx.PutPixel(x + 26, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 26, y + 96, 5, 5, 5);
	gfx.PutPixel(x + 26, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 26, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 27, y + 39, 3, 3, 3);
	gfx.PutPixel(x + 27, y + 40, 134, 134, 134);
	gfx.PutPixel(x + 27, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 62, 229, 229, 229);
	gfx.PutPixel(x + 27, y + 63, 60, 60, 60);
	gfx.PutPixel(x + 27, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 27, y + 65, 3, 3, 3);
	gfx.PutPixel(x + 27, y + 66, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 88, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 89, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 90, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 91, 7, 7, 7);
	gfx.PutPixel(x + 27, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 27, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 28, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 37, 4, 4, 4);
	gfx.PutPixel(x + 28, y + 38, 6, 6, 6);
	gfx.PutPixel(x + 28, y + 39, 162, 162, 162);
	gfx.PutPixel(x + 28, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 61, 210, 210, 210);
	gfx.PutPixel(x + 28, y + 62, 40, 40, 40);
	gfx.PutPixel(x + 28, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 28, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 28, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 29, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 29, y + 37, 1, 1, 1);
	gfx.PutPixel(x + 29, y + 38, 159, 159, 159);
	gfx.PutPixel(x + 29, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 60, 187, 187, 187);
	gfx.PutPixel(x + 29, y + 61, 22, 22, 22);
	gfx.PutPixel(x + 29, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 29, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 29, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 30, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 30, y + 37, 121, 121, 121);
	gfx.PutPixel(x + 30, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 41, 209, 209, 209);
	gfx.PutPixel(x + 30, y + 42, 124, 124, 124);
	gfx.PutPixel(x + 30, y + 43, 246, 246, 246);
	gfx.PutPixel(x + 30, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 59, 159, 159, 159);
	gfx.PutPixel(x + 30, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 61, 1, 1, 1);
	gfx.PutPixel(x + 30, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 30, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 31, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 31, y + 36, 53, 53, 53);
	gfx.PutPixel(x + 31, y + 37, 247, 247, 247);
	gfx.PutPixel(x + 31, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 40, 170, 170, 170);
	gfx.PutPixel(x + 31, y + 41, 13, 13, 13);
	gfx.PutPixel(x + 31, y + 42, 150, 150, 150);
	gfx.PutPixel(x + 31, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 58, 127, 127, 127);
	gfx.PutPixel(x + 31, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 31, y + 60, 4, 4, 4);
	gfx.PutPixel(x + 31, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 31, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 32, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 32, y + 36, 173, 173, 173);
	gfx.PutPixel(x + 32, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 39, 139, 139, 139);
	gfx.PutPixel(x + 32, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 32, y + 41, 132, 132, 132);
	gfx.PutPixel(x + 32, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 56, 246, 246, 246);
	gfx.PutPixel(x + 32, y + 57, 97, 97, 97);
	gfx.PutPixel(x + 32, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 32, y + 59, 6, 6, 6);
	gfx.PutPixel(x + 32, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 32, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 33, y + 35, 46, 46, 46);
	gfx.PutPixel(x + 33, y + 36, 248, 248, 248);
	gfx.PutPixel(x + 33, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 38, 145, 145, 145);
	gfx.PutPixel(x + 33, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 33, y + 40, 106, 106, 106);
	gfx.PutPixel(x + 33, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 55, 229, 229, 229);
	gfx.PutPixel(x + 33, y + 56, 64, 64, 64);
	gfx.PutPixel(x + 33, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 33, y + 58, 7, 7, 7);
	gfx.PutPixel(x + 33, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 33, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 34, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 34, y + 35, 112, 112, 112);
	gfx.PutPixel(x + 34, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 37, 210, 210, 210);
	gfx.PutPixel(x + 34, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 34, y + 39, 75, 75, 75);
	gfx.PutPixel(x + 34, y + 40, 251, 251, 251);
	gfx.PutPixel(x + 34, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 54, 204, 204, 204);
	gfx.PutPixel(x + 34, y + 55, 38, 38, 38);
	gfx.PutPixel(x + 34, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 34, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 34, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 35, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 35, y + 35, 159, 159, 159);
	gfx.PutPixel(x + 35, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 37, 217, 217, 217);
	gfx.PutPixel(x + 35, y + 38, 79, 79, 79);
	gfx.PutPixel(x + 35, y + 39, 233, 233, 233);
	gfx.PutPixel(x + 35, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 53, 171, 171, 171);
	gfx.PutPixel(x + 35, y + 54, 17, 17, 17);
	gfx.PutPixel(x + 35, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 35, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 35, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 33, 7, 7, 7);
	gfx.PutPixel(x + 36, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 36, y + 35, 182, 182, 182);
	gfx.PutPixel(x + 36, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 38, 250, 250, 250);
	gfx.PutPixel(x + 36, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 52, 132, 132, 132);
	gfx.PutPixel(x + 36, y + 53, 2, 2, 2);
	gfx.PutPixel(x + 36, y + 54, 2, 2, 2);
	gfx.PutPixel(x + 36, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 36, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 33, 7, 7, 7);
	gfx.PutPixel(x + 37, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 37, y + 35, 180, 180, 180);
	gfx.PutPixel(x + 37, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 50, 241, 241, 241);
	gfx.PutPixel(x + 37, y + 51, 91, 91, 91);
	gfx.PutPixel(x + 37, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 37, y + 53, 5, 5, 5);
	gfx.PutPixel(x + 37, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 37, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 38, y + 35, 156, 156, 156);
	gfx.PutPixel(x + 38, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 49, 213, 213, 213);
	gfx.PutPixel(x + 38, y + 50, 51, 51, 51);
	gfx.PutPixel(x + 38, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 38, y + 52, 7, 7, 7);
	gfx.PutPixel(x + 38, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 38, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 39, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 39, y + 35, 104, 104, 104);
	gfx.PutPixel(x + 39, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 48, 171, 171, 171);
	gfx.PutPixel(x + 39, y + 49, 20, 20, 20);
	gfx.PutPixel(x + 39, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 39, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 39, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 40, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 34, 2, 2, 2);
	gfx.PutPixel(x + 40, y + 35, 33, 33, 33);
	gfx.PutPixel(x + 40, y + 36, 237, 237, 237);
	gfx.PutPixel(x + 40, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 46, 248, 248, 248);
	gfx.PutPixel(x + 40, y + 47, 117, 117, 117);
	gfx.PutPixel(x + 40, y + 48, 2, 2, 2);
	gfx.PutPixel(x + 40, y + 49, 2, 2, 2);
	gfx.PutPixel(x + 40, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 40, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 41, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 41, y + 36, 130, 130, 130);
	gfx.PutPixel(x + 41, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 45, 214, 214, 214);
	gfx.PutPixel(x + 41, y + 46, 61, 61, 61);
	gfx.PutPixel(x + 41, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 41, y + 48, 6, 6, 6);
	gfx.PutPixel(x + 41, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 41, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 42, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 35, 5, 5, 5);
	gfx.PutPixel(x + 42, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 37, 176, 176, 176);
	gfx.PutPixel(x + 42, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 42, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 42, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 42, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 42, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 42, y + 43, 253, 253, 253);
	gfx.PutPixel(x + 42, y + 44, 150, 150, 150);
	gfx.PutPixel(x + 42, y + 45, 17, 17, 17);
	gfx.PutPixel(x + 42, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 42, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 42, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 36, 3, 3, 3);
	gfx.PutPixel(x + 43, y + 37, 10, 10, 10);
	gfx.PutPixel(x + 43, y + 38, 153, 153, 153);
	gfx.PutPixel(x + 43, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 43, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 43, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 43, y + 42, 204, 204, 204);
	gfx.PutPixel(x + 43, y + 43, 69, 69, 69);
	gfx.PutPixel(x + 43, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 43, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 43, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 63, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 64, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 65, 6, 6, 6);
	gfx.PutPixel(x + 43, y + 66, 6, 6, 6);
	gfx.PutPixel(x + 43, y + 67, 6, 6, 6);
	gfx.PutPixel(x + 43, y + 68, 6, 6, 6);
	gfx.PutPixel(x + 43, y + 69, 6, 6, 6);
	gfx.PutPixel(x + 43, y + 70, 7, 7, 7);
	gfx.PutPixel(x + 43, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 43, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 37, 5, 5, 5);
	gfx.PutPixel(x + 44, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 39, 75, 75, 75);
	gfx.PutPixel(x + 44, y + 40, 159, 159, 159);
	gfx.PutPixel(x + 44, y + 41, 90, 90, 90);
	gfx.PutPixel(x + 44, y + 42, 9, 9, 9);
	gfx.PutPixel(x + 44, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 44, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 58, 4, 4, 4);
	gfx.PutPixel(x + 44, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 64, 3, 3, 3);
	gfx.PutPixel(x + 44, y + 65, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 69, 6, 6, 6);
	gfx.PutPixel(x + 44, y + 70, 1, 1, 1);
	gfx.PutPixel(x + 44, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 44, y + 75, 4, 4, 4);
	gfx.PutPixel(x + 44, y + 76, 7, 7, 7);
	gfx.PutPixel(x + 44, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 44, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 39, 1, 1, 1);
	gfx.PutPixel(x + 45, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 42, 6, 6, 6);
	gfx.PutPixel(x + 45, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 55, 2, 2, 2);
	gfx.PutPixel(x + 45, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 58, 21, 21, 21);
	gfx.PutPixel(x + 45, y + 59, 59, 59, 59);
	gfx.PutPixel(x + 45, y + 60, 101, 101, 101);
	gfx.PutPixel(x + 45, y + 61, 137, 137, 137);
	gfx.PutPixel(x + 45, y + 62, 168, 168, 168);
	gfx.PutPixel(x + 45, y + 63, 184, 184, 184);
	gfx.PutPixel(x + 45, y + 64, 196, 196, 196);
	gfx.PutPixel(x + 45, y + 65, 205, 205, 205);
	gfx.PutPixel(x + 45, y + 66, 206, 206, 206);
	gfx.PutPixel(x + 45, y + 67, 206, 206, 206);
	gfx.PutPixel(x + 45, y + 68, 206, 206, 206);
	gfx.PutPixel(x + 45, y + 69, 204, 204, 204);
	gfx.PutPixel(x + 45, y + 70, 192, 192, 192);
	gfx.PutPixel(x + 45, y + 71, 176, 176, 176);
	gfx.PutPixel(x + 45, y + 72, 146, 146, 146);
	gfx.PutPixel(x + 45, y + 73, 107, 107, 107);
	gfx.PutPixel(x + 45, y + 74, 63, 63, 63);
	gfx.PutPixel(x + 45, y + 75, 23, 23, 23);
	gfx.PutPixel(x + 45, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 45, y + 78, 1, 1, 1);
	gfx.PutPixel(x + 45, y + 79, 7, 7, 7);
	gfx.PutPixel(x + 45, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 45, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 52, 7, 7, 7);
	gfx.PutPixel(x + 46, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 46, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 46, y + 55, 33, 33, 33);
	gfx.PutPixel(x + 46, y + 56, 105, 105, 105);
	gfx.PutPixel(x + 46, y + 57, 178, 178, 178);
	gfx.PutPixel(x + 46, y + 58, 230, 230, 230);
	gfx.PutPixel(x + 46, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 46, y + 75, 232, 232, 232);
	gfx.PutPixel(x + 46, y + 76, 181, 181, 181);
	gfx.PutPixel(x + 46, y + 77, 112, 112, 112);
	gfx.PutPixel(x + 46, y + 78, 42, 42, 42);
	gfx.PutPixel(x + 46, y + 79, 3, 3, 3);
	gfx.PutPixel(x + 46, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 46, y + 81, 3, 3, 3);
	gfx.PutPixel(x + 46, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 46, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 50, 7, 7, 7);
	gfx.PutPixel(x + 47, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 47, y + 52, 2, 2, 2);
	gfx.PutPixel(x + 47, y + 53, 66, 66, 66);
	gfx.PutPixel(x + 47, y + 54, 169, 169, 169);
	gfx.PutPixel(x + 47, y + 55, 242, 242, 242);
	gfx.PutPixel(x + 47, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 47, y + 78, 248, 248, 248);
	gfx.PutPixel(x + 47, y + 79, 191, 191, 191);
	gfx.PutPixel(x + 47, y + 80, 103, 103, 103);
	gfx.PutPixel(x + 47, y + 81, 24, 24, 24);
	gfx.PutPixel(x + 47, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 47, y + 83, 3, 3, 3);
	gfx.PutPixel(x + 47, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 47, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 49, 3, 3, 3);
	gfx.PutPixel(x + 48, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 48, y + 51, 62, 62, 62);
	gfx.PutPixel(x + 48, y + 52, 187, 187, 187);
	gfx.PutPixel(x + 48, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 48, y + 81, 234, 234, 234);
	gfx.PutPixel(x + 48, y + 82, 144, 144, 144);
	gfx.PutPixel(x + 48, y + 83, 27, 27, 27);
	gfx.PutPixel(x + 48, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 48, y + 85, 7, 7, 7);
	gfx.PutPixel(x + 48, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 48, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 49, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 49, y + 49, 25, 25, 25);
	gfx.PutPixel(x + 49, y + 50, 157, 157, 157);
	gfx.PutPixel(x + 49, y + 51, 253, 253, 253);
	gfx.PutPixel(x + 49, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 49, y + 83, 228, 228, 228);
	gfx.PutPixel(x + 49, y + 84, 88, 88, 88);
	gfx.PutPixel(x + 49, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 49, y + 86, 3, 3, 3);
	gfx.PutPixel(x + 49, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 49, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 50, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 46, 6, 6, 6);
	gfx.PutPixel(x + 50, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 50, y + 48, 76, 76, 76);
	gfx.PutPixel(x + 50, y + 49, 226, 226, 226);
	gfx.PutPixel(x + 50, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 85, 159, 159, 159);
	gfx.PutPixel(x + 50, y + 86, 19, 19, 19);
	gfx.PutPixel(x + 50, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 50, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 50, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 51, y + 46, 1, 1, 1);
	gfx.PutPixel(x + 51, y + 47, 126, 126, 126);
	gfx.PutPixel(x + 51, y + 48, 253, 253, 253);
	gfx.PutPixel(x + 51, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 86, 213, 213, 213);
	gfx.PutPixel(x + 51, y + 87, 54, 54, 54);
	gfx.PutPixel(x + 51, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 51, y + 89, 7, 7, 7);
	gfx.PutPixel(x + 51, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 51, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 44, 3, 3, 3);
	gfx.PutPixel(x + 52, y + 45, 7, 7, 7);
	gfx.PutPixel(x + 52, y + 46, 160, 160, 160);
	gfx.PutPixel(x + 52, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 87, 243, 243, 243);
	gfx.PutPixel(x + 52, y + 88, 94, 94, 94);
	gfx.PutPixel(x + 52, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 52, y + 90, 6, 6, 6);
	gfx.PutPixel(x + 52, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 52, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 53, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 43, 3, 3, 3);
	gfx.PutPixel(x + 53, y + 44, 9, 9, 9);
	gfx.PutPixel(x + 53, y + 45, 173, 173, 173);
	gfx.PutPixel(x + 53, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 89, 125, 125, 125);
	gfx.PutPixel(x + 53, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 53, y + 91, 5, 5, 5);
	gfx.PutPixel(x + 53, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 53, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 54, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 42, 4, 4, 4);
	gfx.PutPixel(x + 54, y + 43, 6, 6, 6);
	gfx.PutPixel(x + 54, y + 44, 171, 171, 171);
	gfx.PutPixel(x + 54, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 90, 145, 145, 145);
	gfx.PutPixel(x + 54, y + 91, 1, 1, 1);
	gfx.PutPixel(x + 54, y + 92, 4, 4, 4);
	gfx.PutPixel(x + 54, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 54, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 55, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 41, 6, 6, 6);
	gfx.PutPixel(x + 55, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 55, y + 43, 154, 154, 154);
	gfx.PutPixel(x + 55, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 91, 148, 148, 148);
	gfx.PutPixel(x + 55, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 55, y + 93, 5, 5, 5);
	gfx.PutPixel(x + 55, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 55, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 56, y + 42, 120, 120, 120);
	gfx.PutPixel(x + 56, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 92, 135, 135, 135);
	gfx.PutPixel(x + 56, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 56, y + 94, 7, 7, 7);
	gfx.PutPixel(x + 56, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 56, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 57, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 57, y + 41, 75, 75, 75);
	gfx.PutPixel(x + 57, y + 42, 252, 252, 252);
	gfx.PutPixel(x + 57, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 93, 107, 107, 107);
	gfx.PutPixel(x + 57, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 57, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 57, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 58, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 39, 1, 1, 1);
	gfx.PutPixel(x + 58, y + 40, 29, 29, 29);
	gfx.PutPixel(x + 58, y + 41, 225, 225, 225);
	gfx.PutPixel(x + 58, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 49, 251, 251, 251);
	gfx.PutPixel(x + 58, y + 50, 253, 253, 253);
	gfx.PutPixel(x + 58, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 93, 248, 248, 248);
	gfx.PutPixel(x + 58, y + 94, 65, 65, 65);
	gfx.PutPixel(x + 58, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 58, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 58, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 59, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 59, y + 40, 170, 170, 170);
	gfx.PutPixel(x + 59, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 47, 248, 248, 248);
	gfx.PutPixel(x + 59, y + 48, 158, 158, 158);
	gfx.PutPixel(x + 59, y + 49, 76, 76, 76);
	gfx.PutPixel(x + 59, y + 50, 238, 238, 238);
	gfx.PutPixel(x + 59, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 94, 216, 216, 216);
	gfx.PutPixel(x + 59, y + 95, 21, 21, 21);
	gfx.PutPixel(x + 59, y + 96, 3, 3, 3);
	gfx.PutPixel(x + 59, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 59, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 60, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 60, y + 39, 93, 93, 93);
	gfx.PutPixel(x + 60, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 46, 177, 177, 177);
	gfx.PutPixel(x + 60, y + 47, 49, 49, 49);
	gfx.PutPixel(x + 60, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 60, y + 49, 105, 105, 105);
	gfx.PutPixel(x + 60, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 95, 150, 150, 150);
	gfx.PutPixel(x + 60, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 60, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 60, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 61, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 37, 2, 2, 2);
	gfx.PutPixel(x + 61, y + 38, 24, 24, 24);
	gfx.PutPixel(x + 61, y + 39, 224, 224, 224);
	gfx.PutPixel(x + 61, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 44, 236, 236, 236);
	gfx.PutPixel(x + 61, y + 45, 93, 93, 93);
	gfx.PutPixel(x + 61, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 61, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 61, y + 48, 70, 70, 70);
	gfx.PutPixel(x + 61, y + 49, 249, 249, 249);
	gfx.PutPixel(x + 61, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 95, 253, 253, 253);
	gfx.PutPixel(x + 61, y + 96, 64, 64, 64);
	gfx.PutPixel(x + 61, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 61, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 61, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 62, y + 38, 141, 141, 141);
	gfx.PutPixel(x + 62, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 43, 222, 222, 222);
	gfx.PutPixel(x + 62, y + 44, 49, 49, 49);
	gfx.PutPixel(x + 62, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 62, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 62, y + 47, 35, 35, 35);
	gfx.PutPixel(x + 62, y + 48, 227, 227, 227);
	gfx.PutPixel(x + 62, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 64, 236, 236, 236);
	gfx.PutPixel(x + 62, y + 65, 209, 209, 209);
	gfx.PutPixel(x + 62, y + 66, 185, 185, 185);
	gfx.PutPixel(x + 62, y + 67, 176, 176, 176);
	gfx.PutPixel(x + 62, y + 68, 175, 175, 175);
	gfx.PutPixel(x + 62, y + 69, 176, 176, 176);
	gfx.PutPixel(x + 62, y + 70, 205, 205, 205);
	gfx.PutPixel(x + 62, y + 71, 240, 240, 240);
	gfx.PutPixel(x + 62, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 96, 186, 186, 186);
	gfx.PutPixel(x + 62, y + 97, 3, 3, 3);
	gfx.PutPixel(x + 62, y + 98, 7, 7, 7);
	gfx.PutPixel(x + 62, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 62, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 37, 46, 46, 46);
	gfx.PutPixel(x + 63, y + 38, 245, 245, 245);
	gfx.PutPixel(x + 63, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 42, 239, 239, 239);
	gfx.PutPixel(x + 63, y + 43, 51, 51, 51);
	gfx.PutPixel(x + 63, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 63, y + 46, 11, 11, 11);
	gfx.PutPixel(x + 63, y + 47, 195, 195, 195);
	gfx.PutPixel(x + 63, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 61, 215, 215, 215);
	gfx.PutPixel(x + 63, y + 62, 132, 132, 132);
	gfx.PutPixel(x + 63, y + 63, 65, 65, 65);
	gfx.PutPixel(x + 63, y + 64, 26, 26, 26);
	gfx.PutPixel(x + 63, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 70, 7, 7, 7);
	gfx.PutPixel(x + 63, y + 71, 30, 30, 30);
	gfx.PutPixel(x + 63, y + 72, 74, 74, 74);
	gfx.PutPixel(x + 63, y + 73, 137, 137, 137);
	gfx.PutPixel(x + 63, y + 74, 209, 209, 209);
	gfx.PutPixel(x + 63, y + 75, 253, 253, 253);
	gfx.PutPixel(x + 63, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 97, 71, 71, 71);
	gfx.PutPixel(x + 63, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 63, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 63, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 37, 164, 164, 164);
	gfx.PutPixel(x + 64, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 42, 121, 121, 121);
	gfx.PutPixel(x + 64, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 44, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 46, 154, 154, 154);
	gfx.PutPixel(x + 64, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 59, 220, 220, 220);
	gfx.PutPixel(x + 64, y + 60, 93, 93, 93);
	gfx.PutPixel(x + 64, y + 61, 10, 10, 10);
	gfx.PutPixel(x + 64, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 64, 3, 3, 3);
	gfx.PutPixel(x + 64, y + 65, 6, 6, 6);
	gfx.PutPixel(x + 64, y + 66, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 70, 6, 6, 6);
	gfx.PutPixel(x + 64, y + 71, 2, 2, 2);
	gfx.PutPixel(x + 64, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 74, 7, 7, 7);
	gfx.PutPixel(x + 64, y + 75, 61, 61, 61);
	gfx.PutPixel(x + 64, y + 76, 160, 160, 160);
	gfx.PutPixel(x + 64, y + 77, 245, 245, 245);
	gfx.PutPixel(x + 64, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 97, 170, 170, 170);
	gfx.PutPixel(x + 64, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 64, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 64, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 65, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 35, 1, 1, 1);
	gfx.PutPixel(x + 65, y + 36, 40, 40, 40);
	gfx.PutPixel(x + 65, y + 37, 245, 245, 245);
	gfx.PutPixel(x + 65, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 41, 244, 244, 244);
	gfx.PutPixel(x + 65, y + 42, 36, 36, 36);
	gfx.PutPixel(x + 65, y + 43, 1, 1, 1);
	gfx.PutPixel(x + 65, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 65, y + 45, 110, 110, 110);
	gfx.PutPixel(x + 65, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 58, 163, 163, 163);
	gfx.PutPixel(x + 65, y + 59, 18, 18, 18);
	gfx.PutPixel(x + 65, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 65, y + 61, 5, 5, 5);
	gfx.PutPixel(x + 65, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 74, 6, 6, 6);
	gfx.PutPixel(x + 65, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 65, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 65, y + 77, 43, 43, 43);
	gfx.PutPixel(x + 65, y + 78, 173, 173, 173);
	gfx.PutPixel(x + 65, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 97, 242, 242, 242);
	gfx.PutPixel(x + 65, y + 98, 36, 36, 36);
	gfx.PutPixel(x + 65, y + 99, 2, 2, 2);
	gfx.PutPixel(x + 65, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 65, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 66, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 66, y + 36, 95, 95, 95);
	gfx.PutPixel(x + 66, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 41, 217, 217, 217);
	gfx.PutPixel(x + 66, y + 42, 11, 11, 11);
	gfx.PutPixel(x + 66, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 66, y + 44, 68, 68, 68);
	gfx.PutPixel(x + 66, y + 45, 248, 248, 248);
	gfx.PutPixel(x + 66, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 57, 140, 140, 140);
	gfx.PutPixel(x + 66, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 66, y + 59, 3, 3, 3);
	gfx.PutPixel(x + 66, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 77, 1, 1, 1);
	gfx.PutPixel(x + 66, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 66, y + 79, 99, 99, 99);
	gfx.PutPixel(x + 66, y + 80, 244, 244, 244);
	gfx.PutPixel(x + 66, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 98, 104, 104, 104);
	gfx.PutPixel(x + 66, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 66, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 66, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 67, y + 36, 155, 155, 155);
	gfx.PutPixel(x + 67, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 41, 212, 212, 212);
	gfx.PutPixel(x + 67, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 67, y + 43, 32, 32, 32);
	gfx.PutPixel(x + 67, y + 44, 226, 226, 226);
	gfx.PutPixel(x + 67, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 56, 159, 159, 159);
	gfx.PutPixel(x + 67, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 67, y + 58, 5, 5, 5);
	gfx.PutPixel(x + 67, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 78, 7, 7, 7);
	gfx.PutPixel(x + 67, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 67, y + 80, 68, 68, 68);
	gfx.PutPixel(x + 67, y + 81, 240, 240, 240);
	gfx.PutPixel(x + 67, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 98, 172, 172, 172);
	gfx.PutPixel(x + 67, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 67, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 67, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 34, 6, 6, 6);
	gfx.PutPixel(x + 68, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 68, y + 36, 206, 206, 206);
	gfx.PutPixel(x + 68, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 41, 207, 207, 207);
	gfx.PutPixel(x + 68, y + 42, 9, 9, 9);
	gfx.PutPixel(x + 68, y + 43, 192, 192, 192);
	gfx.PutPixel(x + 68, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 55, 212, 212, 212);
	gfx.PutPixel(x + 68, y + 56, 16, 16, 16);
	gfx.PutPixel(x + 68, y + 57, 2, 2, 2);
	gfx.PutPixel(x + 68, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 68, y + 81, 84, 84, 84);
	gfx.PutPixel(x + 68, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 98, 221, 221, 221);
	gfx.PutPixel(x + 68, y + 99, 15, 15, 15);
	gfx.PutPixel(x + 68, y + 100, 5, 5, 5);
	gfx.PutPixel(x + 68, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 68, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 34, 2, 2, 2);
	gfx.PutPixel(x + 69, y + 35, 33, 33, 33);
	gfx.PutPixel(x + 69, y + 36, 241, 241, 241);
	gfx.PutPixel(x + 69, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 41, 237, 237, 237);
	gfx.PutPixel(x + 69, y + 42, 199, 199, 199);
	gfx.PutPixel(x + 69, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 55, 79, 79, 79);
	gfx.PutPixel(x + 69, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 69, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 80, 7, 7, 7);
	gfx.PutPixel(x + 69, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 69, y + 82, 149, 149, 149);
	gfx.PutPixel(x + 69, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 98, 248, 248, 248);
	gfx.PutPixel(x + 69, y + 99, 42, 42, 42);
	gfx.PutPixel(x + 69, y + 100, 1, 1, 1);
	gfx.PutPixel(x + 69, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 69, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 70, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 70, y + 35, 68, 68, 68);
	gfx.PutPixel(x + 70, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 54, 191, 191, 191);
	gfx.PutPixel(x + 70, y + 55, 2, 2, 2);
	gfx.PutPixel(x + 70, y + 56, 6, 6, 6);
	gfx.PutPixel(x + 70, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 81, 1, 1, 1);
	gfx.PutPixel(x + 70, y + 82, 32, 32, 32);
	gfx.PutPixel(x + 70, y + 83, 237, 237, 237);
	gfx.PutPixel(x + 70, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 70, y + 99, 72, 72, 72);
	gfx.PutPixel(x + 70, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 70, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 70, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 71, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 71, y + 35, 100, 100, 100);
	gfx.PutPixel(x + 71, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 54, 89, 89, 89);
	gfx.PutPixel(x + 71, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 71, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 71, y + 83, 157, 157, 157);
	gfx.PutPixel(x + 71, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 71, y + 99, 93, 93, 93);
	gfx.PutPixel(x + 71, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 71, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 71, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 72, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 72, y + 35, 126, 126, 126);
	gfx.PutPixel(x + 72, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 53, 229, 229, 229);
	gfx.PutPixel(x + 72, y + 54, 21, 21, 21);
	gfx.PutPixel(x + 72, y + 55, 4, 4, 4);
	gfx.PutPixel(x + 72, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 72, y + 83, 82, 82, 82);
	gfx.PutPixel(x + 72, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 72, y + 99, 101, 101, 101);
	gfx.PutPixel(x + 72, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 72, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 72, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 73, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 73, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 73, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 53, 167, 167, 167);
	gfx.PutPixel(x + 73, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 73, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 82, 1, 1, 1);
	gfx.PutPixel(x + 73, y + 83, 40, 40, 40);
	gfx.PutPixel(x + 73, y + 84, 247, 247, 247);
	gfx.PutPixel(x + 73, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 73, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 73, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 73, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 73, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 74, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 74, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 74, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 53, 107, 107, 107);
	gfx.PutPixel(x + 74, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 74, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 82, 4, 4, 4);
	gfx.PutPixel(x + 74, y + 83, 20, 20, 20);
	gfx.PutPixel(x + 74, y + 84, 227, 227, 227);
	gfx.PutPixel(x + 74, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 74, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 74, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 74, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 75, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 75, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 75, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 53, 95, 95, 95);
	gfx.PutPixel(x + 75, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 75, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 82, 6, 6, 6);
	gfx.PutPixel(x + 75, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 84, 213, 213, 213);
	gfx.PutPixel(x + 75, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 75, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 75, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 75, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 76, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 76, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 76, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 53, 95, 95, 95);
	gfx.PutPixel(x + 76, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 76, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 82, 6, 6, 6);
	gfx.PutPixel(x + 76, y + 83, 9, 9, 9);
	gfx.PutPixel(x + 76, y + 84, 214, 214, 214);
	gfx.PutPixel(x + 76, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 76, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 76, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 76, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 77, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 77, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 77, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 53, 108, 108, 108);
	gfx.PutPixel(x + 77, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 77, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 82, 4, 4, 4);
	gfx.PutPixel(x + 77, y + 83, 23, 23, 23);
	gfx.PutPixel(x + 77, y + 84, 231, 231, 231);
	gfx.PutPixel(x + 77, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 77, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 77, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 77, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 78, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 78, y + 35, 135, 135, 135);
	gfx.PutPixel(x + 78, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 53, 145, 145, 145);
	gfx.PutPixel(x + 78, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 78, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 78, y + 83, 53, 53, 53);
	gfx.PutPixel(x + 78, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 99, 99, 99, 99);
	gfx.PutPixel(x + 78, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 78, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 78, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 79, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 79, y + 35, 117, 117, 117);
	gfx.PutPixel(x + 79, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 53, 206, 206, 206);
	gfx.PutPixel(x + 79, y + 54, 6, 6, 6);
	gfx.PutPixel(x + 79, y + 55, 6, 6, 6);
	gfx.PutPixel(x + 79, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 79, y + 83, 123, 123, 123);
	gfx.PutPixel(x + 79, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 99, 86, 86, 86);
	gfx.PutPixel(x + 79, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 79, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 79, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 80, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 80, y + 35, 93, 93, 93);
	gfx.PutPixel(x + 80, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 54, 83, 83, 83);
	gfx.PutPixel(x + 80, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 80, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 81, 2, 2, 2);
	gfx.PutPixel(x + 80, y + 82, 20, 20, 20);
	gfx.PutPixel(x + 80, y + 83, 221, 221, 221);
	gfx.PutPixel(x + 80, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 99, 57, 57, 57);
	gfx.PutPixel(x + 80, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 80, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 80, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 81, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 81, y + 35, 58, 58, 58);
	gfx.PutPixel(x + 81, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 54, 223, 223, 223);
	gfx.PutPixel(x + 81, y + 55, 32, 32, 32);
	gfx.PutPixel(x + 81, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 81, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 80, 4, 4, 4);
	gfx.PutPixel(x + 81, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 81, y + 82, 155, 155, 155);
	gfx.PutPixel(x + 81, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 98, 233, 233, 233);
	gfx.PutPixel(x + 81, y + 99, 25, 25, 25);
	gfx.PutPixel(x + 81, y + 100, 3, 3, 3);
	gfx.PutPixel(x + 81, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 81, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 34, 4, 4, 4);
	gfx.PutPixel(x + 82, y + 35, 21, 21, 21);
	gfx.PutPixel(x + 82, y + 36, 228, 228, 228);
	gfx.PutPixel(x + 82, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 55, 211, 211, 211);
	gfx.PutPixel(x + 82, y + 56, 46, 46, 46);
	gfx.PutPixel(x + 82, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 82, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 82, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 78, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 82, y + 80, 5, 5, 5);
	gfx.PutPixel(x + 82, y + 81, 144, 144, 144);
	gfx.PutPixel(x + 82, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 98, 189, 189, 189);
	gfx.PutPixel(x + 82, y + 99, 1, 1, 1);
	gfx.PutPixel(x + 82, y + 100, 7, 7, 7);
	gfx.PutPixel(x + 82, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 82, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 83, y + 36, 171, 171, 171);
	gfx.PutPixel(x + 83, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 56, 240, 240, 240);
	gfx.PutPixel(x + 83, y + 57, 126, 126, 126);
	gfx.PutPixel(x + 83, y + 58, 24, 24, 24);
	gfx.PutPixel(x + 83, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 83, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 83, y + 61, 2, 2, 2);
	gfx.PutPixel(x + 83, y + 62, 6, 6, 6);
	gfx.PutPixel(x + 83, y + 63, 7, 7, 7);
	gfx.PutPixel(x + 83, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 75, 5, 5, 5);
	gfx.PutPixel(x + 83, y + 76, 1, 1, 1);
	gfx.PutPixel(x + 83, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 83, y + 78, 1, 1, 1);
	gfx.PutPixel(x + 83, y + 79, 60, 60, 60);
	gfx.PutPixel(x + 83, y + 80, 194, 194, 194);
	gfx.PutPixel(x + 83, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 98, 119, 119, 119);
	gfx.PutPixel(x + 83, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 83, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 83, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 84, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 36, 89, 89, 89);
	gfx.PutPixel(x + 84, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 58, 234, 234, 234);
	gfx.PutPixel(x + 84, y + 59, 158, 158, 158);
	gfx.PutPixel(x + 84, y + 60, 84, 84, 84);
	gfx.PutPixel(x + 84, y + 61, 35, 35, 35);
	gfx.PutPixel(x + 84, y + 62, 10, 10, 10);
	gfx.PutPixel(x + 84, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 84, y + 75, 10, 10, 10);
	gfx.PutPixel(x + 84, y + 76, 41, 41, 41);
	gfx.PutPixel(x + 84, y + 77, 101, 101, 101);
	gfx.PutPixel(x + 84, y + 78, 187, 187, 187);
	gfx.PutPixel(x + 84, y + 79, 252, 252, 252);
	gfx.PutPixel(x + 84, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 97, 246, 246, 246);
	gfx.PutPixel(x + 84, y + 98, 43, 43, 43);
	gfx.PutPixel(x + 84, y + 99, 1, 1, 1);
	gfx.PutPixel(x + 84, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 84, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 35, 5, 5, 5);
	gfx.PutPixel(x + 85, y + 36, 16, 16, 16);
	gfx.PutPixel(x + 85, y + 37, 218, 218, 218);
	gfx.PutPixel(x + 85, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 61, 244, 244, 244);
	gfx.PutPixel(x + 85, y + 62, 213, 213, 213);
	gfx.PutPixel(x + 85, y + 63, 181, 181, 181);
	gfx.PutPixel(x + 85, y + 64, 154, 154, 154);
	gfx.PutPixel(x + 85, y + 65, 127, 127, 127);
	gfx.PutPixel(x + 85, y + 66, 115, 115, 115);
	gfx.PutPixel(x + 85, y + 67, 108, 108, 108);
	gfx.PutPixel(x + 85, y + 68, 104, 104, 104);
	gfx.PutPixel(x + 85, y + 69, 103, 103, 103);
	gfx.PutPixel(x + 85, y + 70, 106, 106, 106);
	gfx.PutPixel(x + 85, y + 71, 112, 112, 112);
	gfx.PutPixel(x + 85, y + 72, 123, 123, 123);
	gfx.PutPixel(x + 85, y + 73, 148, 148, 148);
	gfx.PutPixel(x + 85, y + 74, 179, 179, 179);
	gfx.PutPixel(x + 85, y + 75, 214, 214, 214);
	gfx.PutPixel(x + 85, y + 76, 247, 247, 247);
	gfx.PutPixel(x + 85, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 97, 173, 173, 173);
	gfx.PutPixel(x + 85, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 85, y + 99, 7, 7, 7);
	gfx.PutPixel(x + 85, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 85, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 86, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 86, y + 37, 111, 111, 111);
	gfx.PutPixel(x + 86, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 97, 67, 67, 67);
	gfx.PutPixel(x + 86, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 86, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 86, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 36, 5, 5, 5);
	gfx.PutPixel(x + 87, y + 37, 12, 12, 12);
	gfx.PutPixel(x + 87, y + 38, 204, 204, 204);
	gfx.PutPixel(x + 87, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 96, 173, 173, 173);
	gfx.PutPixel(x + 87, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 87, y + 98, 7, 7, 7);
	gfx.PutPixel(x + 87, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 87, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 88, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 88, y + 38, 64, 64, 64);
	gfx.PutPixel(x + 88, y + 39, 249, 249, 249);
	gfx.PutPixel(x + 88, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 95, 234, 234, 234);
	gfx.PutPixel(x + 88, y + 96, 40, 40, 40);
	gfx.PutPixel(x + 88, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 88, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 88, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 89, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 89, y + 39, 116, 116, 116);
	gfx.PutPixel(x + 89, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 95, 90, 90, 90);
	gfx.PutPixel(x + 89, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 89, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 89, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 38, 6, 6, 6);
	gfx.PutPixel(x + 90, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 90, y + 40, 149, 149, 149);
	gfx.PutPixel(x + 90, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 94, 133, 133, 133);
	gfx.PutPixel(x + 90, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 90, y + 96, 7, 7, 7);
	gfx.PutPixel(x + 90, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 90, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 91, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 39, 5, 5, 5);
	gfx.PutPixel(x + 91, y + 40, 3, 3, 3);
	gfx.PutPixel(x + 91, y + 41, 156, 156, 156);
	gfx.PutPixel(x + 91, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 93, 160, 160, 160);
	gfx.PutPixel(x + 91, y + 94, 2, 2, 2);
	gfx.PutPixel(x + 91, y + 95, 5, 5, 5);
	gfx.PutPixel(x + 91, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 91, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 92, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 40, 4, 4, 4);
	gfx.PutPixel(x + 92, y + 41, 2, 2, 2);
	gfx.PutPixel(x + 92, y + 42, 138, 138, 138);
	gfx.PutPixel(x + 92, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 92, 168, 168, 168);
	gfx.PutPixel(x + 92, y + 93, 6, 6, 6);
	gfx.PutPixel(x + 92, y + 94, 4, 4, 4);
	gfx.PutPixel(x + 92, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 92, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 93, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 41, 5, 5, 5);
	gfx.PutPixel(x + 93, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 93, y + 43, 99, 99, 99);
	gfx.PutPixel(x + 93, y + 44, 243, 243, 243);
	gfx.PutPixel(x + 93, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 91, 157, 157, 157);
	gfx.PutPixel(x + 93, y + 92, 6, 6, 6);
	gfx.PutPixel(x + 93, y + 93, 3, 3, 3);
	gfx.PutPixel(x + 93, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 93, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 42, 7, 7, 7);
	gfx.PutPixel(x + 94, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 94, y + 44, 49, 49, 49);
	gfx.PutPixel(x + 94, y + 45, 200, 200, 200);
	gfx.PutPixel(x + 94, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 89, 253, 253, 253);
	gfx.PutPixel(x + 94, y + 90, 124, 124, 124);
	gfx.PutPixel(x + 94, y + 91, 1, 1, 1);
	gfx.PutPixel(x + 94, y + 92, 4, 4, 4);
	gfx.PutPixel(x + 94, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 94, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 95, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 95, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 46, 119, 119, 119);
	gfx.PutPixel(x + 95, y + 47, 240, 240, 240);
	gfx.PutPixel(x + 95, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 88, 223, 223, 223);
	gfx.PutPixel(x + 95, y + 89, 74, 74, 74);
	gfx.PutPixel(x + 95, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 95, y + 91, 6, 6, 6);
	gfx.PutPixel(x + 95, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 95, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 96, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 45, 5, 5, 5);
	gfx.PutPixel(x + 96, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 96, y + 47, 35, 35, 35);
	gfx.PutPixel(x + 96, y + 48, 157, 157, 157);
	gfx.PutPixel(x + 96, y + 49, 248, 248, 248);
	gfx.PutPixel(x + 96, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 86, 250, 250, 250);
	gfx.PutPixel(x + 96, y + 87, 151, 151, 151);
	gfx.PutPixel(x + 96, y + 88, 22, 22, 22);
	gfx.PutPixel(x + 96, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 96, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 96, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 97, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 47, 2, 2, 2);
	gfx.PutPixel(x + 97, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 97, y + 49, 48, 48, 48);
	gfx.PutPixel(x + 97, y + 50, 161, 161, 161);
	gfx.PutPixel(x + 97, y + 51, 246, 246, 246);
	gfx.PutPixel(x + 97, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 84, 251, 251, 251);
	gfx.PutPixel(x + 97, y + 85, 175, 175, 175);
	gfx.PutPixel(x + 97, y + 86, 54, 54, 54);
	gfx.PutPixel(x + 97, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 97, y + 88, 3, 3, 3);
	gfx.PutPixel(x + 97, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 97, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 48, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 49, 1, 1, 1);
	gfx.PutPixel(x + 98, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 98, y + 51, 41, 41, 41);
	gfx.PutPixel(x + 98, y + 52, 138, 138, 138);
	gfx.PutPixel(x + 98, y + 53, 227, 227, 227);
	gfx.PutPixel(x + 98, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 82, 231, 231, 231);
	gfx.PutPixel(x + 98, y + 83, 151, 151, 151);
	gfx.PutPixel(x + 98, y + 84, 52, 52, 52);
	gfx.PutPixel(x + 98, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 98, y + 86, 1, 1, 1);
	gfx.PutPixel(x + 98, y + 87, 7, 7, 7);
	gfx.PutPixel(x + 98, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 98, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 51, 2, 2, 2);
	gfx.PutPixel(x + 99, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 99, y + 53, 19, 19, 19);
	gfx.PutPixel(x + 99, y + 54, 73, 73, 73);
	gfx.PutPixel(x + 99, y + 55, 136, 136, 136);
	gfx.PutPixel(x + 99, y + 56, 193, 193, 193);
	gfx.PutPixel(x + 99, y + 57, 235, 235, 235);
	gfx.PutPixel(x + 99, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 78, 249, 249, 249);
	gfx.PutPixel(x + 99, y + 79, 214, 214, 214);
	gfx.PutPixel(x + 99, y + 80, 156, 156, 156);
	gfx.PutPixel(x + 99, y + 81, 84, 84, 84);
	gfx.PutPixel(x + 99, y + 82, 21, 21, 21);
	gfx.PutPixel(x + 99, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 99, y + 84, 1, 1, 1);
	gfx.PutPixel(x + 99, y + 85, 7, 7, 7);
	gfx.PutPixel(x + 99, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 99, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 100, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 53, 4, 4, 4);
	gfx.PutPixel(x + 100, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 100, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 100, y + 56, 3, 3, 3);
	gfx.PutPixel(x + 100, y + 57, 25, 25, 25);
	gfx.PutPixel(x + 100, y + 58, 60, 60, 60);
	gfx.PutPixel(x + 100, y + 59, 100, 100, 100);
	gfx.PutPixel(x + 100, y + 60, 137, 137, 137);
	gfx.PutPixel(x + 100, y + 61, 171, 171, 171);
	gfx.PutPixel(x + 100, y + 62, 191, 191, 191);
	gfx.PutPixel(x + 100, y + 63, 210, 210, 210);
	gfx.PutPixel(x + 100, y + 64, 219, 219, 219);
	gfx.PutPixel(x + 100, y + 65, 222, 222, 222);
	gfx.PutPixel(x + 100, y + 66, 223, 223, 223);
	gfx.PutPixel(x + 100, y + 67, 223, 223, 223);
	gfx.PutPixel(x + 100, y + 68, 223, 223, 223);
	gfx.PutPixel(x + 100, y + 69, 223, 223, 223);
	gfx.PutPixel(x + 100, y + 70, 223, 223, 223);
	gfx.PutPixel(x + 100, y + 71, 220, 220, 220);
	gfx.PutPixel(x + 100, y + 72, 214, 214, 214);
	gfx.PutPixel(x + 100, y + 73, 200, 200, 200);
	gfx.PutPixel(x + 100, y + 74, 182, 182, 182);
	gfx.PutPixel(x + 100, y + 75, 158, 158, 158);
	gfx.PutPixel(x + 100, y + 76, 123, 123, 123);
	gfx.PutPixel(x + 100, y + 77, 83, 83, 83);
	gfx.PutPixel(x + 100, y + 78, 43, 43, 43);
	gfx.PutPixel(x + 100, y + 79, 10, 10, 10);
	gfx.PutPixel(x + 100, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 100, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 100, y + 82, 4, 4, 4);
	gfx.PutPixel(x + 100, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 100, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 56, 7, 7, 7);
	gfx.PutPixel(x + 101, y + 57, 3, 3, 3);
	gfx.PutPixel(x + 101, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 62, 1, 1, 1);
	gfx.PutPixel(x + 101, y + 63, 9, 9, 9);
	gfx.PutPixel(x + 101, y + 64, 15, 15, 15);
	gfx.PutPixel(x + 101, y + 65, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 66, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 67, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 68, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 69, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 70, 17, 17, 17);
	gfx.PutPixel(x + 101, y + 71, 16, 16, 16);
	gfx.PutPixel(x + 101, y + 72, 12, 12, 12);
	gfx.PutPixel(x + 101, y + 73, 4, 4, 4);
	gfx.PutPixel(x + 101, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 101, y + 78, 1, 1, 1);
	gfx.PutPixel(x + 101, y + 79, 5, 5, 5);
	gfx.PutPixel(x + 101, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 101, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 62, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 63, 6, 6, 6);
	gfx.PutPixel(x + 102, y + 64, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 65, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 66, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 67, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 68, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 69, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 70, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 71, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 72, 5, 5, 5);
	gfx.PutPixel(x + 102, y + 73, 6, 6, 6);
	gfx.PutPixel(x + 102, y + 74, 7, 7, 7);
	gfx.PutPixel(x + 102, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 102, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 40, 6, 6, 6);
	gfx.PutPixel(x + 103, y + 41, 4, 4, 4);
	gfx.PutPixel(x + 103, y + 42, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 43, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 44, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 45, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 46, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 47, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 48, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 49, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 50, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 51, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 52, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 53, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 54, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 55, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 56, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 57, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 59, 3, 3, 3);
	gfx.PutPixel(x + 103, y + 60, 4, 4, 4);
	gfx.PutPixel(x + 103, y + 61, 4, 4, 4);
	gfx.PutPixel(x + 103, y + 62, 4, 4, 4);
	gfx.PutPixel(x + 103, y + 63, 4, 4, 4);
	gfx.PutPixel(x + 103, y + 64, 5, 5, 5);
	gfx.PutPixel(x + 103, y + 65, 5, 5, 5);
	gfx.PutPixel(x + 103, y + 66, 6, 6, 6);
	gfx.PutPixel(x + 103, y + 67, 6, 6, 6);
	gfx.PutPixel(x + 103, y + 68, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 69, 7, 7, 7);
	gfx.PutPixel(x + 103, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 103, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 37, 5, 5, 5);
	gfx.PutPixel(x + 104, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 41, 20, 20, 20);
	gfx.PutPixel(x + 104, y + 42, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 43, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 44, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 45, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 46, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 47, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 48, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 49, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 50, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 51, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 52, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 53, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 54, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 55, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 56, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 57, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 58, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 59, 24, 24, 24);
	gfx.PutPixel(x + 104, y + 60, 23, 23, 23);
	gfx.PutPixel(x + 104, y + 61, 22, 22, 22);
	gfx.PutPixel(x + 104, y + 62, 21, 21, 21);
	gfx.PutPixel(x + 104, y + 63, 19, 19, 19);
	gfx.PutPixel(x + 104, y + 64, 16, 16, 16);
	gfx.PutPixel(x + 104, y + 65, 14, 14, 14);
	gfx.PutPixel(x + 104, y + 66, 10, 10, 10);
	gfx.PutPixel(x + 104, y + 67, 4, 4, 4);
	gfx.PutPixel(x + 104, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 104, y + 77, 1, 1, 1);
	gfx.PutPixel(x + 104, y + 78, 4, 4, 4);
	gfx.PutPixel(x + 104, y + 79, 5, 5, 5);
	gfx.PutPixel(x + 104, y + 80, 7, 7, 7);
	gfx.PutPixel(x + 104, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 104, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 105, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 36, 2, 2, 2);
	gfx.PutPixel(x + 105, y + 37, 10, 10, 10);
	gfx.PutPixel(x + 105, y + 38, 87, 87, 87);
	gfx.PutPixel(x + 105, y + 39, 166, 166, 166);
	gfx.PutPixel(x + 105, y + 40, 215, 215, 215);
	gfx.PutPixel(x + 105, y + 41, 235, 235, 235);
	gfx.PutPixel(x + 105, y + 42, 235, 235, 235);
	gfx.PutPixel(x + 105, y + 43, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 44, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 45, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 46, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 47, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 48, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 49, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 50, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 51, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 52, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 53, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 54, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 55, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 56, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 57, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 58, 236, 236, 236);
	gfx.PutPixel(x + 105, y + 59, 235, 235, 235);
	gfx.PutPixel(x + 105, y + 60, 233, 233, 233);
	gfx.PutPixel(x + 105, y + 61, 231, 231, 231);
	gfx.PutPixel(x + 105, y + 62, 229, 229, 229);
	gfx.PutPixel(x + 105, y + 63, 226, 226, 226);
	gfx.PutPixel(x + 105, y + 64, 221, 221, 221);
	gfx.PutPixel(x + 105, y + 65, 217, 217, 217);
	gfx.PutPixel(x + 105, y + 66, 211, 211, 211);
	gfx.PutPixel(x + 105, y + 67, 200, 200, 200);
	gfx.PutPixel(x + 105, y + 68, 189, 189, 189);
	gfx.PutPixel(x + 105, y + 69, 181, 181, 181);
	gfx.PutPixel(x + 105, y + 70, 170, 170, 170);
	gfx.PutPixel(x + 105, y + 71, 156, 156, 156);
	gfx.PutPixel(x + 105, y + 72, 135, 135, 135);
	gfx.PutPixel(x + 105, y + 73, 119, 119, 119);
	gfx.PutPixel(x + 105, y + 74, 104, 104, 104);
	gfx.PutPixel(x + 105, y + 75, 78, 78, 78);
	gfx.PutPixel(x + 105, y + 76, 61, 61, 61);
	gfx.PutPixel(x + 105, y + 77, 43, 43, 43);
	gfx.PutPixel(x + 105, y + 78, 22, 22, 22);
	gfx.PutPixel(x + 105, y + 79, 11, 11, 11);
	gfx.PutPixel(x + 105, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 105, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 105, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 105, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 105, y + 84, 1, 1, 1);
	gfx.PutPixel(x + 105, y + 85, 5, 5, 5);
	gfx.PutPixel(x + 105, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 105, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 106, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 106, y + 36, 39, 39, 39);
	gfx.PutPixel(x + 106, y + 37, 202, 202, 202);
	gfx.PutPixel(x + 106, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 40, 233, 233, 233);
	gfx.PutPixel(x + 106, y + 41, 228, 228, 228);
	gfx.PutPixel(x + 106, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 77, 248, 248, 248);
	gfx.PutPixel(x + 106, y + 78, 232, 232, 232);
	gfx.PutPixel(x + 106, y + 79, 213, 213, 213);
	gfx.PutPixel(x + 106, y + 80, 185, 185, 185);
	gfx.PutPixel(x + 106, y + 81, 155, 155, 155);
	gfx.PutPixel(x + 106, y + 82, 117, 117, 117);
	gfx.PutPixel(x + 106, y + 83, 79, 79, 79);
	gfx.PutPixel(x + 106, y + 84, 43, 43, 43);
	gfx.PutPixel(x + 106, y + 85, 15, 15, 15);
	gfx.PutPixel(x + 106, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 106, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 106, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 106, y + 89, 4, 4, 4);
	gfx.PutPixel(x + 106, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 106, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 107, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 34, 5, 5, 5);
	gfx.PutPixel(x + 107, y + 35, 13, 13, 13);
	gfx.PutPixel(x + 107, y + 36, 204, 204, 204);
	gfx.PutPixel(x + 107, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 38, 164, 164, 164);
	gfx.PutPixel(x + 107, y + 39, 48, 48, 48);
	gfx.PutPixel(x + 107, y + 40, 5, 5, 5);
	gfx.PutPixel(x + 107, y + 41, 69, 69, 69);
	gfx.PutPixel(x + 107, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 84, 249, 249, 249);
	gfx.PutPixel(x + 107, y + 85, 222, 222, 222);
	gfx.PutPixel(x + 107, y + 86, 179, 179, 179);
	gfx.PutPixel(x + 107, y + 87, 125, 125, 125);
	gfx.PutPixel(x + 107, y + 88, 67, 67, 67);
	gfx.PutPixel(x + 107, y + 89, 20, 20, 20);
	gfx.PutPixel(x + 107, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 107, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 107, y + 92, 4, 4, 4);
	gfx.PutPixel(x + 107, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 107, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 108, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 108, y + 35, 84, 84, 84);
	gfx.PutPixel(x + 108, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 37, 177, 177, 177);
	gfx.PutPixel(x + 108, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 108, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 108, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 108, y + 41, 116, 116, 116);
	gfx.PutPixel(x + 108, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 89, 230, 230, 230);
	gfx.PutPixel(x + 108, y + 90, 169, 169, 169);
	gfx.PutPixel(x + 108, y + 91, 89, 89, 89);
	gfx.PutPixel(x + 108, y + 92, 21, 21, 21);
	gfx.PutPixel(x + 108, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 108, y + 94, 1, 1, 1);
	gfx.PutPixel(x + 108, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 108, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 109, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 109, y + 35, 129, 129, 129);
	gfx.PutPixel(x + 109, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 37, 118, 118, 118);
	gfx.PutPixel(x + 109, y + 38, 22, 22, 22);
	gfx.PutPixel(x + 109, y + 39, 49, 49, 49);
	gfx.PutPixel(x + 109, y + 40, 124, 124, 124);
	gfx.PutPixel(x + 109, y + 41, 245, 245, 245);
	gfx.PutPixel(x + 109, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 92, 229, 229, 229);
	gfx.PutPixel(x + 109, y + 93, 153, 153, 153);
	gfx.PutPixel(x + 109, y + 94, 67, 67, 67);
	gfx.PutPixel(x + 109, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 109, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 109, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 110, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 110, y + 35, 140, 140, 140);
	gfx.PutPixel(x + 110, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 37, 246, 246, 246);
	gfx.PutPixel(x + 110, y + 38, 244, 244, 244);
	gfx.PutPixel(x + 110, y + 39, 252, 252, 252);
	gfx.PutPixel(x + 110, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 94, 254, 254, 254);
	gfx.PutPixel(x + 110, y + 95, 89, 89, 89);
	gfx.PutPixel(x + 110, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 110, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 110, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 111, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 111, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 111, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 95, 226, 226, 226);
	gfx.PutPixel(x + 111, y + 96, 25, 25, 25);
	gfx.PutPixel(x + 111, y + 97, 3, 3, 3);
	gfx.PutPixel(x + 111, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 111, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 112, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 112, y + 35, 139, 139, 139);
	gfx.PutPixel(x + 112, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 96, 115, 115, 115);
	gfx.PutPixel(x + 112, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 112, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 112, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 113, y + 35, 114, 114, 114);
	gfx.PutPixel(x + 113, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 96, 203, 203, 203);
	gfx.PutPixel(x + 113, y + 97, 7, 7, 7);
	gfx.PutPixel(x + 113, y + 98, 6, 6, 6);
	gfx.PutPixel(x + 113, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 113, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 114, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 34, 1, 1, 1);
	gfx.PutPixel(x + 114, y + 35, 44, 44, 44);
	gfx.PutPixel(x + 114, y + 36, 242, 242, 242);
	gfx.PutPixel(x + 114, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 96, 252, 252, 252);
	gfx.PutPixel(x + 114, y + 97, 52, 52, 52);
	gfx.PutPixel(x + 114, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 114, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 114, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 115, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 115, y + 36, 103, 103, 103);
	gfx.PutPixel(x + 115, y + 37, 254, 254, 254);
	gfx.PutPixel(x + 115, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 97, 115, 115, 115);
	gfx.PutPixel(x + 115, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 115, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 115, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 116, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 116, y + 37, 66, 66, 66);
	gfx.PutPixel(x + 116, y + 38, 192, 192, 192);
	gfx.PutPixel(x + 116, y + 39, 252, 252, 252);
	gfx.PutPixel(x + 116, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 97, 175, 175, 175);
	gfx.PutPixel(x + 116, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 116, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 116, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 117, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 117, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 117, y + 39, 51, 51, 51);
	gfx.PutPixel(x + 117, y + 40, 114, 114, 114);
	gfx.PutPixel(x + 117, y + 41, 166, 166, 166);
	gfx.PutPixel(x + 117, y + 42, 200, 200, 200);
	gfx.PutPixel(x + 117, y + 43, 223, 223, 223);
	gfx.PutPixel(x + 117, y + 44, 234, 234, 234);
	gfx.PutPixel(x + 117, y + 45, 242, 242, 242);
	gfx.PutPixel(x + 117, y + 46, 246, 246, 246);
	gfx.PutPixel(x + 117, y + 47, 250, 250, 250);
	gfx.PutPixel(x + 117, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 97, 220, 220, 220);
	gfx.PutPixel(x + 117, y + 98, 15, 15, 15);
	gfx.PutPixel(x + 117, y + 99, 5, 5, 5);
	gfx.PutPixel(x + 117, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 117, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 118, y + 39, 1, 1, 1);
	gfx.PutPixel(x + 118, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 118, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 118, y + 42, 4, 4, 4);
	gfx.PutPixel(x + 118, y + 43, 17, 17, 17);
	gfx.PutPixel(x + 118, y + 44, 23, 23, 23);
	gfx.PutPixel(x + 118, y + 45, 32, 32, 32);
	gfx.PutPixel(x + 118, y + 46, 39, 39, 39);
	gfx.PutPixel(x + 118, y + 47, 47, 47, 47);
	gfx.PutPixel(x + 118, y + 48, 64, 64, 64);
	gfx.PutPixel(x + 118, y + 49, 90, 90, 90);
	gfx.PutPixel(x + 118, y + 50, 115, 115, 115);
	gfx.PutPixel(x + 118, y + 51, 140, 140, 140);
	gfx.PutPixel(x + 118, y + 52, 171, 171, 171);
	gfx.PutPixel(x + 118, y + 53, 195, 195, 195);
	gfx.PutPixel(x + 118, y + 54, 220, 220, 220);
	gfx.PutPixel(x + 118, y + 55, 239, 239, 239);
	gfx.PutPixel(x + 118, y + 56, 254, 254, 254);
	gfx.PutPixel(x + 118, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 97, 249, 249, 249);
	gfx.PutPixel(x + 118, y + 98, 44, 44, 44);
	gfx.PutPixel(x + 118, y + 99, 1, 1, 1);
	gfx.PutPixel(x + 118, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 118, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 119, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 42, 6, 6, 6);
	gfx.PutPixel(x + 119, y + 43, 5, 5, 5);
	gfx.PutPixel(x + 119, y + 44, 4, 4, 4);
	gfx.PutPixel(x + 119, y + 45, 2, 2, 2);
	gfx.PutPixel(x + 119, y + 46, 1, 1, 1);
	gfx.PutPixel(x + 119, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 53, 2, 2, 2);
	gfx.PutPixel(x + 119, y + 54, 15, 15, 15);
	gfx.PutPixel(x + 119, y + 55, 30, 30, 30);
	gfx.PutPixel(x + 119, y + 56, 54, 54, 54);
	gfx.PutPixel(x + 119, y + 57, 80, 80, 80);
	gfx.PutPixel(x + 119, y + 58, 114, 114, 114);
	gfx.PutPixel(x + 119, y + 59, 149, 149, 149);
	gfx.PutPixel(x + 119, y + 60, 183, 183, 183);
	gfx.PutPixel(x + 119, y + 61, 214, 214, 214);
	gfx.PutPixel(x + 119, y + 62, 241, 241, 241);
	gfx.PutPixel(x + 119, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 98, 82, 82, 82);
	gfx.PutPixel(x + 119, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 119, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 119, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 53, 7, 7, 7);
	gfx.PutPixel(x + 120, y + 54, 5, 5, 5);
	gfx.PutPixel(x + 120, y + 55, 3, 3, 3);
	gfx.PutPixel(x + 120, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 61, 11, 11, 11);
	gfx.PutPixel(x + 120, y + 62, 32, 32, 32);
	gfx.PutPixel(x + 120, y + 63, 61, 61, 61);
	gfx.PutPixel(x + 120, y + 64, 100, 100, 100);
	gfx.PutPixel(x + 120, y + 65, 144, 144, 144);
	gfx.PutPixel(x + 120, y + 66, 188, 188, 188);
	gfx.PutPixel(x + 120, y + 67, 227, 227, 227);
	gfx.PutPixel(x + 120, y + 68, 253, 253, 253);
	gfx.PutPixel(x + 120, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 98, 122, 122, 122);
	gfx.PutPixel(x + 120, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 120, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 120, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 60, 7, 7, 7);
	gfx.PutPixel(x + 121, y + 61, 5, 5, 5);
	gfx.PutPixel(x + 121, y + 62, 2, 2, 2);
	gfx.PutPixel(x + 121, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 121, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 121, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 121, y + 66, 1, 1, 1);
	gfx.PutPixel(x + 121, y + 67, 19, 19, 19);
	gfx.PutPixel(x + 121, y + 68, 53, 53, 53);
	gfx.PutPixel(x + 121, y + 69, 101, 101, 101);
	gfx.PutPixel(x + 121, y + 70, 160, 160, 160);
	gfx.PutPixel(x + 121, y + 71, 214, 214, 214);
	gfx.PutPixel(x + 121, y + 72, 252, 252, 252);
	gfx.PutPixel(x + 121, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 98, 161, 161, 161);
	gfx.PutPixel(x + 121, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 121, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 121, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 66, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 67, 4, 4, 4);
	gfx.PutPixel(x + 122, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 122, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 122, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 122, y + 71, 10, 10, 10);
	gfx.PutPixel(x + 122, y + 72, 51, 51, 51);
	gfx.PutPixel(x + 122, y + 73, 117, 117, 117);
	gfx.PutPixel(x + 122, y + 74, 195, 195, 195);
	gfx.PutPixel(x + 122, y + 75, 250, 250, 250);
	gfx.PutPixel(x + 122, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 98, 194, 194, 194);
	gfx.PutPixel(x + 122, y + 99, 2, 2, 2);
	gfx.PutPixel(x + 122, y + 100, 7, 7, 7);
	gfx.PutPixel(x + 122, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 122, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 123, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 71, 5, 5, 5);
	gfx.PutPixel(x + 123, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 123, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 123, y + 74, 3, 3, 3);
	gfx.PutPixel(x + 123, y + 75, 51, 51, 51);
	gfx.PutPixel(x + 123, y + 76, 151, 151, 151);
	gfx.PutPixel(x + 123, y + 77, 244, 244, 244);
	gfx.PutPixel(x + 123, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 98, 220, 220, 220);
	gfx.PutPixel(x + 123, y + 99, 14, 14, 14);
	gfx.PutPixel(x + 123, y + 100, 5, 5, 5);
	gfx.PutPixel(x + 123, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 123, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 74, 7, 7, 7);
	gfx.PutPixel(x + 124, y + 75, 1, 1, 1);
	gfx.PutPixel(x + 124, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 124, y + 77, 43, 43, 43);
	gfx.PutPixel(x + 124, y + 78, 194, 194, 194);
	gfx.PutPixel(x + 124, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 98, 240, 240, 240);
	gfx.PutPixel(x + 124, y + 99, 33, 33, 33);
	gfx.PutPixel(x + 124, y + 100, 2, 2, 2);
	gfx.PutPixel(x + 124, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 124, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 125, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 125, y + 78, 14, 14, 14);
	gfx.PutPixel(x + 125, y + 79, 206, 206, 206);
	gfx.PutPixel(x + 125, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 98, 254, 254, 254);
	gfx.PutPixel(x + 125, y + 99, 51, 51, 51);
	gfx.PutPixel(x + 125, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 125, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 125, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 126, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 126, y + 79, 122, 122, 122);
	gfx.PutPixel(x + 126, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 99, 78, 78, 78);
	gfx.PutPixel(x + 126, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 126, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 126, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 127, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 127, y + 79, 114, 114, 114);
	gfx.PutPixel(x + 127, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 99, 97, 97, 97);
	gfx.PutPixel(x + 127, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 127, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 127, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 128, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 128, y + 79, 119, 119, 119);
	gfx.PutPixel(x + 128, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 128, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 128, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 128, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 129, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 129, y + 79, 141, 141, 141);
	gfx.PutPixel(x + 129, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 129, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 129, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 129, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 130, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 77, 6, 6, 6);
	gfx.PutPixel(x + 130, y + 78, 3, 3, 3);
	gfx.PutPixel(x + 130, y + 79, 198, 198, 198);
	gfx.PutPixel(x + 130, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 130, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 130, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 130, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 131, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 131, y + 78, 70, 70, 70);
	gfx.PutPixel(x + 131, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 99, 83, 83, 83);
	gfx.PutPixel(x + 131, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 131, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 131, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 132, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 132, y + 77, 18, 18, 18);
	gfx.PutPixel(x + 132, y + 78, 208, 208, 208);
	gfx.PutPixel(x + 132, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 98, 252, 252, 252);
	gfx.PutPixel(x + 132, y + 99, 48, 48, 48);
	gfx.PutPixel(x + 132, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 132, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 132, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 133, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 133, y + 76, 18, 18, 18);
	gfx.PutPixel(x + 133, y + 77, 185, 185, 185);
	gfx.PutPixel(x + 133, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 98, 232, 232, 232);
	gfx.PutPixel(x + 133, y + 99, 24, 24, 24);
	gfx.PutPixel(x + 133, y + 100, 4, 4, 4);
	gfx.PutPixel(x + 133, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 133, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 134, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 73, 2, 2, 2);
	gfx.PutPixel(x + 134, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 134, y + 75, 56, 56, 56);
	gfx.PutPixel(x + 134, y + 76, 209, 209, 209);
	gfx.PutPixel(x + 134, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 98, 205, 205, 205);
	gfx.PutPixel(x + 134, y + 99, 6, 6, 6);
	gfx.PutPixel(x + 134, y + 100, 6, 6, 6);
	gfx.PutPixel(x + 134, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 134, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 135, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 70, 5, 5, 5);
	gfx.PutPixel(x + 135, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 135, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 135, y + 73, 34, 34, 34);
	gfx.PutPixel(x + 135, y + 74, 147, 147, 147);
	gfx.PutPixel(x + 135, y + 75, 248, 248, 248);
	gfx.PutPixel(x + 135, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 98, 172, 172, 172);
	gfx.PutPixel(x + 135, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 135, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 135, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 66, 7, 7, 7);
	gfx.PutPixel(x + 136, y + 67, 3, 3, 3);
	gfx.PutPixel(x + 136, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 136, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 136, y + 70, 9, 9, 9);
	gfx.PutPixel(x + 136, y + 71, 64, 64, 64);
	gfx.PutPixel(x + 136, y + 72, 157, 157, 157);
	gfx.PutPixel(x + 136, y + 73, 241, 241, 241);
	gfx.PutPixel(x + 136, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 98, 123, 123, 123);
	gfx.PutPixel(x + 136, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 136, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 136, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 60, 7, 7, 7);
	gfx.PutPixel(x + 137, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 137, y + 62, 3, 3, 3);
	gfx.PutPixel(x + 137, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 137, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 137, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 137, y + 66, 2, 2, 2);
	gfx.PutPixel(x + 137, y + 67, 25, 25, 25);
	gfx.PutPixel(x + 137, y + 68, 74, 74, 74);
	gfx.PutPixel(x + 137, y + 69, 142, 142, 142);
	gfx.PutPixel(x + 137, y + 70, 214, 214, 214);
	gfx.PutPixel(x + 137, y + 71, 254, 254, 254);
	gfx.PutPixel(x + 137, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 97, 236, 236, 236);
	gfx.PutPixel(x + 137, y + 98, 35, 35, 35);
	gfx.PutPixel(x + 137, y + 99, 2, 2, 2);
	gfx.PutPixel(x + 137, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 137, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 50, 7, 7, 7);
	gfx.PutPixel(x + 138, y + 51, 5, 5, 5);
	gfx.PutPixel(x + 138, y + 52, 3, 3, 3);
	gfx.PutPixel(x + 138, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 62, 25, 25, 25);
	gfx.PutPixel(x + 138, y + 63, 53, 53, 53);
	gfx.PutPixel(x + 138, y + 64, 93, 93, 93);
	gfx.PutPixel(x + 138, y + 65, 140, 140, 140);
	gfx.PutPixel(x + 138, y + 66, 192, 192, 192);
	gfx.PutPixel(x + 138, y + 67, 236, 236, 236);
	gfx.PutPixel(x + 138, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 97, 131, 131, 131);
	gfx.PutPixel(x + 138, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 138, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 138, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 43, 7, 7, 7);
	gfx.PutPixel(x + 139, y + 44, 4, 4, 4);
	gfx.PutPixel(x + 139, y + 45, 1, 1, 1);
	gfx.PutPixel(x + 139, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 139, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 139, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 139, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 139, y + 50, 2, 2, 2);
	gfx.PutPixel(x + 139, y + 51, 14, 14, 14);
	gfx.PutPixel(x + 139, y + 52, 24, 24, 24);
	gfx.PutPixel(x + 139, y + 53, 46, 46, 46);
	gfx.PutPixel(x + 139, y + 54, 62, 62, 62);
	gfx.PutPixel(x + 139, y + 55, 84, 84, 84);
	gfx.PutPixel(x + 139, y + 56, 109, 109, 109);
	gfx.PutPixel(x + 139, y + 57, 126, 126, 126);
	gfx.PutPixel(x + 139, y + 58, 152, 152, 152);
	gfx.PutPixel(x + 139, y + 59, 172, 172, 172);
	gfx.PutPixel(x + 139, y + 60, 187, 187, 187);
	gfx.PutPixel(x + 139, y + 61, 208, 208, 208);
	gfx.PutPixel(x + 139, y + 62, 234, 234, 234);
	gfx.PutPixel(x + 139, y + 63, 253, 253, 253);
	gfx.PutPixel(x + 139, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 96, 212, 212, 212);
	gfx.PutPixel(x + 139, y + 97, 17, 17, 17);
	gfx.PutPixel(x + 139, y + 98, 4, 4, 4);
	gfx.PutPixel(x + 139, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 139, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 140, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 39, 6, 6, 6);
	gfx.PutPixel(x + 140, y + 40, 1, 1, 1);
	gfx.PutPixel(x + 140, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 140, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 140, y + 43, 2, 2, 2);
	gfx.PutPixel(x + 140, y + 44, 17, 17, 17);
	gfx.PutPixel(x + 140, y + 45, 42, 42, 42);
	gfx.PutPixel(x + 140, y + 46, 70, 70, 70);
	gfx.PutPixel(x + 140, y + 47, 104, 104, 104);
	gfx.PutPixel(x + 140, y + 48, 135, 135, 135);
	gfx.PutPixel(x + 140, y + 49, 168, 168, 168);
	gfx.PutPixel(x + 140, y + 50, 193, 193, 193);
	gfx.PutPixel(x + 140, y + 51, 218, 218, 218);
	gfx.PutPixel(x + 140, y + 52, 234, 234, 234);
	gfx.PutPixel(x + 140, y + 53, 250, 250, 250);
	gfx.PutPixel(x + 140, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 95, 250, 250, 250);
	gfx.PutPixel(x + 140, y + 96, 66, 66, 66);
	gfx.PutPixel(x + 140, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 140, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 140, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 141, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 37, 2, 2, 2);
	gfx.PutPixel(x + 141, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 141, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 40, 46, 46, 46);
	gfx.PutPixel(x + 141, y + 41, 97, 97, 97);
	gfx.PutPixel(x + 141, y + 42, 147, 147, 147);
	gfx.PutPixel(x + 141, y + 43, 191, 191, 191);
	gfx.PutPixel(x + 141, y + 44, 225, 225, 225);
	gfx.PutPixel(x + 141, y + 45, 248, 248, 248);
	gfx.PutPixel(x + 141, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 95, 111, 111, 111);
	gfx.PutPixel(x + 141, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 141, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 141, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 142, y + 37, 41, 41, 41);
	gfx.PutPixel(x + 142, y + 38, 136, 136, 136);
	gfx.PutPixel(x + 142, y + 39, 218, 218, 218);
	gfx.PutPixel(x + 142, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 94, 134, 134, 134);
	gfx.PutPixel(x + 142, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 142, y + 96, 7, 7, 7);
	gfx.PutPixel(x + 142, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 142, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 143, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 143, y + 36, 87, 87, 87);
	gfx.PutPixel(x + 143, y + 37, 247, 247, 247);
	gfx.PutPixel(x + 143, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 39, 230, 230, 230);
	gfx.PutPixel(x + 143, y + 40, 138, 138, 138);
	gfx.PutPixel(x + 143, y + 41, 76, 76, 76);
	gfx.PutPixel(x + 143, y + 42, 127, 127, 127);
	gfx.PutPixel(x + 143, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 93, 129, 129, 129);
	gfx.PutPixel(x + 143, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 143, y + 95, 5, 5, 5);
	gfx.PutPixel(x + 143, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 143, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 144, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 144, y + 36, 159, 159, 159);
	gfx.PutPixel(x + 144, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 38, 208, 208, 208);
	gfx.PutPixel(x + 144, y + 39, 25, 25, 25);
	gfx.PutPixel(x + 144, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 144, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 144, y + 42, 98, 98, 98);
	gfx.PutPixel(x + 144, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 91, 244, 244, 244);
	gfx.PutPixel(x + 144, y + 92, 99, 99, 99);
	gfx.PutPixel(x + 144, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 144, y + 94, 6, 6, 6);
	gfx.PutPixel(x + 144, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 144, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 145, y + 36, 160, 160, 160);
	gfx.PutPixel(x + 145, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 38, 93, 93, 93);
	gfx.PutPixel(x + 145, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 145, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 145, y + 41, 49, 49, 49);
	gfx.PutPixel(x + 145, y + 42, 219, 219, 219);
	gfx.PutPixel(x + 145, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 90, 204, 204, 204);
	gfx.PutPixel(x + 145, y + 91, 52, 52, 52);
	gfx.PutPixel(x + 145, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 145, y + 93, 7, 7, 7);
	gfx.PutPixel(x + 145, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 145, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 146, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 146, y + 36, 150, 150, 150);
	gfx.PutPixel(x + 146, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 38, 192, 192, 192);
	gfx.PutPixel(x + 146, y + 39, 168, 168, 168);
	gfx.PutPixel(x + 146, y + 40, 190, 190, 190);
	gfx.PutPixel(x + 146, y + 41, 243, 243, 243);
	gfx.PutPixel(x + 146, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 88, 238, 238, 238);
	gfx.PutPixel(x + 146, y + 89, 120, 120, 120);
	gfx.PutPixel(x + 146, y + 90, 10, 10, 10);
	gfx.PutPixel(x + 146, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 146, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 146, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 147, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 147, y + 36, 95, 95, 95);
	gfx.PutPixel(x + 147, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 86, 240, 240, 240);
	gfx.PutPixel(x + 147, y + 87, 145, 145, 145);
	gfx.PutPixel(x + 147, y + 88, 32, 32, 32);
	gfx.PutPixel(x + 147, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 147, y + 90, 5, 5, 5);
	gfx.PutPixel(x + 147, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 147, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 148, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 35, 5, 5, 5);
	gfx.PutPixel(x + 148, y + 36, 14, 14, 14);
	gfx.PutPixel(x + 148, y + 37, 204, 204, 204);
	gfx.PutPixel(x + 148, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 84, 217, 217, 217);
	gfx.PutPixel(x + 148, y + 85, 125, 125, 125);
	gfx.PutPixel(x + 148, y + 86, 32, 32, 32);
	gfx.PutPixel(x + 148, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 148, y + 88, 2, 2, 2);
	gfx.PutPixel(x + 148, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 148, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 149, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 149, y + 37, 41, 41, 41);
	gfx.PutPixel(x + 149, y + 38, 215, 215, 215);
	gfx.PutPixel(x + 149, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 81, 223, 223, 223);
	gfx.PutPixel(x + 149, y + 82, 155, 155, 155);
	gfx.PutPixel(x + 149, y + 83, 73, 73, 73);
	gfx.PutPixel(x + 149, y + 84, 12, 12, 12);
	gfx.PutPixel(x + 149, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 149, y + 86, 3, 3, 3);
	gfx.PutPixel(x + 149, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 149, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 150, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 150, y + 38, 21, 21, 21);
	gfx.PutPixel(x + 150, y + 39, 141, 141, 141);
	gfx.PutPixel(x + 150, y + 40, 239, 239, 239);
	gfx.PutPixel(x + 150, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 76, 254, 254, 254);
	gfx.PutPixel(x + 150, y + 77, 229, 229, 229);
	gfx.PutPixel(x + 150, y + 78, 184, 184, 184);
	gfx.PutPixel(x + 150, y + 79, 126, 126, 126);
	gfx.PutPixel(x + 150, y + 80, 63, 63, 63);
	gfx.PutPixel(x + 150, y + 81, 15, 15, 15);
	gfx.PutPixel(x + 150, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 150, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 150, y + 84, 5, 5, 5);
	gfx.PutPixel(x + 150, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 150, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 151, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 151, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 151, y + 40, 29, 29, 29);
	gfx.PutPixel(x + 151, y + 41, 102, 102, 102);
	gfx.PutPixel(x + 151, y + 42, 172, 172, 172);
	gfx.PutPixel(x + 151, y + 43, 221, 221, 221);
	gfx.PutPixel(x + 151, y + 44, 249, 249, 249);
	gfx.PutPixel(x + 151, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 70, 248, 248, 248);
	gfx.PutPixel(x + 151, y + 71, 231, 231, 231);
	gfx.PutPixel(x + 151, y + 72, 207, 207, 207);
	gfx.PutPixel(x + 151, y + 73, 176, 176, 176);
	gfx.PutPixel(x + 151, y + 74, 138, 138, 138);
	gfx.PutPixel(x + 151, y + 75, 96, 96, 96);
	gfx.PutPixel(x + 151, y + 76, 54, 54, 54);
	gfx.PutPixel(x + 151, y + 77, 20, 20, 20);
	gfx.PutPixel(x + 151, y + 78, 1, 1, 1);
	gfx.PutPixel(x + 151, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 151, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 151, y + 81, 5, 5, 5);
	gfx.PutPixel(x + 151, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 151, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 40, 3, 3, 3);
	gfx.PutPixel(x + 152, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 43, 14, 14, 14);
	gfx.PutPixel(x + 152, y + 44, 44, 44, 44);
	gfx.PutPixel(x + 152, y + 45, 73, 73, 73);
	gfx.PutPixel(x + 152, y + 46, 105, 105, 105);
	gfx.PutPixel(x + 152, y + 47, 124, 124, 124);
	gfx.PutPixel(x + 152, y + 48, 145, 145, 145);
	gfx.PutPixel(x + 152, y + 49, 161, 161, 161);
	gfx.PutPixel(x + 152, y + 50, 170, 170, 170);
	gfx.PutPixel(x + 152, y + 51, 174, 174, 174);
	gfx.PutPixel(x + 152, y + 52, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 53, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 54, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 55, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 56, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 57, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 58, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 59, 176, 176, 176);
	gfx.PutPixel(x + 152, y + 60, 174, 174, 174);
	gfx.PutPixel(x + 152, y + 61, 171, 171, 171);
	gfx.PutPixel(x + 152, y + 62, 166, 166, 166);
	gfx.PutPixel(x + 152, y + 63, 158, 158, 158);
	gfx.PutPixel(x + 152, y + 64, 145, 145, 145);
	gfx.PutPixel(x + 152, y + 65, 128, 128, 128);
	gfx.PutPixel(x + 152, y + 66, 118, 118, 118);
	gfx.PutPixel(x + 152, y + 67, 103, 103, 103);
	gfx.PutPixel(x + 152, y + 68, 79, 79, 79);
	gfx.PutPixel(x + 152, y + 69, 60, 60, 60);
	gfx.PutPixel(x + 152, y + 70, 42, 42, 42);
	gfx.PutPixel(x + 152, y + 71, 21, 21, 21);
	gfx.PutPixel(x + 152, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 152, y + 77, 4, 4, 4);
	gfx.PutPixel(x + 152, y + 78, 7, 7, 7);
	gfx.PutPixel(x + 152, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 152, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 153, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 43, 5, 5, 5);
	gfx.PutPixel(x + 153, y + 44, 1, 1, 1);
	gfx.PutPixel(x + 153, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 153, y + 70, 1, 1, 1);
	gfx.PutPixel(x + 153, y + 71, 4, 4, 4);
	gfx.PutPixel(x + 153, y + 72, 6, 6, 6);
	gfx.PutPixel(x + 153, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 153, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 154, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 154, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 155, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 155, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 156, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 156, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 157, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 157, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 158, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 158, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 159, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 159, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 160, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 160, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 161, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 161, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 162, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 162, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 163, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 163, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 164, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 164, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 165, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 165, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 166, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 166, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 167, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 167, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 168, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 168, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 169, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 169, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 170, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 170, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 171, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 171, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 172, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 172, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 173, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 173, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 174, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 174, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 175, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 175, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 176, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 176, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 177, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 177, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 178, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 178, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 179, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 179, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 180, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 180, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 181, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 181, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 182, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 182, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 183, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 183, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 184, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 184, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 185, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 185, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 186, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 186, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 187, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 187, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 188, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 188, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 189, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 189, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 190, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 39, 5, 5, 5);
	gfx.PutPixel(x + 190, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 190, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 190, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 190, y + 43, 3, 3, 3);
	gfx.PutPixel(x + 190, y + 44, 6, 6, 6);
	gfx.PutPixel(x + 190, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 190, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 191, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 37, 3, 3, 3);
	gfx.PutPixel(x + 191, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 191, y + 39, 16, 16, 16);
	gfx.PutPixel(x + 191, y + 40, 64, 64, 64);
	gfx.PutPixel(x + 191, y + 41, 94, 94, 94);
	gfx.PutPixel(x + 191, y + 42, 59, 59, 59);
	gfx.PutPixel(x + 191, y + 43, 25, 25, 25);
	gfx.PutPixel(x + 191, y + 44, 5, 5, 5);
	gfx.PutPixel(x + 191, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 191, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 191, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 191, y + 48, 2, 2, 2);
	gfx.PutPixel(x + 191, y + 49, 6, 6, 6);
	gfx.PutPixel(x + 191, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 191, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 192, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 192, y + 37, 31, 31, 31);
	gfx.PutPixel(x + 192, y + 38, 142, 142, 142);
	gfx.PutPixel(x + 192, y + 39, 233, 233, 233);
	gfx.PutPixel(x + 192, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 42, 254, 254, 254);
	gfx.PutPixel(x + 192, y + 43, 235, 235, 235);
	gfx.PutPixel(x + 192, y + 44, 201, 201, 201);
	gfx.PutPixel(x + 192, y + 45, 160, 160, 160);
	gfx.PutPixel(x + 192, y + 46, 115, 115, 115);
	gfx.PutPixel(x + 192, y + 47, 70, 70, 70);
	gfx.PutPixel(x + 192, y + 48, 34, 34, 34);
	gfx.PutPixel(x + 192, y + 49, 9, 9, 9);
	gfx.PutPixel(x + 192, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 192, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 192, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 192, y + 53, 2, 2, 2);
	gfx.PutPixel(x + 192, y + 54, 6, 6, 6);
	gfx.PutPixel(x + 192, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 192, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 193, y + 36, 70, 70, 70);
	gfx.PutPixel(x + 193, y + 37, 234, 234, 234);
	gfx.PutPixel(x + 193, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 39, 213, 213, 213);
	gfx.PutPixel(x + 193, y + 40, 156, 156, 156);
	gfx.PutPixel(x + 193, y + 41, 161, 161, 161);
	gfx.PutPixel(x + 193, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 48, 243, 243, 243);
	gfx.PutPixel(x + 193, y + 49, 211, 211, 211);
	gfx.PutPixel(x + 193, y + 50, 170, 170, 170);
	gfx.PutPixel(x + 193, y + 51, 121, 121, 121);
	gfx.PutPixel(x + 193, y + 52, 74, 74, 74);
	gfx.PutPixel(x + 193, y + 53, 35, 35, 35);
	gfx.PutPixel(x + 193, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 193, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 193, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 193, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 193, y + 59, 7, 7, 7);
	gfx.PutPixel(x + 193, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 193, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 34, 2, 2, 2);
	gfx.PutPixel(x + 194, y + 35, 36, 36, 36);
	gfx.PutPixel(x + 194, y + 36, 237, 237, 237);
	gfx.PutPixel(x + 194, y + 37, 247, 247, 247);
	gfx.PutPixel(x + 194, y + 38, 86, 86, 86);
	gfx.PutPixel(x + 194, y + 39, 3, 3, 3);
	gfx.PutPixel(x + 194, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 194, y + 41, 54, 54, 54);
	gfx.PutPixel(x + 194, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 53, 244, 244, 244);
	gfx.PutPixel(x + 194, y + 54, 210, 210, 210);
	gfx.PutPixel(x + 194, y + 55, 164, 164, 164);
	gfx.PutPixel(x + 194, y + 56, 112, 112, 112);
	gfx.PutPixel(x + 194, y + 57, 64, 64, 64);
	gfx.PutPixel(x + 194, y + 58, 24, 24, 24);
	gfx.PutPixel(x + 194, y + 59, 2, 2, 2);
	gfx.PutPixel(x + 194, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 194, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 194, y + 62, 3, 3, 3);
	gfx.PutPixel(x + 194, y + 63, 7, 7, 7);
	gfx.PutPixel(x + 194, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 194, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 195, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 195, y + 35, 125, 125, 125);
	gfx.PutPixel(x + 195, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 37, 122, 122, 122);
	gfx.PutPixel(x + 195, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 195, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 195, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 195, y + 41, 151, 151, 151);
	gfx.PutPixel(x + 195, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 58, 234, 234, 234);
	gfx.PutPixel(x + 195, y + 59, 190, 190, 190);
	gfx.PutPixel(x + 195, y + 60, 136, 136, 136);
	gfx.PutPixel(x + 195, y + 61, 77, 77, 77);
	gfx.PutPixel(x + 195, y + 62, 29, 29, 29);
	gfx.PutPixel(x + 195, y + 63, 2, 2, 2);
	gfx.PutPixel(x + 195, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 195, y + 65, 1, 1, 1);
	gfx.PutPixel(x + 195, y + 66, 6, 6, 6);
	gfx.PutPixel(x + 195, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 195, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 196, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 196, y + 35, 163, 163, 163);
	gfx.PutPixel(x + 196, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 37, 139, 139, 139);
	gfx.PutPixel(x + 196, y + 38, 86, 86, 86);
	gfx.PutPixel(x + 196, y + 39, 110, 110, 110);
	gfx.PutPixel(x + 196, y + 40, 186, 186, 186);
	gfx.PutPixel(x + 196, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 62, 239, 239, 239);
	gfx.PutPixel(x + 196, y + 63, 189, 189, 189);
	gfx.PutPixel(x + 196, y + 64, 121, 121, 121);
	gfx.PutPixel(x + 196, y + 65, 51, 51, 51);
	gfx.PutPixel(x + 196, y + 66, 5, 5, 5);
	gfx.PutPixel(x + 196, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 196, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 196, y + 69, 1, 1, 1);
	gfx.PutPixel(x + 196, y + 70, 5, 5, 5);
	gfx.PutPixel(x + 196, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 196, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 197, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 197, y + 35, 164, 164, 164);
	gfx.PutPixel(x + 197, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 65, 250, 250, 250);
	gfx.PutPixel(x + 197, y + 66, 206, 206, 206);
	gfx.PutPixel(x + 197, y + 67, 154, 154, 154);
	gfx.PutPixel(x + 197, y + 68, 99, 99, 99);
	gfx.PutPixel(x + 197, y + 69, 51, 51, 51);
	gfx.PutPixel(x + 197, y + 70, 15, 15, 15);
	gfx.PutPixel(x + 197, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 197, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 197, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 197, y + 74, 5, 5, 5);
	gfx.PutPixel(x + 197, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 197, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 198, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 198, y + 35, 164, 164, 164);
	gfx.PutPixel(x + 198, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 69, 252, 252, 252);
	gfx.PutPixel(x + 198, y + 70, 222, 222, 222);
	gfx.PutPixel(x + 198, y + 71, 171, 171, 171);
	gfx.PutPixel(x + 198, y + 72, 112, 112, 112);
	gfx.PutPixel(x + 198, y + 73, 56, 56, 56);
	gfx.PutPixel(x + 198, y + 74, 15, 15, 15);
	gfx.PutPixel(x + 198, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 198, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 198, y + 77, 1, 1, 1);
	gfx.PutPixel(x + 198, y + 78, 6, 6, 6);
	gfx.PutPixel(x + 198, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 198, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 199, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 199, y + 35, 164, 164, 164);
	gfx.PutPixel(x + 199, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 73, 254, 254, 254);
	gfx.PutPixel(x + 199, y + 74, 222, 222, 222);
	gfx.PutPixel(x + 199, y + 75, 165, 165, 165);
	gfx.PutPixel(x + 199, y + 76, 100, 100, 100);
	gfx.PutPixel(x + 199, y + 77, 41, 41, 41);
	gfx.PutPixel(x + 199, y + 78, 5, 5, 5);
	gfx.PutPixel(x + 199, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 199, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 199, y + 81, 5, 5, 5);
	gfx.PutPixel(x + 199, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 199, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 200, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 200, y + 35, 136, 136, 136);
	gfx.PutPixel(x + 200, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 77, 248, 248, 248);
	gfx.PutPixel(x + 200, y + 78, 201, 201, 201);
	gfx.PutPixel(x + 200, y + 79, 131, 131, 131);
	gfx.PutPixel(x + 200, y + 80, 60, 60, 60);
	gfx.PutPixel(x + 200, y + 81, 10, 10, 10);
	gfx.PutPixel(x + 200, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 200, y + 83, 1, 1, 1);
	gfx.PutPixel(x + 200, y + 84, 6, 6, 6);
	gfx.PutPixel(x + 200, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 200, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 201, y + 35, 51, 51, 51);
	gfx.PutPixel(x + 201, y + 36, 246, 246, 246);
	gfx.PutPixel(x + 201, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 80, 254, 254, 254);
	gfx.PutPixel(x + 201, y + 81, 214, 214, 214);
	gfx.PutPixel(x + 201, y + 82, 139, 139, 139);
	gfx.PutPixel(x + 201, y + 83, 58, 58, 58);
	gfx.PutPixel(x + 201, y + 84, 7, 7, 7);
	gfx.PutPixel(x + 201, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 201, y + 86, 2, 2, 2);
	gfx.PutPixel(x + 201, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 201, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 202, y + 36, 100, 100, 100);
	gfx.PutPixel(x + 202, y + 37, 250, 250, 250);
	gfx.PutPixel(x + 202, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 83, 253, 253, 253);
	gfx.PutPixel(x + 202, y + 84, 205, 205, 205);
	gfx.PutPixel(x + 202, y + 85, 117, 117, 117);
	gfx.PutPixel(x + 202, y + 86, 33, 33, 33);
	gfx.PutPixel(x + 202, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 202, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 202, y + 89, 7, 7, 7);
	gfx.PutPixel(x + 202, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 202, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 203, y + 37, 56, 56, 56);
	gfx.PutPixel(x + 203, y + 38, 177, 177, 177);
	gfx.PutPixel(x + 203, y + 39, 246, 246, 246);
	gfx.PutPixel(x + 203, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 86, 241, 241, 241);
	gfx.PutPixel(x + 203, y + 87, 162, 162, 162);
	gfx.PutPixel(x + 203, y + 88, 59, 59, 59);
	gfx.PutPixel(x + 203, y + 89, 1, 1, 1);
	gfx.PutPixel(x + 203, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 203, y + 91, 7, 7, 7);
	gfx.PutPixel(x + 203, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 203, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 204, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 37, 1, 1, 1);
	gfx.PutPixel(x + 204, y + 38, 1, 1, 1);
	gfx.PutPixel(x + 204, y + 39, 37, 37, 37);
	gfx.PutPixel(x + 204, y + 40, 96, 96, 96);
	gfx.PutPixel(x + 204, y + 41, 148, 148, 148);
	gfx.PutPixel(x + 204, y + 42, 184, 184, 184);
	gfx.PutPixel(x + 204, y + 43, 210, 210, 210);
	gfx.PutPixel(x + 204, y + 44, 225, 225, 225);
	gfx.PutPixel(x + 204, y + 45, 232, 232, 232);
	gfx.PutPixel(x + 204, y + 46, 235, 235, 235);
	gfx.PutPixel(x + 204, y + 47, 240, 240, 240);
	gfx.PutPixel(x + 204, y + 48, 253, 253, 253);
	gfx.PutPixel(x + 204, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 88, 253, 253, 253);
	gfx.PutPixel(x + 204, y + 89, 183, 183, 183);
	gfx.PutPixel(x + 204, y + 90, 65, 65, 65);
	gfx.PutPixel(x + 204, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 204, y + 92, 1, 1, 1);
	gfx.PutPixel(x + 204, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 204, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 205, y + 39, 2, 2, 2);
	gfx.PutPixel(x + 205, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 205, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 205, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 205, y + 43, 9, 9, 9);
	gfx.PutPixel(x + 205, y + 44, 18, 18, 18);
	gfx.PutPixel(x + 205, y + 45, 22, 22, 22);
	gfx.PutPixel(x + 205, y + 46, 23, 23, 23);
	gfx.PutPixel(x + 205, y + 47, 30, 30, 30);
	gfx.PutPixel(x + 205, y + 48, 52, 52, 52);
	gfx.PutPixel(x + 205, y + 49, 67, 67, 67);
	gfx.PutPixel(x + 205, y + 50, 93, 93, 93);
	gfx.PutPixel(x + 205, y + 51, 116, 116, 116);
	gfx.PutPixel(x + 205, y + 52, 143, 143, 143);
	gfx.PutPixel(x + 205, y + 53, 172, 172, 172);
	gfx.PutPixel(x + 205, y + 54, 195, 195, 195);
	gfx.PutPixel(x + 205, y + 55, 220, 220, 220);
	gfx.PutPixel(x + 205, y + 56, 239, 239, 239);
	gfx.PutPixel(x + 205, y + 57, 254, 254, 254);
	gfx.PutPixel(x + 205, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 90, 254, 254, 254);
	gfx.PutPixel(x + 205, y + 91, 173, 173, 173);
	gfx.PutPixel(x + 205, y + 92, 42, 42, 42);
	gfx.PutPixel(x + 205, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 205, y + 94, 6, 6, 6);
	gfx.PutPixel(x + 205, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 205, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 42, 7, 7, 7);
	gfx.PutPixel(x + 206, y + 43, 6, 6, 6);
	gfx.PutPixel(x + 206, y + 44, 4, 4, 4);
	gfx.PutPixel(x + 206, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 206, y + 46, 3, 3, 3);
	gfx.PutPixel(x + 206, y + 47, 2, 2, 2);
	gfx.PutPixel(x + 206, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 206, y + 54, 2, 2, 2);
	gfx.PutPixel(x + 206, y + 55, 14, 14, 14);
	gfx.PutPixel(x + 206, y + 56, 30, 30, 30);
	gfx.PutPixel(x + 206, y + 57, 54, 54, 54);
	gfx.PutPixel(x + 206, y + 58, 80, 80, 80);
	gfx.PutPixel(x + 206, y + 59, 114, 114, 114);
	gfx.PutPixel(x + 206, y + 60, 150, 150, 150);
	gfx.PutPixel(x + 206, y + 61, 184, 184, 184);
	gfx.PutPixel(x + 206, y + 62, 217, 217, 217);
	gfx.PutPixel(x + 206, y + 63, 243, 243, 243);
	gfx.PutPixel(x + 206, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 92, 243, 243, 243);
	gfx.PutPixel(x + 206, y + 93, 118, 118, 118);
	gfx.PutPixel(x + 206, y + 94, 4, 4, 4);
	gfx.PutPixel(x + 206, y + 95, 2, 2, 2);
	gfx.PutPixel(x + 206, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 206, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 54, 7, 7, 7);
	gfx.PutPixel(x + 207, y + 55, 5, 5, 5);
	gfx.PutPixel(x + 207, y + 56, 3, 3, 3);
	gfx.PutPixel(x + 207, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 62, 12, 12, 12);
	gfx.PutPixel(x + 207, y + 63, 35, 35, 35);
	gfx.PutPixel(x + 207, y + 64, 66, 66, 66);
	gfx.PutPixel(x + 207, y + 65, 108, 108, 108);
	gfx.PutPixel(x + 207, y + 66, 155, 155, 155);
	gfx.PutPixel(x + 207, y + 67, 199, 199, 199);
	gfx.PutPixel(x + 207, y + 68, 238, 238, 238);
	gfx.PutPixel(x + 207, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 94, 180, 180, 180);
	gfx.PutPixel(x + 207, y + 95, 22, 22, 22);
	gfx.PutPixel(x + 207, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 207, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 207, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 61, 7, 7, 7);
	gfx.PutPixel(x + 208, y + 62, 5, 5, 5);
	gfx.PutPixel(x + 208, y + 63, 2, 2, 2);
	gfx.PutPixel(x + 208, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 208, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 208, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 208, y + 67, 4, 4, 4);
	gfx.PutPixel(x + 208, y + 68, 28, 28, 28);
	gfx.PutPixel(x + 208, y + 69, 70, 70, 70);
	gfx.PutPixel(x + 208, y + 70, 124, 124, 124);
	gfx.PutPixel(x + 208, y + 71, 185, 185, 185);
	gfx.PutPixel(x + 208, y + 72, 237, 237, 237);
	gfx.PutPixel(x + 208, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 95, 206, 206, 206);
	gfx.PutPixel(x + 208, y + 96, 28, 28, 28);
	gfx.PutPixel(x + 208, y + 97, 1, 1, 1);
	gfx.PutPixel(x + 208, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 208, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 67, 7, 7, 7);
	gfx.PutPixel(x + 209, y + 68, 3, 3, 3);
	gfx.PutPixel(x + 209, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 209, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 209, y + 71, 1, 1, 1);
	gfx.PutPixel(x + 209, y + 72, 26, 26, 26);
	gfx.PutPixel(x + 209, y + 73, 84, 84, 84);
	gfx.PutPixel(x + 209, y + 74, 162, 162, 162);
	gfx.PutPixel(x + 209, y + 75, 234, 234, 234);
	gfx.PutPixel(x + 209, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 96, 196, 196, 196);
	gfx.PutPixel(x + 209, y + 97, 13, 13, 13);
	gfx.PutPixel(x + 209, y + 98, 4, 4, 4);
	gfx.PutPixel(x + 209, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 209, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 71, 7, 7, 7);
	gfx.PutPixel(x + 210, y + 72, 3, 3, 3);
	gfx.PutPixel(x + 210, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 210, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 210, y + 75, 24, 24, 24);
	gfx.PutPixel(x + 210, y + 76, 106, 106, 106);
	gfx.PutPixel(x + 210, y + 77, 213, 213, 213);
	gfx.PutPixel(x + 210, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 97, 134, 134, 134);
	gfx.PutPixel(x + 210, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 210, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 210, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 211, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 75, 3, 3, 3);
	gfx.PutPixel(x + 211, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 211, y + 77, 10, 10, 10);
	gfx.PutPixel(x + 211, y + 78, 115, 115, 115);
	gfx.PutPixel(x + 211, y + 79, 242, 242, 242);
	gfx.PutPixel(x + 211, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 97, 235, 235, 235);
	gfx.PutPixel(x + 211, y + 98, 31, 31, 31);
	gfx.PutPixel(x + 211, y + 99, 2, 2, 2);
	gfx.PutPixel(x + 211, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 211, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 212, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 77, 5, 5, 5);
	gfx.PutPixel(x + 212, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 212, y + 79, 60, 60, 60);
	gfx.PutPixel(x + 212, y + 80, 236, 236, 236);
	gfx.PutPixel(x + 212, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 98, 89, 89, 89);
	gfx.PutPixel(x + 212, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 212, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 212, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 213, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 213, y + 80, 103, 103, 103);
	gfx.PutPixel(x + 213, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 98, 123, 123, 123);
	gfx.PutPixel(x + 213, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 213, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 213, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 214, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 79, 1, 1, 1);
	gfx.PutPixel(x + 214, y + 80, 37, 37, 37);
	gfx.PutPixel(x + 214, y + 81, 248, 248, 248);
	gfx.PutPixel(x + 214, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 98, 118, 118, 118);
	gfx.PutPixel(x + 214, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 214, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 214, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 215, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 215, y + 80, 52, 52, 52);
	gfx.PutPixel(x + 215, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 98, 87, 87, 87);
	gfx.PutPixel(x + 215, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 215, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 215, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 216, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 216, y + 80, 122, 122, 122);
	gfx.PutPixel(x + 216, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 97, 247, 247, 247);
	gfx.PutPixel(x + 216, y + 98, 43, 43, 43);
	gfx.PutPixel(x + 216, y + 99, 1, 1, 1);
	gfx.PutPixel(x + 216, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 216, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 217, y + 79, 31, 31, 31);
	gfx.PutPixel(x + 217, y + 80, 229, 229, 229);
	gfx.PutPixel(x + 217, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 97, 192, 192, 192);
	gfx.PutPixel(x + 217, y + 98, 3, 3, 3);
	gfx.PutPixel(x + 217, y + 99, 7, 7, 7);
	gfx.PutPixel(x + 217, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 217, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 218, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 218, y + 78, 17, 17, 17);
	gfx.PutPixel(x + 218, y + 79, 192, 192, 192);
	gfx.PutPixel(x + 218, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 97, 91, 91, 91);
	gfx.PutPixel(x + 218, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 218, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 218, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 219, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 75, 5, 5, 5);
	gfx.PutPixel(x + 219, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 219, y + 77, 41, 41, 41);
	gfx.PutPixel(x + 219, y + 78, 201, 201, 201);
	gfx.PutPixel(x + 219, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 219, y + 96, 191, 191, 191);
	gfx.PutPixel(x + 219, y + 97, 5, 5, 5);
	gfx.PutPixel(x + 219, y + 98, 6, 6, 6);
	gfx.PutPixel(x + 219, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 219, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 220, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 73, 4, 4, 4);
	gfx.PutPixel(x + 220, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 220, y + 75, 11, 11, 11);
	gfx.PutPixel(x + 220, y + 76, 114, 114, 114);
	gfx.PutPixel(x + 220, y + 77, 238, 238, 238);
	gfx.PutPixel(x + 220, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 220, y + 95, 240, 240, 240);
	gfx.PutPixel(x + 220, y + 96, 49, 49, 49);
	gfx.PutPixel(x + 220, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 220, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 220, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 221, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 70, 6, 6, 6);
	gfx.PutPixel(x + 221, y + 71, 1, 1, 1);
	gfx.PutPixel(x + 221, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 221, y + 73, 17, 17, 17);
	gfx.PutPixel(x + 221, y + 74, 101, 101, 101);
	gfx.PutPixel(x + 221, y + 75, 213, 213, 213);
	gfx.PutPixel(x + 221, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 221, y + 94, 254, 254, 254);
	gfx.PutPixel(x + 221, y + 95, 89, 89, 89);
	gfx.PutPixel(x + 221, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 221, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 221, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 222, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 67, 5, 5, 5);
	gfx.PutPixel(x + 222, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 222, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 222, y + 70, 6, 6, 6);
	gfx.PutPixel(x + 222, y + 71, 53, 53, 53);
	gfx.PutPixel(x + 222, y + 72, 139, 139, 139);
	gfx.PutPixel(x + 222, y + 73, 226, 226, 226);
	gfx.PutPixel(x + 222, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 222, y + 93, 254, 254, 254);
	gfx.PutPixel(x + 222, y + 94, 103, 103, 103);
	gfx.PutPixel(x + 222, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 222, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 222, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 62, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 63, 4, 4, 4);
	gfx.PutPixel(x + 223, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 223, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 223, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 223, y + 67, 16, 16, 16);
	gfx.PutPixel(x + 223, y + 68, 62, 62, 62);
	gfx.PutPixel(x + 223, y + 69, 131, 131, 131);
	gfx.PutPixel(x + 223, y + 70, 205, 205, 205);
	gfx.PutPixel(x + 223, y + 71, 252, 252, 252);
	gfx.PutPixel(x + 223, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 223, y + 92, 244, 244, 244);
	gfx.PutPixel(x + 223, y + 93, 90, 90, 90);
	gfx.PutPixel(x + 223, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 223, y + 95, 7, 7, 7);
	gfx.PutPixel(x + 223, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 223, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 57, 7, 7, 7);
	gfx.PutPixel(x + 224, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 224, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 224, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 224, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 224, y + 62, 1, 1, 1);
	gfx.PutPixel(x + 224, y + 63, 19, 19, 19);
	gfx.PutPixel(x + 224, y + 64, 56, 56, 56);
	gfx.PutPixel(x + 224, y + 65, 108, 108, 108);
	gfx.PutPixel(x + 224, y + 66, 169, 169, 169);
	gfx.PutPixel(x + 224, y + 67, 224, 224, 224);
	gfx.PutPixel(x + 224, y + 68, 254, 254, 254);
	gfx.PutPixel(x + 224, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 91, 215, 215, 215);
	gfx.PutPixel(x + 224, y + 92, 56, 56, 56);
	gfx.PutPixel(x + 224, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 224, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 224, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 50, 7, 7, 7);
	gfx.PutPixel(x + 225, y + 51, 5, 5, 5);
	gfx.PutPixel(x + 225, y + 52, 2, 2, 2);
	gfx.PutPixel(x + 225, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 225, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 225, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 225, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 225, y + 57, 5, 5, 5);
	gfx.PutPixel(x + 225, y + 58, 22, 22, 22);
	gfx.PutPixel(x + 225, y + 59, 52, 52, 52);
	gfx.PutPixel(x + 225, y + 60, 92, 92, 92);
	gfx.PutPixel(x + 225, y + 61, 138, 138, 138);
	gfx.PutPixel(x + 225, y + 62, 186, 186, 186);
	gfx.PutPixel(x + 225, y + 63, 228, 228, 228);
	gfx.PutPixel(x + 225, y + 64, 253, 253, 253);
	gfx.PutPixel(x + 225, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 89, 254, 254, 254);
	gfx.PutPixel(x + 225, y + 90, 155, 155, 155);
	gfx.PutPixel(x + 225, y + 91, 19, 19, 19);
	gfx.PutPixel(x + 225, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 225, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 225, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 226, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 43, 6, 6, 6);
	gfx.PutPixel(x + 226, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 50, 4, 4, 4);
	gfx.PutPixel(x + 226, y + 51, 17, 17, 17);
	gfx.PutPixel(x + 226, y + 52, 34, 34, 34);
	gfx.PutPixel(x + 226, y + 53, 60, 60, 60);
	gfx.PutPixel(x + 226, y + 54, 94, 94, 94);
	gfx.PutPixel(x + 226, y + 55, 128, 128, 128);
	gfx.PutPixel(x + 226, y + 56, 165, 165, 165);
	gfx.PutPixel(x + 226, y + 57, 201, 201, 201);
	gfx.PutPixel(x + 226, y + 58, 232, 232, 232);
	gfx.PutPixel(x + 226, y + 59, 253, 253, 253);
	gfx.PutPixel(x + 226, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 88, 209, 209, 209);
	gfx.PutPixel(x + 226, y + 89, 73, 73, 73);
	gfx.PutPixel(x + 226, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 226, y + 91, 3, 3, 3);
	gfx.PutPixel(x + 226, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 226, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 39, 2, 2, 2);
	gfx.PutPixel(x + 227, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 227, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 227, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 227, y + 43, 12, 12, 12);
	gfx.PutPixel(x + 227, y + 44, 48, 48, 48);
	gfx.PutPixel(x + 227, y + 45, 64, 64, 64);
	gfx.PutPixel(x + 227, y + 46, 90, 90, 90);
	gfx.PutPixel(x + 227, y + 47, 115, 115, 115);
	gfx.PutPixel(x + 227, y + 48, 141, 141, 141);
	gfx.PutPixel(x + 227, y + 49, 173, 173, 173);
	gfx.PutPixel(x + 227, y + 50, 199, 199, 199);
	gfx.PutPixel(x + 227, y + 51, 224, 224, 224);
	gfx.PutPixel(x + 227, y + 52, 243, 243, 243);
	gfx.PutPixel(x + 227, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 86, 227, 227, 227);
	gfx.PutPixel(x + 227, y + 87, 110, 110, 110);
	gfx.PutPixel(x + 227, y + 88, 11, 11, 11);
	gfx.PutPixel(x + 227, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 227, y + 90, 7, 7, 7);
	gfx.PutPixel(x + 227, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 227, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 228, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 228, y + 39, 34, 34, 34);
	gfx.PutPixel(x + 228, y + 40, 65, 65, 65);
	gfx.PutPixel(x + 228, y + 41, 86, 86, 86);
	gfx.PutPixel(x + 228, y + 42, 124, 124, 124);
	gfx.PutPixel(x + 228, y + 43, 204, 204, 204);
	gfx.PutPixel(x + 228, y + 44, 254, 254, 254);
	gfx.PutPixel(x + 228, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 84, 225, 225, 225);
	gfx.PutPixel(x + 228, y + 85, 120, 120, 120);
	gfx.PutPixel(x + 228, y + 86, 20, 20, 20);
	gfx.PutPixel(x + 228, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 228, y + 88, 5, 5, 5);
	gfx.PutPixel(x + 228, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 228, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 36, 7, 7, 7);
	gfx.PutPixel(x + 229, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 229, y + 38, 77, 77, 77);
	gfx.PutPixel(x + 229, y + 39, 238, 238, 238);
	gfx.PutPixel(x + 229, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 82, 206, 206, 206);
	gfx.PutPixel(x + 229, y + 83, 103, 103, 103);
	gfx.PutPixel(x + 229, y + 84, 18, 18, 18);
	gfx.PutPixel(x + 229, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 229, y + 86, 4, 4, 4);
	gfx.PutPixel(x + 229, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 229, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 230, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 230, y + 37, 105, 105, 105);
	gfx.PutPixel(x + 230, y + 38, 250, 250, 250);
	gfx.PutPixel(x + 230, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 40, 244, 244, 244);
	gfx.PutPixel(x + 230, y + 41, 210, 210, 210);
	gfx.PutPixel(x + 230, y + 42, 213, 213, 213);
	gfx.PutPixel(x + 230, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 79, 241, 241, 241);
	gfx.PutPixel(x + 230, y + 80, 167, 167, 167);
	gfx.PutPixel(x + 230, y + 81, 71, 71, 71);
	gfx.PutPixel(x + 230, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 230, y + 84, 4, 4, 4);
	gfx.PutPixel(x + 230, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 230, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 231, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 231, y + 36, 119, 119, 119);
	gfx.PutPixel(x + 231, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 39, 136, 136, 136);
	gfx.PutPixel(x + 231, y + 40, 34, 34, 34);
	gfx.PutPixel(x + 231, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 231, y + 42, 69, 69, 69);
	gfx.PutPixel(x + 231, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 76, 249, 249, 249);
	gfx.PutPixel(x + 231, y + 77, 195, 195, 195);
	gfx.PutPixel(x + 231, y + 78, 109, 109, 109);
	gfx.PutPixel(x + 231, y + 79, 31, 31, 31);
	gfx.PutPixel(x + 231, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 231, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 231, y + 82, 5, 5, 5);
	gfx.PutPixel(x + 231, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 231, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 232, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 232, y + 36, 182, 182, 182);
	gfx.PutPixel(x + 232, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 38, 142, 142, 142);
	gfx.PutPixel(x + 232, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 232, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 232, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 232, y + 42, 122, 122, 122);
	gfx.PutPixel(x + 232, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 73, 248, 248, 248);
	gfx.PutPixel(x + 232, y + 74, 197, 197, 197);
	gfx.PutPixel(x + 232, y + 75, 121, 121, 121);
	gfx.PutPixel(x + 232, y + 76, 46, 46, 46);
	gfx.PutPixel(x + 232, y + 77, 3, 3, 3);
	gfx.PutPixel(x + 232, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 232, y + 79, 2, 2, 2);
	gfx.PutPixel(x + 232, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 232, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 233, y + 36, 182, 182, 182);
	gfx.PutPixel(x + 233, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 38, 87, 87, 87);
	gfx.PutPixel(x + 233, y + 39, 21, 21, 21);
	gfx.PutPixel(x + 233, y + 40, 48, 48, 48);
	gfx.PutPixel(x + 233, y + 41, 130, 130, 130);
	gfx.PutPixel(x + 233, y + 42, 248, 248, 248);
	gfx.PutPixel(x + 233, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 70, 232, 232, 232);
	gfx.PutPixel(x + 233, y + 71, 167, 167, 167);
	gfx.PutPixel(x + 233, y + 72, 102, 102, 102);
	gfx.PutPixel(x + 233, y + 73, 41, 41, 41);
	gfx.PutPixel(x + 233, y + 74, 4, 4, 4);
	gfx.PutPixel(x + 233, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 233, y + 76, 1, 1, 1);
	gfx.PutPixel(x + 233, y + 77, 7, 7, 7);
	gfx.PutPixel(x + 233, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 233, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 234, y + 36, 181, 181, 181);
	gfx.PutPixel(x + 234, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 38, 243, 243, 243);
	gfx.PutPixel(x + 234, y + 39, 241, 241, 241);
	gfx.PutPixel(x + 234, y + 40, 252, 252, 252);
	gfx.PutPixel(x + 234, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 70, 197, 197, 197);
	gfx.PutPixel(x + 234, y + 71, 79, 79, 79);
	gfx.PutPixel(x + 234, y + 72, 24, 24, 24);
	gfx.PutPixel(x + 234, y + 73, 1, 1, 1);
	gfx.PutPixel(x + 234, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 234, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 234, y + 76, 3, 3, 3);
	gfx.PutPixel(x + 234, y + 77, 7, 7, 7);
	gfx.PutPixel(x + 234, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 234, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 235, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 235, y + 36, 182, 182, 182);
	gfx.PutPixel(x + 235, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 72, 246, 246, 246);
	gfx.PutPixel(x + 235, y + 73, 205, 205, 205);
	gfx.PutPixel(x + 235, y + 74, 144, 144, 144);
	gfx.PutPixel(x + 235, y + 75, 78, 78, 78);
	gfx.PutPixel(x + 235, y + 76, 26, 26, 26);
	gfx.PutPixel(x + 235, y + 77, 1, 1, 1);
	gfx.PutPixel(x + 235, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 235, y + 79, 1, 1, 1);
	gfx.PutPixel(x + 235, y + 80, 6, 6, 6);
	gfx.PutPixel(x + 235, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 235, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 236, y + 36, 176, 176, 176);
	gfx.PutPixel(x + 236, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 76, 237, 237, 237);
	gfx.PutPixel(x + 236, y + 77, 182, 182, 182);
	gfx.PutPixel(x + 236, y + 78, 108, 108, 108);
	gfx.PutPixel(x + 236, y + 79, 43, 43, 43);
	gfx.PutPixel(x + 236, y + 80, 5, 5, 5);
	gfx.PutPixel(x + 236, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 236, y + 82, 1, 1, 1);
	gfx.PutPixel(x + 236, y + 83, 7, 7, 7);
	gfx.PutPixel(x + 236, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 236, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 237, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 237, y + 36, 137, 137, 137);
	gfx.PutPixel(x + 237, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 79, 249, 249, 249);
	gfx.PutPixel(x + 237, y + 80, 200, 200, 200);
	gfx.PutPixel(x + 237, y + 81, 122, 122, 122);
	gfx.PutPixel(x + 237, y + 82, 48, 48, 48);
	gfx.PutPixel(x + 237, y + 83, 4, 4, 4);
	gfx.PutPixel(x + 237, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 237, y + 85, 2, 2, 2);
	gfx.PutPixel(x + 237, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 237, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 238, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 35, 1, 1, 1);
	gfx.PutPixel(x + 238, y + 36, 43, 43, 43);
	gfx.PutPixel(x + 238, y + 37, 238, 238, 238);
	gfx.PutPixel(x + 238, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 82, 250, 250, 250);
	gfx.PutPixel(x + 238, y + 83, 198, 198, 198);
	gfx.PutPixel(x + 238, y + 84, 113, 113, 113);
	gfx.PutPixel(x + 238, y + 85, 33, 33, 33);
	gfx.PutPixel(x + 238, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 238, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 238, y + 88, 6, 6, 6);
	gfx.PutPixel(x + 238, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 238, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 239, y + 37, 74, 74, 74);
	gfx.PutPixel(x + 239, y + 38, 235, 235, 235);
	gfx.PutPixel(x + 239, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 85, 242, 242, 242);
	gfx.PutPixel(x + 239, y + 86, 169, 169, 169);
	gfx.PutPixel(x + 239, y + 87, 71, 71, 71);
	gfx.PutPixel(x + 239, y + 88, 7, 7, 7);
	gfx.PutPixel(x + 239, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 239, y + 90, 5, 5, 5);
	gfx.PutPixel(x + 239, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 239, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 240, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 240, y + 38, 32, 32, 32);
	gfx.PutPixel(x + 240, y + 39, 139, 139, 139);
	gfx.PutPixel(x + 240, y + 40, 223, 223, 223);
	gfx.PutPixel(x + 240, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 88, 205, 205, 205);
	gfx.PutPixel(x + 240, y + 89, 98, 98, 98);
	gfx.PutPixel(x + 240, y + 90, 13, 13, 13);
	gfx.PutPixel(x + 240, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 240, y + 92, 5, 5, 5);
	gfx.PutPixel(x + 240, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 240, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 38, 3, 3, 3);
	gfx.PutPixel(x + 241, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 241, y + 40, 15, 15, 15);
	gfx.PutPixel(x + 241, y + 41, 61, 61, 61);
	gfx.PutPixel(x + 241, y + 42, 108, 108, 108);
	gfx.PutPixel(x + 241, y + 43, 146, 146, 146);
	gfx.PutPixel(x + 241, y + 44, 174, 174, 174);
	gfx.PutPixel(x + 241, y + 45, 189, 189, 189);
	gfx.PutPixel(x + 241, y + 46, 202, 202, 202);
	gfx.PutPixel(x + 241, y + 47, 209, 209, 209);
	gfx.PutPixel(x + 241, y + 48, 215, 215, 215);
	gfx.PutPixel(x + 241, y + 49, 230, 230, 230);
	gfx.PutPixel(x + 241, y + 50, 245, 245, 245);
	gfx.PutPixel(x + 241, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 90, 217, 217, 217);
	gfx.PutPixel(x + 241, y + 91, 100, 100, 100);
	gfx.PutPixel(x + 241, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 241, y + 94, 7, 7, 7);
	gfx.PutPixel(x + 241, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 241, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 242, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 40, 5, 5, 5);
	gfx.PutPixel(x + 242, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 46, 5, 5, 5);
	gfx.PutPixel(x + 242, y + 47, 9, 9, 9);
	gfx.PutPixel(x + 242, y + 48, 13, 13, 13);
	gfx.PutPixel(x + 242, y + 49, 21, 21, 21);
	gfx.PutPixel(x + 242, y + 50, 38, 38, 38);
	gfx.PutPixel(x + 242, y + 51, 57, 57, 57);
	gfx.PutPixel(x + 242, y + 52, 75, 75, 75);
	gfx.PutPixel(x + 242, y + 53, 105, 105, 105);
	gfx.PutPixel(x + 242, y + 54, 129, 129, 129);
	gfx.PutPixel(x + 242, y + 55, 160, 160, 160);
	gfx.PutPixel(x + 242, y + 56, 185, 185, 185);
	gfx.PutPixel(x + 242, y + 57, 213, 213, 213);
	gfx.PutPixel(x + 242, y + 58, 234, 234, 234);
	gfx.PutPixel(x + 242, y + 59, 251, 251, 251);
	gfx.PutPixel(x + 242, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 92, 204, 204, 204);
	gfx.PutPixel(x + 242, y + 93, 68, 68, 68);
	gfx.PutPixel(x + 242, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 242, y + 95, 4, 4, 4);
	gfx.PutPixel(x + 242, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 242, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 45, 7, 7, 7);
	gfx.PutPixel(x + 243, y + 46, 6, 6, 6);
	gfx.PutPixel(x + 243, y + 47, 6, 6, 6);
	gfx.PutPixel(x + 243, y + 48, 5, 5, 5);
	gfx.PutPixel(x + 243, y + 49, 4, 4, 4);
	gfx.PutPixel(x + 243, y + 50, 1, 1, 1);
	gfx.PutPixel(x + 243, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 57, 11, 11, 11);
	gfx.PutPixel(x + 243, y + 58, 24, 24, 24);
	gfx.PutPixel(x + 243, y + 59, 48, 48, 48);
	gfx.PutPixel(x + 243, y + 60, 75, 75, 75);
	gfx.PutPixel(x + 243, y + 61, 111, 111, 111);
	gfx.PutPixel(x + 243, y + 62, 146, 146, 146);
	gfx.PutPixel(x + 243, y + 63, 184, 184, 184);
	gfx.PutPixel(x + 243, y + 64, 218, 218, 218);
	gfx.PutPixel(x + 243, y + 65, 244, 244, 244);
	gfx.PutPixel(x + 243, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 93, 253, 253, 253);
	gfx.PutPixel(x + 243, y + 94, 147, 147, 147);
	gfx.PutPixel(x + 243, y + 95, 13, 13, 13);
	gfx.PutPixel(x + 243, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 243, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 243, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 56, 7, 7, 7);
	gfx.PutPixel(x + 244, y + 57, 5, 5, 5);
	gfx.PutPixel(x + 244, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 244, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 64, 13, 13, 13);
	gfx.PutPixel(x + 244, y + 65, 36, 36, 36);
	gfx.PutPixel(x + 244, y + 66, 71, 71, 71);
	gfx.PutPixel(x + 244, y + 67, 117, 117, 117);
	gfx.PutPixel(x + 244, y + 68, 164, 164, 164);
	gfx.PutPixel(x + 244, y + 69, 211, 211, 211);
	gfx.PutPixel(x + 244, y + 70, 246, 246, 246);
	gfx.PutPixel(x + 244, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 95, 202, 202, 202);
	gfx.PutPixel(x + 244, y + 96, 34, 34, 34);
	gfx.PutPixel(x + 244, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 244, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 244, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 63, 7, 7, 7);
	gfx.PutPixel(x + 245, y + 64, 5, 5, 5);
	gfx.PutPixel(x + 245, y + 65, 2, 2, 2);
	gfx.PutPixel(x + 245, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 245, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 245, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 245, y + 69, 9, 9, 9);
	gfx.PutPixel(x + 245, y + 70, 39, 39, 39);
	gfx.PutPixel(x + 245, y + 71, 86, 86, 86);
	gfx.PutPixel(x + 245, y + 72, 148, 148, 148);
	gfx.PutPixel(x + 245, y + 73, 210, 210, 210);
	gfx.PutPixel(x + 245, y + 74, 251, 251, 251);
	gfx.PutPixel(x + 245, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 96, 221, 221, 221);
	gfx.PutPixel(x + 245, y + 97, 40, 40, 40);
	gfx.PutPixel(x + 245, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 245, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 245, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 246, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 69, 6, 6, 6);
	gfx.PutPixel(x + 246, y + 70, 1, 1, 1);
	gfx.PutPixel(x + 246, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 246, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 246, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 74, 51, 51, 51);
	gfx.PutPixel(x + 246, y + 75, 125, 125, 125);
	gfx.PutPixel(x + 246, y + 76, 209, 209, 209);
	gfx.PutPixel(x + 246, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 97, 208, 208, 208);
	gfx.PutPixel(x + 246, y + 98, 19, 19, 19);
	gfx.PutPixel(x + 246, y + 99, 3, 3, 3);
	gfx.PutPixel(x + 246, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 246, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 247, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 73, 6, 6, 6);
	gfx.PutPixel(x + 247, y + 74, 1, 1, 1);
	gfx.PutPixel(x + 247, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 247, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 77, 72, 72, 72);
	gfx.PutPixel(x + 247, y + 78, 184, 184, 184);
	gfx.PutPixel(x + 247, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 98, 143, 143, 143);
	gfx.PutPixel(x + 247, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 247, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 247, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 248, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 76, 5, 5, 5);
	gfx.PutPixel(x + 248, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 248, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 248, y + 79, 83, 83, 83);
	gfx.PutPixel(x + 248, y + 80, 229, 229, 229);
	gfx.PutPixel(x + 248, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 98, 238, 238, 238);
	gfx.PutPixel(x + 248, y + 99, 33, 33, 33);
	gfx.PutPixel(x + 248, y + 100, 2, 2, 2);
	gfx.PutPixel(x + 248, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 248, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 78, 7, 7, 7);
	gfx.PutPixel(x + 249, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 249, y + 80, 41, 41, 41);
	gfx.PutPixel(x + 249, y + 81, 227, 227, 227);
	gfx.PutPixel(x + 249, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 99, 86, 86, 86);
	gfx.PutPixel(x + 249, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 249, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 249, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 250, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 250, y + 81, 97, 97, 97);
	gfx.PutPixel(x + 250, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 250, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 250, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 250, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 251, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 251, y + 81, 45, 45, 45);
	gfx.PutPixel(x + 251, y + 82, 253, 253, 253);
	gfx.PutPixel(x + 251, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 99, 97, 97, 97);
	gfx.PutPixel(x + 251, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 251, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 251, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 252, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 252, y + 81, 73, 73, 73);
	gfx.PutPixel(x + 252, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 99, 66, 66, 66);
	gfx.PutPixel(x + 252, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 252, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 252, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 253, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 253, y + 81, 154, 154, 154);
	gfx.PutPixel(x + 253, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 98, 227, 227, 227);
	gfx.PutPixel(x + 253, y + 99, 21, 21, 21);
	gfx.PutPixel(x + 253, y + 100, 4, 4, 4);
	gfx.PutPixel(x + 253, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 253, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 254, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 254, y + 80, 65, 65, 65);
	gfx.PutPixel(x + 254, y + 81, 250, 250, 250);
	gfx.PutPixel(x + 254, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 98, 151, 151, 151);
	gfx.PutPixel(x + 254, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 254, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 254, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 77, 7, 7, 7);
	gfx.PutPixel(x + 255, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 255, y + 79, 50, 50, 50);
	gfx.PutPixel(x + 255, y + 80, 227, 227, 227);
	gfx.PutPixel(x + 255, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 97, 244, 244, 244);
	gfx.PutPixel(x + 255, y + 98, 45, 45, 45);
	gfx.PutPixel(x + 255, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 255, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 255, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 256, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 76, 1, 1, 1);
	gfx.PutPixel(x + 256, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 256, y + 78, 87, 87, 87);
	gfx.PutPixel(x + 256, y + 79, 235, 235, 235);
	gfx.PutPixel(x + 256, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 97, 129, 129, 129);
	gfx.PutPixel(x + 256, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 256, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 256, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 73, 7, 7, 7);
	gfx.PutPixel(x + 257, y + 74, 1, 1, 1);
	gfx.PutPixel(x + 257, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 257, y + 76, 42, 42, 42);
	gfx.PutPixel(x + 257, y + 77, 169, 169, 169);
	gfx.PutPixel(x + 257, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 96, 187, 187, 187);
	gfx.PutPixel(x + 257, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 98, 5, 5, 5);
	gfx.PutPixel(x + 257, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 257, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 70, 7, 7, 7);
	gfx.PutPixel(x + 258, y + 71, 2, 2, 2);
	gfx.PutPixel(x + 258, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 258, y + 73, 2, 2, 2);
	gfx.PutPixel(x + 258, y + 74, 54, 54, 54);
	gfx.PutPixel(x + 258, y + 75, 157, 157, 157);
	gfx.PutPixel(x + 258, y + 76, 245, 245, 245);
	gfx.PutPixel(x + 258, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 95, 209, 209, 209);
	gfx.PutPixel(x + 258, y + 96, 26, 26, 26);
	gfx.PutPixel(x + 258, y + 97, 1, 1, 1);
	gfx.PutPixel(x + 258, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 258, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 259, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 67, 5, 5, 5);
	gfx.PutPixel(x + 259, y + 68, 1, 1, 1);
	gfx.PutPixel(x + 259, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 259, y + 70, 1, 1, 1);
	gfx.PutPixel(x + 259, y + 71, 32, 32, 32);
	gfx.PutPixel(x + 259, y + 72, 103, 103, 103);
	gfx.PutPixel(x + 259, y + 73, 192, 192, 192);
	gfx.PutPixel(x + 259, y + 74, 251, 251, 251);
	gfx.PutPixel(x + 259, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 94, 207, 207, 207);
	gfx.PutPixel(x + 259, y + 95, 31, 31, 31);
	gfx.PutPixel(x + 259, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 259, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 259, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 62, 7, 7, 7);
	gfx.PutPixel(x + 260, y + 63, 4, 4, 4);
	gfx.PutPixel(x + 260, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 260, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 260, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 260, y + 67, 11, 11, 11);
	gfx.PutPixel(x + 260, y + 68, 51, 51, 51);
	gfx.PutPixel(x + 260, y + 69, 113, 113, 113);
	gfx.PutPixel(x + 260, y + 70, 184, 184, 184);
	gfx.PutPixel(x + 260, y + 71, 242, 242, 242);
	gfx.PutPixel(x + 260, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 93, 182, 182, 182);
	gfx.PutPixel(x + 260, y + 94, 22, 22, 22);
	gfx.PutPixel(x + 260, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 260, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 260, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 56, 7, 7, 7);
	gfx.PutPixel(x + 261, y + 57, 5, 5, 5);
	gfx.PutPixel(x + 261, y + 58, 2, 2, 2);
	gfx.PutPixel(x + 261, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 261, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 261, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 261, y + 62, 2, 2, 2);
	gfx.PutPixel(x + 261, y + 63, 20, 20, 20);
	gfx.PutPixel(x + 261, y + 64, 56, 56, 56);
	gfx.PutPixel(x + 261, y + 65, 105, 105, 105);
	gfx.PutPixel(x + 261, y + 66, 163, 163, 163);
	gfx.PutPixel(x + 261, y + 67, 217, 217, 217);
	gfx.PutPixel(x + 261, y + 68, 252, 252, 252);
	gfx.PutPixel(x + 261, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 91, 252, 252, 252);
	gfx.PutPixel(x + 261, y + 92, 133, 133, 133);
	gfx.PutPixel(x + 261, y + 93, 5, 5, 5);
	gfx.PutPixel(x + 261, y + 94, 2, 2, 2);
	gfx.PutPixel(x + 261, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 261, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 49, 7, 7, 7);
	gfx.PutPixel(x + 262, y + 50, 5, 5, 5);
	gfx.PutPixel(x + 262, y + 51, 3, 3, 3);
	gfx.PutPixel(x + 262, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 57, 10, 10, 10);
	gfx.PutPixel(x + 262, y + 58, 30, 30, 30);
	gfx.PutPixel(x + 262, y + 59, 61, 61, 61);
	gfx.PutPixel(x + 262, y + 60, 100, 100, 100);
	gfx.PutPixel(x + 262, y + 61, 145, 145, 145);
	gfx.PutPixel(x + 262, y + 62, 191, 191, 191);
	gfx.PutPixel(x + 262, y + 63, 230, 230, 230);
	gfx.PutPixel(x + 262, y + 64, 253, 253, 253);
	gfx.PutPixel(x + 262, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 90, 213, 213, 213);
	gfx.PutPixel(x + 262, y + 91, 68, 68, 68);
	gfx.PutPixel(x + 262, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 262, y + 93, 5, 5, 5);
	gfx.PutPixel(x + 262, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 262, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 263, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 43, 5, 5, 5);
	gfx.PutPixel(x + 263, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 49, 2, 2, 2);
	gfx.PutPixel(x + 263, y + 50, 14, 14, 14);
	gfx.PutPixel(x + 263, y + 51, 28, 28, 28);
	gfx.PutPixel(x + 263, y + 52, 52, 52, 52);
	gfx.PutPixel(x + 263, y + 53, 76, 76, 76);
	gfx.PutPixel(x + 263, y + 54, 112, 112, 112);
	gfx.PutPixel(x + 263, y + 55, 146, 146, 146);
	gfx.PutPixel(x + 263, y + 56, 181, 181, 181);
	gfx.PutPixel(x + 263, y + 57, 212, 212, 212);
	gfx.PutPixel(x + 263, y + 58, 240, 240, 240);
	gfx.PutPixel(x + 263, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 88, 242, 242, 242);
	gfx.PutPixel(x + 263, y + 89, 130, 130, 130);
	gfx.PutPixel(x + 263, y + 90, 15, 15, 15);
	gfx.PutPixel(x + 263, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 263, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 263, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 264, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 40, 6, 6, 6);
	gfx.PutPixel(x + 264, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 264, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 264, y + 43, 17, 17, 17);
	gfx.PutPixel(x + 264, y + 44, 60, 60, 60);
	gfx.PutPixel(x + 264, y + 45, 93, 93, 93);
	gfx.PutPixel(x + 264, y + 46, 116, 116, 116);
	gfx.PutPixel(x + 264, y + 47, 141, 141, 141);
	gfx.PutPixel(x + 264, y + 48, 169, 169, 169);
	gfx.PutPixel(x + 264, y + 49, 192, 192, 192);
	gfx.PutPixel(x + 264, y + 50, 219, 219, 219);
	gfx.PutPixel(x + 264, y + 51, 238, 238, 238);
	gfx.PutPixel(x + 264, y + 52, 253, 253, 253);
	gfx.PutPixel(x + 264, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 86, 250, 250, 250);
	gfx.PutPixel(x + 264, y + 87, 161, 161, 161);
	gfx.PutPixel(x + 264, y + 88, 40, 40, 40);
	gfx.PutPixel(x + 264, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 264, y + 90, 4, 4, 4);
	gfx.PutPixel(x + 264, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 264, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 38, 6, 6, 6);
	gfx.PutPixel(x + 265, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 265, y + 40, 7, 7, 7);
	gfx.PutPixel(x + 265, y + 41, 67, 67, 67);
	gfx.PutPixel(x + 265, y + 42, 154, 154, 154);
	gfx.PutPixel(x + 265, y + 43, 224, 224, 224);
	gfx.PutPixel(x + 265, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 84, 249, 249, 249);
	gfx.PutPixel(x + 265, y + 85, 167, 167, 167);
	gfx.PutPixel(x + 265, y + 86, 52, 52, 52);
	gfx.PutPixel(x + 265, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 265, y + 88, 2, 2, 2);
	gfx.PutPixel(x + 265, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 265, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 266, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 39, 93, 93, 93);
	gfx.PutPixel(x + 266, y + 40, 206, 206, 206);
	gfx.PutPixel(x + 266, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 82, 239, 239, 239);
	gfx.PutPixel(x + 266, y + 83, 152, 152, 152);
	gfx.PutPixel(x + 266, y + 84, 47, 47, 47);
	gfx.PutPixel(x + 266, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 266, y + 86, 1, 1, 1);
	gfx.PutPixel(x + 266, y + 87, 7, 7, 7);
	gfx.PutPixel(x + 266, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 266, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 36, 3, 3, 3);
	gfx.PutPixel(x + 267, y + 37, 64, 64, 64);
	gfx.PutPixel(x + 267, y + 38, 203, 203, 203);
	gfx.PutPixel(x + 267, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 41, 214, 214, 214);
	gfx.PutPixel(x + 267, y + 42, 167, 167, 167);
	gfx.PutPixel(x + 267, y + 43, 193, 193, 193);
	gfx.PutPixel(x + 267, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 80, 213, 213, 213);
	gfx.PutPixel(x + 267, y + 81, 119, 119, 119);
	gfx.PutPixel(x + 267, y + 82, 30, 30, 30);
	gfx.PutPixel(x + 267, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 267, y + 84, 1, 1, 1);
	gfx.PutPixel(x + 267, y + 85, 7, 7, 7);
	gfx.PutPixel(x + 267, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 267, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 268, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 35, 6, 6, 6);
	gfx.PutPixel(x + 268, y + 36, 5, 5, 5);
	gfx.PutPixel(x + 268, y + 37, 201, 201, 201);
	gfx.PutPixel(x + 268, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 39, 234, 234, 234);
	gfx.PutPixel(x + 268, y + 40, 80, 80, 80);
	gfx.PutPixel(x + 268, y + 41, 6, 6, 6);
	gfx.PutPixel(x + 268, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 268, y + 43, 89, 89, 89);
	gfx.PutPixel(x + 268, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 77, 237, 237, 237);
	gfx.PutPixel(x + 268, y + 78, 164, 164, 164);
	gfx.PutPixel(x + 268, y + 79, 73, 73, 73);
	gfx.PutPixel(x + 268, y + 80, 9, 9, 9);
	gfx.PutPixel(x + 268, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 268, y + 82, 3, 3, 3);
	gfx.PutPixel(x + 268, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 268, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 269, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 35, 6, 6, 6);
	gfx.PutPixel(x + 269, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 269, y + 37, 204, 204, 204);
	gfx.PutPixel(x + 269, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 39, 93, 93, 93);
	gfx.PutPixel(x + 269, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 269, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 269, y + 42, 1, 1, 1);
	gfx.PutPixel(x + 269, y + 43, 172, 172, 172);
	gfx.PutPixel(x + 269, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 74, 244, 244, 244);
	gfx.PutPixel(x + 269, y + 75, 184, 184, 184);
	gfx.PutPixel(x + 269, y + 76, 100, 100, 100);
	gfx.PutPixel(x + 269, y + 77, 28, 28, 28);
	gfx.PutPixel(x + 269, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 269, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 269, y + 80, 5, 5, 5);
	gfx.PutPixel(x + 269, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 269, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 270, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 35, 6, 6, 6);
	gfx.PutPixel(x + 270, y + 36, 5, 5, 5);
	gfx.PutPixel(x + 270, y + 37, 203, 203, 203);
	gfx.PutPixel(x + 270, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 39, 93, 93, 93);
	gfx.PutPixel(x + 270, y + 40, 62, 62, 62);
	gfx.PutPixel(x + 270, y + 41, 95, 95, 95);
	gfx.PutPixel(x + 270, y + 42, 187, 187, 187);
	gfx.PutPixel(x + 270, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 71, 240, 240, 240);
	gfx.PutPixel(x + 270, y + 72, 183, 183, 183);
	gfx.PutPixel(x + 270, y + 73, 106, 106, 106);
	gfx.PutPixel(x + 270, y + 74, 36, 36, 36);
	gfx.PutPixel(x + 270, y + 75, 1, 1, 1);
	gfx.PutPixel(x + 270, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 270, y + 77, 3, 3, 3);
	gfx.PutPixel(x + 270, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 270, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 271, y + 37, 172, 172, 172);
	gfx.PutPixel(x + 271, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 68, 227, 227, 227);
	gfx.PutPixel(x + 271, y + 69, 165, 165, 165);
	gfx.PutPixel(x + 271, y + 70, 93, 93, 93);
	gfx.PutPixel(x + 271, y + 71, 31, 31, 31);
	gfx.PutPixel(x + 271, y + 72, 1, 1, 1);
	gfx.PutPixel(x + 271, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 271, y + 74, 2, 2, 2);
	gfx.PutPixel(x + 271, y + 75, 7, 7, 7);
	gfx.PutPixel(x + 271, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 271, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 272, y + 37, 94, 94, 94);
	gfx.PutPixel(x + 272, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 64, 243, 243, 243);
	gfx.PutPixel(x + 272, y + 65, 197, 197, 197);
	gfx.PutPixel(x + 272, y + 66, 132, 132, 132);
	gfx.PutPixel(x + 272, y + 67, 67, 67, 67);
	gfx.PutPixel(x + 272, y + 68, 18, 18, 18);
	gfx.PutPixel(x + 272, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 272, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 272, y + 71, 2, 2, 2);
	gfx.PutPixel(x + 272, y + 72, 7, 7, 7);
	gfx.PutPixel(x + 272, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 272, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 273, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 273, y + 37, 6, 6, 6);
	gfx.PutPixel(x + 273, y + 38, 179, 179, 179);
	gfx.PutPixel(x + 273, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 60, 246, 246, 246);
	gfx.PutPixel(x + 273, y + 61, 206, 206, 206);
	gfx.PutPixel(x + 273, y + 62, 150, 150, 150);
	gfx.PutPixel(x + 273, y + 63, 87, 87, 87);
	gfx.PutPixel(x + 273, y + 64, 34, 34, 34);
	gfx.PutPixel(x + 273, y + 65, 3, 3, 3);
	gfx.PutPixel(x + 273, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 273, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 273, y + 68, 4, 4, 4);
	gfx.PutPixel(x + 273, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 273, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 37, 3, 3, 3);
	gfx.PutPixel(x + 274, y + 38, 15, 15, 15);
	gfx.PutPixel(x + 274, y + 39, 163, 163, 163);
	gfx.PutPixel(x + 274, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 56, 241, 241, 241);
	gfx.PutPixel(x + 274, y + 57, 201, 201, 201);
	gfx.PutPixel(x + 274, y + 58, 146, 146, 146);
	gfx.PutPixel(x + 274, y + 59, 88, 88, 88);
	gfx.PutPixel(x + 274, y + 60, 39, 39, 39);
	gfx.PutPixel(x + 274, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 274, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 274, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 274, y + 64, 2, 2, 2);
	gfx.PutPixel(x + 274, y + 65, 7, 7, 7);
	gfx.PutPixel(x + 274, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 274, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 275, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 275, y + 39, 1, 1, 1);
	gfx.PutPixel(x + 275, y + 40, 68, 68, 68);
	gfx.PutPixel(x + 275, y + 41, 172, 172, 172);
	gfx.PutPixel(x + 275, y + 42, 237, 237, 237);
	gfx.PutPixel(x + 275, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 51, 252, 252, 252);
	gfx.PutPixel(x + 275, y + 52, 223, 223, 223);
	gfx.PutPixel(x + 275, y + 53, 180, 180, 180);
	gfx.PutPixel(x + 275, y + 54, 128, 128, 128);
	gfx.PutPixel(x + 275, y + 55, 74, 74, 74);
	gfx.PutPixel(x + 275, y + 56, 31, 31, 31);
	gfx.PutPixel(x + 275, y + 57, 5, 5, 5);
	gfx.PutPixel(x + 275, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 275, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 275, y + 60, 1, 1, 1);
	gfx.PutPixel(x + 275, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 275, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 275, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 39, 7, 7, 7);
	gfx.PutPixel(x + 276, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 276, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 276, y + 42, 26, 26, 26);
	gfx.PutPixel(x + 276, y + 43, 74, 74, 74);
	gfx.PutPixel(x + 276, y + 44, 120, 120, 120);
	gfx.PutPixel(x + 276, y + 45, 158, 158, 158);
	gfx.PutPixel(x + 276, y + 46, 183, 183, 183);
	gfx.PutPixel(x + 276, y + 47, 198, 198, 198);
	gfx.PutPixel(x + 276, y + 48, 190, 190, 190);
	gfx.PutPixel(x + 276, y + 49, 144, 144, 144);
	gfx.PutPixel(x + 276, y + 50, 94, 94, 94);
	gfx.PutPixel(x + 276, y + 51, 49, 49, 49);
	gfx.PutPixel(x + 276, y + 52, 16, 16, 16);
	gfx.PutPixel(x + 276, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 276, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 276, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 276, y + 56, 3, 3, 3);
	gfx.PutPixel(x + 276, y + 57, 6, 6, 6);
	gfx.PutPixel(x + 276, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 276, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 42, 3, 3, 3);
	gfx.PutPixel(x + 277, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 47, 3, 3, 3);
	gfx.PutPixel(x + 277, y + 48, 1, 1, 1);
	gfx.PutPixel(x + 277, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 277, y + 51, 1, 1, 1);
	gfx.PutPixel(x + 277, y + 52, 4, 4, 4);
	gfx.PutPixel(x + 277, y + 53, 7, 7, 7);
	gfx.PutPixel(x + 277, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 277, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 46, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 47, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 48, 7, 7, 7);
	gfx.PutPixel(x + 278, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 278, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 82, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 83, 7, 7, 7);
	gfx.PutPixel(x + 279, y + 84, 5, 5, 5);
	gfx.PutPixel(x + 279, y + 85, 4, 4, 4);
	gfx.PutPixel(x + 279, y + 86, 2, 2, 2);
	gfx.PutPixel(x + 279, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 279, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 279, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 279, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 279, y + 91, 5, 5, 5);
	gfx.PutPixel(x + 279, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 279, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 280, y + 39, 2, 2, 2);
	gfx.PutPixel(x + 280, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 61, 1, 1, 1);
	gfx.PutPixel(x + 280, y + 62, 1, 1, 1);
	gfx.PutPixel(x + 280, y + 63, 2, 2, 2);
	gfx.PutPixel(x + 280, y + 64, 2, 2, 2);
	gfx.PutPixel(x + 280, y + 65, 3, 3, 3);
	gfx.PutPixel(x + 280, y + 66, 3, 3, 3);
	gfx.PutPixel(x + 280, y + 67, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 68, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 69, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 70, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 71, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 72, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 73, 3, 3, 3);
	gfx.PutPixel(x + 280, y + 74, 1, 1, 1);
	gfx.PutPixel(x + 280, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 83, 4, 4, 4);
	gfx.PutPixel(x + 280, y + 84, 14, 14, 14);
	gfx.PutPixel(x + 280, y + 85, 20, 20, 20);
	gfx.PutPixel(x + 280, y + 86, 32, 32, 32);
	gfx.PutPixel(x + 280, y + 87, 50, 50, 50);
	gfx.PutPixel(x + 280, y + 88, 62, 62, 62);
	gfx.PutPixel(x + 280, y + 89, 79, 79, 79);
	gfx.PutPixel(x + 280, y + 90, 62, 62, 62);
	gfx.PutPixel(x + 280, y + 91, 11, 11, 11);
	gfx.PutPixel(x + 280, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 280, y + 93, 2, 2, 2);
	gfx.PutPixel(x + 280, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 280, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 36, 7, 7, 7);
	gfx.PutPixel(x + 281, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 281, y + 38, 1, 1, 1);
	gfx.PutPixel(x + 281, y + 39, 29, 29, 29);
	gfx.PutPixel(x + 281, y + 40, 63, 63, 63);
	gfx.PutPixel(x + 281, y + 41, 90, 90, 90);
	gfx.PutPixel(x + 281, y + 42, 105, 105, 105);
	gfx.PutPixel(x + 281, y + 43, 106, 106, 106);
	gfx.PutPixel(x + 281, y + 44, 101, 101, 101);
	gfx.PutPixel(x + 281, y + 45, 97, 97, 97);
	gfx.PutPixel(x + 281, y + 46, 93, 93, 93);
	gfx.PutPixel(x + 281, y + 47, 88, 88, 88);
	gfx.PutPixel(x + 281, y + 48, 83, 83, 83);
	gfx.PutPixel(x + 281, y + 49, 75, 75, 75);
	gfx.PutPixel(x + 281, y + 50, 70, 70, 70);
	gfx.PutPixel(x + 281, y + 51, 69, 69, 69);
	gfx.PutPixel(x + 281, y + 52, 67, 67, 67);
	gfx.PutPixel(x + 281, y + 53, 65, 65, 65);
	gfx.PutPixel(x + 281, y + 54, 63, 63, 63);
	gfx.PutPixel(x + 281, y + 55, 60, 60, 60);
	gfx.PutPixel(x + 281, y + 56, 58, 58, 58);
	gfx.PutPixel(x + 281, y + 57, 55, 55, 55);
	gfx.PutPixel(x + 281, y + 58, 53, 53, 53);
	gfx.PutPixel(x + 281, y + 59, 50, 50, 50);
	gfx.PutPixel(x + 281, y + 60, 47, 47, 47);
	gfx.PutPixel(x + 281, y + 61, 44, 44, 44);
	gfx.PutPixel(x + 281, y + 62, 41, 41, 41);
	gfx.PutPixel(x + 281, y + 63, 37, 37, 37);
	gfx.PutPixel(x + 281, y + 64, 32, 32, 32);
	gfx.PutPixel(x + 281, y + 65, 26, 26, 26);
	gfx.PutPixel(x + 281, y + 66, 24, 24, 24);
	gfx.PutPixel(x + 281, y + 67, 23, 23, 23);
	gfx.PutPixel(x + 281, y + 68, 22, 22, 22);
	gfx.PutPixel(x + 281, y + 69, 21, 21, 21);
	gfx.PutPixel(x + 281, y + 70, 20, 20, 20);
	gfx.PutPixel(x + 281, y + 71, 18, 18, 18);
	gfx.PutPixel(x + 281, y + 72, 18, 18, 18);
	gfx.PutPixel(x + 281, y + 73, 25, 25, 25);
	gfx.PutPixel(x + 281, y + 74, 43, 43, 43);
	gfx.PutPixel(x + 281, y + 75, 57, 57, 57);
	gfx.PutPixel(x + 281, y + 76, 68, 68, 68);
	gfx.PutPixel(x + 281, y + 77, 91, 91, 91);
	gfx.PutPixel(x + 281, y + 78, 111, 111, 111);
	gfx.PutPixel(x + 281, y + 79, 125, 125, 125);
	gfx.PutPixel(x + 281, y + 80, 147, 147, 147);
	gfx.PutPixel(x + 281, y + 81, 168, 168, 168);
	gfx.PutPixel(x + 281, y + 82, 183, 183, 183);
	gfx.PutPixel(x + 281, y + 83, 200, 200, 200);
	gfx.PutPixel(x + 281, y + 84, 218, 218, 218);
	gfx.PutPixel(x + 281, y + 85, 229, 229, 229);
	gfx.PutPixel(x + 281, y + 86, 241, 241, 241);
	gfx.PutPixel(x + 281, y + 87, 251, 251, 251);
	gfx.PutPixel(x + 281, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 90, 254, 254, 254);
	gfx.PutPixel(x + 281, y + 91, 214, 214, 214);
	gfx.PutPixel(x + 281, y + 92, 131, 131, 131);
	gfx.PutPixel(x + 281, y + 93, 40, 40, 40);
	gfx.PutPixel(x + 281, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 281, y + 95, 2, 2, 2);
	gfx.PutPixel(x + 281, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 281, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 282, y + 37, 71, 71, 71);
	gfx.PutPixel(x + 282, y + 38, 182, 182, 182);
	gfx.PutPixel(x + 282, y + 39, 241, 241, 241);
	gfx.PutPixel(x + 282, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 57, 254, 254, 254);
	gfx.PutPixel(x + 282, y + 58, 253, 253, 253);
	gfx.PutPixel(x + 282, y + 59, 251, 251, 251);
	gfx.PutPixel(x + 282, y + 60, 250, 250, 250);
	gfx.PutPixel(x + 282, y + 61, 248, 248, 248);
	gfx.PutPixel(x + 282, y + 62, 246, 246, 246);
	gfx.PutPixel(x + 282, y + 63, 244, 244, 244);
	gfx.PutPixel(x + 282, y + 64, 241, 241, 241);
	gfx.PutPixel(x + 282, y + 65, 238, 238, 238);
	gfx.PutPixel(x + 282, y + 66, 236, 236, 236);
	gfx.PutPixel(x + 282, y + 67, 234, 234, 234);
	gfx.PutPixel(x + 282, y + 68, 232, 232, 232);
	gfx.PutPixel(x + 282, y + 69, 230, 230, 230);
	gfx.PutPixel(x + 282, y + 70, 228, 228, 228);
	gfx.PutPixel(x + 282, y + 71, 224, 224, 224);
	gfx.PutPixel(x + 282, y + 72, 225, 225, 225);
	gfx.PutPixel(x + 282, y + 73, 235, 235, 235);
	gfx.PutPixel(x + 282, y + 74, 248, 248, 248);
	gfx.PutPixel(x + 282, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 93, 246, 246, 246);
	gfx.PutPixel(x + 282, y + 94, 159, 159, 159);
	gfx.PutPixel(x + 282, y + 95, 38, 38, 38);
	gfx.PutPixel(x + 282, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 282, y + 97, 7, 7, 7);
	gfx.PutPixel(x + 282, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 282, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 34, 7, 7, 7);
	gfx.PutPixel(x + 283, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 283, y + 36, 125, 125, 125);
	gfx.PutPixel(x + 283, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 40, 213, 213, 213);
	gfx.PutPixel(x + 283, y + 41, 150, 150, 150);
	gfx.PutPixel(x + 283, y + 42, 186, 186, 186);
	gfx.PutPixel(x + 283, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 95, 241, 241, 241);
	gfx.PutPixel(x + 283, y + 96, 107, 107, 107);
	gfx.PutPixel(x + 283, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 283, y + 98, 5, 5, 5);
	gfx.PutPixel(x + 283, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 283, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 284, y + 35, 74, 74, 74);
	gfx.PutPixel(x + 284, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 38, 224, 224, 224);
	gfx.PutPixel(x + 284, y + 39, 85, 85, 85);
	gfx.PutPixel(x + 284, y + 40, 6, 6, 6);
	gfx.PutPixel(x + 284, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 284, y + 42, 121, 121, 121);
	gfx.PutPixel(x + 284, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 97, 142, 142, 142);
	gfx.PutPixel(x + 284, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 284, y + 99, 7, 7, 7);
	gfx.PutPixel(x + 284, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 284, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 33, 7, 7, 7);
	gfx.PutPixel(x + 285, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 285, y + 35, 170, 170, 170);
	gfx.PutPixel(x + 285, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 37, 210, 210, 210);
	gfx.PutPixel(x + 285, y + 38, 31, 31, 31);
	gfx.PutPixel(x + 285, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 285, y + 40, 10, 10, 10);
	gfx.PutPixel(x + 285, y + 41, 125, 125, 125);
	gfx.PutPixel(x + 285, y + 42, 244, 244, 244);
	gfx.PutPixel(x + 285, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 98, 109, 109, 109);
	gfx.PutPixel(x + 285, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 285, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 285, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 286, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 33, 6, 6, 6);
	gfx.PutPixel(x + 286, y + 34, 9, 9, 9);
	gfx.PutPixel(x + 286, y + 35, 217, 217, 217);
	gfx.PutPixel(x + 286, y + 36, 240, 240, 240);
	gfx.PutPixel(x + 286, y + 37, 39, 39, 39);
	gfx.PutPixel(x + 286, y + 38, 0, 0, 0);
	gfx.PutPixel(x + 286, y + 39, 64, 64, 64);
	gfx.PutPixel(x + 286, y + 40, 207, 207, 207);
	gfx.PutPixel(x + 286, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 98, 227, 227, 227);
	gfx.PutPixel(x + 286, y + 99, 24, 24, 24);
	gfx.PutPixel(x + 286, y + 100, 4, 4, 4);
	gfx.PutPixel(x + 286, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 286, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 287, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 33, 6, 6, 6);
	gfx.PutPixel(x + 287, y + 34, 12, 12, 12);
	gfx.PutPixel(x + 287, y + 35, 226, 226, 226);
	gfx.PutPixel(x + 287, y + 36, 217, 217, 217);
	gfx.PutPixel(x + 287, y + 37, 37, 37, 37);
	gfx.PutPixel(x + 287, y + 38, 144, 144, 144);
	gfx.PutPixel(x + 287, y + 39, 252, 252, 252);
	gfx.PutPixel(x + 287, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 99, 81, 81, 81);
	gfx.PutPixel(x + 287, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 287, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 287, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 288, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 33, 6, 6, 6);
	gfx.PutPixel(x + 288, y + 34, 10, 10, 10);
	gfx.PutPixel(x + 288, y + 35, 213, 213, 213);
	gfx.PutPixel(x + 288, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 37, 245, 245, 245);
	gfx.PutPixel(x + 288, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 288, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 288, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 288, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 33, 7, 7, 7);
	gfx.PutPixel(x + 289, y + 34, 1, 1, 1);
	gfx.PutPixel(x + 289, y + 35, 186, 186, 186);
	gfx.PutPixel(x + 289, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 99, 103, 103, 103);
	gfx.PutPixel(x + 289, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 289, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 289, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 290, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 290, y + 35, 125, 125, 125);
	gfx.PutPixel(x + 290, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 99, 100, 100, 100);
	gfx.PutPixel(x + 290, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 290, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 290, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 291, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 34, 1, 1, 1);
	gfx.PutPixel(x + 291, y + 35, 38, 38, 38);
	gfx.PutPixel(x + 291, y + 36, 238, 238, 238);
	gfx.PutPixel(x + 291, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 99, 61, 61, 61);
	gfx.PutPixel(x + 291, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 291, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 291, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 292, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 292, y + 36, 110, 110, 110);
	gfx.PutPixel(x + 292, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 98, 185, 185, 185);
	gfx.PutPixel(x + 292, y + 99, 4, 4, 4);
	gfx.PutPixel(x + 292, y + 100, 6, 6, 6);
	gfx.PutPixel(x + 292, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 292, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 293, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 293, y + 37, 114, 114, 114);
	gfx.PutPixel(x + 293, y + 38, 248, 248, 248);
	gfx.PutPixel(x + 293, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 97, 221, 221, 221);
	gfx.PutPixel(x + 293, y + 98, 38, 38, 38);
	gfx.PutPixel(x + 293, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 293, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 293, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 36, 7, 7, 7);
	gfx.PutPixel(x + 294, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 294, y + 38, 49, 49, 49);
	gfx.PutPixel(x + 294, y + 39, 161, 161, 161);
	gfx.PutPixel(x + 294, y + 40, 234, 234, 234);
	gfx.PutPixel(x + 294, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 56, 254, 254, 254);
	gfx.PutPixel(x + 294, y + 57, 248, 248, 248);
	gfx.PutPixel(x + 294, y + 58, 238, 238, 238);
	gfx.PutPixel(x + 294, y + 59, 228, 228, 228);
	gfx.PutPixel(x + 294, y + 60, 218, 218, 218);
	gfx.PutPixel(x + 294, y + 61, 204, 204, 204);
	gfx.PutPixel(x + 294, y + 62, 187, 187, 187);
	gfx.PutPixel(x + 294, y + 63, 183, 183, 183);
	gfx.PutPixel(x + 294, y + 64, 188, 188, 188);
	gfx.PutPixel(x + 294, y + 65, 196, 196, 196);
	gfx.PutPixel(x + 294, y + 66, 205, 205, 205);
	gfx.PutPixel(x + 294, y + 67, 211, 211, 211);
	gfx.PutPixel(x + 294, y + 68, 216, 216, 216);
	gfx.PutPixel(x + 294, y + 69, 221, 221, 221);
	gfx.PutPixel(x + 294, y + 70, 226, 226, 226);
	gfx.PutPixel(x + 294, y + 71, 230, 230, 230);
	gfx.PutPixel(x + 294, y + 72, 234, 234, 234);
	gfx.PutPixel(x + 294, y + 73, 238, 238, 238);
	gfx.PutPixel(x + 294, y + 74, 243, 243, 243);
	gfx.PutPixel(x + 294, y + 75, 247, 247, 247);
	gfx.PutPixel(x + 294, y + 76, 251, 251, 251);
	gfx.PutPixel(x + 294, y + 77, 253, 253, 253);
	gfx.PutPixel(x + 294, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 96, 184, 184, 184);
	gfx.PutPixel(x + 294, y + 97, 33, 33, 33);
	gfx.PutPixel(x + 294, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 294, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 294, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 38, 1, 1, 1);
	gfx.PutPixel(x + 295, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 295, y + 40, 23, 23, 23);
	gfx.PutPixel(x + 295, y + 41, 70, 70, 70);
	gfx.PutPixel(x + 295, y + 42, 113, 113, 113);
	gfx.PutPixel(x + 295, y + 43, 141, 141, 141);
	gfx.PutPixel(x + 295, y + 44, 162, 162, 162);
	gfx.PutPixel(x + 295, y + 45, 169, 169, 169);
	gfx.PutPixel(x + 295, y + 46, 169, 169, 169);
	gfx.PutPixel(x + 295, y + 47, 169, 169, 169);
	gfx.PutPixel(x + 295, y + 48, 170, 170, 170);
	gfx.PutPixel(x + 295, y + 49, 165, 165, 165);
	gfx.PutPixel(x + 295, y + 50, 147, 147, 147);
	gfx.PutPixel(x + 295, y + 51, 127, 127, 127);
	gfx.PutPixel(x + 295, y + 52, 115, 115, 115);
	gfx.PutPixel(x + 295, y + 53, 101, 101, 101);
	gfx.PutPixel(x + 295, y + 54, 80, 80, 80);
	gfx.PutPixel(x + 295, y + 55, 65, 65, 65);
	gfx.PutPixel(x + 295, y + 56, 56, 56, 56);
	gfx.PutPixel(x + 295, y + 57, 43, 43, 43);
	gfx.PutPixel(x + 295, y + 58, 27, 27, 27);
	gfx.PutPixel(x + 295, y + 59, 20, 20, 20);
	gfx.PutPixel(x + 295, y + 60, 15, 15, 15);
	gfx.PutPixel(x + 295, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 295, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 295, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 295, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 295, y + 65, 2, 2, 2);
	gfx.PutPixel(x + 295, y + 66, 7, 7, 7);
	gfx.PutPixel(x + 295, y + 67, 10, 10, 10);
	gfx.PutPixel(x + 295, y + 68, 14, 14, 14);
	gfx.PutPixel(x + 295, y + 69, 16, 16, 16);
	gfx.PutPixel(x + 295, y + 70, 19, 19, 19);
	gfx.PutPixel(x + 295, y + 71, 21, 21, 21);
	gfx.PutPixel(x + 295, y + 72, 23, 23, 23);
	gfx.PutPixel(x + 295, y + 73, 26, 26, 26);
	gfx.PutPixel(x + 295, y + 74, 36, 36, 36);
	gfx.PutPixel(x + 295, y + 75, 43, 43, 43);
	gfx.PutPixel(x + 295, y + 76, 49, 49, 49);
	gfx.PutPixel(x + 295, y + 77, 54, 54, 54);
	gfx.PutPixel(x + 295, y + 78, 58, 58, 58);
	gfx.PutPixel(x + 295, y + 79, 63, 63, 63);
	gfx.PutPixel(x + 295, y + 80, 67, 67, 67);
	gfx.PutPixel(x + 295, y + 81, 70, 70, 70);
	gfx.PutPixel(x + 295, y + 82, 80, 80, 80);
	gfx.PutPixel(x + 295, y + 83, 91, 91, 91);
	gfx.PutPixel(x + 295, y + 84, 99, 99, 99);
	gfx.PutPixel(x + 295, y + 85, 106, 106, 106);
	gfx.PutPixel(x + 295, y + 86, 112, 112, 112);
	gfx.PutPixel(x + 295, y + 87, 117, 117, 117);
	gfx.PutPixel(x + 295, y + 88, 122, 122, 122);
	gfx.PutPixel(x + 295, y + 89, 127, 127, 127);
	gfx.PutPixel(x + 295, y + 90, 134, 134, 134);
	gfx.PutPixel(x + 295, y + 91, 147, 147, 147);
	gfx.PutPixel(x + 295, y + 92, 155, 155, 155);
	gfx.PutPixel(x + 295, y + 93, 164, 164, 164);
	gfx.PutPixel(x + 295, y + 94, 167, 167, 167);
	gfx.PutPixel(x + 295, y + 95, 86, 86, 86);
	gfx.PutPixel(x + 295, y + 96, 4, 4, 4);
	gfx.PutPixel(x + 295, y + 97, 2, 2, 2);
	gfx.PutPixel(x + 295, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 295, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 40, 3, 3, 3);
	gfx.PutPixel(x + 296, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 57, 1, 1, 1);
	gfx.PutPixel(x + 296, y + 58, 3, 3, 3);
	gfx.PutPixel(x + 296, y + 59, 4, 4, 4);
	gfx.PutPixel(x + 296, y + 60, 5, 5, 5);
	gfx.PutPixel(x + 296, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 296, y + 62, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 63, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 64, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 65, 7, 7, 7);
	gfx.PutPixel(x + 296, y + 66, 6, 6, 6);
	gfx.PutPixel(x + 296, y + 67, 6, 6, 6);
	gfx.PutPixel(x + 296, y + 68, 5, 5, 5);
	gfx.PutPixel(x + 296, y + 69, 5, 5, 5);
	gfx.PutPixel(x + 296, y + 70, 4, 4, 4);
	gfx.PutPixel(x + 296, y + 71, 4, 4, 4);
	gfx.PutPixel(x + 296, y + 72, 4, 4, 4);
	gfx.PutPixel(x + 296, y + 73, 3, 3, 3);
	gfx.PutPixel(x + 296, y + 74, 2, 2, 2);
	gfx.PutPixel(x + 296, y + 75, 1, 1, 1);
	gfx.PutPixel(x + 296, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 296, y + 96, 6, 6, 6);
	gfx.PutPixel(x + 296, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 296, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 88, 7, 7, 7);
	gfx.PutPixel(x + 297, y + 89, 4, 4, 4);
	gfx.PutPixel(x + 297, y + 90, 2, 2, 2);
	gfx.PutPixel(x + 297, y + 91, 1, 1, 1);
	gfx.PutPixel(x + 297, y + 92, 1, 1, 1);
	gfx.PutPixel(x + 297, y + 93, 3, 3, 3);
	gfx.PutPixel(x + 297, y + 94, 6, 6, 6);
	gfx.PutPixel(x + 297, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 297, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 298, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 84, 6, 6, 6);
	gfx.PutPixel(x + 298, y + 85, 1, 1, 1);
	gfx.PutPixel(x + 298, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 298, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 298, y + 88, 4, 4, 4);
	gfx.PutPixel(x + 298, y + 89, 20, 20, 20);
	gfx.PutPixel(x + 298, y + 90, 34, 34, 34);
	gfx.PutPixel(x + 298, y + 91, 41, 41, 41);
	gfx.PutPixel(x + 298, y + 92, 38, 38, 38);
	gfx.PutPixel(x + 298, y + 93, 24, 24, 24);
	gfx.PutPixel(x + 298, y + 94, 5, 5, 5);
	gfx.PutPixel(x + 298, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 298, y + 96, 4, 4, 4);
	gfx.PutPixel(x + 298, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 298, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 299, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 81, 5, 5, 5);
	gfx.PutPixel(x + 299, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 299, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 299, y + 84, 6, 6, 6);
	gfx.PutPixel(x + 299, y + 85, 41, 41, 41);
	gfx.PutPixel(x + 299, y + 86, 94, 94, 94);
	gfx.PutPixel(x + 299, y + 87, 151, 151, 151);
	gfx.PutPixel(x + 299, y + 88, 198, 198, 198);
	gfx.PutPixel(x + 299, y + 89, 229, 229, 229);
	gfx.PutPixel(x + 299, y + 90, 243, 243, 243);
	gfx.PutPixel(x + 299, y + 91, 247, 247, 247);
	gfx.PutPixel(x + 299, y + 92, 245, 245, 245);
	gfx.PutPixel(x + 299, y + 93, 235, 235, 235);
	gfx.PutPixel(x + 299, y + 94, 200, 200, 200);
	gfx.PutPixel(x + 299, y + 95, 117, 117, 117);
	gfx.PutPixel(x + 299, y + 96, 18, 18, 18);
	gfx.PutPixel(x + 299, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 299, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 299, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 300, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 78, 4, 4, 4);
	gfx.PutPixel(x + 300, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 300, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 300, y + 81, 15, 15, 15);
	gfx.PutPixel(x + 300, y + 82, 68, 68, 68);
	gfx.PutPixel(x + 300, y + 83, 138, 138, 138);
	gfx.PutPixel(x + 300, y + 84, 204, 204, 204);
	gfx.PutPixel(x + 300, y + 85, 248, 248, 248);
	gfx.PutPixel(x + 300, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 96, 218, 218, 218);
	gfx.PutPixel(x + 300, y + 97, 57, 57, 57);
	gfx.PutPixel(x + 300, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 300, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 300, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 301, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 75, 5, 5, 5);
	gfx.PutPixel(x + 301, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 301, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 301, y + 78, 18, 18, 18);
	gfx.PutPixel(x + 301, y + 79, 77, 77, 77);
	gfx.PutPixel(x + 301, y + 80, 155, 155, 155);
	gfx.PutPixel(x + 301, y + 81, 224, 224, 224);
	gfx.PutPixel(x + 301, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 97, 237, 237, 237);
	gfx.PutPixel(x + 301, y + 98, 54, 54, 54);
	gfx.PutPixel(x + 301, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 301, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 301, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 302, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 72, 6, 6, 6);
	gfx.PutPixel(x + 302, y + 73, 1, 1, 1);
	gfx.PutPixel(x + 302, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 302, y + 75, 14, 14, 14);
	gfx.PutPixel(x + 302, y + 76, 73, 73, 73);
	gfx.PutPixel(x + 302, y + 77, 155, 155, 155);
	gfx.PutPixel(x + 302, y + 78, 227, 227, 227);
	gfx.PutPixel(x + 302, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 98, 208, 208, 208);
	gfx.PutPixel(x + 302, y + 99, 19, 19, 19);
	gfx.PutPixel(x + 302, y + 100, 4, 4, 4);
	gfx.PutPixel(x + 302, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 302, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 69, 7, 7, 7);
	gfx.PutPixel(x + 303, y + 70, 2, 2, 2);
	gfx.PutPixel(x + 303, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 303, y + 72, 6, 6, 6);
	gfx.PutPixel(x + 303, y + 73, 58, 58, 58);
	gfx.PutPixel(x + 303, y + 74, 142, 142, 142);
	gfx.PutPixel(x + 303, y + 75, 221, 221, 221);
	gfx.PutPixel(x + 303, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 99, 96, 96, 96);
	gfx.PutPixel(x + 303, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 303, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 303, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 304, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 67, 4, 4, 4);
	gfx.PutPixel(x + 304, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 304, y + 69, 1, 1, 1);
	gfx.PutPixel(x + 304, y + 70, 39, 39, 39);
	gfx.PutPixel(x + 304, y + 71, 120, 120, 120);
	gfx.PutPixel(x + 304, y + 72, 205, 205, 205);
	gfx.PutPixel(x + 304, y + 73, 253, 253, 253);
	gfx.PutPixel(x + 304, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 99, 88, 88, 88);
	gfx.PutPixel(x + 304, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 304, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 304, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 305, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 64, 6, 6, 6);
	gfx.PutPixel(x + 305, y + 65, 1, 1, 1);
	gfx.PutPixel(x + 305, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 305, y + 67, 19, 19, 19);
	gfx.PutPixel(x + 305, y + 68, 90, 90, 90);
	gfx.PutPixel(x + 305, y + 69, 181, 181, 181);
	gfx.PutPixel(x + 305, y + 70, 246, 246, 246);
	gfx.PutPixel(x + 305, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 98, 215, 215, 215);
	gfx.PutPixel(x + 305, y + 99, 20, 20, 20);
	gfx.PutPixel(x + 305, y + 100, 4, 4, 4);
	gfx.PutPixel(x + 305, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 305, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 306, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 62, 3, 3, 3);
	gfx.PutPixel(x + 306, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 306, y + 64, 5, 5, 5);
	gfx.PutPixel(x + 306, y + 65, 57, 57, 57);
	gfx.PutPixel(x + 306, y + 66, 146, 146, 146);
	gfx.PutPixel(x + 306, y + 67, 228, 228, 228);
	gfx.PutPixel(x + 306, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 97, 237, 237, 237);
	gfx.PutPixel(x + 306, y + 98, 60, 60, 60);
	gfx.PutPixel(x + 306, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 306, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 306, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 307, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 59, 6, 6, 6);
	gfx.PutPixel(x + 307, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 307, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 307, y + 62, 28, 28, 28);
	gfx.PutPixel(x + 307, y + 63, 108, 108, 108);
	gfx.PutPixel(x + 307, y + 64, 199, 199, 199);
	gfx.PutPixel(x + 307, y + 65, 253, 253, 253);
	gfx.PutPixel(x + 307, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 96, 207, 207, 207);
	gfx.PutPixel(x + 307, y + 97, 49, 49, 49);
	gfx.PutPixel(x + 307, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 307, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 307, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 57, 3, 3, 3);
	gfx.PutPixel(x + 308, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 308, y + 59, 7, 7, 7);
	gfx.PutPixel(x + 308, y + 60, 66, 66, 66);
	gfx.PutPixel(x + 308, y + 61, 159, 159, 159);
	gfx.PutPixel(x + 308, y + 62, 238, 238, 238);
	gfx.PutPixel(x + 308, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 94, 226, 226, 226);
	gfx.PutPixel(x + 308, y + 95, 122, 122, 122);
	gfx.PutPixel(x + 308, y + 96, 13, 13, 13);
	gfx.PutPixel(x + 308, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 308, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 308, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 309, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 53, 5, 5, 5);
	gfx.PutPixel(x + 309, y + 54, 1, 1, 1);
	gfx.PutPixel(x + 309, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 309, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 309, y + 57, 30, 30, 30);
	gfx.PutPixel(x + 309, y + 58, 112, 112, 112);
	gfx.PutPixel(x + 309, y + 59, 205, 205, 205);
	gfx.PutPixel(x + 309, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 91, 224, 224, 224);
	gfx.PutPixel(x + 309, y + 92, 153, 153, 153);
	gfx.PutPixel(x + 309, y + 93, 74, 74, 74);
	gfx.PutPixel(x + 309, y + 94, 16, 16, 16);
	gfx.PutPixel(x + 309, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 309, y + 96, 5, 5, 5);
	gfx.PutPixel(x + 309, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 309, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 310, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 49, 6, 6, 6);
	gfx.PutPixel(x + 310, y + 50, 1, 1, 1);
	gfx.PutPixel(x + 310, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 310, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 310, y + 53, 10, 10, 10);
	gfx.PutPixel(x + 310, y + 54, 45, 45, 45);
	gfx.PutPixel(x + 310, y + 55, 94, 94, 94);
	gfx.PutPixel(x + 310, y + 56, 163, 163, 163);
	gfx.PutPixel(x + 310, y + 57, 239, 239, 239);
	gfx.PutPixel(x + 310, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 88, 224, 224, 224);
	gfx.PutPixel(x + 310, y + 89, 153, 153, 153);
	gfx.PutPixel(x + 310, y + 90, 74, 74, 74);
	gfx.PutPixel(x + 310, y + 91, 16, 16, 16);
	gfx.PutPixel(x + 310, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 310, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 310, y + 94, 4, 4, 4);
	gfx.PutPixel(x + 310, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 310, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 46, 4, 4, 4);
	gfx.PutPixel(x + 311, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 311, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 311, y + 49, 7, 7, 7);
	gfx.PutPixel(x + 311, y + 50, 42, 42, 42);
	gfx.PutPixel(x + 311, y + 51, 97, 97, 97);
	gfx.PutPixel(x + 311, y + 52, 159, 159, 159);
	gfx.PutPixel(x + 311, y + 53, 214, 214, 214);
	gfx.PutPixel(x + 311, y + 54, 250, 250, 250);
	gfx.PutPixel(x + 311, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 85, 224, 224, 224);
	gfx.PutPixel(x + 311, y + 86, 153, 153, 153);
	gfx.PutPixel(x + 311, y + 87, 73, 73, 73);
	gfx.PutPixel(x + 311, y + 88, 16, 16, 16);
	gfx.PutPixel(x + 311, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 311, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 311, y + 91, 4, 4, 4);
	gfx.PutPixel(x + 311, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 311, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 312, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 43, 3, 3, 3);
	gfx.PutPixel(x + 312, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 312, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 312, y + 46, 19, 19, 19);
	gfx.PutPixel(x + 312, y + 47, 73, 73, 73);
	gfx.PutPixel(x + 312, y + 48, 142, 142, 142);
	gfx.PutPixel(x + 312, y + 49, 206, 206, 206);
	gfx.PutPixel(x + 312, y + 50, 248, 248, 248);
	gfx.PutPixel(x + 312, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 82, 224, 224, 224);
	gfx.PutPixel(x + 312, y + 83, 152, 152, 152);
	gfx.PutPixel(x + 312, y + 84, 73, 73, 73);
	gfx.PutPixel(x + 312, y + 85, 16, 16, 16);
	gfx.PutPixel(x + 312, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 312, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 312, y + 88, 4, 4, 4);
	gfx.PutPixel(x + 312, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 312, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 313, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 40, 6, 6, 6);
	gfx.PutPixel(x + 313, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 313, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 313, y + 43, 21, 21, 21);
	gfx.PutPixel(x + 313, y + 44, 86, 86, 86);
	gfx.PutPixel(x + 313, y + 45, 165, 165, 165);
	gfx.PutPixel(x + 313, y + 46, 230, 230, 230);
	gfx.PutPixel(x + 313, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 79, 224, 224, 224);
	gfx.PutPixel(x + 313, y + 80, 152, 152, 152);
	gfx.PutPixel(x + 313, y + 81, 73, 73, 73);
	gfx.PutPixel(x + 313, y + 82, 16, 16, 16);
	gfx.PutPixel(x + 313, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 313, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 313, y + 85, 4, 4, 4);
	gfx.PutPixel(x + 313, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 313, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 38, 7, 7, 7);
	gfx.PutPixel(x + 314, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 314, y + 40, 3, 3, 3);
	gfx.PutPixel(x + 314, y + 41, 62, 62, 62);
	gfx.PutPixel(x + 314, y + 42, 154, 154, 154);
	gfx.PutPixel(x + 314, y + 43, 231, 231, 231);
	gfx.PutPixel(x + 314, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 76, 224, 224, 224);
	gfx.PutPixel(x + 314, y + 77, 152, 152, 152);
	gfx.PutPixel(x + 314, y + 78, 73, 73, 73);
	gfx.PutPixel(x + 314, y + 79, 16, 16, 16);
	gfx.PutPixel(x + 314, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 314, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 314, y + 82, 4, 4, 4);
	gfx.PutPixel(x + 314, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 314, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 315, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 37, 3, 3, 3);
	gfx.PutPixel(x + 315, y + 38, 2, 2, 2);
	gfx.PutPixel(x + 315, y + 39, 77, 77, 77);
	gfx.PutPixel(x + 315, y + 40, 193, 193, 193);
	gfx.PutPixel(x + 315, y + 41, 254, 254, 254);
	gfx.PutPixel(x + 315, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 73, 224, 224, 224);
	gfx.PutPixel(x + 315, y + 74, 152, 152, 152);
	gfx.PutPixel(x + 315, y + 75, 73, 73, 73);
	gfx.PutPixel(x + 315, y + 76, 15, 15, 15);
	gfx.PutPixel(x + 315, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 315, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 315, y + 79, 4, 4, 4);
	gfx.PutPixel(x + 315, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 315, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 316, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 36, 1, 1, 1);
	gfx.PutPixel(x + 316, y + 37, 28, 28, 28);
	gfx.PutPixel(x + 316, y + 38, 176, 176, 176);
	gfx.PutPixel(x + 316, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 43, 208, 208, 208);
	gfx.PutPixel(x + 316, y + 44, 171, 171, 171);
	gfx.PutPixel(x + 316, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 70, 224, 224, 224);
	gfx.PutPixel(x + 316, y + 71, 152, 152, 152);
	gfx.PutPixel(x + 316, y + 72, 73, 73, 73);
	gfx.PutPixel(x + 316, y + 73, 15, 15, 15);
	gfx.PutPixel(x + 316, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 316, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 316, y + 76, 4, 4, 4);
	gfx.PutPixel(x + 316, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 316, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 317, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 35, 4, 4, 4);
	gfx.PutPixel(x + 317, y + 36, 18, 18, 18);
	gfx.PutPixel(x + 317, y + 37, 204, 204, 204);
	gfx.PutPixel(x + 317, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 41, 231, 231, 231);
	gfx.PutPixel(x + 317, y + 42, 104, 104, 104);
	gfx.PutPixel(x + 317, y + 43, 38, 38, 38);
	gfx.PutPixel(x + 317, y + 44, 200, 200, 200);
	gfx.PutPixel(x + 317, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 67, 224, 224, 224);
	gfx.PutPixel(x + 317, y + 68, 152, 152, 152);
	gfx.PutPixel(x + 317, y + 69, 73, 73, 73);
	gfx.PutPixel(x + 317, y + 70, 15, 15, 15);
	gfx.PutPixel(x + 317, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 317, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 317, y + 73, 5, 5, 5);
	gfx.PutPixel(x + 317, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 317, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 318, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 318, y + 36, 81, 81, 81);
	gfx.PutPixel(x + 318, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 39, 251, 251, 251);
	gfx.PutPixel(x + 318, y + 40, 145, 145, 145);
	gfx.PutPixel(x + 318, y + 41, 9, 9, 9);
	gfx.PutPixel(x + 318, y + 42, 41, 41, 41);
	gfx.PutPixel(x + 318, y + 43, 215, 215, 215);
	gfx.PutPixel(x + 318, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 64, 222, 222, 222);
	gfx.PutPixel(x + 318, y + 65, 143, 143, 143);
	gfx.PutPixel(x + 318, y + 66, 73, 73, 73);
	gfx.PutPixel(x + 318, y + 67, 15, 15, 15);
	gfx.PutPixel(x + 318, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 318, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 318, y + 70, 5, 5, 5);
	gfx.PutPixel(x + 318, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 318, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 319, y + 36, 137, 137, 137);
	gfx.PutPixel(x + 319, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 38, 254, 254, 254);
	gfx.PutPixel(x + 319, y + 39, 75, 75, 75);
	gfx.PutPixel(x + 319, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 319, y + 41, 106, 106, 106);
	gfx.PutPixel(x + 319, y + 42, 245, 245, 245);
	gfx.PutPixel(x + 319, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 64, 195, 195, 195);
	gfx.PutPixel(x + 319, y + 65, 29, 29, 29);
	gfx.PutPixel(x + 319, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 319, y + 67, 1, 1, 1);
	gfx.PutPixel(x + 319, y + 68, 7, 7, 7);
	gfx.PutPixel(x + 319, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 319, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 320, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 320, y + 36, 177, 177, 177);
	gfx.PutPixel(x + 320, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 38, 241, 241, 241);
	gfx.PutPixel(x + 320, y + 39, 97, 97, 97);
	gfx.PutPixel(x + 320, y + 40, 180, 180, 180);
	gfx.PutPixel(x + 320, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 65, 243, 243, 243);
	gfx.PutPixel(x + 320, y + 66, 149, 149, 149);
	gfx.PutPixel(x + 320, y + 67, 42, 42, 42);
	gfx.PutPixel(x + 320, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 320, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 320, y + 70, 5, 5, 5);
	gfx.PutPixel(x + 320, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 320, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 321, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 321, y + 36, 142, 142, 142);
	gfx.PutPixel(x + 321, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 67, 246, 246, 246);
	gfx.PutPixel(x + 321, y + 68, 171, 171, 171);
	gfx.PutPixel(x + 321, y + 69, 70, 70, 70);
	gfx.PutPixel(x + 321, y + 70, 11, 11, 11);
	gfx.PutPixel(x + 321, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 321, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 321, y + 73, 1, 1, 1);
	gfx.PutPixel(x + 321, y + 74, 6, 6, 6);
	gfx.PutPixel(x + 321, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 321, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 322, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 35, 2, 2, 2);
	gfx.PutPixel(x + 322, y + 36, 32, 32, 32);
	gfx.PutPixel(x + 322, y + 37, 221, 221, 221);
	gfx.PutPixel(x + 322, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 69, 254, 254, 254);
	gfx.PutPixel(x + 322, y + 70, 219, 219, 219);
	gfx.PutPixel(x + 322, y + 71, 160, 160, 160);
	gfx.PutPixel(x + 322, y + 72, 95, 95, 95);
	gfx.PutPixel(x + 322, y + 73, 39, 39, 39);
	gfx.PutPixel(x + 322, y + 74, 5, 5, 5);
	gfx.PutPixel(x + 322, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 322, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 322, y + 77, 3, 3, 3);
	gfx.PutPixel(x + 322, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 322, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 323, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 323, y + 37, 40, 40, 40);
	gfx.PutPixel(x + 323, y + 38, 204, 204, 204);
	gfx.PutPixel(x + 323, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 73, 247, 247, 247);
	gfx.PutPixel(x + 323, y + 74, 202, 202, 202);
	gfx.PutPixel(x + 323, y + 75, 139, 139, 139);
	gfx.PutPixel(x + 323, y + 76, 73, 73, 73);
	gfx.PutPixel(x + 323, y + 77, 23, 23, 23);
	gfx.PutPixel(x + 323, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 323, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 323, y + 80, 1, 1, 1);
	gfx.PutPixel(x + 323, y + 81, 6, 6, 6);
	gfx.PutPixel(x + 323, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 323, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 324, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 37, 1, 1, 1);
	gfx.PutPixel(x + 324, y + 38, 12, 12, 12);
	gfx.PutPixel(x + 324, y + 39, 121, 121, 121);
	gfx.PutPixel(x + 324, y + 40, 233, 233, 233);
	gfx.PutPixel(x + 324, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 77, 233, 233, 233);
	gfx.PutPixel(x + 324, y + 78, 175, 175, 175);
	gfx.PutPixel(x + 324, y + 79, 107, 107, 107);
	gfx.PutPixel(x + 324, y + 80, 44, 44, 44);
	gfx.PutPixel(x + 324, y + 81, 6, 6, 6);
	gfx.PutPixel(x + 324, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 324, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 324, y + 84, 5, 5, 5);
	gfx.PutPixel(x + 324, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 324, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 325, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 38, 5, 5, 5);
	gfx.PutPixel(x + 325, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 325, y + 40, 25, 25, 25);
	gfx.PutPixel(x + 325, y + 41, 119, 119, 119);
	gfx.PutPixel(x + 325, y + 42, 215, 215, 215);
	gfx.PutPixel(x + 325, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 80, 249, 249, 249);
	gfx.PutPixel(x + 325, y + 81, 203, 203, 203);
	gfx.PutPixel(x + 325, y + 82, 134, 134, 134);
	gfx.PutPixel(x + 325, y + 83, 64, 64, 64);
	gfx.PutPixel(x + 325, y + 84, 13, 13, 13);
	gfx.PutPixel(x + 325, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 325, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 325, y + 87, 4, 4, 4);
	gfx.PutPixel(x + 325, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 325, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 326, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 40, 3, 3, 3);
	gfx.PutPixel(x + 326, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 326, y + 42, 11, 11, 11);
	gfx.PutPixel(x + 326, y + 43, 73, 73, 73);
	gfx.PutPixel(x + 326, y + 44, 158, 158, 158);
	gfx.PutPixel(x + 326, y + 45, 228, 228, 228);
	gfx.PutPixel(x + 326, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 84, 220, 220, 220);
	gfx.PutPixel(x + 326, y + 85, 152, 152, 152);
	gfx.PutPixel(x + 326, y + 86, 76, 76, 76);
	gfx.PutPixel(x + 326, y + 87, 18, 18, 18);
	gfx.PutPixel(x + 326, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 326, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 326, y + 90, 4, 4, 4);
	gfx.PutPixel(x + 326, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 326, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 327, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 42, 5, 5, 5);
	gfx.PutPixel(x + 327, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 327, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 327, y + 45, 18, 18, 18);
	gfx.PutPixel(x + 327, y + 46, 74, 74, 74);
	gfx.PutPixel(x + 327, y + 47, 143, 143, 143);
	gfx.PutPixel(x + 327, y + 48, 208, 208, 208);
	gfx.PutPixel(x + 327, y + 49, 249, 249, 249);
	gfx.PutPixel(x + 327, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 87, 228, 228, 228);
	gfx.PutPixel(x + 327, y + 88, 157, 157, 157);
	gfx.PutPixel(x + 327, y + 89, 75, 75, 75);
	gfx.PutPixel(x + 327, y + 90, 15, 15, 15);
	gfx.PutPixel(x + 327, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 327, y + 92, 1, 1, 1);
	gfx.PutPixel(x + 327, y + 93, 6, 6, 6);
	gfx.PutPixel(x + 327, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 327, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 328, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 45, 4, 4, 4);
	gfx.PutPixel(x + 328, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 328, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 328, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 49, 43, 43, 43);
	gfx.PutPixel(x + 328, y + 50, 98, 98, 98);
	gfx.PutPixel(x + 328, y + 51, 158, 158, 158);
	gfx.PutPixel(x + 328, y + 52, 211, 211, 211);
	gfx.PutPixel(x + 328, y + 53, 247, 247, 247);
	gfx.PutPixel(x + 328, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 90, 223, 223, 223);
	gfx.PutPixel(x + 328, y + 91, 143, 143, 143);
	gfx.PutPixel(x + 328, y + 92, 55, 55, 55);
	gfx.PutPixel(x + 328, y + 93, 4, 4, 4);
	gfx.PutPixel(x + 328, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 328, y + 95, 5, 5, 5);
	gfx.PutPixel(x + 328, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 328, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 329, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 48, 6, 6, 6);
	gfx.PutPixel(x + 329, y + 49, 1, 1, 1);
	gfx.PutPixel(x + 329, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 329, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 329, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 53, 39, 39, 39);
	gfx.PutPixel(x + 329, y + 54, 84, 84, 84);
	gfx.PutPixel(x + 329, y + 55, 136, 136, 136);
	gfx.PutPixel(x + 329, y + 56, 186, 186, 186);
	gfx.PutPixel(x + 329, y + 57, 226, 226, 226);
	gfx.PutPixel(x + 329, y + 58, 253, 253, 253);
	gfx.PutPixel(x + 329, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 92, 252, 252, 252);
	gfx.PutPixel(x + 329, y + 93, 196, 196, 196);
	gfx.PutPixel(x + 329, y + 94, 96, 96, 96);
	gfx.PutPixel(x + 329, y + 95, 16, 16, 16);
	gfx.PutPixel(x + 329, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 329, y + 97, 5, 5, 5);
	gfx.PutPixel(x + 329, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 329, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 330, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 52, 6, 6, 6);
	gfx.PutPixel(x + 330, y + 53, 1, 1, 1);
	gfx.PutPixel(x + 330, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 330, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 330, y + 56, 1, 1, 1);
	gfx.PutPixel(x + 330, y + 57, 17, 17, 17);
	gfx.PutPixel(x + 330, y + 58, 60, 60, 60);
	gfx.PutPixel(x + 330, y + 59, 135, 135, 135);
	gfx.PutPixel(x + 330, y + 60, 210, 210, 210);
	gfx.PutPixel(x + 330, y + 61, 253, 253, 253);
	gfx.PutPixel(x + 330, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 95, 222, 222, 222);
	gfx.PutPixel(x + 330, y + 96, 112, 112, 112);
	gfx.PutPixel(x + 330, y + 97, 13, 13, 13);
	gfx.PutPixel(x + 330, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 330, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 330, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 56, 7, 7, 7);
	gfx.PutPixel(x + 331, y + 57, 4, 4, 4);
	gfx.PutPixel(x + 331, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 331, y + 59, 0, 0, 0);
	gfx.PutPixel(x + 331, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 61, 58, 58, 58);
	gfx.PutPixel(x + 331, y + 62, 137, 137, 137);
	gfx.PutPixel(x + 331, y + 63, 216, 216, 216);
	gfx.PutPixel(x + 331, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 97, 214, 214, 214);
	gfx.PutPixel(x + 331, y + 98, 63, 63, 63);
	gfx.PutPixel(x + 331, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 331, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 331, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 332, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 60, 6, 6, 6);
	gfx.PutPixel(x + 332, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 332, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 332, y + 63, 11, 11, 11);
	gfx.PutPixel(x + 332, y + 64, 69, 69, 69);
	gfx.PutPixel(x + 332, y + 65, 158, 158, 158);
	gfx.PutPixel(x + 332, y + 66, 236, 236, 236);
	gfx.PutPixel(x + 332, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 98, 244, 244, 244);
	gfx.PutPixel(x + 332, y + 99, 57, 57, 57);
	gfx.PutPixel(x + 332, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 332, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 332, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 333, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 63, 5, 5, 5);
	gfx.PutPixel(x + 333, y + 64, 0, 0, 0);
	gfx.PutPixel(x + 333, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 333, y + 66, 26, 26, 26);
	gfx.PutPixel(x + 333, y + 67, 110, 110, 110);
	gfx.PutPixel(x + 333, y + 68, 213, 213, 213);
	gfx.PutPixel(x + 333, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 99, 97, 97, 97);
	gfx.PutPixel(x + 333, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 333, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 333, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 334, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 66, 3, 3, 3);
	gfx.PutPixel(x + 334, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 334, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 334, y + 69, 91, 91, 91);
	gfx.PutPixel(x + 334, y + 70, 237, 237, 237);
	gfx.PutPixel(x + 334, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 98, 230, 230, 230);
	gfx.PutPixel(x + 334, y + 99, 32, 32, 32);
	gfx.PutPixel(x + 334, y + 100, 2, 2, 2);
	gfx.PutPixel(x + 334, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 334, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 335, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 64, 5, 5, 5);
	gfx.PutPixel(x + 335, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 335, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 335, y + 67, 24, 24, 24);
	gfx.PutPixel(x + 335, y + 68, 93, 93, 93);
	gfx.PutPixel(x + 335, y + 69, 173, 173, 173);
	gfx.PutPixel(x + 335, y + 70, 245, 245, 245);
	gfx.PutPixel(x + 335, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 97, 248, 248, 248);
	gfx.PutPixel(x + 335, y + 98, 80, 80, 80);
	gfx.PutPixel(x + 335, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 335, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 335, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 336, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 61, 6, 6, 6);
	gfx.PutPixel(x + 336, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 336, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 336, y + 64, 13, 13, 13);
	gfx.PutPixel(x + 336, y + 65, 73, 73, 73);
	gfx.PutPixel(x + 336, y + 66, 159, 159, 159);
	gfx.PutPixel(x + 336, y + 67, 234, 234, 234);
	gfx.PutPixel(x + 336, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 96, 230, 230, 230);
	gfx.PutPixel(x + 336, y + 97, 72, 72, 72);
	gfx.PutPixel(x + 336, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 336, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 336, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 337, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 58, 6, 6, 6);
	gfx.PutPixel(x + 337, y + 59, 1, 1, 1);
	gfx.PutPixel(x + 337, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 337, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 62, 59, 59, 59);
	gfx.PutPixel(x + 337, y + 63, 141, 141, 141);
	gfx.PutPixel(x + 337, y + 64, 220, 220, 220);
	gfx.PutPixel(x + 337, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 95, 169, 169, 169);
	gfx.PutPixel(x + 337, y + 96, 30, 30, 30);
	gfx.PutPixel(x + 337, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 337, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 337, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 55, 6, 6, 6);
	gfx.PutPixel(x + 338, y + 56, 1, 1, 1);
	gfx.PutPixel(x + 338, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 338, y + 58, 7, 7, 7);
	gfx.PutPixel(x + 338, y + 59, 54, 54, 54);
	gfx.PutPixel(x + 338, y + 60, 132, 132, 132);
	gfx.PutPixel(x + 338, y + 61, 210, 210, 210);
	gfx.PutPixel(x + 338, y + 62, 253, 253, 253);
	gfx.PutPixel(x + 338, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 93, 202, 202, 202);
	gfx.PutPixel(x + 338, y + 94, 74, 74, 74);
	gfx.PutPixel(x + 338, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 338, y + 96, 2, 2, 2);
	gfx.PutPixel(x + 338, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 338, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 52, 5, 5, 5);
	gfx.PutPixel(x + 339, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 339, y + 54, 0, 0, 0);
	gfx.PutPixel(x + 339, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 56, 55, 55, 55);
	gfx.PutPixel(x + 339, y + 57, 129, 129, 129);
	gfx.PutPixel(x + 339, y + 58, 205, 205, 205);
	gfx.PutPixel(x + 339, y + 59, 252, 252, 252);
	gfx.PutPixel(x + 339, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 90, 248, 248, 248);
	gfx.PutPixel(x + 339, y + 91, 196, 196, 196);
	gfx.PutPixel(x + 339, y + 92, 87, 87, 87);
	gfx.PutPixel(x + 339, y + 93, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 339, y + 95, 7, 7, 7);
	gfx.PutPixel(x + 339, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 339, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 340, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 49, 3, 3, 3);
	gfx.PutPixel(x + 340, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 340, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 340, y + 52, 13, 13, 13);
	gfx.PutPixel(x + 340, y + 53, 62, 62, 62);
	gfx.PutPixel(x + 340, y + 54, 135, 135, 135);
	gfx.PutPixel(x + 340, y + 55, 209, 209, 209);
	gfx.PutPixel(x + 340, y + 56, 253, 253, 253);
	gfx.PutPixel(x + 340, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 86, 253, 253, 253);
	gfx.PutPixel(x + 340, y + 87, 218, 218, 218);
	gfx.PutPixel(x + 340, y + 88, 159, 159, 159);
	gfx.PutPixel(x + 340, y + 89, 94, 94, 94);
	gfx.PutPixel(x + 340, y + 90, 39, 39, 39);
	gfx.PutPixel(x + 340, y + 91, 5, 5, 5);
	gfx.PutPixel(x + 340, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 340, y + 93, 6, 6, 6);
	gfx.PutPixel(x + 340, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 340, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 45, 6, 6, 6);
	gfx.PutPixel(x + 341, y + 46, 1, 1, 1);
	gfx.PutPixel(x + 341, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 341, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 341, y + 49, 23, 23, 23);
	gfx.PutPixel(x + 341, y + 50, 77, 77, 77);
	gfx.PutPixel(x + 341, y + 51, 151, 151, 151);
	gfx.PutPixel(x + 341, y + 52, 219, 219, 219);
	gfx.PutPixel(x + 341, y + 53, 254, 254, 254);
	gfx.PutPixel(x + 341, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 83, 242, 242, 242);
	gfx.PutPixel(x + 341, y + 84, 189, 189, 189);
	gfx.PutPixel(x + 341, y + 85, 120, 120, 120);
	gfx.PutPixel(x + 341, y + 86, 56, 56, 56);
	gfx.PutPixel(x + 341, y + 87, 13, 13, 13);
	gfx.PutPixel(x + 341, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 341, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 341, y + 90, 1, 1, 1);
	gfx.PutPixel(x + 341, y + 91, 7, 7, 7);
	gfx.PutPixel(x + 341, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 341, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 41, 7, 7, 7);
	gfx.PutPixel(x + 342, y + 42, 3, 3, 3);
	gfx.PutPixel(x + 342, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 342, y + 44, 0, 0, 0);
	gfx.PutPixel(x + 342, y + 45, 5, 5, 5);
	gfx.PutPixel(x + 342, y + 46, 41, 41, 41);
	gfx.PutPixel(x + 342, y + 47, 103, 103, 103);
	gfx.PutPixel(x + 342, y + 48, 174, 174, 174);
	gfx.PutPixel(x + 342, y + 49, 233, 233, 233);
	gfx.PutPixel(x + 342, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 80, 230, 230, 230);
	gfx.PutPixel(x + 342, y + 81, 167, 167, 167);
	gfx.PutPixel(x + 342, y + 82, 92, 92, 92);
	gfx.PutPixel(x + 342, y + 83, 32, 32, 32);
	gfx.PutPixel(x + 342, y + 84, 2, 2, 2);
	gfx.PutPixel(x + 342, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 342, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 342, y + 87, 5, 5, 5);
	gfx.PutPixel(x + 342, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 342, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 38, 5, 5, 5);
	gfx.PutPixel(x + 343, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 343, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 343, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 343, y + 42, 23, 23, 23);
	gfx.PutPixel(x + 343, y + 43, 73, 73, 73);
	gfx.PutPixel(x + 343, y + 44, 139, 139, 139);
	gfx.PutPixel(x + 343, y + 45, 203, 203, 203);
	gfx.PutPixel(x + 343, y + 46, 247, 247, 247);
	gfx.PutPixel(x + 343, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 77, 225, 225, 225);
	gfx.PutPixel(x + 343, y + 78, 156, 156, 156);
	gfx.PutPixel(x + 343, y + 79, 77, 77, 77);
	gfx.PutPixel(x + 343, y + 80, 20, 20, 20);
	gfx.PutPixel(x + 343, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 343, y + 82, 0, 0, 0);
	gfx.PutPixel(x + 343, y + 83, 2, 2, 2);
	gfx.PutPixel(x + 343, y + 84, 7, 7, 7);
	gfx.PutPixel(x + 343, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 343, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 344, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 37, 1, 1, 1);
	gfx.PutPixel(x + 344, y + 38, 12, 12, 12);
	gfx.PutPixel(x + 344, y + 39, 59, 59, 59);
	gfx.PutPixel(x + 344, y + 40, 118, 118, 118);
	gfx.PutPixel(x + 344, y + 41, 181, 181, 181);
	gfx.PutPixel(x + 344, y + 42, 236, 236, 236);
	gfx.PutPixel(x + 344, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 74, 230, 230, 230);
	gfx.PutPixel(x + 344, y + 75, 158, 158, 158);
	gfx.PutPixel(x + 344, y + 76, 75, 75, 75);
	gfx.PutPixel(x + 344, y + 77, 16, 16, 16);
	gfx.PutPixel(x + 344, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 344, y + 79, 0, 0, 0);
	gfx.PutPixel(x + 344, y + 80, 4, 4, 4);
	gfx.PutPixel(x + 344, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 344, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 345, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 345, y + 37, 48, 48, 48);
	gfx.PutPixel(x + 345, y + 38, 208, 208, 208);
	gfx.PutPixel(x + 345, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 42, 232, 232, 232);
	gfx.PutPixel(x + 345, y + 43, 225, 225, 225);
	gfx.PutPixel(x + 345, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 71, 242, 242, 242);
	gfx.PutPixel(x + 345, y + 72, 175, 175, 175);
	gfx.PutPixel(x + 345, y + 73, 87, 87, 87);
	gfx.PutPixel(x + 345, y + 74, 20, 20, 20);
	gfx.PutPixel(x + 345, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 345, y + 76, 0, 0, 0);
	gfx.PutPixel(x + 345, y + 77, 4, 4, 4);
	gfx.PutPixel(x + 345, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 345, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 346, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 346, y + 36, 47, 47, 47);
	gfx.PutPixel(x + 346, y + 37, 231, 231, 231);
	gfx.PutPixel(x + 346, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 40, 246, 246, 246);
	gfx.PutPixel(x + 346, y + 41, 137, 137, 137);
	gfx.PutPixel(x + 346, y + 42, 65, 65, 65);
	gfx.PutPixel(x + 346, y + 43, 220, 220, 220);
	gfx.PutPixel(x + 346, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 68, 254, 254, 254);
	gfx.PutPixel(x + 346, y + 69, 206, 206, 206);
	gfx.PutPixel(x + 346, y + 70, 115, 115, 115);
	gfx.PutPixel(x + 346, y + 71, 34, 34, 34);
	gfx.PutPixel(x + 346, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 346, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 346, y + 74, 4, 4, 4);
	gfx.PutPixel(x + 346, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 346, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 347, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 34, 5, 5, 5);
	gfx.PutPixel(x + 347, y + 35, 10, 10, 10);
	gfx.PutPixel(x + 347, y + 36, 200, 200, 200);
	gfx.PutPixel(x + 347, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 39, 173, 173, 173);
	gfx.PutPixel(x + 347, y + 40, 29, 29, 29);
	gfx.PutPixel(x + 347, y + 41, 33, 33, 33);
	gfx.PutPixel(x + 347, y + 42, 210, 210, 210);
	gfx.PutPixel(x + 347, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 66, 240, 240, 240);
	gfx.PutPixel(x + 347, y + 67, 161, 161, 161);
	gfx.PutPixel(x + 347, y + 68, 66, 66, 66);
	gfx.PutPixel(x + 347, y + 69, 6, 6, 6);
	gfx.PutPixel(x + 347, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 347, y + 71, 2, 2, 2);
	gfx.PutPixel(x + 347, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 347, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 348, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 348, y + 35, 89, 89, 89);
	gfx.PutPixel(x + 348, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 37, 242, 242, 242);
	gfx.PutPixel(x + 348, y + 38, 83, 83, 83);
	gfx.PutPixel(x + 348, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 348, y + 40, 86, 86, 86);
	gfx.PutPixel(x + 348, y + 41, 239, 239, 239);
	gfx.PutPixel(x + 348, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 64, 209, 209, 209);
	gfx.PutPixel(x + 348, y + 65, 115, 115, 115);
	gfx.PutPixel(x + 348, y + 66, 30, 30, 30);
	gfx.PutPixel(x + 348, y + 67, 0, 0, 0);
	gfx.PutPixel(x + 348, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 348, y + 69, 6, 6, 6);
	gfx.PutPixel(x + 348, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 348, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 349, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 349, y + 35, 147, 147, 147);
	gfx.PutPixel(x + 349, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 37, 179, 179, 179);
	gfx.PutPixel(x + 349, y + 38, 18, 18, 18);
	gfx.PutPixel(x + 349, y + 39, 164, 164, 164);
	gfx.PutPixel(x + 349, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 61, 241, 241, 241);
	gfx.PutPixel(x + 349, y + 62, 165, 165, 165);
	gfx.PutPixel(x + 349, y + 63, 70, 70, 70);
	gfx.PutPixel(x + 349, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 349, y + 66, 2, 2, 2);
	gfx.PutPixel(x + 349, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 349, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 350, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 350, y + 35, 157, 157, 157);
	gfx.PutPixel(x + 350, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 37, 247, 247, 247);
	gfx.PutPixel(x + 350, y + 38, 242, 242, 242);
	gfx.PutPixel(x + 350, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 59, 207, 207, 207);
	gfx.PutPixel(x + 350, y + 60, 115, 115, 115);
	gfx.PutPixel(x + 350, y + 61, 31, 31, 31);
	gfx.PutPixel(x + 350, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 350, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 350, y + 64, 6, 6, 6);
	gfx.PutPixel(x + 350, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 350, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 351, y + 35, 130, 130, 130);
	gfx.PutPixel(x + 351, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 56, 235, 235, 235);
	gfx.PutPixel(x + 351, y + 57, 156, 156, 156);
	gfx.PutPixel(x + 351, y + 58, 65, 65, 65);
	gfx.PutPixel(x + 351, y + 59, 7, 7, 7);
	gfx.PutPixel(x + 351, y + 60, 0, 0, 0);
	gfx.PutPixel(x + 351, y + 61, 2, 2, 2);
	gfx.PutPixel(x + 351, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 351, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 352, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 352, y + 35, 54, 54, 54);
	gfx.PutPixel(x + 352, y + 36, 251, 251, 251);
	gfx.PutPixel(x + 352, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 53, 249, 249, 249);
	gfx.PutPixel(x + 352, y + 54, 190, 190, 190);
	gfx.PutPixel(x + 352, y + 55, 100, 100, 100);
	gfx.PutPixel(x + 352, y + 56, 25, 25, 25);
	gfx.PutPixel(x + 352, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 352, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 352, y + 59, 6, 6, 6);
	gfx.PutPixel(x + 352, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 352, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 353, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 353, y + 36, 125, 125, 125);
	gfx.PutPixel(x + 353, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 50, 254, 254, 254);
	gfx.PutPixel(x + 353, y + 51, 212, 212, 212);
	gfx.PutPixel(x + 353, y + 52, 129, 129, 129);
	gfx.PutPixel(x + 353, y + 53, 46, 46, 46);
	gfx.PutPixel(x + 353, y + 54, 2, 2, 2);
	gfx.PutPixel(x + 353, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 353, y + 56, 3, 3, 3);
	gfx.PutPixel(x + 353, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 353, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 35, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 354, y + 37, 87, 87, 87);
	gfx.PutPixel(x + 354, y + 38, 211, 211, 211);
	gfx.PutPixel(x + 354, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 47, 254, 254, 254);
	gfx.PutPixel(x + 354, y + 48, 216, 216, 216);
	gfx.PutPixel(x + 354, y + 49, 144, 144, 144);
	gfx.PutPixel(x + 354, y + 50, 62, 62, 62);
	gfx.PutPixel(x + 354, y + 51, 9, 9, 9);
	gfx.PutPixel(x + 354, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 354, y + 53, 1, 1, 1);
	gfx.PutPixel(x + 354, y + 54, 7, 7, 7);
	gfx.PutPixel(x + 354, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 354, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 36, 7, 7, 7);
	gfx.PutPixel(x + 355, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 355, y + 38, 10, 10, 10);
	gfx.PutPixel(x + 355, y + 39, 61, 61, 61);
	gfx.PutPixel(x + 355, y + 40, 118, 118, 118);
	gfx.PutPixel(x + 355, y + 41, 160, 160, 160);
	gfx.PutPixel(x + 355, y + 42, 183, 183, 183);
	gfx.PutPixel(x + 355, y + 43, 196, 196, 196);
	gfx.PutPixel(x + 355, y + 44, 204, 204, 204);
	gfx.PutPixel(x + 355, y + 45, 184, 184, 184);
	gfx.PutPixel(x + 355, y + 46, 128, 128, 128);
	gfx.PutPixel(x + 355, y + 47, 61, 61, 61);
	gfx.PutPixel(x + 355, y + 48, 11, 11, 11);
	gfx.PutPixel(x + 355, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 355, y + 50, 0, 0, 0);
	gfx.PutPixel(x + 355, y + 51, 6, 6, 6);
	gfx.PutPixel(x + 355, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 355, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 356, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 38, 5, 5, 5);
	gfx.PutPixel(x + 356, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 41, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 43, 2, 2, 2);
	gfx.PutPixel(x + 356, y + 44, 6, 6, 6);
	gfx.PutPixel(x + 356, y + 45, 1, 1, 1);
	gfx.PutPixel(x + 356, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 47, 0, 0, 0);
	gfx.PutPixel(x + 356, y + 48, 5, 5, 5);
	gfx.PutPixel(x + 356, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 356, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 42, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 43, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 44, 6, 6, 6);
	gfx.PutPixel(x + 357, y + 45, 7, 7, 7);
	gfx.PutPixel(x + 357, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 357, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 358, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 358, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 359, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 359, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 360, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 360, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 361, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 361, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 362, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 362, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 363, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 363, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 364, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 364, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 100, 5, 5, 5);
	gfx.PutPixel(x + 365, y + 101, 4, 4, 4);
	gfx.PutPixel(x + 365, y + 102, 7, 7, 7);
	gfx.PutPixel(x + 365, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 365, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 366, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 97, 3, 3, 3);
	gfx.PutPixel(x + 366, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 366, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 366, y + 100, 11, 11, 11);
	gfx.PutPixel(x + 366, y + 101, 24, 24, 24);
	gfx.PutPixel(x + 366, y + 102, 5, 5, 5);
	gfx.PutPixel(x + 366, y + 103, 1, 1, 1);
	gfx.PutPixel(x + 366, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 366, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 93, 7, 7, 7);
	gfx.PutPixel(x + 367, y + 94, 2, 2, 2);
	gfx.PutPixel(x + 367, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 367, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 367, y + 97, 23, 23, 23);
	gfx.PutPixel(x + 367, y + 98, 76, 76, 76);
	gfx.PutPixel(x + 367, y + 99, 146, 146, 146);
	gfx.PutPixel(x + 367, y + 100, 213, 213, 213);
	gfx.PutPixel(x + 367, y + 101, 238, 238, 238);
	gfx.PutPixel(x + 367, y + 102, 190, 190, 190);
	gfx.PutPixel(x + 367, y + 103, 56, 56, 56);
	gfx.PutPixel(x + 367, y + 104, 0, 0, 0);
	gfx.PutPixel(x + 367, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 367, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 368, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 91, 6, 6, 6);
	gfx.PutPixel(x + 368, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 368, y + 93, 2, 2, 2);
	gfx.PutPixel(x + 368, y + 94, 39, 39, 39);
	gfx.PutPixel(x + 368, y + 95, 103, 103, 103);
	gfx.PutPixel(x + 368, y + 96, 174, 174, 174);
	gfx.PutPixel(x + 368, y + 97, 232, 232, 232);
	gfx.PutPixel(x + 368, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 103, 246, 246, 246);
	gfx.PutPixel(x + 368, y + 104, 83, 83, 83);
	gfx.PutPixel(x + 368, y + 105, 0, 0, 0);
	gfx.PutPixel(x + 368, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 368, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 369, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 90, 3, 3, 3);
	gfx.PutPixel(x + 369, y + 91, 3, 3, 3);
	gfx.PutPixel(x + 369, y + 92, 82, 82, 82);
	gfx.PutPixel(x + 369, y + 93, 188, 188, 188);
	gfx.PutPixel(x + 369, y + 94, 247, 247, 247);
	gfx.PutPixel(x + 369, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 104, 245, 245, 245);
	gfx.PutPixel(x + 369, y + 105, 63, 63, 63);
	gfx.PutPixel(x + 369, y + 106, 0, 0, 0);
	gfx.PutPixel(x + 369, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 369, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 370, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 370, y + 90, 24, 24, 24);
	gfx.PutPixel(x + 370, y + 91, 177, 177, 177);
	gfx.PutPixel(x + 370, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 105, 220, 220, 220);
	gfx.PutPixel(x + 370, y + 106, 25, 25, 25);
	gfx.PutPixel(x + 370, y + 107, 2, 2, 2);
	gfx.PutPixel(x + 370, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 370, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 88, 3, 3, 3);
	gfx.PutPixel(x + 371, y + 89, 21, 21, 21);
	gfx.PutPixel(x + 371, y + 90, 204, 204, 204);
	gfx.PutPixel(x + 371, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 106, 161, 161, 161);
	gfx.PutPixel(x + 371, y + 107, 0, 0, 0);
	gfx.PutPixel(x + 371, y + 108, 7, 7, 7);
	gfx.PutPixel(x + 371, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 371, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 372, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 372, y + 89, 157, 157, 157);
	gfx.PutPixel(x + 372, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 94, 254, 254, 254);
	gfx.PutPixel(x + 372, y + 95, 220, 220, 220);
	gfx.PutPixel(x + 372, y + 96, 154, 154, 154);
	gfx.PutPixel(x + 372, y + 97, 83, 83, 83);
	gfx.PutPixel(x + 372, y + 98, 212, 212, 212);
	gfx.PutPixel(x + 372, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 107, 77, 77, 77);
	gfx.PutPixel(x + 372, y + 108, 0, 0, 0);
	gfx.PutPixel(x + 372, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 372, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 373, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 373, y + 88, 47, 47, 47);
	gfx.PutPixel(x + 373, y + 89, 248, 248, 248);
	gfx.PutPixel(x + 373, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 93, 241, 241, 241);
	gfx.PutPixel(x + 373, y + 94, 75, 75, 75);
	gfx.PutPixel(x + 373, y + 95, 10, 10, 10);
	gfx.PutPixel(x + 373, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 373, y + 97, 0, 0, 0);
	gfx.PutPixel(x + 373, y + 98, 158, 158, 158);
	gfx.PutPixel(x + 373, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 107, 206, 206, 206);
	gfx.PutPixel(x + 373, y + 108, 11, 11, 11);
	gfx.PutPixel(x + 373, y + 109, 5, 5, 5);
	gfx.PutPixel(x + 373, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 373, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 374, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 67, 5, 5, 5);
	gfx.PutPixel(x + 374, y + 68, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 71, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 72, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 74, 3, 3, 3);
	gfx.PutPixel(x + 374, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 88, 116, 116, 116);
	gfx.PutPixel(x + 374, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 93, 137, 137, 137);
	gfx.PutPixel(x + 374, y + 94, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 95, 4, 4, 4);
	gfx.PutPixel(x + 374, y + 96, 4, 4, 4);
	gfx.PutPixel(x + 374, y + 97, 15, 15, 15);
	gfx.PutPixel(x + 374, y + 98, 215, 215, 215);
	gfx.PutPixel(x + 374, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 108, 108, 108, 108);
	gfx.PutPixel(x + 374, y + 109, 0, 0, 0);
	gfx.PutPixel(x + 374, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 374, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 64, 7, 7, 7);
	gfx.PutPixel(x + 375, y + 65, 1, 1, 1);
	gfx.PutPixel(x + 375, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 375, y + 67, 15, 15, 15);
	gfx.PutPixel(x + 375, y + 68, 74, 74, 74);
	gfx.PutPixel(x + 375, y + 69, 125, 125, 125);
	gfx.PutPixel(x + 375, y + 70, 155, 155, 155);
	gfx.PutPixel(x + 375, y + 71, 158, 158, 158);
	gfx.PutPixel(x + 375, y + 72, 132, 132, 132);
	gfx.PutPixel(x + 375, y + 73, 86, 86, 86);
	gfx.PutPixel(x + 375, y + 74, 27, 27, 27);
	gfx.PutPixel(x + 375, y + 75, 0, 0, 0);
	gfx.PutPixel(x + 375, y + 76, 2, 2, 2);
	gfx.PutPixel(x + 375, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 375, y + 88, 158, 158, 158);
	gfx.PutPixel(x + 375, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 92, 227, 227, 227);
	gfx.PutPixel(x + 375, y + 93, 25, 25, 25);
	gfx.PutPixel(x + 375, y + 94, 2, 2, 2);
	gfx.PutPixel(x + 375, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 375, y + 96, 0, 0, 0);
	gfx.PutPixel(x + 375, y + 97, 108, 108, 108);
	gfx.PutPixel(x + 375, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 108, 220, 220, 220);
	gfx.PutPixel(x + 375, y + 109, 19, 19, 19);
	gfx.PutPixel(x + 375, y + 110, 4, 4, 4);
	gfx.PutPixel(x + 375, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 376, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 62, 4, 4, 4);
	gfx.PutPixel(x + 376, y + 63, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 64, 1, 1, 1);
	gfx.PutPixel(x + 376, y + 65, 44, 44, 44);
	gfx.PutPixel(x + 376, y + 66, 132, 132, 132);
	gfx.PutPixel(x + 376, y + 67, 221, 221, 221);
	gfx.PutPixel(x + 376, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 74, 238, 238, 238);
	gfx.PutPixel(x + 376, y + 75, 154, 154, 154);
	gfx.PutPixel(x + 376, y + 76, 39, 39, 39);
	gfx.PutPixel(x + 376, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 78, 6, 6, 6);
	gfx.PutPixel(x + 376, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 376, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 88, 174, 174, 174);
	gfx.PutPixel(x + 376, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 92, 111, 111, 111);
	gfx.PutPixel(x + 376, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 94, 9, 9, 9);
	gfx.PutPixel(x + 376, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 96, 30, 30, 30);
	gfx.PutPixel(x + 376, y + 97, 229, 229, 229);
	gfx.PutPixel(x + 376, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 109, 114, 114, 114);
	gfx.PutPixel(x + 376, y + 110, 0, 0, 0);
	gfx.PutPixel(x + 376, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 59, 7, 7, 7);
	gfx.PutPixel(x + 377, y + 60, 1, 1, 1);
	gfx.PutPixel(x + 377, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 377, y + 62, 16, 16, 16);
	gfx.PutPixel(x + 377, y + 63, 86, 86, 86);
	gfx.PutPixel(x + 377, y + 64, 181, 181, 181);
	gfx.PutPixel(x + 377, y + 65, 248, 248, 248);
	gfx.PutPixel(x + 377, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 76, 243, 243, 243);
	gfx.PutPixel(x + 377, y + 77, 117, 117, 117);
	gfx.PutPixel(x + 377, y + 78, 4, 4, 4);
	gfx.PutPixel(x + 377, y + 79, 3, 3, 3);
	gfx.PutPixel(x + 377, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 377, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 377, y + 88, 167, 167, 167);
	gfx.PutPixel(x + 377, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 91, 205, 205, 205);
	gfx.PutPixel(x + 377, y + 92, 12, 12, 12);
	gfx.PutPixel(x + 377, y + 93, 5, 5, 5);
	gfx.PutPixel(x + 377, y + 94, 3, 3, 3);
	gfx.PutPixel(x + 377, y + 95, 3, 3, 3);
	gfx.PutPixel(x + 377, y + 96, 177, 177, 177);
	gfx.PutPixel(x + 377, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 109, 219, 219, 219);
	gfx.PutPixel(x + 377, y + 110, 17, 17, 17);
	gfx.PutPixel(x + 377, y + 111, 4, 4, 4);
	gfx.PutPixel(x + 378, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 378, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 57, 4, 4, 4);
	gfx.PutPixel(x + 378, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 378, y + 59, 1, 1, 1);
	gfx.PutPixel(x + 378, y + 60, 47, 47, 47);
	gfx.PutPixel(x + 378, y + 61, 136, 136, 136);
	gfx.PutPixel(x + 378, y + 62, 223, 223, 223);
	gfx.PutPixel(x + 378, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 78, 175, 175, 175);
	gfx.PutPixel(x + 378, y + 79, 18, 18, 18);
	gfx.PutPixel(x + 378, y + 80, 0, 0, 0);
	gfx.PutPixel(x + 378, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 378, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 378, y + 88, 145, 145, 145);
	gfx.PutPixel(x + 378, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 91, 152, 152, 152);
	gfx.PutPixel(x + 378, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 378, y + 93, 0, 0, 0);
	gfx.PutPixel(x + 378, y + 94, 4, 4, 4);
	gfx.PutPixel(x + 378, y + 95, 156, 156, 156);
	gfx.PutPixel(x + 378, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 110, 103, 103, 103);
	gfx.PutPixel(x + 378, y + 111, 0, 0, 0);
	gfx.PutPixel(x + 379, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 54, 7, 7, 7);
	gfx.PutPixel(x + 379, y + 55, 1, 1, 1);
	gfx.PutPixel(x + 379, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 379, y + 57, 18, 18, 18);
	gfx.PutPixel(x + 379, y + 58, 90, 90, 90);
	gfx.PutPixel(x + 379, y + 59, 186, 186, 186);
	gfx.PutPixel(x + 379, y + 60, 249, 249, 249);
	gfx.PutPixel(x + 379, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 79, 199, 199, 199);
	gfx.PutPixel(x + 379, y + 80, 24, 24, 24);
	gfx.PutPixel(x + 379, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 379, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 379, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 379, y + 88, 105, 105, 105);
	gfx.PutPixel(x + 379, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 91, 212, 212, 212);
	gfx.PutPixel(x + 379, y + 92, 4, 4, 4);
	gfx.PutPixel(x + 379, y + 93, 43, 43, 43);
	gfx.PutPixel(x + 379, y + 94, 188, 188, 188);
	gfx.PutPixel(x + 379, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 110, 207, 207, 207);
	gfx.PutPixel(x + 379, y + 111, 9, 9, 9);
	gfx.PutPixel(x + 380, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 380, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 52, 4, 4, 4);
	gfx.PutPixel(x + 380, y + 53, 0, 0, 0);
	gfx.PutPixel(x + 380, y + 54, 2, 2, 2);
	gfx.PutPixel(x + 380, y + 55, 50, 50, 50);
	gfx.PutPixel(x + 380, y + 56, 140, 140, 140);
	gfx.PutPixel(x + 380, y + 57, 226, 226, 226);
	gfx.PutPixel(x + 380, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 80, 200, 200, 200);
	gfx.PutPixel(x + 380, y + 81, 20, 20, 20);
	gfx.PutPixel(x + 380, y + 82, 1, 1, 1);
	gfx.PutPixel(x + 380, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 380, y + 87, 0, 0, 0);
	gfx.PutPixel(x + 380, y + 88, 56, 56, 56);
	gfx.PutPixel(x + 380, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 91, 248, 248, 248);
	gfx.PutPixel(x + 380, y + 92, 188, 188, 188);
	gfx.PutPixel(x + 380, y + 93, 242, 242, 242);
	gfx.PutPixel(x + 380, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 111, 81, 81, 81);
	gfx.PutPixel(x + 381, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 49, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 50, 1, 1, 1);
	gfx.PutPixel(x + 381, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 381, y + 52, 20, 20, 20);
	gfx.PutPixel(x + 381, y + 53, 94, 94, 94);
	gfx.PutPixel(x + 381, y + 54, 189, 189, 189);
	gfx.PutPixel(x + 381, y + 55, 250, 250, 250);
	gfx.PutPixel(x + 381, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 81, 181, 181, 181);
	gfx.PutPixel(x + 381, y + 82, 7, 7, 7);
	gfx.PutPixel(x + 381, y + 83, 4, 4, 4);
	gfx.PutPixel(x + 381, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 381, y + 87, 5, 5, 5);
	gfx.PutPixel(x + 381, y + 88, 14, 14, 14);
	gfx.PutPixel(x + 381, y + 89, 217, 217, 217);
	gfx.PutPixel(x + 381, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 111, 175, 175, 175);
	gfx.PutPixel(x + 382, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 47, 4, 4, 4);
	gfx.PutPixel(x + 382, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 382, y + 49, 3, 3, 3);
	gfx.PutPixel(x + 382, y + 50, 53, 53, 53);
	gfx.PutPixel(x + 382, y + 51, 144, 144, 144);
	gfx.PutPixel(x + 382, y + 52, 229, 229, 229);
	gfx.PutPixel(x + 382, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 82, 144, 144, 144);
	gfx.PutPixel(x + 382, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 382, y + 84, 7, 7, 7);
	gfx.PutPixel(x + 382, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 382, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 382, y + 89, 155, 155, 155);
	gfx.PutPixel(x + 382, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 111, 228, 228, 228);
	gfx.PutPixel(x + 383, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 383, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 44, 6, 6, 6);
	gfx.PutPixel(x + 383, y + 45, 1, 1, 1);
	gfx.PutPixel(x + 383, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 383, y + 47, 21, 21, 21);
	gfx.PutPixel(x + 383, y + 48, 98, 98, 98);
	gfx.PutPixel(x + 383, y + 49, 193, 193, 193);
	gfx.PutPixel(x + 383, y + 50, 252, 252, 252);
	gfx.PutPixel(x + 383, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 83, 91, 91, 91);
	gfx.PutPixel(x + 383, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 383, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 383, y + 88, 0, 0, 0);
	gfx.PutPixel(x + 383, y + 89, 85, 85, 85);
	gfx.PutPixel(x + 383, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 111, 249, 249, 249);
	gfx.PutPixel(x + 384, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 384, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 41, 6, 6, 6);
	gfx.PutPixel(x + 384, y + 42, 1, 1, 1);
	gfx.PutPixel(x + 384, y + 43, 0, 0, 0);
	gfx.PutPixel(x + 384, y + 44, 4, 4, 4);
	gfx.PutPixel(x + 384, y + 45, 56, 56, 56);
	gfx.PutPixel(x + 384, y + 46, 148, 148, 148);
	gfx.PutPixel(x + 384, y + 47, 231, 231, 231);
	gfx.PutPixel(x + 384, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 83, 232, 232, 232);
	gfx.PutPixel(x + 384, y + 84, 36, 36, 36);
	gfx.PutPixel(x + 384, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 384, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 384, y + 88, 3, 3, 3);
	gfx.PutPixel(x + 384, y + 89, 28, 28, 28);
	gfx.PutPixel(x + 384, y + 90, 236, 236, 236);
	gfx.PutPixel(x + 384, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 111, 254, 254, 254);
	gfx.PutPixel(x + 385, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 38, 4, 4, 4);
	gfx.PutPixel(x + 385, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 385, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 385, y + 41, 5, 5, 5);
	gfx.PutPixel(x + 385, y + 42, 40, 40, 40);
	gfx.PutPixel(x + 385, y + 43, 105, 105, 105);
	gfx.PutPixel(x + 385, y + 44, 195, 195, 195);
	gfx.PutPixel(x + 385, y + 45, 252, 252, 252);
	gfx.PutPixel(x + 385, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 84, 175, 175, 175);
	gfx.PutPixel(x + 385, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 385, y + 86, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 385, y + 88, 7, 7, 7);
	gfx.PutPixel(x + 385, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 385, y + 90, 180, 180, 180);
	gfx.PutPixel(x + 385, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 111, 245, 245, 245);
	gfx.PutPixel(x + 386, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 386, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 34, 5, 5, 5);
	gfx.PutPixel(x + 386, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 386, y + 36, 0, 0, 0);
	gfx.PutPixel(x + 386, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 386, y + 38, 21, 21, 21);
	gfx.PutPixel(x + 386, y + 39, 69, 69, 69);
	gfx.PutPixel(x + 386, y + 40, 135, 135, 135);
	gfx.PutPixel(x + 386, y + 41, 202, 202, 202);
	gfx.PutPixel(x + 386, y + 42, 248, 248, 248);
	gfx.PutPixel(x + 386, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 85, 93, 93, 93);
	gfx.PutPixel(x + 386, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 386, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 386, y + 89, 0, 0, 0);
	gfx.PutPixel(x + 386, y + 90, 99, 99, 99);
	gfx.PutPixel(x + 386, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 111, 209, 209, 209);
	gfx.PutPixel(x + 387, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 387, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 30, 5, 5, 5);
	gfx.PutPixel(x + 387, y + 31, 0, 0, 0);
	gfx.PutPixel(x + 387, y + 32, 0, 0, 0);
	gfx.PutPixel(x + 387, y + 33, 0, 0, 0);
	gfx.PutPixel(x + 387, y + 34, 13, 13, 13);
	gfx.PutPixel(x + 387, y + 35, 53, 53, 53);
	gfx.PutPixel(x + 387, y + 36, 111, 111, 111);
	gfx.PutPixel(x + 387, y + 37, 176, 176, 176);
	gfx.PutPixel(x + 387, y + 38, 231, 231, 231);
	gfx.PutPixel(x + 387, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 85, 202, 202, 202);
	gfx.PutPixel(x + 387, y + 86, 5, 5, 5);
	gfx.PutPixel(x + 387, y + 87, 6, 6, 6);
	gfx.PutPixel(x + 387, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 387, y + 89, 4, 4, 4);
	gfx.PutPixel(x + 387, y + 90, 21, 21, 21);
	gfx.PutPixel(x + 387, y + 91, 224, 224, 224);
	gfx.PutPixel(x + 387, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 111, 125, 125, 125);
	gfx.PutPixel(x + 388, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 388, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 25, 5, 5, 5);
	gfx.PutPixel(x + 388, y + 26, 2, 2, 2);
	gfx.PutPixel(x + 388, y + 27, 0, 0, 0);
	gfx.PutPixel(x + 388, y + 28, 0, 0, 0);
	gfx.PutPixel(x + 388, y + 29, 0, 0, 0);
	gfx.PutPixel(x + 388, y + 30, 16, 16, 16);
	gfx.PutPixel(x + 388, y + 31, 53, 53, 53);
	gfx.PutPixel(x + 388, y + 32, 105, 105, 105);
	gfx.PutPixel(x + 388, y + 33, 165, 165, 165);
	gfx.PutPixel(x + 388, y + 34, 220, 220, 220);
	gfx.PutPixel(x + 388, y + 35, 252, 252, 252);
	gfx.PutPixel(x + 388, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 85, 209, 209, 209);
	gfx.PutPixel(x + 388, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 87, 6, 6, 6);
	gfx.PutPixel(x + 388, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 388, y + 90, 0, 0, 0);
	gfx.PutPixel(x + 388, y + 91, 116, 116, 116);
	gfx.PutPixel(x + 388, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 110, 214, 214, 214);
	gfx.PutPixel(x + 388, y + 111, 20, 20, 20);
	gfx.PutPixel(x + 389, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 389, y + 18, 7, 7, 7);
	gfx.PutPixel(x + 389, y + 19, 5, 5, 5);
	gfx.PutPixel(x + 389, y + 20, 2, 2, 2);
	gfx.PutPixel(x + 389, y + 21, 0, 0, 0);
	gfx.PutPixel(x + 389, y + 22, 0, 0, 0);
	gfx.PutPixel(x + 389, y + 23, 0, 0, 0);
	gfx.PutPixel(x + 389, y + 24, 0, 0, 0);
	gfx.PutPixel(x + 389, y + 25, 10, 10, 10);
	gfx.PutPixel(x + 389, y + 26, 34, 34, 34);
	gfx.PutPixel(x + 389, y + 27, 74, 74, 74);
	gfx.PutPixel(x + 389, y + 28, 124, 124, 124);
	gfx.PutPixel(x + 389, y + 29, 176, 176, 176);
	gfx.PutPixel(x + 389, y + 30, 224, 224, 224);
	gfx.PutPixel(x + 389, y + 31, 253, 253, 253);
	gfx.PutPixel(x + 389, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 85, 201, 201, 201);
	gfx.PutPixel(x + 389, y + 86, 5, 5, 5);
	gfx.PutPixel(x + 389, y + 87, 6, 6, 6);
	gfx.PutPixel(x + 389, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 389, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 389, y + 90, 5, 5, 5);
	gfx.PutPixel(x + 389, y + 91, 9, 9, 9);
	gfx.PutPixel(x + 389, y + 92, 194, 194, 194);
	gfx.PutPixel(x + 389, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 109, 235, 235, 235);
	gfx.PutPixel(x + 389, y + 110, 55, 55, 55);
	gfx.PutPixel(x + 389, y + 111, 0, 0, 0);
	gfx.PutPixel(x + 390, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 16, 2, 2, 2);
	gfx.PutPixel(x + 390, y + 17, 0, 0, 0);
	gfx.PutPixel(x + 390, y + 18, 1, 1, 1);
	gfx.PutPixel(x + 390, y + 19, 17, 17, 17);
	gfx.PutPixel(x + 390, y + 20, 37, 37, 37);
	gfx.PutPixel(x + 390, y + 21, 60, 60, 60);
	gfx.PutPixel(x + 390, y + 22, 93, 93, 93);
	gfx.PutPixel(x + 390, y + 23, 130, 130, 130);
	gfx.PutPixel(x + 390, y + 24, 172, 172, 172);
	gfx.PutPixel(x + 390, y + 25, 213, 213, 213);
	gfx.PutPixel(x + 390, y + 26, 243, 243, 243);
	gfx.PutPixel(x + 390, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 85, 187, 187, 187);
	gfx.PutPixel(x + 390, y + 86, 1, 1, 1);
	gfx.PutPixel(x + 390, y + 87, 7, 7, 7);
	gfx.PutPixel(x + 390, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 390, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 390, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 390, y + 91, 0, 0, 0);
	gfx.PutPixel(x + 390, y + 92, 34, 34, 34);
	gfx.PutPixel(x + 390, y + 93, 218, 218, 218);
	gfx.PutPixel(x + 390, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 108, 211, 211, 211);
	gfx.PutPixel(x + 390, y + 109, 49, 49, 49);
	gfx.PutPixel(x + 390, y + 110, 0, 0, 0);
	gfx.PutPixel(x + 390, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 391, y + 13, 4, 4, 4);
	gfx.PutPixel(x + 391, y + 14, 0, 0, 0);
	gfx.PutPixel(x + 391, y + 15, 0, 0, 0);
	gfx.PutPixel(x + 391, y + 16, 30, 30, 30);
	gfx.PutPixel(x + 391, y + 17, 99, 99, 99);
	gfx.PutPixel(x + 391, y + 18, 175, 175, 175);
	gfx.PutPixel(x + 391, y + 19, 227, 227, 227);
	gfx.PutPixel(x + 391, y + 20, 245, 245, 245);
	gfx.PutPixel(x + 391, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 85, 164, 164, 164);
	gfx.PutPixel(x + 391, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 391, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 92, 0, 0, 0);
	gfx.PutPixel(x + 391, y + 93, 32, 32, 32);
	gfx.PutPixel(x + 391, y + 94, 190, 190, 190);
	gfx.PutPixel(x + 391, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 106, 241, 241, 241);
	gfx.PutPixel(x + 391, y + 107, 133, 133, 133);
	gfx.PutPixel(x + 391, y + 108, 16, 16, 16);
	gfx.PutPixel(x + 391, y + 109, 0, 0, 0);
	gfx.PutPixel(x + 391, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 391, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 392, y + 11, 3, 3, 3);
	gfx.PutPixel(x + 392, y + 12, 0, 0, 0);
	gfx.PutPixel(x + 392, y + 13, 12, 12, 12);
	gfx.PutPixel(x + 392, y + 14, 82, 82, 82);
	gfx.PutPixel(x + 392, y + 15, 173, 173, 173);
	gfx.PutPixel(x + 392, y + 16, 240, 240, 240);
	gfx.PutPixel(x + 392, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 85, 137, 137, 137);
	gfx.PutPixel(x + 392, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 392, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 93, 2, 2, 2);
	gfx.PutPixel(x + 392, y + 94, 6, 6, 6);
	gfx.PutPixel(x + 392, y + 95, 93, 93, 93);
	gfx.PutPixel(x + 392, y + 96, 192, 192, 192);
	gfx.PutPixel(x + 392, y + 97, 243, 243, 243);
	gfx.PutPixel(x + 392, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 102, 252, 252, 252);
	gfx.PutPixel(x + 392, y + 103, 230, 230, 230);
	gfx.PutPixel(x + 392, y + 104, 186, 186, 186);
	gfx.PutPixel(x + 392, y + 105, 121, 121, 121);
	gfx.PutPixel(x + 392, y + 106, 35, 35, 35);
	gfx.PutPixel(x + 392, y + 107, 0, 0, 0);
	gfx.PutPixel(x + 392, y + 108, 4, 4, 4);
	gfx.PutPixel(x + 392, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 392, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 393, y + 9, 4, 4, 4);
	gfx.PutPixel(x + 393, y + 10, 0, 0, 0);
	gfx.PutPixel(x + 393, y + 11, 23, 23, 23);
	gfx.PutPixel(x + 393, y + 12, 119, 119, 119);
	gfx.PutPixel(x + 393, y + 13, 219, 219, 219);
	gfx.PutPixel(x + 393, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 85, 98, 98, 98);
	gfx.PutPixel(x + 393, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 393, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 94, 6, 6, 6);
	gfx.PutPixel(x + 393, y + 95, 0, 0, 0);
	gfx.PutPixel(x + 393, y + 96, 3, 3, 3);
	gfx.PutPixel(x + 393, y + 97, 34, 34, 34);
	gfx.PutPixel(x + 393, y + 98, 65, 65, 65);
	gfx.PutPixel(x + 393, y + 99, 84, 84, 84);
	gfx.PutPixel(x + 393, y + 100, 87, 87, 87);
	gfx.PutPixel(x + 393, y + 101, 70, 70, 70);
	gfx.PutPixel(x + 393, y + 102, 50, 50, 50);
	gfx.PutPixel(x + 393, y + 103, 20, 20, 20);
	gfx.PutPixel(x + 393, y + 104, 2, 2, 2);
	gfx.PutPixel(x + 393, y + 105, 0, 0, 0);
	gfx.PutPixel(x + 393, y + 106, 3, 3, 3);
	gfx.PutPixel(x + 393, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 393, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 9, 13, 13, 13);
	gfx.PutPixel(x + 394, y + 10, 118, 118, 118);
	gfx.PutPixel(x + 394, y + 11, 231, 231, 231);
	gfx.PutPixel(x + 394, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 84, 254, 254, 254);
	gfx.PutPixel(x + 394, y + 85, 54, 54, 54);
	gfx.PutPixel(x + 394, y + 86, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 96, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 97, 2, 2, 2);
	gfx.PutPixel(x + 394, y + 98, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 99, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 100, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 101, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 102, 0, 0, 0);
	gfx.PutPixel(x + 394, y + 103, 4, 4, 4);
	gfx.PutPixel(x + 394, y + 104, 7, 7, 7);
	gfx.PutPixel(x + 394, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 394, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 395, y + 6, 5, 5, 5);
	gfx.PutPixel(x + 395, y + 7, 0, 0, 0);
	gfx.PutPixel(x + 395, y + 8, 66, 66, 66);
	gfx.PutPixel(x + 395, y + 9, 212, 212, 212);
	gfx.PutPixel(x + 395, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 84, 216, 216, 216);
	gfx.PutPixel(x + 395, y + 85, 13, 13, 13);
	gfx.PutPixel(x + 395, y + 86, 5, 5, 5);
	gfx.PutPixel(x + 395, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 395, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 396, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 396, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 396, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 396, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 396, y + 5, 4, 4, 4);
	gfx.PutPixel(x + 396, y + 6, 0, 0, 0);
	gfx.PutPixel(x + 396, y + 7, 121, 121, 121);
	gfx.PutPixel(x + 396, y + 8, 252, 252, 252);
	gfx.PutPixel(x + 396, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 84, 139, 139, 139);
	gfx.PutPixel(x + 396, y + 85, 0, 0, 0);
	gfx.PutPixel(x + 396, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 396, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 397, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 397, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 397, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 397, y + 4, 5, 5, 5);
	gfx.PutPixel(x + 397, y + 5, 0, 0, 0);
	gfx.PutPixel(x + 397, y + 6, 143, 143, 143);
	gfx.PutPixel(x + 397, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 14, 190, 190, 190);
	gfx.PutPixel(x + 397, y + 15, 212, 212, 212);
	gfx.PutPixel(x + 397, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 83, 236, 236, 236);
	gfx.PutPixel(x + 397, y + 84, 37, 37, 37);
	gfx.PutPixel(x + 397, y + 85, 1, 1, 1);
	gfx.PutPixel(x + 397, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 397, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 398, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 398, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 398, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 398, y + 4, 0, 0, 0);
	gfx.PutPixel(x + 398, y + 5, 118, 118, 118);
	gfx.PutPixel(x + 398, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 12, 230, 230, 230);
	gfx.PutPixel(x + 398, y + 13, 100, 100, 100);
	gfx.PutPixel(x + 398, y + 14, 0, 0, 0);
	gfx.PutPixel(x + 398, y + 15, 144, 144, 144);
	gfx.PutPixel(x + 398, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 82, 250, 250, 250);
	gfx.PutPixel(x + 398, y + 83, 91, 91, 91);
	gfx.PutPixel(x + 398, y + 84, 0, 0, 0);
	gfx.PutPixel(x + 398, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 398, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 399, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 399, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 399, y + 3, 0, 0, 0);
	gfx.PutPixel(x + 399, y + 4, 49, 49, 49);
	gfx.PutPixel(x + 399, y + 5, 246, 246, 246);
	gfx.PutPixel(x + 399, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 11, 163, 163, 163);
	gfx.PutPixel(x + 399, y + 12, 25, 25, 25);
	gfx.PutPixel(x + 399, y + 13, 0, 0, 0);
	gfx.PutPixel(x + 399, y + 14, 121, 121, 121);
	gfx.PutPixel(x + 399, y + 15, 250, 250, 250);
	gfx.PutPixel(x + 399, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 79, 242, 242, 242);
	gfx.PutPixel(x + 399, y + 80, 198, 198, 198);
	gfx.PutPixel(x + 399, y + 81, 139, 139, 139);
	gfx.PutPixel(x + 399, y + 82, 51, 51, 51);
	gfx.PutPixel(x + 399, y + 83, 0, 0, 0);
	gfx.PutPixel(x + 399, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 399, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 400, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 400, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 400, y + 3, 0, 0, 0);
	gfx.PutPixel(x + 400, y + 4, 158, 158, 158);
	gfx.PutPixel(x + 400, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 9, 240, 240, 240);
	gfx.PutPixel(x + 400, y + 10, 93, 93, 93);
	gfx.PutPixel(x + 400, y + 11, 0, 0, 0);
	gfx.PutPixel(x + 400, y + 12, 0, 0, 0);
	gfx.PutPixel(x + 400, y + 13, 116, 116, 116);
	gfx.PutPixel(x + 400, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 71, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 72, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 73, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 74, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 75, 243, 243, 243);
	gfx.PutPixel(x + 400, y + 76, 200, 200, 200);
	gfx.PutPixel(x + 400, y + 77, 142, 142, 142);
	gfx.PutPixel(x + 400, y + 78, 81, 81, 81);
	gfx.PutPixel(x + 400, y + 79, 32, 32, 32);
	gfx.PutPixel(x + 400, y + 80, 4, 4, 4);
	gfx.PutPixel(x + 400, y + 81, 0, 0, 0);
	gfx.PutPixel(x + 400, y + 82, 2, 2, 2);
	gfx.PutPixel(x + 400, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 400, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 401, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 401, y + 2, 3, 3, 3);
	gfx.PutPixel(x + 401, y + 3, 19, 19, 19);
	gfx.PutPixel(x + 401, y + 4, 225, 225, 225);
	gfx.PutPixel(x + 401, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 8, 226, 226, 226);
	gfx.PutPixel(x + 401, y + 9, 53, 53, 53);
	gfx.PutPixel(x + 401, y + 10, 0, 0, 0);
	gfx.PutPixel(x + 401, y + 11, 0, 0, 0);
	gfx.PutPixel(x + 401, y + 12, 72, 72, 72);
	gfx.PutPixel(x + 401, y + 13, 252, 252, 252);
	gfx.PutPixel(x + 401, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 67, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 68, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 69, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 70, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 71, 244, 244, 244);
	gfx.PutPixel(x + 401, y + 72, 202, 202, 202);
	gfx.PutPixel(x + 401, y + 73, 145, 145, 145);
	gfx.PutPixel(x + 401, y + 74, 84, 84, 84);
	gfx.PutPixel(x + 401, y + 75, 34, 34, 34);
	gfx.PutPixel(x + 401, y + 76, 5, 5, 5);
	gfx.PutPixel(x + 401, y + 77, 0, 0, 0);
	gfx.PutPixel(x + 401, y + 78, 0, 0, 0);
	gfx.PutPixel(x + 401, y + 79, 2, 2, 2);
	gfx.PutPixel(x + 401, y + 80, 7, 7, 7);
	gfx.PutPixel(x + 401, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 401, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 402, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 402, y + 2, 0, 0, 0);
	gfx.PutPixel(x + 402, y + 3, 45, 45, 45);
	gfx.PutPixel(x + 402, y + 4, 250, 250, 250);
	gfx.PutPixel(x + 402, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 7, 239, 239, 239);
	gfx.PutPixel(x + 402, y + 8, 52, 52, 52);
	gfx.PutPixel(x + 402, y + 9, 0, 0, 0);
	gfx.PutPixel(x + 402, y + 10, 1, 1, 1);
	gfx.PutPixel(x + 402, y + 11, 24, 24, 24);
	gfx.PutPixel(x + 402, y + 12, 221, 221, 221);
	gfx.PutPixel(x + 402, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 63, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 64, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 65, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 66, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 67, 246, 246, 246);
	gfx.PutPixel(x + 402, y + 68, 205, 205, 205);
	gfx.PutPixel(x + 402, y + 69, 148, 148, 148);
	gfx.PutPixel(x + 402, y + 70, 87, 87, 87);
	gfx.PutPixel(x + 402, y + 71, 36, 36, 36);
	gfx.PutPixel(x + 402, y + 72, 5, 5, 5);
	gfx.PutPixel(x + 402, y + 73, 0, 0, 0);
	gfx.PutPixel(x + 402, y + 74, 0, 0, 0);
	gfx.PutPixel(x + 402, y + 75, 2, 2, 2);
	gfx.PutPixel(x + 402, y + 76, 7, 7, 7);
	gfx.PutPixel(x + 402, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 402, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 403, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 403, y + 2, 0, 0, 0);
	gfx.PutPixel(x + 403, y + 3, 48, 48, 48);
	gfx.PutPixel(x + 403, y + 4, 253, 253, 253);
	gfx.PutPixel(x + 403, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 7, 127, 127, 127);
	gfx.PutPixel(x + 403, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 403, y + 9, 6, 6, 6);
	gfx.PutPixel(x + 403, y + 10, 0, 0, 0);
	gfx.PutPixel(x + 403, y + 11, 157, 157, 157);
	gfx.PutPixel(x + 403, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 59, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 60, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 61, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 62, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 63, 247, 247, 247);
	gfx.PutPixel(x + 403, y + 64, 207, 207, 207);
	gfx.PutPixel(x + 403, y + 65, 150, 150, 150);
	gfx.PutPixel(x + 403, y + 66, 89, 89, 89);
	gfx.PutPixel(x + 403, y + 67, 38, 38, 38);
	gfx.PutPixel(x + 403, y + 68, 6, 6, 6);
	gfx.PutPixel(x + 403, y + 69, 0, 0, 0);
	gfx.PutPixel(x + 403, y + 70, 0, 0, 0);
	gfx.PutPixel(x + 403, y + 71, 2, 2, 2);
	gfx.PutPixel(x + 403, y + 72, 6, 6, 6);
	gfx.PutPixel(x + 403, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 403, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 404, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 404, y + 2, 1, 1, 1);
	gfx.PutPixel(x + 404, y + 3, 31, 31, 31);
	gfx.PutPixel(x + 404, y + 4, 239, 239, 239);
	gfx.PutPixel(x + 404, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 7, 95, 95, 95);
	gfx.PutPixel(x + 404, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 404, y + 9, 0, 0, 0);
	gfx.PutPixel(x + 404, y + 10, 76, 76, 76);
	gfx.PutPixel(x + 404, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 53, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 54, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 55, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 56, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 57, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 58, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 59, 230, 230, 230);
	gfx.PutPixel(x + 404, y + 60, 165, 165, 165);
	gfx.PutPixel(x + 404, y + 61, 159, 159, 159);
	gfx.PutPixel(x + 404, y + 62, 92, 92, 92);
	gfx.PutPixel(x + 404, y + 63, 40, 40, 40);
	gfx.PutPixel(x + 404, y + 64, 7, 7, 7);
	gfx.PutPixel(x + 404, y + 65, 0, 0, 0);
	gfx.PutPixel(x + 404, y + 66, 0, 0, 0);
	gfx.PutPixel(x + 404, y + 67, 1, 1, 1);
	gfx.PutPixel(x + 404, y + 68, 6, 6, 6);
	gfx.PutPixel(x + 404, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 404, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 405, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 405, y + 2, 6, 6, 6);
	gfx.PutPixel(x + 405, y + 3, 3, 3, 3);
	gfx.PutPixel(x + 405, y + 4, 198, 198, 198);
	gfx.PutPixel(x + 405, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 7, 162, 162, 162);
	gfx.PutPixel(x + 405, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 405, y + 9, 11, 11, 11);
	gfx.PutPixel(x + 405, y + 10, 209, 209, 209);
	gfx.PutPixel(x + 405, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 50, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 51, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 52, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 53, 249, 249, 249);
	gfx.PutPixel(x + 405, y + 54, 205, 205, 205);
	gfx.PutPixel(x + 405, y + 55, 137, 137, 137);
	gfx.PutPixel(x + 405, y + 56, 91, 91, 91);
	gfx.PutPixel(x + 405, y + 57, 68, 68, 68);
	gfx.PutPixel(x + 405, y + 58, 56, 56, 56);
	gfx.PutPixel(x + 405, y + 59, 36, 36, 36);
	gfx.PutPixel(x + 405, y + 60, 2, 2, 2);
	gfx.PutPixel(x + 405, y + 61, 0, 0, 0);
	gfx.PutPixel(x + 405, y + 62, 0, 0, 0);
	gfx.PutPixel(x + 405, y + 63, 1, 1, 1);
	gfx.PutPixel(x + 405, y + 64, 6, 6, 6);
	gfx.PutPixel(x + 405, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 405, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 406, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 406, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 406, y + 3, 0, 0, 0);
	gfx.PutPixel(x + 406, y + 4, 133, 133, 133);
	gfx.PutPixel(x + 406, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 7, 223, 223, 223);
	gfx.PutPixel(x + 406, y + 8, 69, 69, 69);
	gfx.PutPixel(x + 406, y + 9, 161, 161, 161);
	gfx.PutPixel(x + 406, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 46, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 47, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 48, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 49, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 50, 236, 236, 236);
	gfx.PutPixel(x + 406, y + 51, 180, 180, 180);
	gfx.PutPixel(x + 406, y + 52, 109, 109, 109);
	gfx.PutPixel(x + 406, y + 53, 45, 45, 45);
	gfx.PutPixel(x + 406, y + 54, 6, 6, 6);
	gfx.PutPixel(x + 406, y + 55, 0, 0, 0);
	gfx.PutPixel(x + 406, y + 56, 0, 0, 0);
	gfx.PutPixel(x + 406, y + 57, 0, 0, 0);
	gfx.PutPixel(x + 406, y + 58, 0, 0, 0);
	gfx.PutPixel(x + 406, y + 59, 2, 2, 2);
	gfx.PutPixel(x + 406, y + 60, 7, 7, 7);
	gfx.PutPixel(x + 406, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 406, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 407, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 407, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 407, y + 3, 0, 0, 0);
	gfx.PutPixel(x + 407, y + 4, 64, 64, 64);
	gfx.PutPixel(x + 407, y + 5, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 43, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 44, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 45, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 46, 253, 253, 253);
	gfx.PutPixel(x + 407, y + 47, 217, 217, 217);
	gfx.PutPixel(x + 407, y + 48, 153, 153, 153);
	gfx.PutPixel(x + 407, y + 49, 82, 82, 82);
	gfx.PutPixel(x + 407, y + 50, 26, 26, 26);
	gfx.PutPixel(x + 407, y + 51, 0, 0, 0);
	gfx.PutPixel(x + 407, y + 52, 0, 0, 0);
	gfx.PutPixel(x + 407, y + 53, 1, 1, 1);
	gfx.PutPixel(x + 407, y + 54, 6, 6, 6);
	gfx.PutPixel(x + 407, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 407, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 408, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 408, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 408, y + 3, 4, 4, 4);
	gfx.PutPixel(x + 408, y + 4, 15, 15, 15);
	gfx.PutPixel(x + 408, y + 5, 221, 221, 221);
	gfx.PutPixel(x + 408, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 41, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 42, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 43, 245, 245, 245);
	gfx.PutPixel(x + 408, y + 44, 194, 194, 194);
	gfx.PutPixel(x + 408, y + 45, 125, 125, 125);
	gfx.PutPixel(x + 408, y + 46, 58, 58, 58);
	gfx.PutPixel(x + 408, y + 47, 12, 12, 12);
	gfx.PutPixel(x + 408, y + 48, 0, 0, 0);
	gfx.PutPixel(x + 408, y + 49, 0, 0, 0);
	gfx.PutPixel(x + 408, y + 50, 3, 3, 3);
	gfx.PutPixel(x + 408, y + 51, 7, 7, 7);
	gfx.PutPixel(x + 408, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 408, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 409, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 409, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 409, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 409, y + 4, 0, 0, 0);
	gfx.PutPixel(x + 409, y + 5, 153, 153, 153);
	gfx.PutPixel(x + 409, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 38, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 39, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 40, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 41, 213, 213, 213);
	gfx.PutPixel(x + 409, y + 42, 124, 124, 124);
	gfx.PutPixel(x + 409, y + 43, 40, 40, 40);
	gfx.PutPixel(x + 409, y + 44, 2, 2, 2);
	gfx.PutPixel(x + 409, y + 45, 0, 0, 0);
	gfx.PutPixel(x + 409, y + 46, 0, 0, 0);
	gfx.PutPixel(x + 409, y + 47, 5, 5, 5);
	gfx.PutPixel(x + 409, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 409, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 410, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 410, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 410, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 410, y + 4, 0, 0, 0);
	gfx.PutPixel(x + 410, y + 5, 63, 63, 63);
	gfx.PutPixel(x + 410, y + 6, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 35, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 36, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 37, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 38, 239, 239, 239);
	gfx.PutPixel(x + 410, y + 39, 164, 164, 164);
	gfx.PutPixel(x + 410, y + 40, 71, 71, 71);
	gfx.PutPixel(x + 410, y + 41, 9, 9, 9);
	gfx.PutPixel(x + 410, y + 42, 0, 0, 0);
	gfx.PutPixel(x + 410, y + 43, 2, 2, 2);
	gfx.PutPixel(x + 410, y + 44, 7, 7, 7);
	gfx.PutPixel(x + 410, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 410, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 411, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 411, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 411, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 411, y + 4, 6, 6, 6);
	gfx.PutPixel(x + 411, y + 5, 2, 2, 2);
	gfx.PutPixel(x + 411, y + 6, 184, 184, 184);
	gfx.PutPixel(x + 411, y + 7, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 33, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 34, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 35, 253, 253, 253);
	gfx.PutPixel(x + 411, y + 36, 201, 201, 201);
	gfx.PutPixel(x + 411, y + 37, 109, 109, 109);
	gfx.PutPixel(x + 411, y + 38, 30, 30, 30);
	gfx.PutPixel(x + 411, y + 39, 0, 0, 0);
	gfx.PutPixel(x + 411, y + 40, 0, 0, 0);
	gfx.PutPixel(x + 411, y + 41, 5, 5, 5);
	gfx.PutPixel(x + 411, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 411, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 412, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 412, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 412, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 412, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 412, y + 5, 0, 0, 0);
	gfx.PutPixel(x + 412, y + 6, 58, 58, 58);
	gfx.PutPixel(x + 412, y + 7, 249, 249, 249);
	gfx.PutPixel(x + 412, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 30, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 31, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 32, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 33, 231, 231, 231);
	gfx.PutPixel(x + 412, y + 34, 149, 149, 149);
	gfx.PutPixel(x + 412, y + 35, 59, 59, 59);
	gfx.PutPixel(x + 412, y + 36, 5, 5, 5);
	gfx.PutPixel(x + 412, y + 37, 0, 0, 0);
	gfx.PutPixel(x + 412, y + 38, 3, 3, 3);
	gfx.PutPixel(x + 412, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 412, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 413, y + 6, 0, 0, 0);
	gfx.PutPixel(x + 413, y + 7, 120, 120, 120);
	gfx.PutPixel(x + 413, y + 8, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 28, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 29, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 30, 250, 250, 250);
	gfx.PutPixel(x + 413, y + 31, 188, 188, 188);
	gfx.PutPixel(x + 413, y + 32, 95, 95, 95);
	gfx.PutPixel(x + 413, y + 33, 21, 21, 21);
	gfx.PutPixel(x + 413, y + 34, 0, 0, 0);
	gfx.PutPixel(x + 413, y + 35, 0, 0, 0);
	gfx.PutPixel(x + 413, y + 36, 6, 6, 6);
	gfx.PutPixel(x + 413, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 413, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 414, y + 6, 5, 5, 5);
	gfx.PutPixel(x + 414, y + 7, 0, 0, 0);
	gfx.PutPixel(x + 414, y + 8, 141, 141, 141);
	gfx.PutPixel(x + 414, y + 9, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 10, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 25, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 26, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 27, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 28, 222, 222, 222);
	gfx.PutPixel(x + 414, y + 29, 135, 135, 135);
	gfx.PutPixel(x + 414, y + 30, 47, 47, 47);
	gfx.PutPixel(x + 414, y + 31, 2, 2, 2);
	gfx.PutPixel(x + 414, y + 32, 0, 0, 0);
	gfx.PutPixel(x + 414, y + 33, 4, 4, 4);
	gfx.PutPixel(x + 414, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 414, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 7, 5, 5, 5);
	gfx.PutPixel(x + 415, y + 8, 0, 0, 0);
	gfx.PutPixel(x + 415, y + 9, 105, 105, 105);
	gfx.PutPixel(x + 415, y + 10, 239, 239, 239);
	gfx.PutPixel(x + 415, y + 11, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 12, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 21, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 22, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 23, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 24, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 25, 243, 243, 243);
	gfx.PutPixel(x + 415, y + 26, 175, 175, 175);
	gfx.PutPixel(x + 415, y + 27, 82, 82, 82);
	gfx.PutPixel(x + 415, y + 28, 14, 14, 14);
	gfx.PutPixel(x + 415, y + 29, 0, 0, 0);
	gfx.PutPixel(x + 415, y + 30, 1, 1, 1);
	gfx.PutPixel(x + 415, y + 31, 7, 7, 7);
	gfx.PutPixel(x + 415, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 415, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 416, y + 8, 6, 6, 6);
	gfx.PutPixel(x + 416, y + 9, 0, 0, 0);
	gfx.PutPixel(x + 416, y + 10, 33, 33, 33);
	gfx.PutPixel(x + 416, y + 11, 138, 138, 138);
	gfx.PutPixel(x + 416, y + 12, 221, 221, 221);
	gfx.PutPixel(x + 416, y + 13, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 14, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 15, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 16, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 17, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 18, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 19, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 20, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 21, 244, 244, 244);
	gfx.PutPixel(x + 416, y + 22, 210, 210, 210);
	gfx.PutPixel(x + 416, y + 23, 157, 157, 157);
	gfx.PutPixel(x + 416, y + 24, 92, 92, 92);
	gfx.PutPixel(x + 416, y + 25, 33, 33, 33);
	gfx.PutPixel(x + 416, y + 26, 0, 0, 0);
	gfx.PutPixel(x + 416, y + 27, 0, 0, 0);
	gfx.PutPixel(x + 416, y + 28, 5, 5, 5);
	gfx.PutPixel(x + 416, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 416, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 417, y + 10, 2, 2, 2);
	gfx.PutPixel(x + 417, y + 11, 0, 0, 0);
	gfx.PutPixel(x + 417, y + 12, 14, 14, 14);
	gfx.PutPixel(x + 417, y + 13, 56, 56, 56);
	gfx.PutPixel(x + 417, y + 14, 99, 99, 99);
	gfx.PutPixel(x + 417, y + 15, 124, 124, 124);
	gfx.PutPixel(x + 417, y + 16, 131, 131, 131);
	gfx.PutPixel(x + 417, y + 17, 131, 131, 131);
	gfx.PutPixel(x + 417, y + 18, 123, 123, 123);
	gfx.PutPixel(x + 417, y + 19, 102, 102, 102);
	gfx.PutPixel(x + 417, y + 20, 68, 68, 68);
	gfx.PutPixel(x + 417, y + 21, 35, 35, 35);
	gfx.PutPixel(x + 417, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 23, 0, 0, 0);
	gfx.PutPixel(x + 417, y + 24, 0, 0, 0);
	gfx.PutPixel(x + 417, y + 25, 2, 2, 2);
	gfx.PutPixel(x + 417, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 417, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 418, y + 12, 4, 4, 4);
	gfx.PutPixel(x + 418, y + 13, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 14, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 15, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 16, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 17, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 18, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 19, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 20, 0, 0, 0);
	gfx.PutPixel(x + 418, y + 21, 2, 2, 2);
	gfx.PutPixel(x + 418, y + 22, 6, 6, 6);
	gfx.PutPixel(x + 418, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 418, y + 111, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 0, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 1, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 2, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 3, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 4, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 5, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 6, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 7, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 8, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 9, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 10, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 11, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 12, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 13, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 14, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 15, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 16, 7, 7, 7);
	gfx.PutPixel(x + 419, y + 17, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 18, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 19, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 20, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 21, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 22, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 23, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 24, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 25, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 26, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 27, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 28, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 29, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 30, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 31, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 32, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 33, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 34, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 35, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 36, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 37, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 38, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 39, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 40, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 41, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 42, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 43, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 44, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 45, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 46, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 47, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 48, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 49, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 50, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 51, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 52, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 53, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 54, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 55, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 56, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 57, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 58, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 59, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 60, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 61, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 62, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 63, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 64, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 65, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 66, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 67, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 68, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 69, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 70, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 71, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 72, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 73, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 74, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 75, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 76, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 77, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 78, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 79, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 80, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 81, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 82, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 83, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 84, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 85, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 86, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 87, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 88, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 89, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 90, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 91, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 92, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 93, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 94, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 95, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 96, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 97, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 98, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 99, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 100, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 101, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 102, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 103, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 104, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 105, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 106, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 107, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 108, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 109, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 110, 8, 8, 8);
	gfx.PutPixel(x + 419, y + 111, 8, 8, 8);
}
void Game::DrawGameOver(int x, int y)
{
	gfx.PutPixel(x + 95, y + 152, 254, 21, 254);
	gfx.PutPixel(x + 95, y + 153, 254, 86, 254);
	gfx.PutPixel(x + 95, y + 154, 254, 147, 254);
	gfx.PutPixel(x + 95, y + 155, 254, 193, 254);
	gfx.PutPixel(x + 95, y + 156, 254, 229, 254);
	gfx.PutPixel(x + 95, y + 157, 254, 246, 254);
	gfx.PutPixel(x + 95, y + 158, 254, 250, 254);
	gfx.PutPixel(x + 95, y + 159, 254, 239, 254);
	gfx.PutPixel(x + 95, y + 160, 254, 216, 254);
	gfx.PutPixel(x + 95, y + 161, 254, 176, 254);
	gfx.PutPixel(x + 95, y + 162, 254, 117, 254);
	gfx.PutPixel(x + 95, y + 163, 254, 38, 254);
	gfx.PutPixel(x + 96, y + 149, 254, 17, 254);
	gfx.PutPixel(x + 96, y + 150, 254, 106, 254);
	gfx.PutPixel(x + 96, y + 151, 254, 193, 254);
	gfx.PutPixel(x + 96, y + 152, 254, 252, 254);
	gfx.PutPixel(x + 96, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 164, 254, 182, 254);
	gfx.PutPixel(x + 96, y + 165, 254, 45, 254);
	gfx.PutPixel(x + 97, y + 147, 254, 55, 254);
	gfx.PutPixel(x + 97, y + 148, 254, 164, 254);
	gfx.PutPixel(x + 97, y + 149, 254, 247, 254);
	gfx.PutPixel(x + 97, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 165, 254, 249, 254);
	gfx.PutPixel(x + 97, y + 166, 254, 117, 254);
	gfx.PutPixel(x + 98, y + 145, 254, 77, 254);
	gfx.PutPixel(x + 98, y + 146, 254, 196, 254);
	gfx.PutPixel(x + 98, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 167, 254, 150, 254);
	gfx.PutPixel(x + 99, y + 143, 254, 76, 254);
	gfx.PutPixel(x + 99, y + 144, 254, 204, 254);
	gfx.PutPixel(x + 99, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 168, 254, 124, 254);
	gfx.PutPixel(x + 100, y + 141, 254, 53, 254);
	gfx.PutPixel(x + 100, y + 142, 254, 194, 254);
	gfx.PutPixel(x + 100, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 168, 254, 251, 254);
	gfx.PutPixel(x + 100, y + 169, 254, 52, 254);
	gfx.PutPixel(x + 101, y + 139, 254, 25, 254);
	gfx.PutPixel(x + 101, y + 140, 254, 164, 254);
	gfx.PutPixel(x + 101, y + 141, 254, 253, 254);
	gfx.PutPixel(x + 101, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 169, 254, 183, 254);
	gfx.PutPixel(x + 102, y + 137, 254, 3, 254);
	gfx.PutPixel(x + 102, y + 138, 254, 115, 254);
	gfx.PutPixel(x + 102, y + 139, 254, 241, 254);
	gfx.PutPixel(x + 102, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 153, 254, 251, 254);
	gfx.PutPixel(x + 102, y + 154, 254, 187, 254);
	gfx.PutPixel(x + 102, y + 155, 254, 119, 254);
	gfx.PutPixel(x + 102, y + 156, 254, 61, 254);
	gfx.PutPixel(x + 102, y + 157, 254, 25, 254);
	gfx.PutPixel(x + 102, y + 158, 254, 5, 254);
	gfx.PutPixel(x + 102, y + 159, 254, 6, 254);
	gfx.PutPixel(x + 102, y + 160, 254, 36, 254);
	gfx.PutPixel(x + 102, y + 161, 254, 101, 254);
	gfx.PutPixel(x + 102, y + 162, 254, 219, 254);
	gfx.PutPixel(x + 102, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 102, y + 170, 254, 52, 254);
	gfx.PutPixel(x + 103, y + 136, 254, 52, 254);
	gfx.PutPixel(x + 103, y + 137, 254, 207, 254);
	gfx.PutPixel(x + 103, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 151, 254, 219, 254);
	gfx.PutPixel(x + 103, y + 152, 254, 111, 254);
	gfx.PutPixel(x + 103, y + 153, 254, 22, 254);
	gfx.PutPixel(x + 103, y + 162, 254, 15, 254);
	gfx.PutPixel(x + 103, y + 163, 254, 193, 254);
	gfx.PutPixel(x + 103, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 103, y + 170, 254, 161, 254);
	gfx.PutPixel(x + 104, y + 134, 254, 8, 254);
	gfx.PutPixel(x + 104, y + 135, 254, 141, 254);
	gfx.PutPixel(x + 104, y + 136, 254, 252, 254);
	gfx.PutPixel(x + 104, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 149, 254, 202, 254);
	gfx.PutPixel(x + 104, y + 150, 254, 78, 254);
	gfx.PutPixel(x + 104, y + 151, 254, 1, 254);
	gfx.PutPixel(x + 104, y + 163, 254, 29, 254);
	gfx.PutPixel(x + 104, y + 164, 254, 243, 254);
	gfx.PutPixel(x + 104, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 104, y + 170, 254, 233, 254);
	gfx.PutPixel(x + 104, y + 171, 254, 0, 254);
	gfx.PutPixel(x + 105, y + 133, 254, 57, 254);
	gfx.PutPixel(x + 105, y + 134, 254, 217, 254);
	gfx.PutPixel(x + 105, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 147, 254, 211, 254);
	gfx.PutPixel(x + 105, y + 148, 254, 80, 254);
	gfx.PutPixel(x + 105, y + 164, 254, 163, 254);
	gfx.PutPixel(x + 105, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 105, y + 171, 254, 51, 254);
	gfx.PutPixel(x + 106, y + 131, 254, 5, 254);
	gfx.PutPixel(x + 106, y + 132, 254, 135, 254);
	gfx.PutPixel(x + 106, y + 133, 254, 252, 254);
	gfx.PutPixel(x + 106, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 145, 254, 235, 254);
	gfx.PutPixel(x + 106, y + 146, 254, 104, 254);
	gfx.PutPixel(x + 106, y + 147, 254, 2, 254);
	gfx.PutPixel(x + 106, y + 164, 254, 92, 254);
	gfx.PutPixel(x + 106, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 106, y + 171, 254, 111, 254);
	gfx.PutPixel(x + 106, y + 293, 254, 15, 254);
	gfx.PutPixel(x + 106, y + 294, 254, 66, 254);
	gfx.PutPixel(x + 106, y + 295, 254, 118, 254);
	gfx.PutPixel(x + 106, y + 296, 254, 160, 254);
	gfx.PutPixel(x + 106, y + 297, 254, 193, 254);
	gfx.PutPixel(x + 106, y + 298, 254, 223, 254);
	gfx.PutPixel(x + 106, y + 299, 254, 238, 254);
	gfx.PutPixel(x + 106, y + 300, 254, 248, 254);
	gfx.PutPixel(x + 106, y + 301, 254, 251, 254);
	gfx.PutPixel(x + 106, y + 302, 254, 242, 254);
	gfx.PutPixel(x + 106, y + 303, 254, 231, 254);
	gfx.PutPixel(x + 106, y + 304, 254, 206, 254);
	gfx.PutPixel(x + 106, y + 305, 254, 173, 254);
	gfx.PutPixel(x + 106, y + 306, 254, 134, 254);
	gfx.PutPixel(x + 106, y + 307, 254, 76, 254);
	gfx.PutPixel(x + 106, y + 308, 254, 16, 254);
	gfx.PutPixel(x + 107, y + 130, 254, 39, 254);
	gfx.PutPixel(x + 107, y + 131, 254, 204, 254);
	gfx.PutPixel(x + 107, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 143, 254, 253, 254);
	gfx.PutPixel(x + 107, y + 144, 254, 149, 254);
	gfx.PutPixel(x + 107, y + 145, 254, 16, 254);
	gfx.PutPixel(x + 107, y + 164, 254, 52, 254);
	gfx.PutPixel(x + 107, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 171, 254, 148, 254);
	gfx.PutPixel(x + 107, y + 289, 254, 0, 254);
	gfx.PutPixel(x + 107, y + 290, 254, 57, 254);
	gfx.PutPixel(x + 107, y + 291, 254, 132, 254);
	gfx.PutPixel(x + 107, y + 292, 254, 202, 254);
	gfx.PutPixel(x + 107, y + 293, 254, 252, 254);
	gfx.PutPixel(x + 107, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 107, y + 308, 254, 251, 254);
	gfx.PutPixel(x + 107, y + 309, 254, 180, 254);
	gfx.PutPixel(x + 107, y + 310, 254, 82, 254);
	gfx.PutPixel(x + 107, y + 311, 254, 1, 254);
	gfx.PutPixel(x + 108, y + 129, 254, 99, 254);
	gfx.PutPixel(x + 108, y + 130, 254, 245, 254);
	gfx.PutPixel(x + 108, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 142, 254, 212, 254);
	gfx.PutPixel(x + 108, y + 143, 254, 57, 254);
	gfx.PutPixel(x + 108, y + 164, 254, 27, 254);
	gfx.PutPixel(x + 108, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 171, 254, 183, 254);
	gfx.PutPixel(x + 108, y + 287, 254, 44, 254);
	gfx.PutPixel(x + 108, y + 288, 254, 139, 254);
	gfx.PutPixel(x + 108, y + 289, 254, 225, 254);
	gfx.PutPixel(x + 108, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 311, 254, 213, 254);
	gfx.PutPixel(x + 108, y + 312, 254, 79, 254);
	gfx.PutPixel(x + 109, y + 127, 254, 10, 254);
	gfx.PutPixel(x + 109, y + 128, 254, 162, 254);
	gfx.PutPixel(x + 109, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 140, 254, 251, 254);
	gfx.PutPixel(x + 109, y + 141, 254, 130, 254);
	gfx.PutPixel(x + 109, y + 142, 254, 5, 254);
	gfx.PutPixel(x + 109, y + 164, 254, 13, 254);
	gfx.PutPixel(x + 109, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 171, 254, 214, 254);
	gfx.PutPixel(x + 109, y + 284, 254, 5, 254);
	gfx.PutPixel(x + 109, y + 285, 254, 91, 254);
	gfx.PutPixel(x + 109, y + 286, 254, 196, 254);
	gfx.PutPixel(x + 109, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 313, 254, 171, 254);
	gfx.PutPixel(x + 109, y + 314, 254, 18, 254);
	gfx.PutPixel(x + 110, y + 126, 254, 42, 254);
	gfx.PutPixel(x + 110, y + 127, 254, 213, 254);
	gfx.PutPixel(x + 110, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 139, 254, 216, 254);
	gfx.PutPixel(x + 110, y + 140, 254, 54, 254);
	gfx.PutPixel(x + 110, y + 164, 254, 5, 254);
	gfx.PutPixel(x + 110, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 171, 254, 229, 254);
	gfx.PutPixel(x + 110, y + 282, 254, 11, 254);
	gfx.PutPixel(x + 110, y + 283, 254, 114, 254);
	gfx.PutPixel(x + 110, y + 284, 254, 228, 254);
	gfx.PutPixel(x + 110, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 314, 254, 219, 254);
	gfx.PutPixel(x + 110, y + 315, 254, 40, 254);
	gfx.PutPixel(x + 111, y + 125, 254, 95, 254);
	gfx.PutPixel(x + 111, y + 126, 254, 244, 254);
	gfx.PutPixel(x + 111, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 138, 254, 152, 254);
	gfx.PutPixel(x + 111, y + 139, 254, 8, 254);
	gfx.PutPixel(x + 111, y + 164, 254, 1, 254);
	gfx.PutPixel(x + 111, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 171, 254, 240, 254);
	gfx.PutPixel(x + 111, y + 280, 254, 9, 254);
	gfx.PutPixel(x + 111, y + 281, 254, 121, 254);
	gfx.PutPixel(x + 111, y + 282, 254, 236, 254);
	gfx.PutPixel(x + 111, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 315, 254, 232, 254);
	gfx.PutPixel(x + 111, y + 316, 254, 37, 254);
	gfx.PutPixel(x + 112, y + 123, 254, 7, 254);
	gfx.PutPixel(x + 112, y + 124, 254, 157, 254);
	gfx.PutPixel(x + 112, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 136, 254, 239, 254);
	gfx.PutPixel(x + 112, y + 137, 254, 84, 254);
	gfx.PutPixel(x + 112, y + 164, 254, 2, 254);
	gfx.PutPixel(x + 112, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 171, 254, 251, 254);
	gfx.PutPixel(x + 112, y + 278, 254, 3, 254);
	gfx.PutPixel(x + 112, y + 279, 254, 104, 254);
	gfx.PutPixel(x + 112, y + 280, 254, 230, 254);
	gfx.PutPixel(x + 112, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 316, 254, 221, 254);
	gfx.PutPixel(x + 112, y + 317, 254, 15, 254);
	gfx.PutPixel(x + 113, y + 122, 254, 27, 254);
	gfx.PutPixel(x + 113, y + 123, 254, 201, 254);
	gfx.PutPixel(x + 113, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 135, 254, 201, 254);
	gfx.PutPixel(x + 113, y + 136, 254, 31, 254);
	gfx.PutPixel(x + 113, y + 164, 254, 9, 254);
	gfx.PutPixel(x + 113, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 171, 254, 250, 254);
	gfx.PutPixel(x + 113, y + 277, 254, 65, 254);
	gfx.PutPixel(x + 113, y + 278, 254, 212, 254);
	gfx.PutPixel(x + 113, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 317, 254, 168, 254);
	gfx.PutPixel(x + 114, y + 121, 254, 58, 254);
	gfx.PutPixel(x + 114, y + 122, 254, 231, 254);
	gfx.PutPixel(x + 114, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 134, 254, 151, 254);
	gfx.PutPixel(x + 114, y + 135, 254, 6, 254);
	gfx.PutPixel(x + 114, y + 164, 254, 22, 254);
	gfx.PutPixel(x + 114, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 171, 254, 243, 254);
	gfx.PutPixel(x + 114, y + 275, 254, 25, 254);
	gfx.PutPixel(x + 114, y + 276, 254, 169, 254);
	gfx.PutPixel(x + 114, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 294, 254, 251, 254);
	gfx.PutPixel(x + 114, y + 295, 254, 206, 254);
	gfx.PutPixel(x + 114, y + 296, 254, 148, 254);
	gfx.PutPixel(x + 114, y + 297, 254, 99, 254);
	gfx.PutPixel(x + 114, y + 298, 254, 63, 254);
	gfx.PutPixel(x + 114, y + 299, 254, 30, 254);
	gfx.PutPixel(x + 114, y + 300, 254, 14, 254);
	gfx.PutPixel(x + 114, y + 301, 254, 2, 254);
	gfx.PutPixel(x + 114, y + 302, 254, 7, 254);
	gfx.PutPixel(x + 114, y + 303, 254, 19, 254);
	gfx.PutPixel(x + 114, y + 304, 254, 33, 254);
	gfx.PutPixel(x + 114, y + 305, 254, 70, 254);
	gfx.PutPixel(x + 114, y + 306, 254, 113, 254);
	gfx.PutPixel(x + 114, y + 307, 254, 173, 254);
	gfx.PutPixel(x + 114, y + 308, 254, 244, 254);
	gfx.PutPixel(x + 114, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 318, 254, 69, 254);
	gfx.PutPixel(x + 115, y + 120, 254, 102, 254);
	gfx.PutPixel(x + 115, y + 121, 254, 249, 254);
	gfx.PutPixel(x + 115, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 132, 254, 245, 254);
	gfx.PutPixel(x + 115, y + 133, 254, 93, 254);
	gfx.PutPixel(x + 115, y + 164, 254, 44, 254);
	gfx.PutPixel(x + 115, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 171, 254, 228, 254);
	gfx.PutPixel(x + 115, y + 274, 254, 96, 254);
	gfx.PutPixel(x + 115, y + 275, 254, 238, 254);
	gfx.PutPixel(x + 115, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 291, 254, 242, 254);
	gfx.PutPixel(x + 115, y + 292, 254, 166, 254);
	gfx.PutPixel(x + 115, y + 293, 254, 88, 254);
	gfx.PutPixel(x + 115, y + 294, 254, 15, 254);
	gfx.PutPixel(x + 115, y + 308, 254, 9, 254);
	gfx.PutPixel(x + 115, y + 309, 254, 102, 254);
	gfx.PutPixel(x + 115, y + 310, 254, 229, 254);
	gfx.PutPixel(x + 115, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 318, 254, 206, 254);
	gfx.PutPixel(x + 116, y + 118, 254, 4, 254);
	gfx.PutPixel(x + 116, y + 119, 254, 151, 254);
	gfx.PutPixel(x + 116, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 131, 254, 226, 254);
	gfx.PutPixel(x + 116, y + 132, 254, 50, 254);
	gfx.PutPixel(x + 116, y + 164, 254, 69, 254);
	gfx.PutPixel(x + 116, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 171, 254, 207, 254);
	gfx.PutPixel(x + 116, y + 272, 254, 25, 254);
	gfx.PutPixel(x + 116, y + 273, 254, 181, 254);
	gfx.PutPixel(x + 116, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 289, 254, 190, 254);
	gfx.PutPixel(x + 116, y + 290, 254, 93, 254);
	gfx.PutPixel(x + 116, y + 291, 254, 10, 254);
	gfx.PutPixel(x + 116, y + 310, 254, 10, 254);
	gfx.PutPixel(x + 116, y + 311, 254, 180, 254);
	gfx.PutPixel(x + 116, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 319, 254, 65, 254);
	gfx.PutPixel(x + 117, y + 117, 254, 16, 254);
	gfx.PutPixel(x + 117, y + 118, 254, 191, 254);
	gfx.PutPixel(x + 117, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 130, 254, 196, 254);
	gfx.PutPixel(x + 117, y + 131, 254, 22, 254);
	gfx.PutPixel(x + 117, y + 164, 254, 104, 254);
	gfx.PutPixel(x + 117, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 171, 254, 183, 254);
	gfx.PutPixel(x + 117, y + 271, 254, 77, 254);
	gfx.PutPixel(x + 117, y + 272, 254, 235, 254);
	gfx.PutPixel(x + 117, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 286, 254, 247, 254);
	gfx.PutPixel(x + 117, y + 287, 254, 156, 254);
	gfx.PutPixel(x + 117, y + 288, 254, 44, 254);
	gfx.PutPixel(x + 117, y + 311, 254, 4, 254);
	gfx.PutPixel(x + 117, y + 312, 254, 190, 254);
	gfx.PutPixel(x + 117, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 319, 254, 162, 254);
	gfx.PutPixel(x + 118, y + 116, 254, 32, 254);
	gfx.PutPixel(x + 118, y + 117, 254, 214, 254);
	gfx.PutPixel(x + 118, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 129, 254, 158, 254);
	gfx.PutPixel(x + 118, y + 130, 254, 5, 254);
	gfx.PutPixel(x + 118, y + 164, 254, 140, 254);
	gfx.PutPixel(x + 118, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 171, 254, 150, 254);
	gfx.PutPixel(x + 118, y + 269, 254, 6, 254);
	gfx.PutPixel(x + 118, y + 270, 254, 146, 254);
	gfx.PutPixel(x + 118, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 284, 254, 247, 254);
	gfx.PutPixel(x + 118, y + 285, 254, 145, 254);
	gfx.PutPixel(x + 118, y + 286, 254, 24, 254);
	gfx.PutPixel(x + 118, y + 312, 254, 44, 254);
	gfx.PutPixel(x + 118, y + 313, 254, 252, 254);
	gfx.PutPixel(x + 118, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 319, 254, 239, 254);
	gfx.PutPixel(x + 118, y + 320, 254, 3, 254);
	gfx.PutPixel(x + 119, y + 115, 254, 53, 254);
	gfx.PutPixel(x + 119, y + 116, 254, 232, 254);
	gfx.PutPixel(x + 119, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 127, 254, 253, 254);
	gfx.PutPixel(x + 119, y + 128, 254, 120, 254);
	gfx.PutPixel(x + 119, y + 164, 254, 185, 254);
	gfx.PutPixel(x + 119, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 171, 254, 116, 254);
	gfx.PutPixel(x + 119, y + 268, 254, 29, 254);
	gfx.PutPixel(x + 119, y + 269, 254, 203, 254);
	gfx.PutPixel(x + 119, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 282, 254, 252, 254);
	gfx.PutPixel(x + 119, y + 283, 254, 156, 254);
	gfx.PutPixel(x + 119, y + 284, 254, 26, 254);
	gfx.PutPixel(x + 119, y + 313, 254, 178, 254);
	gfx.PutPixel(x + 119, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 320, 254, 45, 254);
	gfx.PutPixel(x + 120, y + 114, 254, 80, 254);
	gfx.PutPixel(x + 120, y + 115, 254, 245, 254);
	gfx.PutPixel(x + 120, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 126, 254, 246, 254);
	gfx.PutPixel(x + 120, y + 127, 254, 85, 254);
	gfx.PutPixel(x + 120, y + 164, 254, 232, 254);
	gfx.PutPixel(x + 120, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 171, 254, 75, 254);
	gfx.PutPixel(x + 120, y + 267, 254, 66, 254);
	gfx.PutPixel(x + 120, y + 268, 254, 234, 254);
	gfx.PutPixel(x + 120, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 281, 254, 191, 254);
	gfx.PutPixel(x + 120, y + 282, 254, 48, 254);
	gfx.PutPixel(x + 120, y + 313, 254, 107, 254);
	gfx.PutPixel(x + 120, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 320, 254, 96, 254);
	gfx.PutPixel(x + 121, y + 113, 254, 112, 254);
	gfx.PutPixel(x + 121, y + 114, 254, 252, 254);
	gfx.PutPixel(x + 121, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 125, 254, 236, 254);
	gfx.PutPixel(x + 121, y + 126, 254, 56, 254);
	gfx.PutPixel(x + 121, y + 163, 254, 30, 254);
	gfx.PutPixel(x + 121, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 171, 254, 29, 254);
	gfx.PutPixel(x + 121, y + 266, 254, 113, 254);
	gfx.PutPixel(x + 121, y + 267, 254, 251, 254);
	gfx.PutPixel(x + 121, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 279, 254, 234, 254);
	gfx.PutPixel(x + 121, y + 280, 254, 93, 254);
	gfx.PutPixel(x + 121, y + 313, 254, 48, 254);
	gfx.PutPixel(x + 121, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 320, 254, 147, 254);
	gfx.PutPixel(x + 122, y + 112, 254, 137, 254);
	gfx.PutPixel(x + 122, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 124, 254, 225, 254);
	gfx.PutPixel(x + 122, y + 125, 254, 40, 254);
	gfx.PutPixel(x + 122, y + 163, 254, 92, 254);
	gfx.PutPixel(x + 122, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 170, 254, 236, 254);
	gfx.PutPixel(x + 122, y + 264, 254, 3, 254);
	gfx.PutPixel(x + 122, y + 265, 254, 151, 254);
	gfx.PutPixel(x + 122, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 278, 254, 163, 254);
	gfx.PutPixel(x + 122, y + 279, 254, 19, 254);
	gfx.PutPixel(x + 122, y + 313, 254, 26, 254);
	gfx.PutPixel(x + 122, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 320, 254, 191, 254);
	gfx.PutPixel(x + 123, y + 110, 254, 2, 254);
	gfx.PutPixel(x + 123, y + 111, 254, 155, 254);
	gfx.PutPixel(x + 123, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 123, 254, 211, 254);
	gfx.PutPixel(x + 123, y + 124, 254, 26, 254);
	gfx.PutPixel(x + 123, y + 163, 254, 155, 254);
	gfx.PutPixel(x + 123, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 170, 254, 179, 254);
	gfx.PutPixel(x + 123, y + 263, 254, 13, 254);
	gfx.PutPixel(x + 123, y + 264, 254, 184, 254);
	gfx.PutPixel(x + 123, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 276, 254, 236, 254);
	gfx.PutPixel(x + 123, y + 277, 254, 84, 254);
	gfx.PutPixel(x + 123, y + 313, 254, 7, 254);
	gfx.PutPixel(x + 123, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 320, 254, 210, 254);
	gfx.PutPixel(x + 124, y + 109, 254, 6, 254);
	gfx.PutPixel(x + 124, y + 110, 254, 172, 254);
	gfx.PutPixel(x + 124, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 122, 254, 194, 254);
	gfx.PutPixel(x + 124, y + 123, 254, 16, 254);
	gfx.PutPixel(x + 124, y + 163, 254, 218, 254);
	gfx.PutPixel(x + 124, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 170, 254, 120, 254);
	gfx.PutPixel(x + 124, y + 262, 254, 25, 254);
	gfx.PutPixel(x + 124, y + 263, 254, 209, 254);
	gfx.PutPixel(x + 124, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 275, 254, 187, 254);
	gfx.PutPixel(x + 124, y + 276, 254, 25, 254);
	gfx.PutPixel(x + 124, y + 313, 254, 6, 254);
	gfx.PutPixel(x + 124, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 320, 254, 227, 254);
	gfx.PutPixel(x + 125, y + 108, 254, 11, 254);
	gfx.PutPixel(x + 125, y + 109, 254, 187, 254);
	gfx.PutPixel(x + 125, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 121, 254, 175, 254);
	gfx.PutPixel(x + 125, y + 122, 254, 7, 254);
	gfx.PutPixel(x + 125, y + 162, 254, 25, 254);
	gfx.PutPixel(x + 125, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 170, 254, 61, 254);
	gfx.PutPixel(x + 125, y + 261, 254, 36, 254);
	gfx.PutPixel(x + 125, y + 262, 254, 222, 254);
	gfx.PutPixel(x + 125, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 273, 254, 253, 254);
	gfx.PutPixel(x + 125, y + 274, 254, 129, 254);
	gfx.PutPixel(x + 125, y + 275, 254, 1, 254);
	gfx.PutPixel(x + 125, y + 313, 254, 20, 254);
	gfx.PutPixel(x + 125, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 320, 254, 243, 254);
	gfx.PutPixel(x + 126, y + 107, 254, 17, 254);
	gfx.PutPixel(x + 126, y + 108, 254, 200, 254);
	gfx.PutPixel(x + 126, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 120, 254, 153, 254);
	gfx.PutPixel(x + 126, y + 121, 254, 2, 254);
	gfx.PutPixel(x + 126, y + 162, 254, 93, 254);
	gfx.PutPixel(x + 126, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 169, 254, 248, 254);
	gfx.PutPixel(x + 126, y + 170, 254, 7, 254);
	gfx.PutPixel(x + 126, y + 260, 254, 48, 254);
	gfx.PutPixel(x + 126, y + 261, 254, 232, 254);
	gfx.PutPixel(x + 126, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 272, 254, 236, 254);
	gfx.PutPixel(x + 126, y + 273, 254, 70, 254);
	gfx.PutPixel(x + 126, y + 313, 254, 35, 254);
	gfx.PutPixel(x + 126, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 320, 254, 251, 254);
	gfx.PutPixel(x + 127, y + 106, 254, 20, 254);
	gfx.PutPixel(x + 127, y + 107, 254, 210, 254);
	gfx.PutPixel(x + 127, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 119, 254, 138, 254);
	gfx.PutPixel(x + 127, y + 162, 254, 171, 254);
	gfx.PutPixel(x + 127, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 169, 254, 198, 254);
	gfx.PutPixel(x + 127, y + 259, 254, 59, 254);
	gfx.PutPixel(x + 127, y + 260, 254, 240, 254);
	gfx.PutPixel(x + 127, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 271, 254, 214, 254);
	gfx.PutPixel(x + 127, y + 272, 254, 35, 254);
	gfx.PutPixel(x + 127, y + 313, 254, 68, 254);
	gfx.PutPixel(x + 127, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 320, 254, 241, 254);
	gfx.PutPixel(x + 128, y + 105, 254, 21, 254);
	gfx.PutPixel(x + 128, y + 106, 254, 211, 254);
	gfx.PutPixel(x + 128, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 118, 254, 129, 254);
	gfx.PutPixel(x + 128, y + 161, 254, 6, 254);
	gfx.PutPixel(x + 128, y + 162, 254, 242, 254);
	gfx.PutPixel(x + 128, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 169, 254, 128, 254);
	gfx.PutPixel(x + 128, y + 258, 254, 62, 254);
	gfx.PutPixel(x + 128, y + 259, 254, 243, 254);
	gfx.PutPixel(x + 128, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 270, 254, 182, 254);
	gfx.PutPixel(x + 128, y + 271, 254, 14, 254);
	gfx.PutPixel(x + 128, y + 313, 254, 110, 254);
	gfx.PutPixel(x + 128, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 320, 254, 229, 254);
	gfx.PutPixel(x + 129, y + 104, 254, 23, 254);
	gfx.PutPixel(x + 129, y + 105, 254, 213, 254);
	gfx.PutPixel(x + 129, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 117, 254, 120, 254);
	gfx.PutPixel(x + 129, y + 161, 254, 73, 254);
	gfx.PutPixel(x + 129, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 169, 254, 52, 254);
	gfx.PutPixel(x + 129, y + 257, 254, 66, 254);
	gfx.PutPixel(x + 129, y + 258, 254, 244, 254);
	gfx.PutPixel(x + 129, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 269, 254, 156, 254);
	gfx.PutPixel(x + 129, y + 270, 254, 3, 254);
	gfx.PutPixel(x + 129, y + 313, 254, 152, 254);
	gfx.PutPixel(x + 129, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 320, 254, 218, 254);
	gfx.PutPixel(x + 130, y + 103, 254, 24, 254);
	gfx.PutPixel(x + 130, y + 104, 254, 215, 254);
	gfx.PutPixel(x + 130, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 116, 254, 111, 254);
	gfx.PutPixel(x + 130, y + 161, 254, 152, 254);
	gfx.PutPixel(x + 130, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 168, 254, 230, 254);
	gfx.PutPixel(x + 130, y + 169, 254, 0, 254);
	gfx.PutPixel(x + 130, y + 256, 254, 68, 254);
	gfx.PutPixel(x + 130, y + 257, 254, 245, 254);
	gfx.PutPixel(x + 130, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 268, 254, 131, 254);
	gfx.PutPixel(x + 130, y + 313, 254, 210, 254);
	gfx.PutPixel(x + 130, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 320, 254, 197, 254);
	gfx.PutPixel(x + 131, y + 102, 254, 25, 254);
	gfx.PutPixel(x + 131, y + 103, 254, 216, 254);
	gfx.PutPixel(x + 131, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 114, 254, 253, 254);
	gfx.PutPixel(x + 131, y + 115, 254, 103, 254);
	gfx.PutPixel(x + 131, y + 160, 254, 2, 254);
	gfx.PutPixel(x + 131, y + 161, 254, 230, 254);
	gfx.PutPixel(x + 131, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 168, 254, 157, 254);
	gfx.PutPixel(x + 131, y + 255, 254, 61, 254);
	gfx.PutPixel(x + 131, y + 256, 254, 244, 254);
	gfx.PutPixel(x + 131, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 267, 254, 110, 254);
	gfx.PutPixel(x + 131, y + 312, 254, 21, 254);
	gfx.PutPixel(x + 131, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 320, 254, 164, 254);
	gfx.PutPixel(x + 132, y + 101, 254, 24, 254);
	gfx.PutPixel(x + 132, y + 102, 254, 217, 254);
	gfx.PutPixel(x + 132, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 113, 254, 252, 254);
	gfx.PutPixel(x + 132, y + 114, 254, 94, 254);
	gfx.PutPixel(x + 132, y + 160, 254, 72, 254);
	gfx.PutPixel(x + 132, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 168, 254, 81, 254);
	gfx.PutPixel(x + 132, y + 254, 254, 54, 254);
	gfx.PutPixel(x + 132, y + 255, 254, 242, 254);
	gfx.PutPixel(x + 132, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 265, 254, 253, 254);
	gfx.PutPixel(x + 132, y + 266, 254, 104, 254);
	gfx.PutPixel(x + 132, y + 312, 254, 87, 254);
	gfx.PutPixel(x + 132, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 132, y + 320, 254, 130, 254);
	gfx.PutPixel(x + 133, y + 100, 254, 18, 254);
	gfx.PutPixel(x + 133, y + 101, 254, 211, 254);
	gfx.PutPixel(x + 133, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 112, 254, 252, 254);
	gfx.PutPixel(x + 133, y + 113, 254, 92, 254);
	gfx.PutPixel(x + 133, y + 160, 254, 166, 254);
	gfx.PutPixel(x + 133, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 167, 254, 245, 254);
	gfx.PutPixel(x + 133, y + 168, 254, 11, 254);
	gfx.PutPixel(x + 133, y + 253, 254, 48, 254);
	gfx.PutPixel(x + 133, y + 254, 254, 238, 254);
	gfx.PutPixel(x + 133, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 264, 254, 253, 254);
	gfx.PutPixel(x + 133, y + 265, 254, 99, 254);
	gfx.PutPixel(x + 133, y + 312, 254, 164, 254);
	gfx.PutPixel(x + 133, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 133, y + 320, 254, 95, 254);
	gfx.PutPixel(x + 134, y + 99, 254, 13, 254);
	gfx.PutPixel(x + 134, y + 100, 254, 203, 254);
	gfx.PutPixel(x + 134, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 111, 254, 253, 254);
	gfx.PutPixel(x + 134, y + 112, 254, 100, 254);
	gfx.PutPixel(x + 134, y + 159, 254, 13, 254);
	gfx.PutPixel(x + 134, y + 160, 254, 245, 254);
	gfx.PutPixel(x + 134, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 167, 254, 168, 254);
	gfx.PutPixel(x + 134, y + 252, 254, 32, 254);
	gfx.PutPixel(x + 134, y + 253, 254, 231, 254);
	gfx.PutPixel(x + 134, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 264, 254, 104, 254);
	gfx.PutPixel(x + 134, y + 311, 254, 9, 254);
	gfx.PutPixel(x + 134, y + 312, 254, 243, 254);
	gfx.PutPixel(x + 134, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 134, y + 320, 254, 52, 254);
	gfx.PutPixel(x + 135, y + 98, 254, 9, 254);
	gfx.PutPixel(x + 135, y + 99, 254, 194, 254);
	gfx.PutPixel(x + 135, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 111, 254, 107, 254);
	gfx.PutPixel(x + 135, y + 159, 254, 98, 254);
	gfx.PutPixel(x + 135, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 167, 254, 77, 254);
	gfx.PutPixel(x + 135, y + 251, 254, 15, 254);
	gfx.PutPixel(x + 135, y + 252, 254, 213, 254);
	gfx.PutPixel(x + 135, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 263, 254, 117, 254);
	gfx.PutPixel(x + 135, y + 311, 254, 87, 254);
	gfx.PutPixel(x + 135, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 135, y + 319, 254, 246, 254);
	gfx.PutPixel(x + 135, y + 320, 254, 4, 254);
	gfx.PutPixel(x + 136, y + 97, 254, 6, 254);
	gfx.PutPixel(x + 136, y + 98, 254, 184, 254);
	gfx.PutPixel(x + 136, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 110, 254, 115, 254);
	gfx.PutPixel(x + 136, y + 159, 254, 192, 254);
	gfx.PutPixel(x + 136, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 166, 254, 236, 254);
	gfx.PutPixel(x + 136, y + 167, 254, 4, 254);
	gfx.PutPixel(x + 136, y + 250, 254, 4, 254);
	gfx.PutPixel(x + 136, y + 251, 254, 189, 254);
	gfx.PutPixel(x + 136, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 262, 254, 134, 254);
	gfx.PutPixel(x + 136, y + 311, 254, 180, 254);
	gfx.PutPixel(x + 136, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 136, y + 319, 254, 195, 254);
	gfx.PutPixel(x + 137, y + 96, 254, 2, 254);
	gfx.PutPixel(x + 137, y + 97, 254, 174, 254);
	gfx.PutPixel(x + 137, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 109, 254, 123, 254);
	gfx.PutPixel(x + 137, y + 158, 254, 42, 254);
	gfx.PutPixel(x + 137, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 166, 254, 151, 254);
	gfx.PutPixel(x + 137, y + 250, 254, 160, 254);
	gfx.PutPixel(x + 137, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 261, 254, 162, 254);
	gfx.PutPixel(x + 137, y + 310, 254, 32, 254);
	gfx.PutPixel(x + 137, y + 311, 254, 252, 254);
	gfx.PutPixel(x + 137, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 137, y + 319, 254, 139, 254);
	gfx.PutPixel(x + 138, y + 96, 254, 153, 254);
	gfx.PutPixel(x + 138, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 108, 254, 131, 254);
	gfx.PutPixel(x + 138, y + 158, 254, 148, 254);
	gfx.PutPixel(x + 138, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 166, 254, 59, 254);
	gfx.PutPixel(x + 138, y + 249, 254, 126, 254);
	gfx.PutPixel(x + 138, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 260, 254, 188, 254);
	gfx.PutPixel(x + 138, y + 261, 254, 4, 254);
	gfx.PutPixel(x + 138, y + 310, 254, 138, 254);
	gfx.PutPixel(x + 138, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 319, 254, 73, 254);
	gfx.PutPixel(x + 139, y + 95, 254, 125, 254);
	gfx.PutPixel(x + 139, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 107, 254, 143, 254);
	gfx.PutPixel(x + 139, y + 157, 254, 13, 254);
	gfx.PutPixel(x + 139, y + 158, 254, 242, 254);
	gfx.PutPixel(x + 139, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 165, 254, 216, 254);
	gfx.PutPixel(x + 139, y + 166, 254, 0, 254);
	gfx.PutPixel(x + 139, y + 248, 254, 92, 254);
	gfx.PutPixel(x + 139, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 259, 254, 214, 254);
	gfx.PutPixel(x + 139, y + 260, 254, 14, 254);
	gfx.PutPixel(x + 139, y + 309, 254, 8, 254);
	gfx.PutPixel(x + 139, y + 310, 254, 236, 254);
	gfx.PutPixel(x + 139, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 318, 254, 243, 254);
	gfx.PutPixel(x + 139, y + 319, 254, 7, 254);
	gfx.PutPixel(x + 140, y + 94, 254, 96, 254);
	gfx.PutPixel(x + 140, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 106, 254, 167, 254);
	gfx.PutPixel(x + 140, y + 107, 254, 0, 254);
	gfx.PutPixel(x + 140, y + 157, 254, 107, 254);
	gfx.PutPixel(x + 140, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 165, 254, 111, 254);
	gfx.PutPixel(x + 140, y + 247, 254, 46, 254);
	gfx.PutPixel(x + 140, y + 248, 254, 246, 254);
	gfx.PutPixel(x + 140, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 258, 254, 238, 254);
	gfx.PutPixel(x + 140, y + 259, 254, 36, 254);
	gfx.PutPixel(x + 140, y + 309, 254, 109, 254);
	gfx.PutPixel(x + 140, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 318, 254, 173, 254);
	gfx.PutPixel(x + 141, y + 93, 254, 71, 254);
	gfx.PutPixel(x + 141, y + 94, 254, 250, 254);
	gfx.PutPixel(x + 141, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 105, 254, 187, 254);
	gfx.PutPixel(x + 141, y + 106, 254, 5, 254);
	gfx.PutPixel(x + 141, y + 157, 254, 214, 254);
	gfx.PutPixel(x + 141, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 164, 254, 244, 254);
	gfx.PutPixel(x + 141, y + 165, 254, 15, 254);
	gfx.PutPixel(x + 141, y + 246, 254, 11, 254);
	gfx.PutPixel(x + 141, y + 247, 254, 216, 254);
	gfx.PutPixel(x + 141, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 257, 254, 253, 254);
	gfx.PutPixel(x + 141, y + 258, 254, 73, 254);
	gfx.PutPixel(x + 141, y + 308, 254, 5, 254);
	gfx.PutPixel(x + 141, y + 309, 254, 227, 254);
	gfx.PutPixel(x + 141, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 318, 254, 95, 254);
	gfx.PutPixel(x + 142, y + 92, 254, 50, 254);
	gfx.PutPixel(x + 142, y + 93, 254, 242, 254);
	gfx.PutPixel(x + 142, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 104, 254, 205, 254);
	gfx.PutPixel(x + 142, y + 105, 254, 12, 254);
	gfx.PutPixel(x + 142, y + 140, 254, 27, 254);
	gfx.PutPixel(x + 142, y + 141, 254, 135, 254);
	gfx.PutPixel(x + 142, y + 142, 254, 189, 254);
	gfx.PutPixel(x + 142, y + 143, 254, 169, 254);
	gfx.PutPixel(x + 142, y + 144, 254, 91, 254);
	gfx.PutPixel(x + 142, y + 145, 254, 0, 254);
	gfx.PutPixel(x + 142, y + 156, 254, 72, 254);
	gfx.PutPixel(x + 142, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 164, 254, 154, 254);
	gfx.PutPixel(x + 142, y + 246, 254, 164, 254);
	gfx.PutPixel(x + 142, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 257, 254, 134, 254);
	gfx.PutPixel(x + 142, y + 308, 254, 102, 254);
	gfx.PutPixel(x + 142, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 317, 254, 247, 254);
	gfx.PutPixel(x + 142, y + 318, 254, 16, 254);
	gfx.PutPixel(x + 143, y + 91, 254, 27, 254);
	gfx.PutPixel(x + 143, y + 92, 254, 230, 254);
	gfx.PutPixel(x + 143, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 103, 254, 220, 254);
	gfx.PutPixel(x + 143, y + 104, 254, 22, 254);
	gfx.PutPixel(x + 143, y + 139, 254, 110, 254);
	gfx.PutPixel(x + 143, y + 140, 254, 244, 254);
	gfx.PutPixel(x + 143, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 145, 254, 119, 254);
	gfx.PutPixel(x + 143, y + 156, 254, 194, 254);
	gfx.PutPixel(x + 143, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 164, 254, 47, 254);
	gfx.PutPixel(x + 143, y + 245, 254, 100, 254);
	gfx.PutPixel(x + 143, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 256, 254, 220, 254);
	gfx.PutPixel(x + 143, y + 257, 254, 5, 254);
	gfx.PutPixel(x + 143, y + 307, 254, 6, 254);
	gfx.PutPixel(x + 143, y + 308, 254, 224, 254);
	gfx.PutPixel(x + 143, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 317, 254, 167, 254);
	gfx.PutPixel(x + 144, y + 90, 254, 9, 254);
	gfx.PutPixel(x + 144, y + 91, 254, 204, 254);
	gfx.PutPixel(x + 144, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 102, 254, 233, 254);
	gfx.PutPixel(x + 144, y + 103, 254, 35, 254);
	gfx.PutPixel(x + 144, y + 137, 254, 8, 254);
	gfx.PutPixel(x + 144, y + 138, 254, 170, 254);
	gfx.PutPixel(x + 144, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 145, 254, 220, 254);
	gfx.PutPixel(x + 144, y + 155, 254, 59, 254);
	gfx.PutPixel(x + 144, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 163, 254, 197, 254);
	gfx.PutPixel(x + 144, y + 244, 254, 45, 254);
	gfx.PutPixel(x + 144, y + 245, 254, 246, 254);
	gfx.PutPixel(x + 144, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 256, 254, 123, 254);
	gfx.PutPixel(x + 144, y + 307, 254, 116, 254);
	gfx.PutPixel(x + 144, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 317, 254, 70, 254);
	gfx.PutPixel(x + 145, y + 89, 254, 0, 254);
	gfx.PutPixel(x + 145, y + 90, 254, 170, 254);
	gfx.PutPixel(x + 145, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 101, 254, 243, 254);
	gfx.PutPixel(x + 145, y + 102, 254, 51, 254);
	gfx.PutPixel(x + 145, y + 136, 254, 17, 254);
	gfx.PutPixel(x + 145, y + 137, 254, 199, 254);
	gfx.PutPixel(x + 145, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 145, 254, 248, 254);
	gfx.PutPixel(x + 145, y + 155, 254, 179, 254);
	gfx.PutPixel(x + 145, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 163, 254, 81, 254);
	gfx.PutPixel(x + 145, y + 243, 254, 11, 254);
	gfx.PutPixel(x + 145, y + 244, 254, 215, 254);
	gfx.PutPixel(x + 145, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 256, 254, 91, 254);
	gfx.PutPixel(x + 145, y + 306, 254, 18, 254);
	gfx.PutPixel(x + 145, y + 307, 254, 238, 254);
	gfx.PutPixel(x + 145, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 316, 254, 225, 254);
	gfx.PutPixel(x + 145, y + 317, 254, 1, 254);
	gfx.PutPixel(x + 146, y + 89, 254, 126, 254);
	gfx.PutPixel(x + 146, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 100, 254, 252, 254);
	gfx.PutPixel(x + 146, y + 101, 254, 77, 254);
	gfx.PutPixel(x + 146, y + 135, 254, 21, 254);
	gfx.PutPixel(x + 146, y + 136, 254, 211, 254);
	gfx.PutPixel(x + 146, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 145, 254, 230, 254);
	gfx.PutPixel(x + 146, y + 154, 254, 45, 254);
	gfx.PutPixel(x + 146, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 162, 254, 213, 254);
	gfx.PutPixel(x + 146, y + 163, 254, 0, 254);
	gfx.PutPixel(x + 146, y + 243, 254, 153, 254);
	gfx.PutPixel(x + 146, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 256, 254, 122, 254);
	gfx.PutPixel(x + 146, y + 306, 254, 143, 254);
	gfx.PutPixel(x + 146, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 316, 254, 124, 254);
	gfx.PutPixel(x + 147, y + 88, 254, 83, 254);
	gfx.PutPixel(x + 147, y + 89, 254, 253, 254);
	gfx.PutPixel(x + 147, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 100, 254, 112, 254);
	gfx.PutPixel(x + 147, y + 134, 254, 19, 254);
	gfx.PutPixel(x + 147, y + 135, 254, 211, 254);
	gfx.PutPixel(x + 147, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 145, 254, 203, 254);
	gfx.PutPixel(x + 147, y + 154, 254, 170, 254);
	gfx.PutPixel(x + 147, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 162, 254, 94, 254);
	gfx.PutPixel(x + 147, y + 242, 254, 70, 254);
	gfx.PutPixel(x + 147, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 256, 254, 119, 254);
	gfx.PutPixel(x + 147, y + 305, 254, 45, 254);
	gfx.PutPixel(x + 147, y + 306, 254, 250, 254);
	gfx.PutPixel(x + 147, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 315, 254, 244, 254);
	gfx.PutPixel(x + 147, y + 316, 254, 18, 254);
	gfx.PutPixel(x + 148, y + 87, 254, 46, 254);
	gfx.PutPixel(x + 148, y + 88, 254, 244, 254);
	gfx.PutPixel(x + 148, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 99, 254, 150, 254);
	gfx.PutPixel(x + 148, y + 133, 254, 12, 254);
	gfx.PutPixel(x + 148, y + 134, 254, 202, 254);
	gfx.PutPixel(x + 148, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 145, 254, 148, 254);
	gfx.PutPixel(x + 148, y + 153, 254, 51, 254);
	gfx.PutPixel(x + 148, y + 154, 254, 253, 254);
	gfx.PutPixel(x + 148, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 161, 254, 223, 254);
	gfx.PutPixel(x + 148, y + 162, 254, 3, 254);
	gfx.PutPixel(x + 148, y + 241, 254, 14, 254);
	gfx.PutPixel(x + 148, y + 242, 254, 226, 254);
	gfx.PutPixel(x + 148, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 255, 254, 253, 254);
	gfx.PutPixel(x + 148, y + 256, 254, 42, 254);
	gfx.PutPixel(x + 148, y + 304, 254, 0, 254);
	gfx.PutPixel(x + 148, y + 305, 254, 197, 254);
	gfx.PutPixel(x + 148, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 315, 254, 149, 254);
	gfx.PutPixel(x + 149, y + 86, 254, 15, 254);
	gfx.PutPixel(x + 149, y + 87, 254, 220, 254);
	gfx.PutPixel(x + 149, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 98, 254, 189, 254);
	gfx.PutPixel(x + 149, y + 99, 254, 3, 254);
	gfx.PutPixel(x + 149, y + 132, 254, 2, 254);
	gfx.PutPixel(x + 149, y + 133, 254, 184, 254);
	gfx.PutPixel(x + 149, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 145, 254, 86, 254);
	gfx.PutPixel(x + 149, y + 153, 254, 186, 254);
	gfx.PutPixel(x + 149, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 161, 254, 106, 254);
	gfx.PutPixel(x + 149, y + 241, 254, 154, 254);
	gfx.PutPixel(x + 149, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 255, 254, 159, 254);
	gfx.PutPixel(x + 149, y + 304, 254, 100, 254);
	gfx.PutPixel(x + 149, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 314, 254, 252, 254);
	gfx.PutPixel(x + 149, y + 315, 254, 36, 254);
	gfx.PutPixel(x + 150, y + 85, 254, 0, 254);
	gfx.PutPixel(x + 150, y + 86, 254, 178, 254);
	gfx.PutPixel(x + 150, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 97, 254, 221, 254);
	gfx.PutPixel(x + 150, y + 98, 254, 18, 254);
	gfx.PutPixel(x + 150, y + 132, 254, 148, 254);
	gfx.PutPixel(x + 150, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 144, 254, 249, 254);
	gfx.PutPixel(x + 150, y + 145, 254, 18, 254);
	gfx.PutPixel(x + 150, y + 152, 254, 66, 254);
	gfx.PutPixel(x + 150, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 160, 254, 230, 254);
	gfx.PutPixel(x + 150, y + 161, 254, 7, 254);
	gfx.PutPixel(x + 150, y + 240, 254, 62, 254);
	gfx.PutPixel(x + 150, y + 241, 254, 253, 254);
	gfx.PutPixel(x + 150, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 254, 254, 213, 254);
	gfx.PutPixel(x + 150, y + 255, 254, 14, 254);
	gfx.PutPixel(x + 150, y + 303, 254, 24, 254);
	gfx.PutPixel(x + 150, y + 304, 254, 237, 254);
	gfx.PutPixel(x + 150, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 314, 254, 168, 254);
	gfx.PutPixel(x + 151, y + 85, 254, 118, 254);
	gfx.PutPixel(x + 151, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 96, 254, 242, 254);
	gfx.PutPixel(x + 151, y + 97, 254, 43, 254);
	gfx.PutPixel(x + 151, y + 131, 254, 109, 254);
	gfx.PutPixel(x + 151, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 144, 254, 178, 254);
	gfx.PutPixel(x + 151, y + 152, 254, 201, 254);
	gfx.PutPixel(x + 151, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 160, 254, 104, 254);
	gfx.PutPixel(x + 151, y + 239, 254, 7, 254);
	gfx.PutPixel(x + 151, y + 240, 254, 216, 254);
	gfx.PutPixel(x + 151, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 253, 254, 232, 254);
	gfx.PutPixel(x + 151, y + 254, 254, 32, 254);
	gfx.PutPixel(x + 151, y + 303, 254, 179, 254);
	gfx.PutPixel(x + 151, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 313, 254, 251, 254);
	gfx.PutPixel(x + 151, y + 314, 254, 38, 254);
	gfx.PutPixel(x + 152, y + 84, 254, 56, 254);
	gfx.PutPixel(x + 152, y + 85, 254, 250, 254);
	gfx.PutPixel(x + 152, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 96, 254, 84, 254);
	gfx.PutPixel(x + 152, y + 130, 254, 68, 254);
	gfx.PutPixel(x + 152, y + 131, 254, 251, 254);
	gfx.PutPixel(x + 152, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 144, 254, 87, 254);
	gfx.PutPixel(x + 152, y + 151, 254, 86, 254);
	gfx.PutPixel(x + 152, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 159, 254, 217, 254);
	gfx.PutPixel(x + 152, y + 160, 254, 3, 254);
	gfx.PutPixel(x + 152, y + 239, 254, 126, 254);
	gfx.PutPixel(x + 152, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 252, 254, 231, 254);
	gfx.PutPixel(x + 152, y + 253, 254, 46, 254);
	gfx.PutPixel(x + 152, y + 302, 254, 97, 254);
	gfx.PutPixel(x + 152, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 313, 254, 158, 254);
	gfx.PutPixel(x + 153, y + 83, 254, 15, 254);
	gfx.PutPixel(x + 153, y + 84, 254, 223, 254);
	gfx.PutPixel(x + 153, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 95, 254, 141, 254);
	gfx.PutPixel(x + 153, y + 129, 254, 26, 254);
	gfx.PutPixel(x + 153, y + 130, 254, 233, 254);
	gfx.PutPixel(x + 153, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 143, 254, 231, 254);
	gfx.PutPixel(x + 153, y + 144, 254, 6, 254);
	gfx.PutPixel(x + 153, y + 150, 254, 10, 254);
	gfx.PutPixel(x + 153, y + 151, 254, 226, 254);
	gfx.PutPixel(x + 153, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 159, 254, 84, 254);
	gfx.PutPixel(x + 153, y + 238, 254, 32, 254);
	gfx.PutPixel(x + 153, y + 239, 254, 246, 254);
	gfx.PutPixel(x + 153, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 251, 254, 222, 254);
	gfx.PutPixel(x + 153, y + 252, 254, 35, 254);
	gfx.PutPixel(x + 153, y + 301, 254, 30, 254);
	gfx.PutPixel(x + 153, y + 302, 254, 240, 254);
	gfx.PutPixel(x + 153, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 312, 254, 248, 254);
	gfx.PutPixel(x + 153, y + 313, 254, 31, 254);
	gfx.PutPixel(x + 154, y + 83, 254, 163, 254);
	gfx.PutPixel(x + 154, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 93, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 94, 254, 192, 254);
	gfx.PutPixel(x + 154, y + 95, 254, 3, 254);
	gfx.PutPixel(x + 154, y + 128, 254, 3, 254);
	gfx.PutPixel(x + 154, y + 129, 254, 194, 254);
	gfx.PutPixel(x + 154, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 143, 254, 127, 254);
	gfx.PutPixel(x + 154, y + 150, 254, 135, 254);
	gfx.PutPixel(x + 154, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 158, 254, 200, 254);
	gfx.PutPixel(x + 154, y + 238, 254, 176, 254);
	gfx.PutPixel(x + 154, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 250, 254, 212, 254);
	gfx.PutPixel(x + 154, y + 251, 254, 24, 254);
	gfx.PutPixel(x + 154, y + 300, 254, 3, 254);
	gfx.PutPixel(x + 154, y + 301, 254, 196, 254);
	gfx.PutPixel(x + 154, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 312, 254, 139, 254);
	gfx.PutPixel(x + 155, y + 82, 254, 84, 254);
	gfx.PutPixel(x + 155, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 92, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 93, 254, 233, 254);
	gfx.PutPixel(x + 155, y + 94, 254, 24, 254);
	gfx.PutPixel(x + 155, y + 128, 254, 135, 254);
	gfx.PutPixel(x + 155, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 142, 254, 247, 254);
	gfx.PutPixel(x + 155, y + 143, 254, 22, 254);
	gfx.PutPixel(x + 155, y + 149, 254, 37, 254);
	gfx.PutPixel(x + 155, y + 150, 254, 248, 254);
	gfx.PutPixel(x + 155, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 158, 254, 63, 254);
	gfx.PutPixel(x + 155, y + 237, 254, 68, 254);
	gfx.PutPixel(x + 155, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 249, 254, 225, 254);
	gfx.PutPixel(x + 155, y + 250, 254, 26, 254);
	gfx.PutPixel(x + 155, y + 300, 254, 133, 254);
	gfx.PutPixel(x + 155, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 311, 254, 230, 254);
	gfx.PutPixel(x + 155, y + 312, 254, 12, 254);
	gfx.PutPixel(x + 156, y + 81, 254, 21, 254);
	gfx.PutPixel(x + 156, y + 82, 254, 236, 254);
	gfx.PutPixel(x + 156, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 92, 254, 253, 254);
	gfx.PutPixel(x + 156, y + 93, 254, 70, 254);
	gfx.PutPixel(x + 156, y + 127, 254, 66, 254);
	gfx.PutPixel(x + 156, y + 128, 254, 253, 254);
	gfx.PutPixel(x + 156, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 142, 254, 146, 254);
	gfx.PutPixel(x + 156, y + 149, 254, 183, 254);
	gfx.PutPixel(x + 156, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 157, 254, 175, 254);
	gfx.PutPixel(x + 156, y + 236, 254, 1, 254);
	gfx.PutPixel(x + 156, y + 237, 254, 208, 254);
	gfx.PutPixel(x + 156, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 248, 254, 249, 254);
	gfx.PutPixel(x + 156, y + 249, 254, 55, 254);
	gfx.PutPixel(x + 156, y + 299, 254, 68, 254);
	gfx.PutPixel(x + 156, y + 300, 254, 253, 254);
	gfx.PutPixel(x + 156, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 311, 254, 92, 254);
	gfx.PutPixel(x + 157, y + 81, 254, 169, 254);
	gfx.PutPixel(x + 157, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 91, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 92, 254, 135, 254);
	gfx.PutPixel(x + 157, y + 126, 254, 14, 254);
	gfx.PutPixel(x + 157, y + 127, 254, 225, 254);
	gfx.PutPixel(x + 157, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 141, 254, 246, 254);
	gfx.PutPixel(x + 157, y + 142, 254, 24, 254);
	gfx.PutPixel(x + 157, y + 148, 254, 87, 254);
	gfx.PutPixel(x + 157, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 156, 254, 244, 254);
	gfx.PutPixel(x + 157, y + 157, 254, 30, 254);
	gfx.PutPixel(x + 157, y + 236, 254, 97, 254);
	gfx.PutPixel(x + 157, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 248, 254, 128, 254);
	gfx.PutPixel(x + 157, y + 298, 254, 27, 254);
	gfx.PutPixel(x + 157, y + 299, 254, 232, 254);
	gfx.PutPixel(x + 157, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 310, 254, 195, 254);
	gfx.PutPixel(x + 157, y + 311, 254, 0, 254);
	gfx.PutPixel(x + 158, y + 80, 254, 78, 254);
	gfx.PutPixel(x + 158, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 90, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 91, 254, 202, 254);
	gfx.PutPixel(x + 158, y + 92, 254, 4, 254);
	gfx.PutPixel(x + 158, y + 126, 254, 161, 254);
	gfx.PutPixel(x + 158, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 141, 254, 140, 254);
	gfx.PutPixel(x + 158, y + 147, 254, 21, 254);
	gfx.PutPixel(x + 158, y + 148, 254, 235, 254);
	gfx.PutPixel(x + 158, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 156, 254, 120, 254);
	gfx.PutPixel(x + 158, y + 235, 254, 6, 254);
	gfx.PutPixel(x + 158, y + 236, 254, 225, 254);
	gfx.PutPixel(x + 158, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 247, 254, 220, 254);
	gfx.PutPixel(x + 158, y + 248, 254, 5, 254);
	gfx.PutPixel(x + 158, y + 297, 254, 7, 254);
	gfx.PutPixel(x + 158, y + 298, 254, 201, 254);
	gfx.PutPixel(x + 158, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 309, 254, 247, 254);
	gfx.PutPixel(x + 158, y + 310, 254, 41, 254);
	gfx.PutPixel(x + 159, y + 79, 254, 9, 254);
	gfx.PutPixel(x + 159, y + 80, 254, 224, 254);
	gfx.PutPixel(x + 159, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 90, 254, 245, 254);
	gfx.PutPixel(x + 159, y + 91, 254, 39, 254);
	gfx.PutPixel(x + 159, y + 125, 254, 81, 254);
	gfx.PutPixel(x + 159, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 140, 254, 237, 254);
	gfx.PutPixel(x + 159, y + 141, 254, 17, 254);
	gfx.PutPixel(x + 159, y + 147, 254, 172, 254);
	gfx.PutPixel(x + 159, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 155, 254, 213, 254);
	gfx.PutPixel(x + 159, y + 156, 254, 4, 254);
	gfx.PutPixel(x + 159, y + 235, 254, 109, 254);
	gfx.PutPixel(x + 159, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 247, 254, 90, 254);
	gfx.PutPixel(x + 159, y + 297, 254, 158, 254);
	gfx.PutPixel(x + 159, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 309, 254, 121, 254);
	gfx.PutPixel(x + 160, y + 79, 254, 133, 254);
	gfx.PutPixel(x + 160, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 89, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 90, 254, 107, 254);
	gfx.PutPixel(x + 160, y + 124, 254, 14, 254);
	gfx.PutPixel(x + 160, y + 125, 254, 230, 254);
	gfx.PutPixel(x + 160, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 140, 254, 110, 254);
	gfx.PutPixel(x + 160, y + 146, 254, 86, 254);
	gfx.PutPixel(x + 160, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 155, 254, 63, 254);
	gfx.PutPixel(x + 160, y + 234, 254, 8, 254);
	gfx.PutPixel(x + 160, y + 235, 254, 230, 254);
	gfx.PutPixel(x + 160, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 246, 254, 207, 254);
	gfx.PutPixel(x + 160, y + 247, 254, 0, 254);
	gfx.PutPixel(x + 160, y + 296, 254, 109, 254);
	gfx.PutPixel(x + 160, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 308, 254, 203, 254);
	gfx.PutPixel(x + 160, y + 309, 254, 3, 254);
	gfx.PutPixel(x + 161, y + 78, 254, 33, 254);
	gfx.PutPixel(x + 161, y + 79, 254, 248, 254);
	gfx.PutPixel(x + 161, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 88, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 89, 254, 192, 254);
	gfx.PutPixel(x + 161, y + 90, 254, 0, 254);
	gfx.PutPixel(x + 161, y + 124, 254, 148, 254);
	gfx.PutPixel(x + 161, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 139, 254, 215, 254);
	gfx.PutPixel(x + 161, y + 140, 254, 4, 254);
	gfx.PutPixel(x + 161, y + 145, 254, 22, 254);
	gfx.PutPixel(x + 161, y + 146, 254, 234, 254);
	gfx.PutPixel(x + 161, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 154, 254, 160, 254);
	gfx.PutPixel(x + 161, y + 234, 254, 108, 254);
	gfx.PutPixel(x + 161, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 246, 254, 84, 254);
	gfx.PutPixel(x + 161, y + 295, 254, 76, 254);
	gfx.PutPixel(x + 161, y + 296, 254, 251, 254);
	gfx.PutPixel(x + 161, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 307, 254, 247, 254);
	gfx.PutPixel(x + 161, y + 308, 254, 44, 254);
	gfx.PutPixel(x + 162, y + 78, 254, 169, 254);
	gfx.PutPixel(x + 162, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 88, 254, 246, 254);
	gfx.PutPixel(x + 162, y + 89, 254, 36, 254);
	gfx.PutPixel(x + 162, y + 123, 254, 52, 254);
	gfx.PutPixel(x + 162, y + 124, 254, 252, 254);
	gfx.PutPixel(x + 162, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 139, 254, 65, 254);
	gfx.PutPixel(x + 162, y + 144, 254, 0, 254);
	gfx.PutPixel(x + 162, y + 145, 254, 183, 254);
	gfx.PutPixel(x + 162, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 153, 254, 225, 254);
	gfx.PutPixel(x + 162, y + 154, 254, 14, 254);
	gfx.PutPixel(x + 162, y + 233, 254, 4, 254);
	gfx.PutPixel(x + 162, y + 234, 254, 224, 254);
	gfx.PutPixel(x + 162, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 245, 254, 228, 254);
	gfx.PutPixel(x + 162, y + 246, 254, 3, 254);
	gfx.PutPixel(x + 162, y + 294, 254, 49, 254);
	gfx.PutPixel(x + 162, y + 295, 254, 243, 254);
	gfx.PutPixel(x + 162, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 162, y + 307, 254, 109, 254);
	gfx.PutPixel(x + 163, y + 77, 254, 54, 254);
	gfx.PutPixel(x + 163, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 87, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 88, 254, 119, 254);
	gfx.PutPixel(x + 163, y + 122, 254, 1, 254);
	gfx.PutPixel(x + 163, y + 123, 254, 204, 254);
	gfx.PutPixel(x + 163, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 138, 254, 162, 254);
	gfx.PutPixel(x + 163, y + 144, 254, 115, 254);
	gfx.PutPixel(x + 163, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 152, 254, 253, 254);
	gfx.PutPixel(x + 163, y + 153, 254, 65, 254);
	gfx.PutPixel(x + 163, y + 233, 254, 92, 254);
	gfx.PutPixel(x + 163, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 245, 254, 127, 254);
	gfx.PutPixel(x + 163, y + 293, 254, 29, 254);
	gfx.PutPixel(x + 163, y + 294, 254, 229, 254);
	gfx.PutPixel(x + 163, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 163, y + 306, 254, 176, 254);
	gfx.PutPixel(x + 164, y + 77, 254, 183, 254);
	gfx.PutPixel(x + 164, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 87, 254, 215, 254);
	gfx.PutPixel(x + 164, y + 88, 254, 4, 254);
	gfx.PutPixel(x + 164, y + 122, 254, 95, 254);
	gfx.PutPixel(x + 164, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 137, 254, 236, 254);
	gfx.PutPixel(x + 164, y + 138, 254, 20, 254);
	gfx.PutPixel(x + 164, y + 143, 254, 59, 254);
	gfx.PutPixel(x + 164, y + 144, 254, 250, 254);
	gfx.PutPixel(x + 164, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 152, 254, 142, 254);
	gfx.PutPixel(x + 164, y + 233, 254, 204, 254);
	gfx.PutPixel(x + 164, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 245, 254, 30, 254);
	gfx.PutPixel(x + 164, y + 292, 254, 20, 254);
	gfx.PutPixel(x + 164, y + 293, 254, 215, 254);
	gfx.PutPixel(x + 164, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 164, y + 305, 254, 225, 254);
	gfx.PutPixel(x + 164, y + 306, 254, 17, 254);
	gfx.PutPixel(x + 165, y + 76, 254, 54, 254);
	gfx.PutPixel(x + 165, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 86, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 87, 254, 69, 254);
	gfx.PutPixel(x + 165, y + 121, 254, 7, 254);
	gfx.PutPixel(x + 165, y + 122, 254, 225, 254);
	gfx.PutPixel(x + 165, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 137, 254, 88, 254);
	gfx.PutPixel(x + 165, y + 142, 254, 24, 254);
	gfx.PutPixel(x + 165, y + 143, 254, 229, 254);
	gfx.PutPixel(x + 165, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 151, 254, 212, 254);
	gfx.PutPixel(x + 165, y + 152, 254, 7, 254);
	gfx.PutPixel(x + 165, y + 232, 254, 58, 254);
	gfx.PutPixel(x + 165, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 244, 254, 203, 254);
	gfx.PutPixel(x + 165, y + 291, 254, 14, 254);
	gfx.PutPixel(x + 165, y + 292, 254, 205, 254);
	gfx.PutPixel(x + 165, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 165, y + 304, 254, 248, 254);
	gfx.PutPixel(x + 165, y + 305, 254, 57, 254);
	gfx.PutPixel(x + 166, y + 76, 254, 170, 254);
	gfx.PutPixel(x + 166, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 85, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 86, 254, 177, 254);
	gfx.PutPixel(x + 166, y + 121, 254, 115, 254);
	gfx.PutPixel(x + 166, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 136, 254, 170, 254);
	gfx.PutPixel(x + 166, y + 141, 254, 7, 254);
	gfx.PutPixel(x + 166, y + 142, 254, 200, 254);
	gfx.PutPixel(x + 166, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 150, 254, 247, 254);
	gfx.PutPixel(x + 166, y + 151, 254, 48, 254);
	gfx.PutPixel(x + 166, y + 232, 254, 161, 254);
	gfx.PutPixel(x + 166, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 244, 254, 120, 254);
	gfx.PutPixel(x + 166, y + 290, 254, 8, 254);
	gfx.PutPixel(x + 166, y + 291, 254, 193, 254);
	gfx.PutPixel(x + 166, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 166, y + 304, 254, 101, 254);
	gfx.PutPixel(x + 167, y + 75, 254, 31, 254);
	gfx.PutPixel(x + 167, y + 76, 254, 251, 254);
	gfx.PutPixel(x + 167, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 85, 254, 252, 254);
	gfx.PutPixel(x + 167, y + 86, 254, 43, 254);
	gfx.PutPixel(x + 167, y + 120, 254, 12, 254);
	gfx.PutPixel(x + 167, y + 121, 254, 236, 254);
	gfx.PutPixel(x + 167, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 135, 254, 229, 254);
	gfx.PutPixel(x + 167, y + 136, 254, 20, 254);
	gfx.PutPixel(x + 167, y + 140, 254, 0, 254);
	gfx.PutPixel(x + 167, y + 141, 254, 166, 254);
	gfx.PutPixel(x + 167, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 150, 254, 104, 254);
	gfx.PutPixel(x + 167, y + 231, 254, 13, 254);
	gfx.PutPixel(x + 167, y + 232, 254, 244, 254);
	gfx.PutPixel(x + 167, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 244, 254, 45, 254);
	gfx.PutPixel(x + 167, y + 289, 254, 8, 254);
	gfx.PutPixel(x + 167, y + 290, 254, 186, 254);
	gfx.PutPixel(x + 167, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 167, y + 303, 254, 150, 254);
	gfx.PutPixel(x + 168, y + 75, 254, 128, 254);
	gfx.PutPixel(x + 168, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 85, 254, 160, 254);
	gfx.PutPixel(x + 168, y + 120, 254, 111, 254);
	gfx.PutPixel(x + 168, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 134, 254, 252, 254);
	gfx.PutPixel(x + 168, y + 135, 254, 66, 254);
	gfx.PutPixel(x + 168, y + 140, 254, 139, 254);
	gfx.PutPixel(x + 168, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 149, 254, 162, 254);
	gfx.PutPixel(x + 168, y + 231, 254, 98, 254);
	gfx.PutPixel(x + 168, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 243, 254, 236, 254);
	gfx.PutPixel(x + 168, y + 244, 254, 0, 254);
	gfx.PutPixel(x + 168, y + 288, 254, 10, 254);
	gfx.PutPixel(x + 168, y + 289, 254, 189, 254);
	gfx.PutPixel(x + 168, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 168, y + 302, 254, 192, 254);
	gfx.PutPixel(x + 168, y + 303, 254, 4, 254);
	gfx.PutPixel(x + 169, y + 74, 254, 0, 254);
	gfx.PutPixel(x + 169, y + 75, 254, 222, 254);
	gfx.PutPixel(x + 169, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 84, 254, 253, 254);
	gfx.PutPixel(x + 169, y + 85, 254, 38, 254);
	gfx.PutPixel(x + 169, y + 119, 254, 2, 254);
	gfx.PutPixel(x + 169, y + 120, 254, 222, 254);
	gfx.PutPixel(x + 169, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 134, 254, 131, 254);
	gfx.PutPixel(x + 169, y + 139, 254, 120, 254);
	gfx.PutPixel(x + 169, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 148, 254, 202, 254);
	gfx.PutPixel(x + 169, y + 149, 254, 8, 254);
	gfx.PutPixel(x + 169, y + 231, 254, 183, 254);
	gfx.PutPixel(x + 169, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 243, 254, 175, 254);
	gfx.PutPixel(x + 169, y + 287, 254, 11, 254);
	gfx.PutPixel(x + 169, y + 288, 254, 192, 254);
	gfx.PutPixel(x + 169, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 169, y + 301, 254, 214, 254);
	gfx.PutPixel(x + 169, y + 302, 254, 18, 254);
	gfx.PutPixel(x + 170, y + 74, 254, 48, 254);
	gfx.PutPixel(x + 170, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 84, 254, 175, 254);
	gfx.PutPixel(x + 170, y + 119, 254, 84, 254);
	gfx.PutPixel(x + 170, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 133, 254, 186, 254);
	gfx.PutPixel(x + 170, y + 134, 254, 2, 254);
	gfx.PutPixel(x + 170, y + 138, 254, 116, 254);
	gfx.PutPixel(x + 170, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 147, 254, 229, 254);
	gfx.PutPixel(x + 170, y + 148, 254, 25, 254);
	gfx.PutPixel(x + 170, y + 230, 254, 15, 254);
	gfx.PutPixel(x + 170, y + 231, 254, 249, 254);
	gfx.PutPixel(x + 170, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 243, 254, 119, 254);
	gfx.PutPixel(x + 170, y + 286, 254, 18, 254);
	gfx.PutPixel(x + 170, y + 287, 254, 198, 254);
	gfx.PutPixel(x + 170, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 170, y + 300, 254, 229, 254);
	gfx.PutPixel(x + 170, y + 301, 254, 30, 254);
	gfx.PutPixel(x + 171, y + 74, 254, 119, 254);
	gfx.PutPixel(x + 171, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 84, 254, 67, 254);
	gfx.PutPixel(x + 171, y + 119, 254, 190, 254);
	gfx.PutPixel(x + 171, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 132, 254, 221, 254);
	gfx.PutPixel(x + 171, y + 133, 254, 17, 254);
	gfx.PutPixel(x + 171, y + 137, 254, 133, 254);
	gfx.PutPixel(x + 171, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 146, 254, 241, 254);
	gfx.PutPixel(x + 171, y + 147, 254, 49, 254);
	gfx.PutPixel(x + 171, y + 230, 254, 80, 254);
	gfx.PutPixel(x + 171, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 243, 254, 72, 254);
	gfx.PutPixel(x + 171, y + 285, 254, 29, 254);
	gfx.PutPixel(x + 171, y + 286, 254, 214, 254);
	gfx.PutPixel(x + 171, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 171, y + 299, 254, 240, 254);
	gfx.PutPixel(x + 171, y + 300, 254, 46, 254);
	gfx.PutPixel(x + 172, y + 74, 254, 174, 254);
	gfx.PutPixel(x + 172, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 83, 254, 227, 254);
	gfx.PutPixel(x + 172, y + 84, 254, 1, 254);
	gfx.PutPixel(x + 172, y + 118, 254, 29, 254);
	gfx.PutPixel(x + 172, y + 119, 254, 253, 254);
	gfx.PutPixel(x + 172, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 131, 254, 240, 254);
	gfx.PutPixel(x + 172, y + 132, 254, 45, 254);
	gfx.PutPixel(x + 172, y + 135, 254, 8, 254);
	gfx.PutPixel(x + 172, y + 136, 254, 164, 254);
	gfx.PutPixel(x + 172, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 145, 254, 249, 254);
	gfx.PutPixel(x + 172, y + 146, 254, 67, 254);
	gfx.PutPixel(x + 172, y + 230, 254, 145, 254);
	gfx.PutPixel(x + 172, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 243, 254, 33, 254);
	gfx.PutPixel(x + 172, y + 284, 254, 44, 254);
	gfx.PutPixel(x + 172, y + 285, 254, 228, 254);
	gfx.PutPixel(x + 172, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 172, y + 298, 254, 245, 254);
	gfx.PutPixel(x + 172, y + 299, 254, 63, 254);
	gfx.PutPixel(x + 173, y + 74, 254, 218, 254);
	gfx.PutPixel(x + 173, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 83, 254, 145, 254);
	gfx.PutPixel(x + 173, y + 118, 254, 106, 254);
	gfx.PutPixel(x + 173, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 130, 254, 249, 254);
	gfx.PutPixel(x + 173, y + 131, 254, 66, 254);
	gfx.PutPixel(x + 173, y + 134, 254, 40, 254);
	gfx.PutPixel(x + 173, y + 135, 254, 206, 254);
	gfx.PutPixel(x + 173, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 144, 254, 251, 254);
	gfx.PutPixel(x + 173, y + 145, 254, 84, 254);
	gfx.PutPixel(x + 173, y + 158, 254, 12, 254);
	gfx.PutPixel(x + 173, y + 159, 254, 31, 254);
	gfx.PutPixel(x + 173, y + 160, 254, 51, 254);
	gfx.PutPixel(x + 173, y + 161, 254, 70, 254);
	gfx.PutPixel(x + 173, y + 162, 254, 89, 254);
	gfx.PutPixel(x + 173, y + 163, 254, 107, 254);
	gfx.PutPixel(x + 173, y + 164, 254, 122, 254);
	gfx.PutPixel(x + 173, y + 165, 254, 137, 254);
	gfx.PutPixel(x + 173, y + 166, 254, 152, 254);
	gfx.PutPixel(x + 173, y + 167, 254, 167, 254);
	gfx.PutPixel(x + 173, y + 168, 254, 182, 254);
	gfx.PutPixel(x + 173, y + 169, 254, 192, 254);
	gfx.PutPixel(x + 173, y + 170, 254, 202, 254);
	gfx.PutPixel(x + 173, y + 171, 254, 211, 254);
	gfx.PutPixel(x + 173, y + 172, 254, 221, 254);
	gfx.PutPixel(x + 173, y + 173, 254, 230, 254);
	gfx.PutPixel(x + 173, y + 174, 254, 237, 254);
	gfx.PutPixel(x + 173, y + 175, 254, 241, 254);
	gfx.PutPixel(x + 173, y + 176, 254, 244, 254);
	gfx.PutPixel(x + 173, y + 177, 254, 247, 254);
	gfx.PutPixel(x + 173, y + 178, 254, 251, 254);
	gfx.PutPixel(x + 173, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 181, 254, 251, 254);
	gfx.PutPixel(x + 173, y + 182, 254, 249, 254);
	gfx.PutPixel(x + 173, y + 183, 254, 246, 254);
	gfx.PutPixel(x + 173, y + 184, 254, 241, 254);
	gfx.PutPixel(x + 173, y + 185, 254, 234, 254);
	gfx.PutPixel(x + 173, y + 186, 254, 226, 254);
	gfx.PutPixel(x + 173, y + 187, 254, 218, 254);
	gfx.PutPixel(x + 173, y + 188, 254, 208, 254);
	gfx.PutPixel(x + 173, y + 189, 254, 194, 254);
	gfx.PutPixel(x + 173, y + 190, 254, 179, 254);
	gfx.PutPixel(x + 173, y + 191, 254, 163, 254);
	gfx.PutPixel(x + 173, y + 192, 254, 144, 254);
	gfx.PutPixel(x + 173, y + 193, 254, 119, 254);
	gfx.PutPixel(x + 173, y + 194, 254, 94, 254);
	gfx.PutPixel(x + 173, y + 195, 254, 66, 254);
	gfx.PutPixel(x + 173, y + 196, 254, 30, 254);
	gfx.PutPixel(x + 173, y + 197, 254, 0, 254);
	gfx.PutPixel(x + 173, y + 230, 254, 195, 254);
	gfx.PutPixel(x + 173, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 243, 254, 8, 254);
	gfx.PutPixel(x + 173, y + 283, 254, 74, 254);
	gfx.PutPixel(x + 173, y + 284, 254, 240, 254);
	gfx.PutPixel(x + 173, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 173, y + 297, 254, 246, 254);
	gfx.PutPixel(x + 173, y + 298, 254, 68, 254);
	gfx.PutPixel(x + 174, y + 74, 254, 240, 254);
	gfx.PutPixel(x + 174, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 83, 254, 83, 254);
	gfx.PutPixel(x + 174, y + 118, 254, 173, 254);
	gfx.PutPixel(x + 174, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 129, 254, 251, 254);
	gfx.PutPixel(x + 174, y + 130, 254, 86, 254);
	gfx.PutPixel(x + 174, y + 132, 254, 8, 254);
	gfx.PutPixel(x + 174, y + 133, 254, 126, 254);
	gfx.PutPixel(x + 174, y + 134, 254, 245, 254);
	gfx.PutPixel(x + 174, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 143, 254, 252, 254);
	gfx.PutPixel(x + 174, y + 144, 254, 90, 254);
	gfx.PutPixel(x + 174, y + 145, 254, 15, 254);
	gfx.PutPixel(x + 174, y + 146, 254, 35, 254);
	gfx.PutPixel(x + 174, y + 147, 254, 56, 254);
	gfx.PutPixel(x + 174, y + 148, 254, 77, 254);
	gfx.PutPixel(x + 174, y + 149, 254, 97, 254);
	gfx.PutPixel(x + 174, y + 150, 254, 118, 254);
	gfx.PutPixel(x + 174, y + 151, 254, 137, 254);
	gfx.PutPixel(x + 174, y + 152, 254, 156, 254);
	gfx.PutPixel(x + 174, y + 153, 254, 174, 254);
	gfx.PutPixel(x + 174, y + 154, 254, 192, 254);
	gfx.PutPixel(x + 174, y + 155, 254, 211, 254);
	gfx.PutPixel(x + 174, y + 156, 254, 230, 254);
	gfx.PutPixel(x + 174, y + 157, 254, 248, 254);
	gfx.PutPixel(x + 174, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 197, 254, 240, 254);
	gfx.PutPixel(x + 174, y + 198, 254, 188, 254);
	gfx.PutPixel(x + 174, y + 199, 254, 123, 254);
	gfx.PutPixel(x + 174, y + 200, 254, 36, 254);
	gfx.PutPixel(x + 174, y + 230, 254, 233, 254);
	gfx.PutPixel(x + 174, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 243, 254, 5, 254);
	gfx.PutPixel(x + 174, y + 282, 254, 117, 254);
	gfx.PutPixel(x + 174, y + 283, 254, 252, 254);
	gfx.PutPixel(x + 174, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 174, y + 296, 254, 247, 254);
	gfx.PutPixel(x + 174, y + 297, 254, 72, 254);
	gfx.PutPixel(x + 175, y + 74, 254, 249, 254);
	gfx.PutPixel(x + 175, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 83, 254, 33, 254);
	gfx.PutPixel(x + 175, y + 118, 254, 221, 254);
	gfx.PutPixel(x + 175, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 128, 254, 250, 254);
	gfx.PutPixel(x + 175, y + 129, 254, 102, 254);
	gfx.PutPixel(x + 175, y + 130, 254, 59, 254);
	gfx.PutPixel(x + 175, y + 131, 254, 135, 254);
	gfx.PutPixel(x + 175, y + 132, 254, 234, 254);
	gfx.PutPixel(x + 175, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 143, 254, 251, 254);
	gfx.PutPixel(x + 175, y + 144, 254, 249, 254);
	gfx.PutPixel(x + 175, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 198, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 199, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 200, 254, 253, 254);
	gfx.PutPixel(x + 175, y + 201, 254, 136, 254);
	gfx.PutPixel(x + 175, y + 230, 254, 250, 254);
	gfx.PutPixel(x + 175, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 243, 254, 22, 254);
	gfx.PutPixel(x + 175, y + 280, 254, 8, 254);
	gfx.PutPixel(x + 175, y + 281, 254, 161, 254);
	gfx.PutPixel(x + 175, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 175, y + 295, 254, 247, 254);
	gfx.PutPixel(x + 175, y + 296, 254, 75, 254);
	gfx.PutPixel(x + 176, y + 74, 254, 231, 254);
	gfx.PutPixel(x + 176, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 83, 254, 9, 254);
	gfx.PutPixel(x + 176, y + 118, 254, 246, 254);
	gfx.PutPixel(x + 176, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 198, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 199, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 200, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 201, 254, 239, 254);
	gfx.PutPixel(x + 176, y + 230, 254, 244, 254);
	gfx.PutPixel(x + 176, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 243, 254, 39, 254);
	gfx.PutPixel(x + 176, y + 279, 254, 40, 254);
	gfx.PutPixel(x + 176, y + 280, 254, 210, 254);
	gfx.PutPixel(x + 176, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 176, y + 294, 254, 241, 254);
	gfx.PutPixel(x + 176, y + 295, 254, 64, 254);
	gfx.PutPixel(x + 177, y + 74, 254, 188, 254);
	gfx.PutPixel(x + 177, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 83, 254, 8, 254);
	gfx.PutPixel(x + 177, y + 118, 254, 245, 254);
	gfx.PutPixel(x + 177, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 198, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 199, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 200, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 201, 254, 166, 254);
	gfx.PutPixel(x + 177, y + 230, 254, 229, 254);
	gfx.PutPixel(x + 177, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 243, 254, 68, 254);
	gfx.PutPixel(x + 177, y + 278, 254, 94, 254);
	gfx.PutPixel(x + 177, y + 279, 254, 243, 254);
	gfx.PutPixel(x + 177, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 177, y + 293, 254, 234, 254);
	gfx.PutPixel(x + 177, y + 294, 254, 51, 254);
	gfx.PutPixel(x + 178, y + 74, 254, 122, 254);
	gfx.PutPixel(x + 178, y + 75, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 83, 254, 36, 254);
	gfx.PutPixel(x + 178, y + 118, 254, 225, 254);
	gfx.PutPixel(x + 178, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 198, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 199, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 200, 254, 231, 254);
	gfx.PutPixel(x + 178, y + 201, 254, 27, 254);
	gfx.PutPixel(x + 178, y + 230, 254, 195, 254);
	gfx.PutPixel(x + 178, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 243, 254, 127, 254);
	gfx.PutPixel(x + 178, y + 276, 254, 19, 254);
	gfx.PutPixel(x + 178, y + 277, 254, 167, 254);
	gfx.PutPixel(x + 178, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 178, y + 292, 254, 223, 254);
	gfx.PutPixel(x + 178, y + 293, 254, 38, 254);
	gfx.PutPixel(x + 179, y + 74, 254, 27, 254);
	gfx.PutPixel(x + 179, y + 75, 254, 251, 254);
	gfx.PutPixel(x + 179, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 83, 254, 118, 254);
	gfx.PutPixel(x + 179, y + 118, 254, 177, 254);
	gfx.PutPixel(x + 179, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 198, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 199, 254, 231, 254);
	gfx.PutPixel(x + 179, y + 200, 254, 47, 254);
	gfx.PutPixel(x + 179, y + 230, 254, 145, 254);
	gfx.PutPixel(x + 179, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 243, 254, 186, 254);
	gfx.PutPixel(x + 179, y + 275, 254, 85, 254);
	gfx.PutPixel(x + 179, y + 276, 254, 233, 254);
	gfx.PutPixel(x + 179, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 179, y + 291, 254, 197, 254);
	gfx.PutPixel(x + 179, y + 292, 254, 22, 254);
	gfx.PutPixel(x + 180, y + 75, 254, 140, 254);
	gfx.PutPixel(x + 180, y + 76, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 83, 254, 241, 254);
	gfx.PutPixel(x + 180, y + 84, 254, 38, 254);
	gfx.PutPixel(x + 180, y + 118, 254, 95, 254);
	gfx.PutPixel(x + 180, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 173, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 174, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 175, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 176, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 177, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 178, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 179, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 180, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 181, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 182, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 183, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 184, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 185, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 186, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 187, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 188, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 189, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 190, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 191, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 192, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 193, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 194, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 195, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 196, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 197, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 198, 254, 182, 254);
	gfx.PutPixel(x + 180, y + 199, 254, 27, 254);
	gfx.PutPixel(x + 180, y + 230, 254, 73, 254);
	gfx.PutPixel(x + 180, y + 231, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 243, 254, 250, 254);
	gfx.PutPixel(x + 180, y + 244, 254, 30, 254);
	gfx.PutPixel(x + 180, y + 273, 254, 31, 254);
	gfx.PutPixel(x + 180, y + 274, 254, 174, 254);
	gfx.PutPixel(x + 180, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 180, y + 290, 254, 162, 254);
	gfx.PutPixel(x + 180, y + 291, 254, 6, 254);
	gfx.PutPixel(x + 181, y + 75, 254, 9, 254);
	gfx.PutPixel(x + 181, y + 76, 254, 195, 254);
	gfx.PutPixel(x + 181, y + 77, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 84, 254, 211, 254);
	gfx.PutPixel(x + 181, y + 118, 254, 6, 254);
	gfx.PutPixel(x + 181, y + 119, 254, 212, 254);
	gfx.PutPixel(x + 181, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 144, 254, 252, 254);
	gfx.PutPixel(x + 181, y + 145, 254, 233, 254);
	gfx.PutPixel(x + 181, y + 146, 254, 215, 254);
	gfx.PutPixel(x + 181, y + 147, 254, 196, 254);
	gfx.PutPixel(x + 181, y + 148, 254, 177, 254);
	gfx.PutPixel(x + 181, y + 149, 254, 161, 254);
	gfx.PutPixel(x + 181, y + 150, 254, 144, 254);
	gfx.PutPixel(x + 181, y + 151, 254, 128, 254);
	gfx.PutPixel(x + 181, y + 152, 254, 113, 254);
	gfx.PutPixel(x + 181, y + 153, 254, 99, 254);
	gfx.PutPixel(x + 181, y + 154, 254, 87, 254);
	gfx.PutPixel(x + 181, y + 155, 254, 74, 254);
	gfx.PutPixel(x + 181, y + 156, 254, 62, 254);
	gfx.PutPixel(x + 181, y + 157, 254, 53, 254);
	gfx.PutPixel(x + 181, y + 158, 254, 45, 254);
	gfx.PutPixel(x + 181, y + 159, 254, 37, 254);
	gfx.PutPixel(x + 181, y + 160, 254, 29, 254);
	gfx.PutPixel(x + 181, y + 161, 254, 23, 254);
	gfx.PutPixel(x + 181, y + 162, 254, 18, 254);
	gfx.PutPixel(x + 181, y + 163, 254, 13, 254);
	gfx.PutPixel(x + 181, y + 164, 254, 8, 254);
	gfx.PutPixel(x + 181, y + 165, 254, 5, 254);
	gfx.PutPixel(x + 181, y + 166, 254, 3, 254);
	gfx.PutPixel(x + 181, y + 167, 254, 2, 254);
	gfx.PutPixel(x + 181, y + 168, 254, 0, 254);
	gfx.PutPixel(x + 181, y + 172, 254, 0, 254);
	gfx.PutPixel(x + 181, y + 173, 254, 2, 254);
	gfx.PutPixel(x + 181, y + 174, 254, 5, 254);
	gfx.PutPixel(x + 181, y + 175, 254, 8, 254);
	gfx.PutPixel(x + 181, y + 176, 254, 12, 254);
	gfx.PutPixel(x + 181, y + 177, 254, 17, 254);
	gfx.PutPixel(x + 181, y + 178, 254, 24, 254);
	gfx.PutPixel(x + 181, y + 179, 254, 32, 254);
	gfx.PutPixel(x + 181, y + 180, 254, 41, 254);
	gfx.PutPixel(x + 181, y + 181, 254, 52, 254);
	gfx.PutPixel(x + 181, y + 182, 254, 65, 254);
	gfx.PutPixel(x + 181, y + 183, 254, 79, 254);
	gfx.PutPixel(x + 181, y + 184, 254, 93, 254);
	gfx.PutPixel(x + 181, y + 185, 254, 110, 254);
	gfx.PutPixel(x + 181, y + 186, 254, 128, 254);
	gfx.PutPixel(x + 181, y + 187, 254, 148, 254);
	gfx.PutPixel(x + 181, y + 188, 254, 167, 254);
	gfx.PutPixel(x + 181, y + 189, 254, 187, 254);
	gfx.PutPixel(x + 181, y + 190, 254, 206, 254);
	gfx.PutPixel(x + 181, y + 191, 254, 220, 254);
	gfx.PutPixel(x + 181, y + 192, 254, 231, 254);
	gfx.PutPixel(x + 181, y + 193, 254, 241, 254);
	gfx.PutPixel(x + 181, y + 194, 254, 250, 254);
	gfx.PutPixel(x + 181, y + 195, 254, 230, 254);
	gfx.PutPixel(x + 181, y + 196, 254, 166, 254);
	gfx.PutPixel(x + 181, y + 197, 254, 63, 254);
	gfx.PutPixel(x + 181, y + 230, 254, 2, 254);
	gfx.PutPixel(x + 181, y + 231, 254, 223, 254);
	gfx.PutPixel(x + 181, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 244, 254, 145, 254);
	gfx.PutPixel(x + 181, y + 271, 254, 9, 254);
	gfx.PutPixel(x + 181, y + 272, 254, 130, 254);
	gfx.PutPixel(x + 181, y + 273, 254, 246, 254);
	gfx.PutPixel(x + 181, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 181, y + 288, 254, 251, 254);
	gfx.PutPixel(x + 181, y + 289, 254, 119, 254);
	gfx.PutPixel(x + 182, y + 76, 254, 13, 254);
	gfx.PutPixel(x + 182, y + 77, 254, 168, 254);
	gfx.PutPixel(x + 182, y + 78, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 79, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 80, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 81, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 82, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 83, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 84, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 85, 254, 6, 254);
	gfx.PutPixel(x + 182, y + 119, 254, 37, 254);
	gfx.PutPixel(x + 182, y + 120, 254, 219, 254);
	gfx.PutPixel(x + 182, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 134, 254, 251, 254);
	gfx.PutPixel(x + 182, y + 135, 254, 226, 254);
	gfx.PutPixel(x + 182, y + 136, 254, 198, 254);
	gfx.PutPixel(x + 182, y + 137, 254, 170, 254);
	gfx.PutPixel(x + 182, y + 138, 254, 143, 254);
	gfx.PutPixel(x + 182, y + 139, 254, 116, 254);
	gfx.PutPixel(x + 182, y + 140, 254, 91, 254);
	gfx.PutPixel(x + 182, y + 141, 254, 65, 254);
	gfx.PutPixel(x + 182, y + 142, 254, 42, 254);
	gfx.PutPixel(x + 182, y + 143, 254, 20, 254);
	gfx.PutPixel(x + 182, y + 144, 254, 1, 254);
	gfx.PutPixel(x + 182, y + 231, 254, 76, 254);
	gfx.PutPixel(x + 182, y + 232, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 233, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 237, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 238, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 244, 254, 246, 254);
	gfx.PutPixel(x + 182, y + 245, 254, 51, 254);
	gfx.PutPixel(x + 182, y + 269, 254, 5, 254);
	gfx.PutPixel(x + 182, y + 270, 254, 104, 254);
	gfx.PutPixel(x + 182, y + 271, 254, 226, 254);
	gfx.PutPixel(x + 182, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 182, y + 287, 254, 229, 254);
	gfx.PutPixel(x + 182, y + 288, 254, 62, 254);
	gfx.PutPixel(x + 183, y + 78, 254, 63, 254);
	gfx.PutPixel(x + 183, y + 79, 254, 160, 254);
	gfx.PutPixel(x + 183, y + 80, 254, 218, 254);
	gfx.PutPixel(x + 183, y + 81, 254, 244, 254);
	gfx.PutPixel(x + 183, y + 82, 254, 242, 254);
	gfx.PutPixel(x + 183, y + 83, 254, 204, 254);
	gfx.PutPixel(x + 183, y + 84, 254, 109, 254);
	gfx.PutPixel(x + 183, y + 120, 254, 9, 254);
	gfx.PutPixel(x + 183, y + 121, 254, 108, 254);
	gfx.PutPixel(x + 183, y + 122, 254, 185, 254);
	gfx.PutPixel(x + 183, y + 123, 254, 230, 254);
	gfx.PutPixel(x + 183, y + 124, 254, 248, 254);
	gfx.PutPixel(x + 183, y + 125, 254, 248, 254);
	gfx.PutPixel(x + 183, y + 126, 254, 234, 254);
	gfx.PutPixel(x + 183, y + 127, 254, 211, 254);
	gfx.PutPixel(x + 183, y + 128, 254, 184, 254);
	gfx.PutPixel(x + 183, y + 129, 254, 154, 254);
	gfx.PutPixel(x + 183, y + 130, 254, 123, 254);
	gfx.PutPixel(x + 183, y + 131, 254, 90, 254);
	gfx.PutPixel(x + 183, y + 132, 254, 59, 254);
	gfx.PutPixel(x + 183, y + 133, 254, 30, 254);
	gfx.PutPixel(x + 183, y + 134, 254, 3, 254);
	gfx.PutPixel(x + 183, y + 232, 254, 90, 254);
	gfx.PutPixel(x + 183, y + 233, 254, 234, 254);
	gfx.PutPixel(x + 183, y + 234, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 235, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 236, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 237, 254, 227, 254);
	gfx.PutPixel(x + 183, y + 238, 254, 229, 254);
	gfx.PutPixel(x + 183, y + 239, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 245, 254, 226, 254);
	gfx.PutPixel(x + 183, y + 246, 254, 23, 254);
	gfx.PutPixel(x + 183, y + 267, 254, 6, 254);
	gfx.PutPixel(x + 183, y + 268, 254, 104, 254);
	gfx.PutPixel(x + 183, y + 269, 254, 225, 254);
	gfx.PutPixel(x + 183, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 183, y + 286, 254, 185, 254);
	gfx.PutPixel(x + 183, y + 287, 254, 21, 254);
	gfx.PutPixel(x + 184, y + 233, 254, 5, 254);
	gfx.PutPixel(x + 184, y + 234, 254, 59, 254);
	gfx.PutPixel(x + 184, y + 235, 254, 83, 254);
	gfx.PutPixel(x + 184, y + 236, 254, 50, 254);
	gfx.PutPixel(x + 184, y + 238, 254, 17, 254);
	gfx.PutPixel(x + 184, y + 239, 254, 237, 254);
	gfx.PutPixel(x + 184, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 246, 254, 224, 254);
	gfx.PutPixel(x + 184, y + 247, 254, 54, 254);
	gfx.PutPixel(x + 184, y + 265, 254, 33, 254);
	gfx.PutPixel(x + 184, y + 266, 254, 136, 254);
	gfx.PutPixel(x + 184, y + 267, 254, 233, 254);
	gfx.PutPixel(x + 184, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 184, y + 284, 254, 246, 254);
	gfx.PutPixel(x + 184, y + 285, 254, 115, 254);
	gfx.PutPixel(x + 184, y + 286, 254, 1, 254);
	gfx.PutPixel(x + 185, y + 239, 254, 131, 254);
	gfx.PutPixel(x + 185, y + 240, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 247, 254, 249, 254);
	gfx.PutPixel(x + 185, y + 248, 254, 140, 254);
	gfx.PutPixel(x + 185, y + 249, 254, 26, 254);
	gfx.PutPixel(x + 185, y + 262, 254, 33, 254);
	gfx.PutPixel(x + 185, y + 263, 254, 109, 254);
	gfx.PutPixel(x + 185, y + 264, 254, 188, 254);
	gfx.PutPixel(x + 185, y + 265, 254, 253, 254);
	gfx.PutPixel(x + 185, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 185, y + 283, 254, 200, 254);
	gfx.PutPixel(x + 185, y + 284, 254, 40, 254);
	gfx.PutPixel(x + 186, y + 239, 254, 19, 254);
	gfx.PutPixel(x + 186, y + 240, 254, 236, 254);
	gfx.PutPixel(x + 186, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 249, 254, 248, 254);
	gfx.PutPixel(x + 186, y + 250, 254, 175, 254);
	gfx.PutPixel(x + 186, y + 251, 254, 111, 254);
	gfx.PutPixel(x + 186, y + 252, 254, 53, 254);
	gfx.PutPixel(x + 186, y + 253, 254, 29, 254);
	gfx.PutPixel(x + 186, y + 254, 254, 10, 254);
	gfx.PutPixel(x + 186, y + 255, 254, 6, 254);
	gfx.PutPixel(x + 186, y + 256, 254, 23, 254);
	gfx.PutPixel(x + 186, y + 257, 254, 39, 254);
	gfx.PutPixel(x + 186, y + 258, 254, 65, 254);
	gfx.PutPixel(x + 186, y + 259, 254, 112, 254);
	gfx.PutPixel(x + 186, y + 260, 254, 159, 254);
	gfx.PutPixel(x + 186, y + 261, 254, 213, 254);
	gfx.PutPixel(x + 186, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 186, y + 281, 254, 244, 254);
	gfx.PutPixel(x + 186, y + 282, 254, 118, 254);
	gfx.PutPixel(x + 186, y + 283, 254, 2, 254);
	gfx.PutPixel(x + 187, y + 240, 254, 96, 254);
	gfx.PutPixel(x + 187, y + 241, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 187, y + 280, 254, 169, 254);
	gfx.PutPixel(x + 187, y + 281, 254, 28, 254);
	gfx.PutPixel(x + 188, y + 241, 254, 169, 254);
	gfx.PutPixel(x + 188, y + 242, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 188, y + 278, 254, 208, 254);
	gfx.PutPixel(x + 188, y + 279, 254, 65, 254);
	gfx.PutPixel(x + 189, y + 241, 254, 7, 254);
	gfx.PutPixel(x + 189, y + 242, 254, 196, 254);
	gfx.PutPixel(x + 189, y + 243, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 189, y + 276, 254, 212, 254);
	gfx.PutPixel(x + 189, y + 277, 254, 84, 254);
	gfx.PutPixel(x + 189, y + 278, 254, 0, 254);
	gfx.PutPixel(x + 190, y + 242, 254, 13, 254);
	gfx.PutPixel(x + 190, y + 243, 254, 175, 254);
	gfx.PutPixel(x + 190, y + 244, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 245, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 190, y + 274, 254, 207, 254);
	gfx.PutPixel(x + 190, y + 275, 254, 90, 254);
	gfx.PutPixel(x + 190, y + 276, 254, 1, 254);
	gfx.PutPixel(x + 191, y + 243, 254, 0, 254);
	gfx.PutPixel(x + 191, y + 244, 254, 120, 254);
	gfx.PutPixel(x + 191, y + 245, 254, 245, 254);
	gfx.PutPixel(x + 191, y + 246, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 247, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 271, 254, 252, 254);
	gfx.PutPixel(x + 191, y + 272, 254, 174, 254);
	gfx.PutPixel(x + 191, y + 273, 254, 62, 254);
	gfx.PutPixel(x + 192, y + 245, 254, 27, 254);
	gfx.PutPixel(x + 192, y + 246, 254, 160, 254);
	gfx.PutPixel(x + 192, y + 247, 254, 252, 254);
	gfx.PutPixel(x + 192, y + 248, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 269, 254, 207, 254);
	gfx.PutPixel(x + 192, y + 270, 254, 117, 254);
	gfx.PutPixel(x + 192, y + 271, 254, 29, 254);
	gfx.PutPixel(x + 193, y + 247, 254, 27, 254);
	gfx.PutPixel(x + 193, y + 248, 254, 121, 254);
	gfx.PutPixel(x + 193, y + 249, 254, 216, 254);
	gfx.PutPixel(x + 193, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 265, 254, 244, 254);
	gfx.PutPixel(x + 193, y + 266, 254, 185, 254);
	gfx.PutPixel(x + 193, y + 267, 254, 119, 254);
	gfx.PutPixel(x + 193, y + 268, 254, 40, 254);
	gfx.PutPixel(x + 194, y + 250, 254, 40, 254);
	gfx.PutPixel(x + 194, y + 251, 254, 104, 254);
	gfx.PutPixel(x + 194, y + 252, 254, 152, 254);
	gfx.PutPixel(x + 194, y + 253, 254, 192, 254);
	gfx.PutPixel(x + 194, y + 254, 254, 218, 254);
	gfx.PutPixel(x + 194, y + 255, 254, 238, 254);
	gfx.PutPixel(x + 194, y + 256, 254, 247, 254);
	gfx.PutPixel(x + 194, y + 257, 254, 251, 254);
	gfx.PutPixel(x + 194, y + 258, 254, 240, 254);
	gfx.PutPixel(x + 194, y + 259, 254, 226, 254);
	gfx.PutPixel(x + 194, y + 260, 254, 213, 254);
	gfx.PutPixel(x + 194, y + 261, 254, 183, 254);
	gfx.PutPixel(x + 194, y + 262, 254, 143, 254);
	gfx.PutPixel(x + 194, y + 263, 254, 103, 254);
	gfx.PutPixel(x + 194, y + 264, 254, 58, 254);
	gfx.PutPixel(x + 194, y + 265, 254, 5, 254);
	gfx.PutPixel(x + 223, y + 255, 254, 2, 254);
	gfx.PutPixel(x + 223, y + 256, 254, 39, 254);
	gfx.PutPixel(x + 223, y + 257, 254, 60, 254);
	gfx.PutPixel(x + 223, y + 258, 254, 77, 254);
	gfx.PutPixel(x + 223, y + 259, 254, 75, 254);
	gfx.PutPixel(x + 223, y + 260, 254, 62, 254);
	gfx.PutPixel(x + 223, y + 261, 254, 38, 254);
	gfx.PutPixel(x + 223, y + 262, 254, 4, 254);
	gfx.PutPixel(x + 224, y + 253, 254, 80, 254);
	gfx.PutPixel(x + 224, y + 254, 254, 179, 254);
	gfx.PutPixel(x + 224, y + 255, 254, 240, 254);
	gfx.PutPixel(x + 224, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 224, y + 262, 254, 251, 254);
	gfx.PutPixel(x + 224, y + 263, 254, 212, 254);
	gfx.PutPixel(x + 224, y + 264, 254, 149, 254);
	gfx.PutPixel(x + 224, y + 265, 254, 77, 254);
	gfx.PutPixel(x + 224, y + 266, 254, 3, 254);
	gfx.PutPixel(x + 225, y + 149, 254, 16, 254);
	gfx.PutPixel(x + 225, y + 150, 254, 86, 254);
	gfx.PutPixel(x + 225, y + 151, 254, 147, 254);
	gfx.PutPixel(x + 225, y + 152, 254, 193, 254);
	gfx.PutPixel(x + 225, y + 153, 254, 227, 254);
	gfx.PutPixel(x + 225, y + 154, 254, 244, 254);
	gfx.PutPixel(x + 225, y + 155, 254, 249, 254);
	gfx.PutPixel(x + 225, y + 156, 254, 234, 254);
	gfx.PutPixel(x + 225, y + 157, 254, 210, 254);
	gfx.PutPixel(x + 225, y + 158, 254, 159, 254);
	gfx.PutPixel(x + 225, y + 159, 254, 97, 254);
	gfx.PutPixel(x + 225, y + 160, 254, 11, 254);
	gfx.PutPixel(x + 225, y + 252, 254, 124, 254);
	gfx.PutPixel(x + 225, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 225, y + 266, 254, 229, 254);
	gfx.PutPixel(x + 225, y + 267, 254, 128, 254);
	gfx.PutPixel(x + 225, y + 268, 254, 18, 254);
	gfx.PutPixel(x + 226, y + 146, 254, 6, 254);
	gfx.PutPixel(x + 226, y + 147, 254, 89, 254);
	gfx.PutPixel(x + 226, y + 148, 254, 187, 254);
	gfx.PutPixel(x + 226, y + 149, 254, 251, 254);
	gfx.PutPixel(x + 226, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 160, 254, 241, 254);
	gfx.PutPixel(x + 226, y + 161, 254, 132, 254);
	gfx.PutPixel(x + 226, y + 162, 254, 6, 254);
	gfx.PutPixel(x + 226, y + 252, 254, 200, 254);
	gfx.PutPixel(x + 226, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 226, y + 268, 254, 242, 254);
	gfx.PutPixel(x + 226, y + 269, 254, 133, 254);
	gfx.PutPixel(x + 226, y + 270, 254, 16, 254);
	gfx.PutPixel(x + 227, y + 144, 254, 17, 254);
	gfx.PutPixel(x + 227, y + 145, 254, 126, 254);
	gfx.PutPixel(x + 227, y + 146, 254, 235, 254);
	gfx.PutPixel(x + 227, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 162, 254, 212, 254);
	gfx.PutPixel(x + 227, y + 163, 254, 28, 254);
	gfx.PutPixel(x + 227, y + 252, 254, 147, 254);
	gfx.PutPixel(x + 227, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 270, 254, 239, 254);
	gfx.PutPixel(x + 227, y + 271, 254, 120, 254);
	gfx.PutPixel(x + 227, y + 272, 254, 9, 254);
	gfx.PutPixel(x + 228, y + 142, 254, 13, 254);
	gfx.PutPixel(x + 228, y + 143, 254, 131, 254);
	gfx.PutPixel(x + 228, y + 144, 254, 243, 254);
	gfx.PutPixel(x + 228, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 163, 254, 218, 254);
	gfx.PutPixel(x + 228, y + 164, 254, 20, 254);
	gfx.PutPixel(x + 228, y + 252, 254, 18, 254);
	gfx.PutPixel(x + 228, y + 253, 254, 213, 254);
	gfx.PutPixel(x + 228, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 272, 254, 228, 254);
	gfx.PutPixel(x + 228, y + 273, 254, 97, 254);
	gfx.PutPixel(x + 228, y + 274, 254, 1, 254);
	gfx.PutPixel(x + 229, y + 140, 254, 2, 254);
	gfx.PutPixel(x + 229, y + 141, 254, 106, 254);
	gfx.PutPixel(x + 229, y + 142, 254, 234, 254);
	gfx.PutPixel(x + 229, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 164, 254, 164, 254);
	gfx.PutPixel(x + 229, y + 253, 254, 12, 254);
	gfx.PutPixel(x + 229, y + 254, 254, 135, 254);
	gfx.PutPixel(x + 229, y + 255, 254, 240, 254);
	gfx.PutPixel(x + 229, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 274, 254, 208, 254);
	gfx.PutPixel(x + 229, y + 275, 254, 68, 254);
	gfx.PutPixel(x + 230, y + 139, 254, 53, 254);
	gfx.PutPixel(x + 230, y + 140, 254, 204, 254);
	gfx.PutPixel(x + 230, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 165, 254, 49, 254);
	gfx.PutPixel(x + 230, y + 255, 254, 11, 254);
	gfx.PutPixel(x + 230, y + 256, 254, 95, 254);
	gfx.PutPixel(x + 230, y + 257, 254, 179, 254);
	gfx.PutPixel(x + 230, y + 258, 254, 246, 254);
	gfx.PutPixel(x + 230, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 276, 254, 180, 254);
	gfx.PutPixel(x + 230, y + 277, 254, 41, 254);
	gfx.PutPixel(x + 231, y + 137, 254, 7, 254);
	gfx.PutPixel(x + 231, y + 138, 254, 140, 254);
	gfx.PutPixel(x + 231, y + 139, 254, 252, 254);
	gfx.PutPixel(x + 231, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 165, 254, 136, 254);
	gfx.PutPixel(x + 231, y + 258, 254, 8, 254);
	gfx.PutPixel(x + 231, y + 259, 254, 70, 254);
	gfx.PutPixel(x + 231, y + 260, 254, 142, 254);
	gfx.PutPixel(x + 231, y + 261, 254, 228, 254);
	gfx.PutPixel(x + 231, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 277, 254, 251, 254);
	gfx.PutPixel(x + 231, y + 278, 254, 148, 254);
	gfx.PutPixel(x + 231, y + 279, 254, 19, 254);
	gfx.PutPixel(x + 232, y + 136, 254, 51, 254);
	gfx.PutPixel(x + 232, y + 137, 254, 213, 254);
	gfx.PutPixel(x + 232, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 165, 254, 203, 254);
	gfx.PutPixel(x + 232, y + 261, 254, 2, 254);
	gfx.PutPixel(x + 232, y + 262, 254, 81, 254);
	gfx.PutPixel(x + 232, y + 263, 254, 197, 254);
	gfx.PutPixel(x + 232, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 279, 254, 239, 254);
	gfx.PutPixel(x + 232, y + 280, 254, 114, 254);
	gfx.PutPixel(x + 232, y + 281, 254, 5, 254);
	gfx.PutPixel(x + 233, y + 134, 254, 0, 254);
	gfx.PutPixel(x + 233, y + 135, 254, 113, 254);
	gfx.PutPixel(x + 233, y + 136, 254, 249, 254);
	gfx.PutPixel(x + 233, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 152, 254, 208, 254);
	gfx.PutPixel(x + 233, y + 153, 254, 123, 254);
	gfx.PutPixel(x + 233, y + 154, 254, 57, 254);
	gfx.PutPixel(x + 233, y + 155, 254, 20, 254);
	gfx.PutPixel(x + 233, y + 156, 254, 7, 254);
	gfx.PutPixel(x + 233, y + 157, 254, 56, 254);
	gfx.PutPixel(x + 233, y + 158, 254, 212, 254);
	gfx.PutPixel(x + 233, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 165, 254, 230, 254);
	gfx.PutPixel(x + 233, y + 264, 254, 65, 254);
	gfx.PutPixel(x + 233, y + 265, 254, 194, 254);
	gfx.PutPixel(x + 233, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 281, 254, 218, 254);
	gfx.PutPixel(x + 233, y + 282, 254, 78, 254);
	gfx.PutPixel(x + 234, y + 133, 254, 16, 254);
	gfx.PutPixel(x + 234, y + 134, 254, 179, 254);
	gfx.PutPixel(x + 234, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 149, 254, 253, 254);
	gfx.PutPixel(x + 234, y + 150, 254, 171, 254);
	gfx.PutPixel(x + 234, y + 151, 254, 54, 254);
	gfx.PutPixel(x + 234, y + 158, 254, 61, 254);
	gfx.PutPixel(x + 234, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 165, 254, 250, 254);
	gfx.PutPixel(x + 234, y + 266, 254, 69, 254);
	gfx.PutPixel(x + 234, y + 267, 254, 201, 254);
	gfx.PutPixel(x + 234, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 283, 254, 187, 254);
	gfx.PutPixel(x + 234, y + 284, 254, 44, 254);
	gfx.PutPixel(x + 235, y + 132, 254, 43, 254);
	gfx.PutPixel(x + 235, y + 133, 254, 219, 254);
	gfx.PutPixel(x + 235, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 148, 254, 186, 254);
	gfx.PutPixel(x + 235, y + 149, 254, 47, 254);
	gfx.PutPixel(x + 235, y + 158, 254, 10, 254);
	gfx.PutPixel(x + 235, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 165, 254, 243, 254);
	gfx.PutPixel(x + 235, y + 268, 254, 84, 254);
	gfx.PutPixel(x + 235, y + 269, 254, 217, 254);
	gfx.PutPixel(x + 235, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 284, 254, 251, 254);
	gfx.PutPixel(x + 235, y + 285, 254, 150, 254);
	gfx.PutPixel(x + 235, y + 286, 254, 19, 254);
	gfx.PutPixel(x + 236, y + 131, 254, 81, 254);
	gfx.PutPixel(x + 236, y + 132, 254, 243, 254);
	gfx.PutPixel(x + 236, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 146, 254, 227, 254);
	gfx.PutPixel(x + 236, y + 147, 254, 85, 254);
	gfx.PutPixel(x + 236, y + 158, 254, 9, 254);
	gfx.PutPixel(x + 236, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 165, 254, 229, 254);
	gfx.PutPixel(x + 236, y + 269, 254, 3, 254);
	gfx.PutPixel(x + 236, y + 270, 254, 104, 254);
	gfx.PutPixel(x + 236, y + 271, 254, 232, 254);
	gfx.PutPixel(x + 236, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 286, 254, 238, 254);
	gfx.PutPixel(x + 236, y + 287, 254, 112, 254);
	gfx.PutPixel(x + 236, y + 288, 254, 4, 254);
	gfx.PutPixel(x + 237, y + 130, 254, 117, 254);
	gfx.PutPixel(x + 237, y + 131, 254, 253, 254);
	gfx.PutPixel(x + 237, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 145, 254, 158, 254);
	gfx.PutPixel(x + 237, y + 146, 254, 14, 254);
	gfx.PutPixel(x + 237, y + 158, 254, 36, 254);
	gfx.PutPixel(x + 237, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 165, 254, 214, 254);
	gfx.PutPixel(x + 237, y + 271, 254, 13, 254);
	gfx.PutPixel(x + 237, y + 272, 254, 135, 254);
	gfx.PutPixel(x + 237, y + 273, 254, 246, 254);
	gfx.PutPixel(x + 237, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 288, 254, 216, 254);
	gfx.PutPixel(x + 237, y + 289, 254, 75, 254);
	gfx.PutPixel(x + 238, y + 128, 254, 2, 254);
	gfx.PutPixel(x + 238, y + 129, 254, 153, 254);
	gfx.PutPixel(x + 238, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 143, 254, 235, 254);
	gfx.PutPixel(x + 238, y + 144, 254, 77, 254);
	gfx.PutPixel(x + 238, y + 158, 254, 78, 254);
	gfx.PutPixel(x + 238, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 165, 254, 181, 254);
	gfx.PutPixel(x + 238, y + 273, 254, 30, 254);
	gfx.PutPixel(x + 238, y + 274, 254, 167, 254);
	gfx.PutPixel(x + 238, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 290, 254, 184, 254);
	gfx.PutPixel(x + 238, y + 291, 254, 41, 254);
	gfx.PutPixel(x + 239, y + 127, 254, 7, 254);
	gfx.PutPixel(x + 239, y + 128, 254, 176, 254);
	gfx.PutPixel(x + 239, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 142, 254, 193, 254);
	gfx.PutPixel(x + 239, y + 143, 254, 26, 254);
	gfx.PutPixel(x + 239, y + 158, 254, 135, 254);
	gfx.PutPixel(x + 239, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 165, 254, 141, 254);
	gfx.PutPixel(x + 239, y + 275, 254, 58, 254);
	gfx.PutPixel(x + 239, y + 276, 254, 203, 254);
	gfx.PutPixel(x + 239, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 291, 254, 250, 254);
	gfx.PutPixel(x + 239, y + 292, 254, 147, 254);
	gfx.PutPixel(x + 239, y + 293, 254, 18, 254);
	gfx.PutPixel(x + 240, y + 126, 254, 14, 254);
	gfx.PutPixel(x + 240, y + 127, 254, 193, 254);
	gfx.PutPixel(x + 240, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 141, 254, 139, 254);
	gfx.PutPixel(x + 240, y + 142, 254, 2, 254);
	gfx.PutPixel(x + 240, y + 158, 254, 200, 254);
	gfx.PutPixel(x + 240, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 165, 254, 101, 254);
	gfx.PutPixel(x + 240, y + 276, 254, 0, 254);
	gfx.PutPixel(x + 240, y + 277, 254, 97, 254);
	gfx.PutPixel(x + 240, y + 278, 254, 231, 254);
	gfx.PutPixel(x + 240, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 293, 254, 238, 254);
	gfx.PutPixel(x + 240, y + 294, 254, 112, 254);
	gfx.PutPixel(x + 240, y + 295, 254, 4, 254);
	gfx.PutPixel(x + 241, y + 125, 254, 15, 254);
	gfx.PutPixel(x + 241, y + 126, 254, 202, 254);
	gfx.PutPixel(x + 241, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 139, 254, 246, 254);
	gfx.PutPixel(x + 241, y + 140, 254, 91, 254);
	gfx.PutPixel(x + 241, y + 157, 254, 20, 254);
	gfx.PutPixel(x + 241, y + 158, 254, 252, 254);
	gfx.PutPixel(x + 241, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 165, 254, 48, 254);
	gfx.PutPixel(x + 241, y + 278, 254, 13, 254);
	gfx.PutPixel(x + 241, y + 279, 254, 139, 254);
	gfx.PutPixel(x + 241, y + 280, 254, 249, 254);
	gfx.PutPixel(x + 241, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 295, 254, 217, 254);
	gfx.PutPixel(x + 241, y + 296, 254, 78, 254);
	gfx.PutPixel(x + 242, y + 124, 254, 16, 254);
	gfx.PutPixel(x + 242, y + 125, 254, 203, 254);
	gfx.PutPixel(x + 242, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 138, 254, 231, 254);
	gfx.PutPixel(x + 242, y + 139, 254, 52, 254);
	gfx.PutPixel(x + 242, y + 157, 254, 100, 254);
	gfx.PutPixel(x + 242, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 164, 254, 238, 254);
	gfx.PutPixel(x + 242, y + 165, 254, 1, 254);
	gfx.PutPixel(x + 242, y + 280, 254, 39, 254);
	gfx.PutPixel(x + 242, y + 281, 254, 184, 254);
	gfx.PutPixel(x + 242, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 297, 254, 188, 254);
	gfx.PutPixel(x + 242, y + 298, 254, 46, 254);
	gfx.PutPixel(x + 243, y + 123, 254, 16, 254);
	gfx.PutPixel(x + 243, y + 124, 254, 203, 254);
	gfx.PutPixel(x + 243, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 137, 254, 213, 254);
	gfx.PutPixel(x + 243, y + 138, 254, 31, 254);
	gfx.PutPixel(x + 243, y + 157, 254, 187, 254);
	gfx.PutPixel(x + 243, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 164, 254, 178, 254);
	gfx.PutPixel(x + 243, y + 282, 254, 79, 254);
	gfx.PutPixel(x + 243, y + 283, 254, 222, 254);
	gfx.PutPixel(x + 243, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 298, 254, 252, 254);
	gfx.PutPixel(x + 243, y + 299, 254, 153, 254);
	gfx.PutPixel(x + 243, y + 300, 254, 22, 254);
	gfx.PutPixel(x + 244, y + 122, 254, 16, 254);
	gfx.PutPixel(x + 244, y + 123, 254, 204, 254);
	gfx.PutPixel(x + 244, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 136, 254, 194, 254);
	gfx.PutPixel(x + 244, y + 137, 254, 15, 254);
	gfx.PutPixel(x + 244, y + 156, 254, 28, 254);
	gfx.PutPixel(x + 244, y + 157, 254, 252, 254);
	gfx.PutPixel(x + 244, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 164, 254, 108, 254);
	gfx.PutPixel(x + 244, y + 283, 254, 8, 254);
	gfx.PutPixel(x + 244, y + 284, 254, 128, 254);
	gfx.PutPixel(x + 244, y + 285, 254, 246, 254);
	gfx.PutPixel(x + 244, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 300, 254, 241, 254);
	gfx.PutPixel(x + 244, y + 301, 254, 120, 254);
	gfx.PutPixel(x + 244, y + 302, 254, 7, 254);
	gfx.PutPixel(x + 245, y + 121, 254, 16, 254);
	gfx.PutPixel(x + 245, y + 122, 254, 204, 254);
	gfx.PutPixel(x + 245, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 135, 254, 179, 254);
	gfx.PutPixel(x + 245, y + 136, 254, 8, 254);
	gfx.PutPixel(x + 245, y + 156, 254, 125, 254);
	gfx.PutPixel(x + 245, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 163, 254, 253, 254);
	gfx.PutPixel(x + 245, y + 164, 254, 25, 254);
	gfx.PutPixel(x + 245, y + 285, 254, 33, 254);
	gfx.PutPixel(x + 245, y + 286, 254, 178, 254);
	gfx.PutPixel(x + 245, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 302, 254, 223, 254);
	gfx.PutPixel(x + 245, y + 303, 254, 88, 254);
	gfx.PutPixel(x + 245, y + 304, 254, 0, 254);
	gfx.PutPixel(x + 246, y + 120, 254, 8, 254);
	gfx.PutPixel(x + 246, y + 121, 254, 197, 254);
	gfx.PutPixel(x + 246, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 134, 254, 163, 254);
	gfx.PutPixel(x + 246, y + 135, 254, 3, 254);
	gfx.PutPixel(x + 246, y + 155, 254, 2, 254);
	gfx.PutPixel(x + 246, y + 156, 254, 226, 254);
	gfx.PutPixel(x + 246, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 163, 254, 195, 254);
	gfx.PutPixel(x + 246, y + 287, 254, 77, 254);
	gfx.PutPixel(x + 246, y + 288, 254, 222, 254);
	gfx.PutPixel(x + 246, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 304, 254, 203, 254);
	gfx.PutPixel(x + 246, y + 305, 254, 69, 254);
	gfx.PutPixel(x + 247, y + 119, 254, 3, 254);
	gfx.PutPixel(x + 247, y + 120, 254, 178, 254);
	gfx.PutPixel(x + 247, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 133, 254, 156, 254);
	gfx.PutPixel(x + 247, y + 134, 254, 1, 254);
	gfx.PutPixel(x + 247, y + 155, 254, 82, 254);
	gfx.PutPixel(x + 247, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 163, 254, 109, 254);
	gfx.PutPixel(x + 247, y + 288, 254, 8, 254);
	gfx.PutPixel(x + 247, y + 289, 254, 130, 254);
	gfx.PutPixel(x + 247, y + 290, 254, 248, 254);
	gfx.PutPixel(x + 247, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 306, 254, 198, 254);
	gfx.PutPixel(x + 247, y + 307, 254, 81, 254);
	gfx.PutPixel(x + 247, y + 308, 254, 1, 254);
	gfx.PutPixel(x + 248, y + 119, 254, 158, 254);
	gfx.PutPixel(x + 248, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 132, 254, 151, 254);
	gfx.PutPixel(x + 248, y + 133, 254, 0, 254);
	gfx.PutPixel(x + 248, y + 155, 254, 196, 254);
	gfx.PutPixel(x + 248, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 162, 254, 247, 254);
	gfx.PutPixel(x + 248, y + 163, 254, 18, 254);
	gfx.PutPixel(x + 248, y + 290, 254, 37, 254);
	gfx.PutPixel(x + 248, y + 291, 254, 186, 254);
	gfx.PutPixel(x + 248, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 308, 254, 217, 254);
	gfx.PutPixel(x + 248, y + 309, 254, 104, 254);
	gfx.PutPixel(x + 248, y + 310, 254, 11, 254);
	gfx.PutPixel(x + 249, y + 118, 254, 134, 254);
	gfx.PutPixel(x + 249, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 131, 254, 150, 254);
	gfx.PutPixel(x + 249, y + 132, 254, 0, 254);
	gfx.PutPixel(x + 249, y + 154, 254, 56, 254);
	gfx.PutPixel(x + 249, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 162, 254, 163, 254);
	gfx.PutPixel(x + 249, y + 292, 254, 88, 254);
	gfx.PutPixel(x + 249, y + 293, 254, 230, 254);
	gfx.PutPixel(x + 249, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 310, 254, 242, 254);
	gfx.PutPixel(x + 249, y + 311, 254, 153, 254);
	gfx.PutPixel(x + 249, y + 312, 254, 50, 254);
	gfx.PutPixel(x + 250, y + 117, 254, 109, 254);
	gfx.PutPixel(x + 250, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 130, 254, 156, 254);
	gfx.PutPixel(x + 250, y + 131, 254, 0, 254);
	gfx.PutPixel(x + 250, y + 154, 254, 179, 254);
	gfx.PutPixel(x + 250, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 162, 254, 60, 254);
	gfx.PutPixel(x + 250, y + 293, 254, 14, 254);
	gfx.PutPixel(x + 250, y + 294, 254, 151, 254);
	gfx.PutPixel(x + 250, y + 295, 254, 253, 254);
	gfx.PutPixel(x + 250, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 313, 254, 224, 254);
	gfx.PutPixel(x + 250, y + 314, 254, 154, 254);
	gfx.PutPixel(x + 250, y + 315, 254, 85, 254);
	gfx.PutPixel(x + 250, y + 316, 254, 40, 254);
	gfx.PutPixel(x + 250, y + 317, 254, 12, 254);
	gfx.PutPixel(x + 250, y + 318, 254, 0, 254);
	gfx.PutPixel(x + 251, y + 116, 254, 79, 254);
	gfx.PutPixel(x + 251, y + 117, 254, 253, 254);
	gfx.PutPixel(x + 251, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 129, 254, 163, 254);
	gfx.PutPixel(x + 251, y + 130, 254, 1, 254);
	gfx.PutPixel(x + 251, y + 153, 254, 53, 254);
	gfx.PutPixel(x + 251, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 161, 254, 209, 254);
	gfx.PutPixel(x + 251, y + 295, 254, 60, 254);
	gfx.PutPixel(x + 251, y + 296, 254, 216, 254);
	gfx.PutPixel(x + 251, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 319, 254, 253, 254);
	gfx.PutPixel(x + 251, y + 320, 254, 250, 254);
	gfx.PutPixel(x + 251, y + 321, 254, 245, 254);
	gfx.PutPixel(x + 251, y + 322, 254, 231, 254);
	gfx.PutPixel(x + 251, y + 323, 254, 199, 254);
	gfx.PutPixel(x + 251, y + 324, 254, 137, 254);
	gfx.PutPixel(x + 251, y + 325, 254, 27, 254);
	gfx.PutPixel(x + 252, y + 115, 254, 43, 254);
	gfx.PutPixel(x + 252, y + 116, 254, 242, 254);
	gfx.PutPixel(x + 252, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 128, 254, 179, 254);
	gfx.PutPixel(x + 252, y + 129, 254, 2, 254);
	gfx.PutPixel(x + 252, y + 153, 254, 181, 254);
	gfx.PutPixel(x + 252, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 161, 254, 93, 254);
	gfx.PutPixel(x + 252, y + 296, 254, 8, 254);
	gfx.PutPixel(x + 252, y + 297, 254, 145, 254);
	gfx.PutPixel(x + 252, y + 298, 254, 253, 254);
	gfx.PutPixel(x + 252, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 322, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 323, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 324, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 325, 254, 218, 254);
	gfx.PutPixel(x + 253, y + 114, 254, 18, 254);
	gfx.PutPixel(x + 253, y + 115, 254, 221, 254);
	gfx.PutPixel(x + 253, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 127, 254, 200, 254);
	gfx.PutPixel(x + 253, y + 128, 254, 9, 254);
	gfx.PutPixel(x + 253, y + 152, 254, 55, 254);
	gfx.PutPixel(x + 253, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 160, 254, 226, 254);
	gfx.PutPixel(x + 253, y + 161, 254, 4, 254);
	gfx.PutPixel(x + 253, y + 298, 254, 75, 254);
	gfx.PutPixel(x + 253, y + 299, 254, 242, 254);
	gfx.PutPixel(x + 253, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 322, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 323, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 324, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 325, 254, 215, 254);
	gfx.PutPixel(x + 254, y + 113, 254, 3, 254);
	gfx.PutPixel(x + 254, y + 114, 254, 189, 254);
	gfx.PutPixel(x + 254, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 126, 254, 217, 254);
	gfx.PutPixel(x + 254, y + 127, 254, 19, 254);
	gfx.PutPixel(x + 254, y + 152, 254, 184, 254);
	gfx.PutPixel(x + 254, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 160, 254, 113, 254);
	gfx.PutPixel(x + 254, y + 299, 254, 86, 254);
	gfx.PutPixel(x + 254, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 322, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 323, 255, 255, 255);
	gfx.PutPixel(x + 254, y + 324, 254, 253, 254);
	gfx.PutPixel(x + 254, y + 325, 254, 72, 254);
	gfx.PutPixel(x + 255, y + 113, 254, 148, 254);
	gfx.PutPixel(x + 255, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 125, 254, 232, 254);
	gfx.PutPixel(x + 255, y + 126, 254, 33, 254);
	gfx.PutPixel(x + 255, y + 151, 254, 69, 254);
	gfx.PutPixel(x + 255, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 159, 254, 231, 254);
	gfx.PutPixel(x + 255, y + 160, 254, 9, 254);
	gfx.PutPixel(x + 255, y + 299, 254, 29, 254);
	gfx.PutPixel(x + 255, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 322, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 323, 255, 255, 255);
	gfx.PutPixel(x + 255, y + 324, 254, 118, 254);
	gfx.PutPixel(x + 256, y + 112, 254, 100, 254);
	gfx.PutPixel(x + 256, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 124, 254, 242, 254);
	gfx.PutPixel(x + 256, y + 125, 254, 50, 254);
	gfx.PutPixel(x + 256, y + 150, 254, 2, 254);
	gfx.PutPixel(x + 256, y + 151, 254, 210, 254);
	gfx.PutPixel(x + 256, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 159, 254, 111, 254);
	gfx.PutPixel(x + 256, y + 298, 254, 46, 254);
	gfx.PutPixel(x + 256, y + 299, 254, 192, 254);
	gfx.PutPixel(x + 256, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 322, 255, 255, 255);
	gfx.PutPixel(x + 256, y + 323, 254, 131, 254);
	gfx.PutPixel(x + 257, y + 111, 254, 44, 254);
	gfx.PutPixel(x + 257, y + 112, 254, 246, 254);
	gfx.PutPixel(x + 257, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 123, 254, 250, 254);
	gfx.PutPixel(x + 257, y + 124, 254, 71, 254);
	gfx.PutPixel(x + 257, y + 150, 254, 101, 254);
	gfx.PutPixel(x + 257, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 158, 254, 226, 254);
	gfx.PutPixel(x + 257, y + 159, 254, 6, 254);
	gfx.PutPixel(x + 257, y + 295, 254, 3, 254);
	gfx.PutPixel(x + 257, y + 296, 254, 86, 254);
	gfx.PutPixel(x + 257, y + 297, 254, 194, 254);
	gfx.PutPixel(x + 257, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 257, y + 321, 254, 249, 254);
	gfx.PutPixel(x + 257, y + 322, 254, 105, 254);
	gfx.PutPixel(x + 258, y + 110, 254, 9, 254);
	gfx.PutPixel(x + 258, y + 111, 254, 213, 254);
	gfx.PutPixel(x + 258, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 123, 254, 112, 254);
	gfx.PutPixel(x + 258, y + 149, 254, 12, 254);
	gfx.PutPixel(x + 258, y + 150, 254, 231, 254);
	gfx.PutPixel(x + 258, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 158, 254, 100, 254);
	gfx.PutPixel(x + 258, y + 293, 254, 44, 254);
	gfx.PutPixel(x + 258, y + 294, 254, 137, 254);
	gfx.PutPixel(x + 258, y + 295, 254, 230, 254);
	gfx.PutPixel(x + 258, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 258, y + 320, 254, 215, 254);
	gfx.PutPixel(x + 258, y + 321, 254, 50, 254);
	gfx.PutPixel(x + 259, y + 110, 254, 157, 254);
	gfx.PutPixel(x + 259, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 122, 254, 164, 254);
	gfx.PutPixel(x + 259, y + 149, 254, 134, 254);
	gfx.PutPixel(x + 259, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 157, 254, 208, 254);
	gfx.PutPixel(x + 259, y + 158, 254, 2, 254);
	gfx.PutPixel(x + 259, y + 290, 254, 30, 254);
	gfx.PutPixel(x + 259, y + 291, 254, 118, 254);
	gfx.PutPixel(x + 259, y + 292, 254, 208, 254);
	gfx.PutPixel(x + 259, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 259, y + 318, 254, 239, 254);
	gfx.PutPixel(x + 259, y + 319, 254, 116, 254);
	gfx.PutPixel(x + 259, y + 320, 254, 6, 254);
	gfx.PutPixel(x + 260, y + 109, 254, 89, 254);
	gfx.PutPixel(x + 260, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 121, 254, 206, 254);
	gfx.PutPixel(x + 260, y + 122, 254, 8, 254);
	gfx.PutPixel(x + 260, y + 148, 254, 41, 254);
	gfx.PutPixel(x + 260, y + 149, 254, 248, 254);
	gfx.PutPixel(x + 260, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 157, 254, 65, 254);
	gfx.PutPixel(x + 260, y + 287, 254, 22, 254);
	gfx.PutPixel(x + 260, y + 288, 254, 107, 254);
	gfx.PutPixel(x + 260, y + 289, 254, 194, 254);
	gfx.PutPixel(x + 260, y + 290, 254, 253, 254);
	gfx.PutPixel(x + 260, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 314, 254, 253, 254);
	gfx.PutPixel(x + 260, y + 315, 254, 226, 254);
	gfx.PutPixel(x + 260, y + 316, 254, 187, 254);
	gfx.PutPixel(x + 260, y + 317, 254, 99, 254);
	gfx.PutPixel(x + 260, y + 318, 254, 13, 254);
	gfx.PutPixel(x + 261, y + 108, 254, 35, 254);
	gfx.PutPixel(x + 261, y + 109, 254, 241, 254);
	gfx.PutPixel(x + 261, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 120, 254, 235, 254);
	gfx.PutPixel(x + 261, y + 121, 254, 31, 254);
	gfx.PutPixel(x + 261, y + 147, 254, 0, 254);
	gfx.PutPixel(x + 261, y + 148, 254, 194, 254);
	gfx.PutPixel(x + 261, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 156, 254, 176, 254);
	gfx.PutPixel(x + 261, y + 284, 254, 17, 254);
	gfx.PutPixel(x + 261, y + 285, 254, 99, 254);
	gfx.PutPixel(x + 261, y + 286, 254, 186, 254);
	gfx.PutPixel(x + 261, y + 287, 254, 251, 254);
	gfx.PutPixel(x + 261, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 309, 254, 229, 254);
	gfx.PutPixel(x + 261, y + 310, 254, 162, 254);
	gfx.PutPixel(x + 261, y + 311, 254, 107, 254);
	gfx.PutPixel(x + 261, y + 312, 254, 54, 254);
	gfx.PutPixel(x + 261, y + 313, 254, 24, 254);
	gfx.PutPixel(x + 261, y + 314, 254, 3, 254);
	gfx.PutPixel(x + 262, y + 107, 254, 3, 254);
	gfx.PutPixel(x + 262, y + 108, 254, 201, 254);
	gfx.PutPixel(x + 262, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 119, 254, 251, 254);
	gfx.PutPixel(x + 262, y + 120, 254, 68, 254);
	gfx.PutPixel(x + 262, y + 147, 254, 101, 254);
	gfx.PutPixel(x + 262, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 155, 254, 245, 254);
	gfx.PutPixel(x + 262, y + 156, 254, 32, 254);
	gfx.PutPixel(x + 262, y + 281, 254, 13, 254);
	gfx.PutPixel(x + 262, y + 282, 254, 92, 254);
	gfx.PutPixel(x + 262, y + 283, 254, 180, 254);
	gfx.PutPixel(x + 262, y + 284, 254, 249, 254);
	gfx.PutPixel(x + 262, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 306, 254, 209, 254);
	gfx.PutPixel(x + 262, y + 307, 254, 124, 254);
	gfx.PutPixel(x + 262, y + 308, 254, 47, 254);
	gfx.PutPixel(x + 262, y + 309, 254, 0, 254);
	gfx.PutPixel(x + 263, y + 107, 254, 123, 254);
	gfx.PutPixel(x + 263, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 119, 254, 118, 254);
	gfx.PutPixel(x + 263, y + 146, 254, 23, 254);
	gfx.PutPixel(x + 263, y + 147, 254, 238, 254);
	gfx.PutPixel(x + 263, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 155, 254, 119, 254);
	gfx.PutPixel(x + 263, y + 278, 254, 8, 254);
	gfx.PutPixel(x + 263, y + 279, 254, 85, 254);
	gfx.PutPixel(x + 263, y + 280, 254, 173, 254);
	gfx.PutPixel(x + 263, y + 281, 254, 246, 254);
	gfx.PutPixel(x + 263, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 303, 254, 224, 254);
	gfx.PutPixel(x + 263, y + 304, 254, 129, 254);
	gfx.PutPixel(x + 263, y + 305, 254, 38, 254);
	gfx.PutPixel(x + 264, y + 106, 254, 47, 254);
	gfx.PutPixel(x + 264, y + 107, 254, 249, 254);
	gfx.PutPixel(x + 264, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 118, 254, 185, 254);
	gfx.PutPixel(x + 264, y + 119, 254, 0, 254);
	gfx.PutPixel(x + 264, y + 146, 254, 167, 254);
	gfx.PutPixel(x + 264, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 154, 254, 209, 254);
	gfx.PutPixel(x + 264, y + 155, 254, 3, 254);
	gfx.PutPixel(x + 264, y + 275, 254, 4, 254);
	gfx.PutPixel(x + 264, y + 276, 254, 74, 254);
	gfx.PutPixel(x + 264, y + 277, 254, 163, 254);
	gfx.PutPixel(x + 264, y + 278, 254, 242, 254);
	gfx.PutPixel(x + 264, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 300, 254, 245, 254);
	gfx.PutPixel(x + 264, y + 301, 254, 163, 254);
	gfx.PutPixel(x + 264, y + 302, 254, 67, 254);
	gfx.PutPixel(x + 264, y + 303, 254, 1, 254);
	gfx.PutPixel(x + 265, y + 105, 254, 3, 254);
	gfx.PutPixel(x + 265, y + 106, 254, 206, 254);
	gfx.PutPixel(x + 265, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 117, 254, 235, 254);
	gfx.PutPixel(x + 265, y + 118, 254, 25, 254);
	gfx.PutPixel(x + 265, y + 145, 254, 84, 254);
	gfx.PutPixel(x + 265, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 153, 254, 252, 254);
	gfx.PutPixel(x + 265, y + 154, 254, 56, 254);
	gfx.PutPixel(x + 265, y + 272, 254, 1, 254);
	gfx.PutPixel(x + 265, y + 273, 254, 65, 254);
	gfx.PutPixel(x + 265, y + 274, 254, 152, 254);
	gfx.PutPixel(x + 265, y + 275, 254, 236, 254);
	gfx.PutPixel(x + 265, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 298, 254, 202, 254);
	gfx.PutPixel(x + 265, y + 299, 254, 102, 254);
	gfx.PutPixel(x + 265, y + 300, 254, 14, 254);
	gfx.PutPixel(x + 266, y + 105, 254, 118, 254);
	gfx.PutPixel(x + 266, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 117, 254, 81, 254);
	gfx.PutPixel(x + 266, y + 144, 254, 26, 254);
	gfx.PutPixel(x + 266, y + 145, 254, 236, 254);
	gfx.PutPixel(x + 266, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 153, 254, 139, 254);
	gfx.PutPixel(x + 266, y + 270, 254, 55, 254);
	gfx.PutPixel(x + 266, y + 271, 254, 144, 254);
	gfx.PutPixel(x + 266, y + 272, 254, 229, 254);
	gfx.PutPixel(x + 266, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 295, 254, 239, 254);
	gfx.PutPixel(x + 266, y + 296, 254, 148, 254);
	gfx.PutPixel(x + 266, y + 297, 254, 48, 254);
	gfx.PutPixel(x + 267, y + 104, 254, 35, 254);
	gfx.PutPixel(x + 267, y + 105, 254, 245, 254);
	gfx.PutPixel(x + 267, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 116, 254, 157, 254);
	gfx.PutPixel(x + 267, y + 143, 254, 0, 254);
	gfx.PutPixel(x + 267, y + 144, 254, 186, 254);
	gfx.PutPixel(x + 267, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 152, 254, 213, 254);
	gfx.PutPixel(x + 267, y + 153, 254, 7, 254);
	gfx.PutPixel(x + 267, y + 267, 254, 46, 254);
	gfx.PutPixel(x + 267, y + 268, 254, 135, 254);
	gfx.PutPixel(x + 267, y + 269, 254, 222, 254);
	gfx.PutPixel(x + 267, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 292, 254, 252, 254);
	gfx.PutPixel(x + 267, y + 293, 254, 188, 254);
	gfx.PutPixel(x + 267, y + 294, 254, 93, 254);
	gfx.PutPixel(x + 267, y + 295, 254, 9, 254);
	gfx.PutPixel(x + 268, y + 104, 254, 183, 254);
	gfx.PutPixel(x + 268, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 115, 254, 219, 254);
	gfx.PutPixel(x + 268, y + 116, 254, 11, 254);
	gfx.PutPixel(x + 268, y + 143, 254, 112, 254);
	gfx.PutPixel(x + 268, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 151, 254, 250, 254);
	gfx.PutPixel(x + 268, y + 152, 254, 52, 254);
	gfx.PutPixel(x + 268, y + 264, 254, 34, 254);
	gfx.PutPixel(x + 268, y + 265, 254, 124, 254);
	gfx.PutPixel(x + 268, y + 266, 254, 213, 254);
	gfx.PutPixel(x + 268, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 290, 254, 209, 254);
	gfx.PutPixel(x + 268, y + 291, 254, 118, 254);
	gfx.PutPixel(x + 268, y + 292, 254, 28, 254);
	gfx.PutPixel(x + 269, y + 103, 254, 80, 254);
	gfx.PutPixel(x + 269, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 114, 254, 251, 254);
	gfx.PutPixel(x + 269, y + 115, 254, 55, 254);
	gfx.PutPixel(x + 269, y + 142, 254, 44, 254);
	gfx.PutPixel(x + 269, y + 143, 254, 247, 254);
	gfx.PutPixel(x + 269, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 151, 254, 126, 254);
	gfx.PutPixel(x + 269, y + 261, 254, 18, 254);
	gfx.PutPixel(x + 269, y + 262, 254, 106, 254);
	gfx.PutPixel(x + 269, y + 263, 254, 198, 254);
	gfx.PutPixel(x + 269, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 287, 254, 219, 254);
	gfx.PutPixel(x + 269, y + 288, 254, 132, 254);
	gfx.PutPixel(x + 269, y + 289, 254, 43, 254);
	gfx.PutPixel(x + 270, y + 102, 254, 8, 254);
	gfx.PutPixel(x + 270, y + 103, 254, 223, 254);
	gfx.PutPixel(x + 270, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 114, 254, 139, 254);
	gfx.PutPixel(x + 270, y + 141, 254, 11, 254);
	gfx.PutPixel(x + 270, y + 142, 254, 213, 254);
	gfx.PutPixel(x + 270, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 150, 254, 192, 254);
	gfx.PutPixel(x + 270, y + 151, 254, 2, 254);
	gfx.PutPixel(x + 270, y + 258, 254, 2, 254);
	gfx.PutPixel(x + 270, y + 259, 254, 76, 254);
	gfx.PutPixel(x + 270, y + 260, 254, 173, 254);
	gfx.PutPixel(x + 270, y + 261, 254, 249, 254);
	gfx.PutPixel(x + 270, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 284, 254, 224, 254);
	gfx.PutPixel(x + 270, y + 285, 254, 140, 254);
	gfx.PutPixel(x + 270, y + 286, 254, 53, 254);
	gfx.PutPixel(x + 271, y + 102, 254, 123, 254);
	gfx.PutPixel(x + 271, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 113, 254, 225, 254);
	gfx.PutPixel(x + 271, y + 114, 254, 10, 254);
	gfx.PutPixel(x + 271, y + 141, 254, 170, 254);
	gfx.PutPixel(x + 271, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 149, 254, 236, 254);
	gfx.PutPixel(x + 271, y + 150, 254, 27, 254);
	gfx.PutPixel(x + 271, y + 256, 254, 27, 254);
	gfx.PutPixel(x + 271, y + 257, 254, 128, 254);
	gfx.PutPixel(x + 271, y + 258, 254, 227, 254);
	gfx.PutPixel(x + 271, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 281, 254, 225, 254);
	gfx.PutPixel(x + 271, y + 282, 254, 142, 254);
	gfx.PutPixel(x + 271, y + 283, 254, 56, 254);
	gfx.PutPixel(x + 271, y + 284, 254, 0, 254);
	gfx.PutPixel(x + 272, y + 101, 254, 22, 254);
	gfx.PutPixel(x + 272, y + 102, 254, 242, 254);
	gfx.PutPixel(x + 272, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 113, 254, 78, 254);
	gfx.PutPixel(x + 272, y + 140, 254, 115, 254);
	gfx.PutPixel(x + 272, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 149, 254, 78, 254);
	gfx.PutPixel(x + 272, y + 254, 254, 43, 254);
	gfx.PutPixel(x + 272, y + 255, 254, 163, 254);
	gfx.PutPixel(x + 272, y + 256, 254, 250, 254);
	gfx.PutPixel(x + 272, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 278, 254, 223, 254);
	gfx.PutPixel(x + 272, y + 279, 254, 139, 254);
	gfx.PutPixel(x + 272, y + 280, 254, 55, 254);
	gfx.PutPixel(x + 272, y + 281, 254, 0, 254);
	gfx.PutPixel(x + 273, y + 101, 254, 150, 254);
	gfx.PutPixel(x + 273, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 112, 254, 176, 254);
	gfx.PutPixel(x + 273, y + 139, 254, 63, 254);
	gfx.PutPixel(x + 273, y + 140, 254, 251, 254);
	gfx.PutPixel(x + 273, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 148, 254, 137, 254);
	gfx.PutPixel(x + 273, y + 252, 254, 30, 254);
	gfx.PutPixel(x + 273, y + 253, 254, 165, 254);
	gfx.PutPixel(x + 273, y + 254, 254, 253, 254);
	gfx.PutPixel(x + 273, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 275, 254, 221, 254);
	gfx.PutPixel(x + 273, y + 276, 254, 136, 254);
	gfx.PutPixel(x + 273, y + 277, 254, 52, 254);
	gfx.PutPixel(x + 274, y + 100, 254, 32, 254);
	gfx.PutPixel(x + 274, y + 101, 254, 249, 254);
	gfx.PutPixel(x + 274, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 111, 254, 243, 254);
	gfx.PutPixel(x + 274, y + 112, 254, 29, 254);
	gfx.PutPixel(x + 274, y + 138, 254, 27, 254);
	gfx.PutPixel(x + 274, y + 139, 254, 232, 254);
	gfx.PutPixel(x + 274, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 147, 254, 188, 254);
	gfx.PutPixel(x + 274, y + 148, 254, 2, 254);
	gfx.PutPixel(x + 274, y + 250, 254, 0, 254);
	gfx.PutPixel(x + 274, y + 251, 254, 111, 254);
	gfx.PutPixel(x + 274, y + 252, 254, 245, 254);
	gfx.PutPixel(x + 274, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 272, 254, 222, 254);
	gfx.PutPixel(x + 274, y + 273, 254, 137, 254);
	gfx.PutPixel(x + 274, y + 274, 254, 51, 254);
	gfx.PutPixel(x + 275, y + 100, 254, 157, 254);
	gfx.PutPixel(x + 275, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 111, 254, 115, 254);
	gfx.PutPixel(x + 275, y + 137, 254, 10, 254);
	gfx.PutPixel(x + 275, y + 138, 254, 202, 254);
	gfx.PutPixel(x + 275, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 146, 254, 226, 254);
	gfx.PutPixel(x + 275, y + 147, 254, 20, 254);
	gfx.PutPixel(x + 275, y + 249, 254, 7, 254);
	gfx.PutPixel(x + 275, y + 250, 254, 174, 254);
	gfx.PutPixel(x + 275, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 269, 254, 229, 254);
	gfx.PutPixel(x + 275, y + 270, 254, 142, 254);
	gfx.PutPixel(x + 275, y + 271, 254, 53, 254);
	gfx.PutPixel(x + 276, y + 99, 254, 33, 254);
	gfx.PutPixel(x + 276, y + 100, 254, 251, 254);
	gfx.PutPixel(x + 276, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 110, 254, 214, 254);
	gfx.PutPixel(x + 276, y + 111, 254, 3, 254);
	gfx.PutPixel(x + 276, y + 136, 254, 2, 254);
	gfx.PutPixel(x + 276, y + 137, 254, 178, 254);
	gfx.PutPixel(x + 276, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 145, 254, 245, 254);
	gfx.PutPixel(x + 276, y + 146, 254, 52, 254);
	gfx.PutPixel(x + 276, y + 249, 254, 177, 254);
	gfx.PutPixel(x + 276, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 266, 254, 239, 254);
	gfx.PutPixel(x + 276, y + 267, 254, 156, 254);
	gfx.PutPixel(x + 276, y + 268, 254, 65, 254);
	gfx.PutPixel(x + 276, y + 269, 254, 1, 254);
	gfx.PutPixel(x + 277, y + 99, 254, 148, 254);
	gfx.PutPixel(x + 277, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 110, 254, 79, 254);
	gfx.PutPixel(x + 277, y + 136, 254, 149, 254);
	gfx.PutPixel(x + 277, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 145, 254, 86, 254);
	gfx.PutPixel(x + 277, y + 248, 254, 94, 254);
	gfx.PutPixel(x + 277, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 263, 254, 250, 254);
	gfx.PutPixel(x + 277, y + 264, 254, 179, 254);
	gfx.PutPixel(x + 277, y + 265, 254, 84, 254);
	gfx.PutPixel(x + 277, y + 266, 254, 7, 254);
	gfx.PutPixel(x + 278, y + 98, 254, 20, 254);
	gfx.PutPixel(x + 278, y + 99, 254, 245, 254);
	gfx.PutPixel(x + 278, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 109, 254, 194, 254);
	gfx.PutPixel(x + 278, y + 135, 254, 118, 254);
	gfx.PutPixel(x + 278, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 144, 254, 127, 254);
	gfx.PutPixel(x + 278, y + 248, 254, 198, 254);
	gfx.PutPixel(x + 278, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 261, 254, 214, 254);
	gfx.PutPixel(x + 278, y + 262, 254, 114, 254);
	gfx.PutPixel(x + 278, y + 263, 254, 22, 254);
	gfx.PutPixel(x + 279, y + 98, 254, 122, 254);
	gfx.PutPixel(x + 279, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 109, 254, 58, 254);
	gfx.PutPixel(x + 279, y + 134, 254, 87, 254);
	gfx.PutPixel(x + 279, y + 135, 254, 253, 254);
	gfx.PutPixel(x + 279, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 143, 254, 167, 254);
	gfx.PutPixel(x + 279, y + 144, 254, 0, 254);
	gfx.PutPixel(x + 279, y + 248, 254, 242, 254);
	gfx.PutPixel(x + 279, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 258, 254, 252, 254);
	gfx.PutPixel(x + 279, y + 259, 254, 172, 254);
	gfx.PutPixel(x + 279, y + 260, 254, 62, 254);
	gfx.PutPixel(x + 280, y + 97, 254, 2, 254);
	gfx.PutPixel(x + 280, y + 98, 254, 225, 254);
	gfx.PutPixel(x + 280, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 108, 254, 190, 254);
	gfx.PutPixel(x + 280, y + 133, 254, 73, 254);
	gfx.PutPixel(x + 280, y + 134, 254, 248, 254);
	gfx.PutPixel(x + 280, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 142, 254, 191, 254);
	gfx.PutPixel(x + 280, y + 143, 254, 6, 254);
	gfx.PutPixel(x + 280, y + 248, 254, 203, 254);
	gfx.PutPixel(x + 280, y + 249, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 250, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 256, 254, 249, 254);
	gfx.PutPixel(x + 280, y + 257, 254, 156, 254);
	gfx.PutPixel(x + 280, y + 258, 254, 36, 254);
	gfx.PutPixel(x + 281, y + 97, 254, 76, 254);
	gfx.PutPixel(x + 281, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 108, 254, 66, 254);
	gfx.PutPixel(x + 281, y + 132, 254, 72, 254);
	gfx.PutPixel(x + 281, y + 133, 254, 247, 254);
	gfx.PutPixel(x + 281, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 141, 254, 210, 254);
	gfx.PutPixel(x + 281, y + 142, 254, 15, 254);
	gfx.PutPixel(x + 281, y + 248, 254, 24, 254);
	gfx.PutPixel(x + 281, y + 249, 254, 156, 254);
	gfx.PutPixel(x + 281, y + 250, 254, 239, 254);
	gfx.PutPixel(x + 281, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 281, y + 253, 254, 237, 254);
	gfx.PutPixel(x + 281, y + 254, 254, 185, 254);
	gfx.PutPixel(x + 281, y + 255, 254, 110, 254);
	gfx.PutPixel(x + 281, y + 256, 254, 24, 254);
	gfx.PutPixel(x + 282, y + 97, 254, 171, 254);
	gfx.PutPixel(x + 282, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 107, 254, 201, 254);
	gfx.PutPixel(x + 282, y + 131, 254, 71, 254);
	gfx.PutPixel(x + 282, y + 132, 254, 247, 254);
	gfx.PutPixel(x + 282, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 282, y + 140, 254, 226, 254);
	gfx.PutPixel(x + 282, y + 141, 254, 27, 254);
	gfx.PutPixel(x + 282, y + 251, 254, 12, 254);
	gfx.PutPixel(x + 282, y + 252, 254, 10, 254);
	gfx.PutPixel(x + 283, y + 96, 254, 15, 254);
	gfx.PutPixel(x + 283, y + 97, 254, 247, 254);
	gfx.PutPixel(x + 283, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 107, 254, 94, 254);
	gfx.PutPixel(x + 283, y + 130, 254, 70, 254);
	gfx.PutPixel(x + 283, y + 131, 254, 247, 254);
	gfx.PutPixel(x + 283, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 283, y + 139, 254, 233, 254);
	gfx.PutPixel(x + 283, y + 140, 254, 39, 254);
	gfx.PutPixel(x + 284, y + 96, 254, 94, 254);
	gfx.PutPixel(x + 284, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 106, 254, 234, 254);
	gfx.PutPixel(x + 284, y + 107, 254, 6, 254);
	gfx.PutPixel(x + 284, y + 129, 254, 70, 254);
	gfx.PutPixel(x + 284, y + 130, 254, 246, 254);
	gfx.PutPixel(x + 284, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 284, y + 138, 254, 237, 254);
	gfx.PutPixel(x + 284, y + 139, 254, 46, 254);
	gfx.PutPixel(x + 285, y + 96, 254, 171, 254);
	gfx.PutPixel(x + 285, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 106, 254, 140, 254);
	gfx.PutPixel(x + 285, y + 128, 254, 95, 254);
	gfx.PutPixel(x + 285, y + 129, 254, 249, 254);
	gfx.PutPixel(x + 285, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 285, y + 137, 254, 242, 254);
	gfx.PutPixel(x + 285, y + 138, 254, 54, 254);
	gfx.PutPixel(x + 286, y + 95, 254, 4, 254);
	gfx.PutPixel(x + 286, y + 96, 254, 241, 254);
	gfx.PutPixel(x + 286, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 106, 254, 49, 254);
	gfx.PutPixel(x + 286, y + 127, 254, 130, 254);
	gfx.PutPixel(x + 286, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 136, 254, 241, 254);
	gfx.PutPixel(x + 286, y + 137, 254, 59, 254);
	gfx.PutPixel(x + 287, y + 95, 254, 54, 254);
	gfx.PutPixel(x + 287, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 105, 254, 215, 254);
	gfx.PutPixel(x + 287, y + 125, 254, 5, 254);
	gfx.PutPixel(x + 287, y + 126, 254, 163, 254);
	gfx.PutPixel(x + 287, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 135, 254, 240, 254);
	gfx.PutPixel(x + 287, y + 136, 254, 56, 254);
	gfx.PutPixel(x + 288, y + 95, 254, 113, 254);
	gfx.PutPixel(x + 288, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 105, 254, 137, 254);
	gfx.PutPixel(x + 288, y + 124, 254, 17, 254);
	gfx.PutPixel(x + 288, y + 125, 254, 192, 254);
	gfx.PutPixel(x + 288, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 134, 254, 238, 254);
	gfx.PutPixel(x + 288, y + 135, 254, 53, 254);
	gfx.PutPixel(x + 289, y + 95, 254, 162, 254);
	gfx.PutPixel(x + 289, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 105, 254, 74, 254);
	gfx.PutPixel(x + 289, y + 114, 254, 2, 254);
	gfx.PutPixel(x + 289, y + 115, 254, 34, 254);
	gfx.PutPixel(x + 289, y + 116, 254, 71, 254);
	gfx.PutPixel(x + 289, y + 117, 254, 108, 254);
	gfx.PutPixel(x + 289, y + 118, 254, 143, 254);
	gfx.PutPixel(x + 289, y + 119, 254, 167, 254);
	gfx.PutPixel(x + 289, y + 120, 254, 185, 254);
	gfx.PutPixel(x + 289, y + 121, 254, 202, 254);
	gfx.PutPixel(x + 289, y + 122, 254, 219, 254);
	gfx.PutPixel(x + 289, y + 123, 254, 236, 254);
	gfx.PutPixel(x + 289, y + 124, 254, 252, 254);
	gfx.PutPixel(x + 289, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 134, 254, 253, 254);
	gfx.PutPixel(x + 289, y + 135, 254, 235, 254);
	gfx.PutPixel(x + 289, y + 136, 254, 211, 254);
	gfx.PutPixel(x + 289, y + 137, 254, 187, 254);
	gfx.PutPixel(x + 289, y + 138, 254, 161, 254);
	gfx.PutPixel(x + 289, y + 139, 254, 130, 254);
	gfx.PutPixel(x + 289, y + 140, 254, 97, 254);
	gfx.PutPixel(x + 289, y + 141, 254, 64, 254);
	gfx.PutPixel(x + 289, y + 142, 254, 30, 254);
	gfx.PutPixel(x + 289, y + 143, 254, 1, 254);
	gfx.PutPixel(x + 290, y + 95, 254, 198, 254);
	gfx.PutPixel(x + 290, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 105, 254, 38, 254);
	gfx.PutPixel(x + 290, y + 110, 254, 30, 254);
	gfx.PutPixel(x + 290, y + 111, 254, 101, 254);
	gfx.PutPixel(x + 290, y + 112, 254, 160, 254);
	gfx.PutPixel(x + 290, y + 113, 254, 214, 254);
	gfx.PutPixel(x + 290, y + 114, 254, 249, 254);
	gfx.PutPixel(x + 290, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 143, 254, 245, 254);
	gfx.PutPixel(x + 290, y + 144, 254, 206, 254);
	gfx.PutPixel(x + 290, y + 145, 254, 165, 254);
	gfx.PutPixel(x + 290, y + 146, 254, 124, 254);
	gfx.PutPixel(x + 290, y + 147, 254, 79, 254);
	gfx.PutPixel(x + 290, y + 148, 254, 30, 254);
	gfx.PutPixel(x + 291, y + 95, 254, 229, 254);
	gfx.PutPixel(x + 291, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 105, 254, 23, 254);
	gfx.PutPixel(x + 291, y + 107, 254, 8, 254);
	gfx.PutPixel(x + 291, y + 108, 254, 104, 254);
	gfx.PutPixel(x + 291, y + 109, 254, 200, 254);
	gfx.PutPixel(x + 291, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 149, 254, 237, 254);
	gfx.PutPixel(x + 291, y + 150, 254, 189, 254);
	gfx.PutPixel(x + 291, y + 151, 254, 138, 254);
	gfx.PutPixel(x + 291, y + 152, 254, 82, 254);
	gfx.PutPixel(x + 291, y + 153, 254, 26, 254);
	gfx.PutPixel(x + 292, y + 95, 254, 243, 254);
	gfx.PutPixel(x + 292, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 105, 254, 33, 254);
	gfx.PutPixel(x + 292, y + 106, 254, 110, 254);
	gfx.PutPixel(x + 292, y + 107, 254, 234, 254);
	gfx.PutPixel(x + 292, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 154, 254, 225, 254);
	gfx.PutPixel(x + 292, y + 155, 254, 168, 254);
	gfx.PutPixel(x + 292, y + 156, 254, 106, 254);
	gfx.PutPixel(x + 292, y + 157, 254, 42, 254);
	gfx.PutPixel(x + 292, y + 158, 254, 0, 254);
	gfx.PutPixel(x + 293, y + 95, 254, 251, 254);
	gfx.PutPixel(x + 293, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 105, 254, 226, 254);
	gfx.PutPixel(x + 293, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 158, 254, 233, 254);
	gfx.PutPixel(x + 293, y + 159, 254, 170, 254);
	gfx.PutPixel(x + 293, y + 160, 254, 101, 254);
	gfx.PutPixel(x + 293, y + 161, 254, 29, 254);
	gfx.PutPixel(x + 294, y + 95, 254, 237, 254);
	gfx.PutPixel(x + 294, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 162, 254, 213, 254);
	gfx.PutPixel(x + 294, y + 163, 254, 139, 254);
	gfx.PutPixel(x + 294, y + 164, 254, 60, 254);
	gfx.PutPixel(x + 294, y + 165, 254, 1, 254);
	gfx.PutPixel(x + 295, y + 95, 254, 199, 254);
	gfx.PutPixel(x + 295, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 165, 254, 231, 254);
	gfx.PutPixel(x + 295, y + 166, 254, 152, 254);
	gfx.PutPixel(x + 295, y + 167, 254, 65, 254);
	gfx.PutPixel(x + 295, y + 168, 254, 1, 254);
	gfx.PutPixel(x + 296, y + 95, 254, 123, 254);
	gfx.PutPixel(x + 296, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 168, 254, 225, 254);
	gfx.PutPixel(x + 296, y + 169, 254, 125, 254);
	gfx.PutPixel(x + 296, y + 170, 254, 22, 254);
	gfx.PutPixel(x + 297, y + 95, 254, 13, 254);
	gfx.PutPixel(x + 297, y + 96, 254, 217, 254);
	gfx.PutPixel(x + 297, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 123, 254, 192, 254);
	gfx.PutPixel(x + 297, y + 124, 254, 32, 254);
	gfx.PutPixel(x + 297, y + 125, 254, 0, 254);
	gfx.PutPixel(x + 297, y + 131, 254, 5, 254);
	gfx.PutPixel(x + 297, y + 132, 254, 13, 254);
	gfx.PutPixel(x + 297, y + 133, 254, 29, 254);
	gfx.PutPixel(x + 297, y + 134, 254, 50, 254);
	gfx.PutPixel(x + 297, y + 135, 254, 72, 254);
	gfx.PutPixel(x + 297, y + 136, 254, 102, 254);
	gfx.PutPixel(x + 297, y + 137, 254, 132, 254);
	gfx.PutPixel(x + 297, y + 138, 254, 164, 254);
	gfx.PutPixel(x + 297, y + 139, 254, 201, 254);
	gfx.PutPixel(x + 297, y + 140, 254, 238, 254);
	gfx.PutPixel(x + 297, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 170, 254, 247, 254);
	gfx.PutPixel(x + 297, y + 171, 254, 145, 254);
	gfx.PutPixel(x + 297, y + 172, 254, 19, 254);
	gfx.PutPixel(x + 298, y + 96, 254, 17, 254);
	gfx.PutPixel(x + 298, y + 97, 254, 127, 254);
	gfx.PutPixel(x + 298, y + 98, 254, 193, 254);
	gfx.PutPixel(x + 298, y + 99, 254, 216, 254);
	gfx.PutPixel(x + 298, y + 100, 254, 207, 254);
	gfx.PutPixel(x + 298, y + 101, 254, 168, 254);
	gfx.PutPixel(x + 298, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 121, 254, 253, 254);
	gfx.PutPixel(x + 298, y + 122, 254, 132, 254);
	gfx.PutPixel(x + 298, y + 123, 254, 2, 254);
	gfx.PutPixel(x + 298, y + 141, 254, 22, 254);
	gfx.PutPixel(x + 298, y + 142, 254, 67, 254);
	gfx.PutPixel(x + 298, y + 143, 254, 111, 254);
	gfx.PutPixel(x + 298, y + 144, 254, 156, 254);
	gfx.PutPixel(x + 298, y + 145, 254, 201, 254);
	gfx.PutPixel(x + 298, y + 146, 254, 244, 254);
	gfx.PutPixel(x + 298, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 172, 254, 234, 254);
	gfx.PutPixel(x + 298, y + 173, 254, 60, 254);
	gfx.PutPixel(x + 299, y + 101, 254, 29, 254);
	gfx.PutPixel(x + 299, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 120, 254, 225, 254);
	gfx.PutPixel(x + 299, y + 121, 254, 65, 254);
	gfx.PutPixel(x + 299, y + 146, 254, 1, 254);
	gfx.PutPixel(x + 299, y + 147, 254, 35, 254);
	gfx.PutPixel(x + 299, y + 148, 254, 84, 254);
	gfx.PutPixel(x + 299, y + 149, 254, 135, 254);
	gfx.PutPixel(x + 299, y + 150, 254, 186, 254);
	gfx.PutPixel(x + 299, y + 151, 254, 237, 254);
	gfx.PutPixel(x + 299, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 173, 254, 230, 254);
	gfx.PutPixel(x + 300, y + 101, 254, 33, 254);
	gfx.PutPixel(x + 300, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 119, 254, 160, 254);
	gfx.PutPixel(x + 300, y + 120, 254, 14, 254);
	gfx.PutPixel(x + 300, y + 151, 254, 0, 254);
	gfx.PutPixel(x + 300, y + 152, 254, 37, 254);
	gfx.PutPixel(x + 300, y + 153, 254, 91, 254);
	gfx.PutPixel(x + 300, y + 154, 254, 146, 254);
	gfx.PutPixel(x + 300, y + 155, 254, 204, 254);
	gfx.PutPixel(x + 300, y + 156, 254, 250, 254);
	gfx.PutPixel(x + 300, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 173, 254, 227, 254);
	gfx.PutPixel(x + 301, y + 101, 254, 21, 254);
	gfx.PutPixel(x + 301, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 117, 254, 217, 254);
	gfx.PutPixel(x + 301, y + 118, 254, 67, 254);
	gfx.PutPixel(x + 301, y + 156, 254, 9, 254);
	gfx.PutPixel(x + 301, y + 157, 254, 64, 254);
	gfx.PutPixel(x + 301, y + 158, 254, 125, 254);
	gfx.PutPixel(x + 301, y + 159, 254, 186, 254);
	gfx.PutPixel(x + 301, y + 160, 254, 244, 254);
	gfx.PutPixel(x + 301, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 173, 254, 117, 254);
	gfx.PutPixel(x + 302, y + 101, 254, 0, 254);
	gfx.PutPixel(x + 302, y + 102, 254, 240, 254);
	gfx.PutPixel(x + 302, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 115, 254, 240, 254);
	gfx.PutPixel(x + 302, y + 116, 254, 122, 254);
	gfx.PutPixel(x + 302, y + 117, 254, 6, 254);
	gfx.PutPixel(x + 302, y + 160, 254, 4, 254);
	gfx.PutPixel(x + 302, y + 161, 254, 58, 254);
	gfx.PutPixel(x + 302, y + 162, 254, 124, 254);
	gfx.PutPixel(x + 302, y + 163, 254, 193, 254);
	gfx.PutPixel(x + 302, y + 164, 254, 249, 254);
	gfx.PutPixel(x + 302, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 171, 254, 221, 254);
	gfx.PutPixel(x + 302, y + 172, 254, 105, 254);
	gfx.PutPixel(x + 303, y + 102, 254, 165, 254);
	gfx.PutPixel(x + 303, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 113, 254, 244, 254);
	gfx.PutPixel(x + 303, y + 114, 254, 143, 254);
	gfx.PutPixel(x + 303, y + 115, 254, 20, 254);
	gfx.PutPixel(x + 303, y + 164, 254, 8, 254);
	gfx.PutPixel(x + 303, y + 165, 254, 61, 254);
	gfx.PutPixel(x + 303, y + 166, 254, 105, 254);
	gfx.PutPixel(x + 303, y + 167, 254, 121, 254);
	gfx.PutPixel(x + 303, y + 168, 254, 120, 254);
	gfx.PutPixel(x + 303, y + 169, 254, 94, 254);
	gfx.PutPixel(x + 303, y + 170, 254, 56, 254);
	gfx.PutPixel(x + 303, y + 171, 254, 0, 254);
	gfx.PutPixel(x + 304, y + 102, 254, 34, 254);
	gfx.PutPixel(x + 304, y + 103, 254, 239, 254);
	gfx.PutPixel(x + 304, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 111, 254, 222, 254);
	gfx.PutPixel(x + 304, y + 112, 254, 124, 254);
	gfx.PutPixel(x + 304, y + 113, 254, 18, 254);
	gfx.PutPixel(x + 305, y + 103, 254, 41, 254);
	gfx.PutPixel(x + 305, y + 104, 254, 179, 254);
	gfx.PutPixel(x + 305, y + 105, 254, 242, 254);
	gfx.PutPixel(x + 305, y + 106, 254, 245, 254);
	gfx.PutPixel(x + 305, y + 107, 254, 225, 254);
	gfx.PutPixel(x + 305, y + 108, 254, 184, 254);
	gfx.PutPixel(x + 305, y + 109, 254, 128, 254);
	gfx.PutPixel(x + 305, y + 110, 254, 55, 254);
	gfx.PutPixel(x + 312, y + 294, 254, 13, 254);
	gfx.PutPixel(x + 312, y + 295, 254, 73, 254);
	gfx.PutPixel(x + 312, y + 296, 254, 119, 254);
	gfx.PutPixel(x + 312, y + 297, 254, 164, 254);
	gfx.PutPixel(x + 312, y + 298, 254, 204, 254);
	gfx.PutPixel(x + 312, y + 299, 254, 223, 254);
	gfx.PutPixel(x + 312, y + 300, 254, 237, 254);
	gfx.PutPixel(x + 312, y + 301, 254, 250, 254);
	gfx.PutPixel(x + 312, y + 302, 254, 246, 254);
	gfx.PutPixel(x + 312, y + 303, 254, 232, 254);
	gfx.PutPixel(x + 312, y + 304, 254, 213, 254);
	gfx.PutPixel(x + 312, y + 305, 254, 172, 254);
	gfx.PutPixel(x + 312, y + 306, 254, 125, 254);
	gfx.PutPixel(x + 312, y + 307, 254, 60, 254);
	gfx.PutPixel(x + 312, y + 308, 254, 1, 254);
	gfx.PutPixel(x + 313, y + 291, 254, 26, 254);
	gfx.PutPixel(x + 313, y + 292, 254, 114, 254);
	gfx.PutPixel(x + 313, y + 293, 254, 189, 254);
	gfx.PutPixel(x + 313, y + 294, 254, 249, 254);
	gfx.PutPixel(x + 313, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 308, 254, 231, 254);
	gfx.PutPixel(x + 313, y + 309, 254, 137, 254);
	gfx.PutPixel(x + 313, y + 310, 254, 20, 254);
	gfx.PutPixel(x + 314, y + 289, 254, 65, 254);
	gfx.PutPixel(x + 314, y + 290, 254, 171, 254);
	gfx.PutPixel(x + 314, y + 291, 254, 250, 254);
	gfx.PutPixel(x + 314, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 314, y + 310, 254, 244, 254);
	gfx.PutPixel(x + 314, y + 311, 254, 124, 254);
	gfx.PutPixel(x + 314, y + 312, 254, 3, 254);
	gfx.PutPixel(x + 315, y + 287, 254, 69, 254);
	gfx.PutPixel(x + 315, y + 288, 254, 201, 254);
	gfx.PutPixel(x + 315, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 315, y + 312, 254, 202, 254);
	gfx.PutPixel(x + 315, y + 313, 254, 33, 254);
	gfx.PutPixel(x + 316, y + 285, 254, 51, 254);
	gfx.PutPixel(x + 316, y + 286, 254, 191, 254);
	gfx.PutPixel(x + 316, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 316, y + 313, 254, 233, 254);
	gfx.PutPixel(x + 316, y + 314, 254, 55, 254);
	gfx.PutPixel(x + 317, y + 283, 254, 14, 254);
	gfx.PutPixel(x + 317, y + 284, 254, 147, 254);
	gfx.PutPixel(x + 317, y + 285, 254, 252, 254);
	gfx.PutPixel(x + 317, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 317, y + 314, 254, 243, 254);
	gfx.PutPixel(x + 317, y + 315, 254, 56, 254);
	gfx.PutPixel(x + 318, y + 282, 254, 78, 254);
	gfx.PutPixel(x + 318, y + 283, 254, 230, 254);
	gfx.PutPixel(x + 318, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 318, y + 315, 254, 237, 254);
	gfx.PutPixel(x + 318, y + 316, 254, 39, 254);
	gfx.PutPixel(x + 319, y + 280, 254, 9, 254);
	gfx.PutPixel(x + 319, y + 281, 254, 151, 254);
	gfx.PutPixel(x + 319, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 319, y + 316, 254, 208, 254);
	gfx.PutPixel(x + 319, y + 317, 254, 6, 254);
	gfx.PutPixel(x + 320, y + 279, 254, 42, 254);
	gfx.PutPixel(x + 320, y + 280, 254, 214, 254);
	gfx.PutPixel(x + 320, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 320, y + 317, 254, 138, 254);
	gfx.PutPixel(x + 321, y + 278, 254, 76, 254);
	gfx.PutPixel(x + 321, y + 279, 254, 241, 254);
	gfx.PutPixel(x + 321, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 293, 254, 216, 254);
	gfx.PutPixel(x + 321, y + 294, 254, 143, 254);
	gfx.PutPixel(x + 321, y + 295, 254, 91, 254);
	gfx.PutPixel(x + 321, y + 296, 254, 43, 254);
	gfx.PutPixel(x + 321, y + 297, 254, 21, 254);
	gfx.PutPixel(x + 321, y + 298, 254, 5, 254);
	gfx.PutPixel(x + 321, y + 299, 254, 12, 254);
	gfx.PutPixel(x + 321, y + 300, 254, 34, 254);
	gfx.PutPixel(x + 321, y + 301, 254, 68, 254);
	gfx.PutPixel(x + 321, y + 302, 254, 135, 254);
	gfx.PutPixel(x + 321, y + 303, 254, 207, 254);
	gfx.PutPixel(x + 321, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 321, y + 317, 254, 249, 254);
	gfx.PutPixel(x + 321, y + 318, 254, 39, 254);
	gfx.PutPixel(x + 322, y + 277, 254, 120, 254);
	gfx.PutPixel(x + 322, y + 278, 254, 252, 254);
	gfx.PutPixel(x + 322, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 290, 254, 239, 254);
	gfx.PutPixel(x + 322, y + 291, 254, 138, 254);
	gfx.PutPixel(x + 322, y + 292, 254, 44, 254);
	gfx.PutPixel(x + 322, y + 304, 254, 59, 254);
	gfx.PutPixel(x + 322, y + 305, 254, 202, 254);
	gfx.PutPixel(x + 322, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 318, 254, 175, 254);
	gfx.PutPixel(x + 323, y + 275, 254, 0, 254);
	gfx.PutPixel(x + 323, y + 276, 254, 147, 254);
	gfx.PutPixel(x + 323, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 289, 254, 201, 254);
	gfx.PutPixel(x + 323, y + 290, 254, 21, 254);
	gfx.PutPixel(x + 323, y + 305, 254, 3, 254);
	gfx.PutPixel(x + 323, y + 306, 254, 143, 254);
	gfx.PutPixel(x + 323, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 318, 254, 253, 254);
	gfx.PutPixel(x + 323, y + 319, 254, 41, 254);
	gfx.PutPixel(x + 324, y + 274, 254, 3, 254);
	gfx.PutPixel(x + 324, y + 275, 254, 164, 254);
	gfx.PutPixel(x + 324, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 290, 254, 234, 254);
	gfx.PutPixel(x + 324, y + 291, 254, 71, 254);
	gfx.PutPixel(x + 324, y + 307, 254, 139, 254);
	gfx.PutPixel(x + 324, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 319, 254, 153, 254);
	gfx.PutPixel(x + 325, y + 273, 254, 4, 254);
	gfx.PutPixel(x + 325, y + 274, 254, 175, 254);
	gfx.PutPixel(x + 325, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 291, 254, 251, 254);
	gfx.PutPixel(x + 325, y + 292, 254, 85, 254);
	gfx.PutPixel(x + 325, y + 307, 254, 1, 254);
	gfx.PutPixel(x + 325, y + 308, 254, 191, 254);
	gfx.PutPixel(x + 325, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 319, 254, 239, 254);
	gfx.PutPixel(x + 325, y + 320, 254, 7, 254);
	gfx.PutPixel(x + 326, y + 272, 254, 2, 254);
	gfx.PutPixel(x + 326, y + 273, 254, 169, 254);
	gfx.PutPixel(x + 326, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 284, 254, 239, 254);
	gfx.PutPixel(x + 326, y + 285, 254, 165, 254);
	gfx.PutPixel(x + 326, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 292, 254, 237, 254);
	gfx.PutPixel(x + 326, y + 293, 254, 12, 254);
	gfx.PutPixel(x + 326, y + 308, 254, 39, 254);
	gfx.PutPixel(x + 326, y + 309, 254, 251, 254);
	gfx.PutPixel(x + 326, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 320, 254, 82, 254);
	gfx.PutPixel(x + 327, y + 271, 254, 0, 254);
	gfx.PutPixel(x + 327, y + 272, 254, 161, 254);
	gfx.PutPixel(x + 327, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 283, 254, 198, 254);
	gfx.PutPixel(x + 327, y + 284, 254, 30, 254);
	gfx.PutPixel(x + 327, y + 285, 254, 138, 254);
	gfx.PutPixel(x + 327, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 293, 254, 82, 254);
	gfx.PutPixel(x + 327, y + 309, 254, 164, 254);
	gfx.PutPixel(x + 327, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 320, 254, 155, 254);
	gfx.PutPixel(x + 328, y + 271, 254, 140, 254);
	gfx.PutPixel(x + 328, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 282, 254, 151, 254);
	gfx.PutPixel(x + 328, y + 283, 254, 5, 254);
	gfx.PutPixel(x + 328, y + 285, 254, 199, 254);
	gfx.PutPixel(x + 328, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 293, 254, 117, 254);
	gfx.PutPixel(x + 328, y + 309, 254, 60, 254);
	gfx.PutPixel(x + 328, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 320, 254, 224, 254);
	gfx.PutPixel(x + 329, y + 270, 254, 105, 254);
	gfx.PutPixel(x + 329, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 280, 254, 252, 254);
	gfx.PutPixel(x + 329, y + 281, 254, 111, 254);
	gfx.PutPixel(x + 329, y + 284, 254, 8, 254);
	gfx.PutPixel(x + 329, y + 285, 254, 249, 254);
	gfx.PutPixel(x + 329, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 293, 254, 115, 254);
	gfx.PutPixel(x + 329, y + 309, 254, 0, 254);
	gfx.PutPixel(x + 329, y + 310, 254, 228, 254);
	gfx.PutPixel(x + 329, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 321, 254, 24, 254);
	gfx.PutPixel(x + 330, y + 269, 254, 74, 254);
	gfx.PutPixel(x + 330, y + 270, 254, 251, 254);
	gfx.PutPixel(x + 330, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 279, 254, 248, 254);
	gfx.PutPixel(x + 330, y + 280, 254, 83, 254);
	gfx.PutPixel(x + 330, y + 284, 254, 63, 254);
	gfx.PutPixel(x + 330, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 293, 254, 103, 254);
	gfx.PutPixel(x + 330, y + 310, 254, 162, 254);
	gfx.PutPixel(x + 330, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 321, 254, 76, 254);
	gfx.PutPixel(x + 331, y + 268, 254, 36, 254);
	gfx.PutPixel(x + 331, y + 269, 254, 240, 254);
	gfx.PutPixel(x + 331, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 278, 254, 247, 254);
	gfx.PutPixel(x + 331, y + 279, 254, 74, 254);
	gfx.PutPixel(x + 331, y + 284, 254, 124, 254);
	gfx.PutPixel(x + 331, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 293, 254, 79, 254);
	gfx.PutPixel(x + 331, y + 310, 254, 109, 254);
	gfx.PutPixel(x + 331, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 321, 254, 121, 254);
	gfx.PutPixel(x + 332, y + 267, 254, 9, 254);
	gfx.PutPixel(x + 332, y + 268, 254, 209, 254);
	gfx.PutPixel(x + 332, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 277, 254, 250, 254);
	gfx.PutPixel(x + 332, y + 278, 254, 74, 254);
	gfx.PutPixel(x + 332, y + 284, 254, 185, 254);
	gfx.PutPixel(x + 332, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 293, 254, 49, 254);
	gfx.PutPixel(x + 332, y + 310, 254, 67, 254);
	gfx.PutPixel(x + 332, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 321, 254, 159, 254);
	gfx.PutPixel(x + 333, y + 267, 254, 163, 254);
	gfx.PutPixel(x + 333, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 277, 254, 95, 254);
	gfx.PutPixel(x + 333, y + 283, 254, 8, 254);
	gfx.PutPixel(x + 333, y + 284, 254, 246, 254);
	gfx.PutPixel(x + 333, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 293, 254, 13, 254);
	gfx.PutPixel(x + 333, y + 310, 254, 39, 254);
	gfx.PutPixel(x + 333, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 321, 254, 192, 254);
	gfx.PutPixel(x + 334, y + 266, 254, 95, 254);
	gfx.PutPixel(x + 334, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 276, 254, 120, 254);
	gfx.PutPixel(x + 334, y + 283, 254, 71, 254);
	gfx.PutPixel(x + 334, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 292, 254, 223, 254);
	gfx.PutPixel(x + 334, y + 310, 254, 17, 254);
	gfx.PutPixel(x + 334, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 321, 254, 215, 254);
	gfx.PutPixel(x + 335, y + 265, 254, 33, 254);
	gfx.PutPixel(x + 335, y + 266, 254, 241, 254);
	gfx.PutPixel(x + 335, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 275, 254, 145, 254);
	gfx.PutPixel(x + 335, y + 283, 254, 142, 254);
	gfx.PutPixel(x + 335, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 292, 254, 176, 254);
	gfx.PutPixel(x + 335, y + 310, 254, 7, 254);
	gfx.PutPixel(x + 335, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 321, 254, 236, 254);
	gfx.PutPixel(x + 336, y + 264, 254, 2, 254);
	gfx.PutPixel(x + 336, y + 265, 254, 196, 254);
	gfx.PutPixel(x + 336, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 274, 254, 184, 254);
	gfx.PutPixel(x + 336, y + 275, 254, 1, 254);
	gfx.PutPixel(x + 336, y + 283, 254, 214, 254);
	gfx.PutPixel(x + 336, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 292, 254, 120, 254);
	gfx.PutPixel(x + 336, y + 310, 254, 1, 254);
	gfx.PutPixel(x + 336, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 321, 254, 246, 254);
	gfx.PutPixel(x + 337, y + 264, 254, 118, 254);
	gfx.PutPixel(x + 337, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 273, 254, 224, 254);
	gfx.PutPixel(x + 337, y + 274, 254, 18, 254);
	gfx.PutPixel(x + 337, y + 282, 254, 29, 254);
	gfx.PutPixel(x + 337, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 292, 254, 63, 254);
	gfx.PutPixel(x + 337, y + 310, 254, 5, 254);
	gfx.PutPixel(x + 337, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 321, 254, 252, 254);
	gfx.PutPixel(x + 338, y + 263, 254, 36, 254);
	gfx.PutPixel(x + 338, y + 264, 254, 246, 254);
	gfx.PutPixel(x + 338, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 272, 254, 248, 254);
	gfx.PutPixel(x + 338, y + 273, 254, 52, 254);
	gfx.PutPixel(x + 338, y + 282, 254, 101, 254);
	gfx.PutPixel(x + 338, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 291, 254, 246, 254);
	gfx.PutPixel(x + 338, y + 292, 254, 8, 254);
	gfx.PutPixel(x + 338, y + 310, 254, 14, 254);
	gfx.PutPixel(x + 338, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 321, 254, 249, 254);
	gfx.PutPixel(x + 339, y + 262, 254, 0, 254);
	gfx.PutPixel(x + 339, y + 263, 254, 192, 254);
	gfx.PutPixel(x + 339, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 272, 254, 103, 254);
	gfx.PutPixel(x + 339, y + 282, 254, 178, 254);
	gfx.PutPixel(x + 339, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 291, 254, 183, 254);
	gfx.PutPixel(x + 339, y + 310, 254, 22, 254);
	gfx.PutPixel(x + 339, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 321, 254, 241, 254);
	gfx.PutPixel(x + 340, y + 262, 254, 99, 254);
	gfx.PutPixel(x + 340, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 271, 254, 173, 254);
	gfx.PutPixel(x + 340, y + 281, 254, 12, 254);
	gfx.PutPixel(x + 340, y + 282, 254, 247, 254);
	gfx.PutPixel(x + 340, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 291, 254, 111, 254);
	gfx.PutPixel(x + 340, y + 310, 254, 42, 254);
	gfx.PutPixel(x + 340, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 321, 254, 226, 254);
	gfx.PutPixel(x + 341, y + 261, 254, 16, 254);
	gfx.PutPixel(x + 341, y + 262, 254, 234, 254);
	gfx.PutPixel(x + 341, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 270, 254, 234, 254);
	gfx.PutPixel(x + 341, y + 271, 254, 21, 254);
	gfx.PutPixel(x + 341, y + 281, 254, 86, 254);
	gfx.PutPixel(x + 341, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 291, 254, 38, 254);
	gfx.PutPixel(x + 341, y + 310, 254, 68, 254);
	gfx.PutPixel(x + 341, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 321, 254, 201, 254);
	gfx.PutPixel(x + 342, y + 261, 254, 146, 254);
	gfx.PutPixel(x + 342, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 270, 254, 85, 254);
	gfx.PutPixel(x + 342, y + 281, 254, 167, 254);
	gfx.PutPixel(x + 342, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 290, 254, 221, 254);
	gfx.PutPixel(x + 342, y + 310, 254, 94, 254);
	gfx.PutPixel(x + 342, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 342, y + 321, 254, 174, 254);
	gfx.PutPixel(x + 343, y + 260, 254, 43, 254);
	gfx.PutPixel(x + 343, y + 261, 254, 251, 254);
	gfx.PutPixel(x + 343, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 269, 254, 169, 254);
	gfx.PutPixel(x + 343, y + 280, 254, 6, 254);
	gfx.PutPixel(x + 343, y + 281, 254, 241, 254);
	gfx.PutPixel(x + 343, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 290, 254, 149, 254);
	gfx.PutPixel(x + 343, y + 310, 254, 128, 254);
	gfx.PutPixel(x + 343, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 343, y + 321, 254, 136, 254);
	gfx.PutPixel(x + 344, y + 260, 254, 179, 254);
	gfx.PutPixel(x + 344, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 268, 254, 238, 254);
	gfx.PutPixel(x + 344, y + 269, 254, 20, 254);
	gfx.PutPixel(x + 344, y + 280, 254, 74, 254);
	gfx.PutPixel(x + 344, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 290, 254, 66, 254);
	gfx.PutPixel(x + 344, y + 310, 254, 170, 254);
	gfx.PutPixel(x + 344, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 344, y + 321, 254, 94, 254);
	gfx.PutPixel(x + 345, y + 259, 254, 60, 254);
	gfx.PutPixel(x + 345, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 268, 254, 111, 254);
	gfx.PutPixel(x + 345, y + 280, 254, 159, 254);
	gfx.PutPixel(x + 345, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 289, 254, 230, 254);
	gfx.PutPixel(x + 345, y + 290, 254, 1, 254);
	gfx.PutPixel(x + 345, y + 310, 254, 213, 254);
	gfx.PutPixel(x + 345, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 345, y + 321, 254, 44, 254);
	gfx.PutPixel(x + 346, y + 259, 254, 197, 254);
	gfx.PutPixel(x + 346, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 267, 254, 212, 254);
	gfx.PutPixel(x + 346, y + 268, 254, 3, 254);
	gfx.PutPixel(x + 346, y + 279, 254, 8, 254);
	gfx.PutPixel(x + 346, y + 280, 254, 241, 254);
	gfx.PutPixel(x + 346, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 289, 254, 145, 254);
	gfx.PutPixel(x + 346, y + 309, 254, 9, 254);
	gfx.PutPixel(x + 346, y + 310, 254, 250, 254);
	gfx.PutPixel(x + 346, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 346, y + 320, 254, 238, 254);
	gfx.PutPixel(x + 346, y + 321, 254, 1, 254);
	gfx.PutPixel(x + 347, y + 258, 254, 70, 254);
	gfx.PutPixel(x + 347, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 267, 254, 68, 254);
	gfx.PutPixel(x + 347, y + 279, 254, 85, 254);
	gfx.PutPixel(x + 347, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 289, 254, 56, 254);
	gfx.PutPixel(x + 347, y + 309, 254, 64, 254);
	gfx.PutPixel(x + 347, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 347, y + 320, 254, 178, 254);
	gfx.PutPixel(x + 348, y + 258, 254, 194, 254);
	gfx.PutPixel(x + 348, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 266, 254, 176, 254);
	gfx.PutPixel(x + 348, y + 279, 254, 176, 254);
	gfx.PutPixel(x + 348, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 288, 254, 222, 254);
	gfx.PutPixel(x + 348, y + 309, 254, 123, 254);
	gfx.PutPixel(x + 348, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 348, y + 320, 254, 100, 254);
	gfx.PutPixel(x + 349, y + 93, 254, 16, 254);
	gfx.PutPixel(x + 349, y + 94, 254, 124, 254);
	gfx.PutPixel(x + 349, y + 95, 254, 193, 254);
	gfx.PutPixel(x + 349, y + 96, 254, 230, 254);
	gfx.PutPixel(x + 349, y + 97, 254, 248, 254);
	gfx.PutPixel(x + 349, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 99, 254, 250, 254);
	gfx.PutPixel(x + 349, y + 100, 254, 241, 254);
	gfx.PutPixel(x + 349, y + 101, 254, 226, 254);
	gfx.PutPixel(x + 349, y + 102, 254, 207, 254);
	gfx.PutPixel(x + 349, y + 103, 254, 185, 254);
	gfx.PutPixel(x + 349, y + 104, 254, 159, 254);
	gfx.PutPixel(x + 349, y + 105, 254, 130, 254);
	gfx.PutPixel(x + 349, y + 106, 254, 100, 254);
	gfx.PutPixel(x + 349, y + 107, 254, 72, 254);
	gfx.PutPixel(x + 349, y + 108, 254, 47, 254);
	gfx.PutPixel(x + 349, y + 109, 254, 27, 254);
	gfx.PutPixel(x + 349, y + 110, 254, 7, 254);
	gfx.PutPixel(x + 349, y + 257, 254, 62, 254);
	gfx.PutPixel(x + 349, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 265, 254, 253, 254);
	gfx.PutPixel(x + 349, y + 266, 254, 45, 254);
	gfx.PutPixel(x + 349, y + 278, 254, 17, 254);
	gfx.PutPixel(x + 349, y + 279, 254, 249, 254);
	gfx.PutPixel(x + 349, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 288, 254, 134, 254);
	gfx.PutPixel(x + 349, y + 309, 254, 185, 254);
	gfx.PutPixel(x + 349, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 349, y + 319, 254, 252, 254);
	gfx.PutPixel(x + 349, y + 320, 254, 21, 254);
	gfx.PutPixel(x + 350, y + 93, 254, 204, 254);
	gfx.PutPixel(x + 350, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 111, 254, 247, 254);
	gfx.PutPixel(x + 350, y + 112, 254, 231, 254);
	gfx.PutPixel(x + 350, y + 113, 254, 217, 254);
	gfx.PutPixel(x + 350, y + 114, 254, 203, 254);
	gfx.PutPixel(x + 350, y + 115, 254, 189, 254);
	gfx.PutPixel(x + 350, y + 116, 254, 176, 254);
	gfx.PutPixel(x + 350, y + 117, 254, 166, 254);
	gfx.PutPixel(x + 350, y + 118, 254, 156, 254);
	gfx.PutPixel(x + 350, y + 119, 254, 145, 254);
	gfx.PutPixel(x + 350, y + 120, 254, 134, 254);
	gfx.PutPixel(x + 350, y + 121, 254, 127, 254);
	gfx.PutPixel(x + 350, y + 122, 254, 118, 254);
	gfx.PutPixel(x + 350, y + 123, 254, 110, 254);
	gfx.PutPixel(x + 350, y + 124, 254, 102, 254);
	gfx.PutPixel(x + 350, y + 125, 254, 94, 254);
	gfx.PutPixel(x + 350, y + 126, 254, 89, 254);
	gfx.PutPixel(x + 350, y + 127, 254, 84, 254);
	gfx.PutPixel(x + 350, y + 128, 254, 78, 254);
	gfx.PutPixel(x + 350, y + 129, 254, 73, 254);
	gfx.PutPixel(x + 350, y + 130, 254, 68, 254);
	gfx.PutPixel(x + 350, y + 131, 254, 63, 254);
	gfx.PutPixel(x + 350, y + 132, 254, 59, 254);
	gfx.PutPixel(x + 350, y + 133, 254, 54, 254);
	gfx.PutPixel(x + 350, y + 134, 254, 49, 254);
	gfx.PutPixel(x + 350, y + 135, 254, 46, 254);
	gfx.PutPixel(x + 350, y + 136, 254, 42, 254);
	gfx.PutPixel(x + 350, y + 137, 254, 38, 254);
	gfx.PutPixel(x + 350, y + 138, 254, 34, 254);
	gfx.PutPixel(x + 350, y + 139, 254, 29, 254);
	gfx.PutPixel(x + 350, y + 140, 254, 26, 254);
	gfx.PutPixel(x + 350, y + 141, 254, 22, 254);
	gfx.PutPixel(x + 350, y + 142, 254, 19, 254);
	gfx.PutPixel(x + 350, y + 143, 254, 14, 254);
	gfx.PutPixel(x + 350, y + 144, 254, 13, 254);
	gfx.PutPixel(x + 350, y + 145, 254, 11, 254);
	gfx.PutPixel(x + 350, y + 146, 254, 9, 254);
	gfx.PutPixel(x + 350, y + 147, 254, 6, 254);
	gfx.PutPixel(x + 350, y + 148, 254, 4, 254);
	gfx.PutPixel(x + 350, y + 149, 254, 3, 254);
	gfx.PutPixel(x + 350, y + 150, 254, 1, 254);
	gfx.PutPixel(x + 350, y + 151, 254, 0, 254);
	gfx.PutPixel(x + 350, y + 257, 254, 180, 254);
	gfx.PutPixel(x + 350, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 265, 254, 172, 254);
	gfx.PutPixel(x + 350, y + 278, 254, 101, 254);
	gfx.PutPixel(x + 350, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 288, 254, 38, 254);
	gfx.PutPixel(x + 350, y + 308, 254, 10, 254);
	gfx.PutPixel(x + 350, y + 309, 254, 247, 254);
	gfx.PutPixel(x + 350, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 350, y + 319, 254, 194, 254);
	gfx.PutPixel(x + 351, y + 93, 254, 238, 254);
	gfx.PutPixel(x + 351, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 158, 254, 249, 254);
	gfx.PutPixel(x + 351, y + 159, 254, 240, 254);
	gfx.PutPixel(x + 351, y + 160, 254, 232, 254);
	gfx.PutPixel(x + 351, y + 161, 254, 216, 254);
	gfx.PutPixel(x + 351, y + 162, 254, 188, 254);
	gfx.PutPixel(x + 351, y + 163, 254, 143, 254);
	gfx.PutPixel(x + 351, y + 164, 254, 69, 254);
	gfx.PutPixel(x + 351, y + 165, 254, 0, 254);
	gfx.PutPixel(x + 351, y + 256, 254, 36, 254);
	gfx.PutPixel(x + 351, y + 257, 254, 253, 254);
	gfx.PutPixel(x + 351, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 264, 254, 253, 254);
	gfx.PutPixel(x + 351, y + 265, 254, 44, 254);
	gfx.PutPixel(x + 351, y + 278, 254, 198, 254);
	gfx.PutPixel(x + 351, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 287, 254, 188, 254);
	gfx.PutPixel(x + 351, y + 308, 254, 76, 254);
	gfx.PutPixel(x + 351, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 319, 254, 113, 254);
	gfx.PutPixel(x + 352, y + 93, 254, 136, 254);
	gfx.PutPixel(x + 352, y + 94, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 165, 254, 188, 254);
	gfx.PutPixel(x + 352, y + 166, 254, 6, 254);
	gfx.PutPixel(x + 352, y + 256, 254, 144, 254);
	gfx.PutPixel(x + 352, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 264, 254, 171, 254);
	gfx.PutPixel(x + 352, y + 277, 254, 43, 254);
	gfx.PutPixel(x + 352, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 287, 254, 83, 254);
	gfx.PutPixel(x + 352, y + 308, 254, 151, 254);
	gfx.PutPixel(x + 352, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 318, 254, 252, 254);
	gfx.PutPixel(x + 352, y + 319, 254, 29, 254);
	gfx.PutPixel(x + 353, y + 93, 254, 2, 254);
	gfx.PutPixel(x + 353, y + 94, 254, 147, 254);
	gfx.PutPixel(x + 353, y + 95, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 96, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 97, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 166, 254, 84, 254);
	gfx.PutPixel(x + 353, y + 255, 254, 9, 254);
	gfx.PutPixel(x + 353, y + 256, 254, 240, 254);
	gfx.PutPixel(x + 353, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 264, 254, 50, 254);
	gfx.PutPixel(x + 353, y + 277, 254, 145, 254);
	gfx.PutPixel(x + 353, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 286, 254, 229, 254);
	gfx.PutPixel(x + 353, y + 287, 254, 3, 254);
	gfx.PutPixel(x + 353, y + 307, 254, 2, 254);
	gfx.PutPixel(x + 353, y + 308, 254, 231, 254);
	gfx.PutPixel(x + 353, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 318, 254, 177, 254);
	gfx.PutPixel(x + 354, y + 95, 254, 49, 254);
	gfx.PutPixel(x + 354, y + 96, 254, 163, 254);
	gfx.PutPixel(x + 354, y + 97, 254, 245, 254);
	gfx.PutPixel(x + 354, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 166, 254, 80, 254);
	gfx.PutPixel(x + 354, y + 255, 254, 92, 254);
	gfx.PutPixel(x + 354, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 263, 254, 197, 254);
	gfx.PutPixel(x + 354, y + 276, 254, 7, 254);
	gfx.PutPixel(x + 354, y + 277, 254, 237, 254);
	gfx.PutPixel(x + 354, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 286, 254, 129, 254);
	gfx.PutPixel(x + 354, y + 307, 254, 65, 254);
	gfx.PutPixel(x + 354, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 318, 254, 69, 254);
	gfx.PutPixel(x + 355, y + 97, 254, 9, 254);
	gfx.PutPixel(x + 355, y + 98, 254, 77, 254);
	gfx.PutPixel(x + 355, y + 99, 254, 144, 254);
	gfx.PutPixel(x + 355, y + 100, 254, 204, 254);
	gfx.PutPixel(x + 355, y + 101, 254, 249, 254);
	gfx.PutPixel(x + 355, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 165, 254, 199, 254);
	gfx.PutPixel(x + 355, y + 166, 254, 4, 254);
	gfx.PutPixel(x + 355, y + 255, 254, 188, 254);
	gfx.PutPixel(x + 355, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 263, 254, 88, 254);
	gfx.PutPixel(x + 355, y + 276, 254, 92, 254);
	gfx.PutPixel(x + 355, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 285, 254, 251, 254);
	gfx.PutPixel(x + 355, y + 286, 254, 27, 254);
	gfx.PutPixel(x + 355, y + 307, 254, 151, 254);
	gfx.PutPixel(x + 355, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 317, 254, 216, 254);
	gfx.PutPixel(x + 355, y + 318, 254, 0, 254);
	gfx.PutPixel(x + 356, y + 101, 254, 6, 254);
	gfx.PutPixel(x + 356, y + 102, 254, 50, 254);
	gfx.PutPixel(x + 356, y + 103, 254, 93, 254);
	gfx.PutPixel(x + 356, y + 104, 254, 134, 254);
	gfx.PutPixel(x + 356, y + 105, 254, 170, 254);
	gfx.PutPixel(x + 356, y + 106, 254, 206, 254);
	gfx.PutPixel(x + 356, y + 107, 254, 238, 254);
	gfx.PutPixel(x + 356, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 163, 254, 251, 254);
	gfx.PutPixel(x + 356, y + 164, 254, 147, 254);
	gfx.PutPixel(x + 356, y + 165, 254, 11, 254);
	gfx.PutPixel(x + 356, y + 254, 254, 28, 254);
	gfx.PutPixel(x + 356, y + 255, 254, 253, 254);
	gfx.PutPixel(x + 356, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 262, 254, 228, 254);
	gfx.PutPixel(x + 356, y + 263, 254, 4, 254);
	gfx.PutPixel(x + 356, y + 276, 254, 195, 254);
	gfx.PutPixel(x + 356, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 285, 254, 170, 254);
	gfx.PutPixel(x + 356, y + 306, 254, 7, 254);
	gfx.PutPixel(x + 356, y + 307, 254, 238, 254);
	gfx.PutPixel(x + 356, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 317, 254, 110, 254);
	gfx.PutPixel(x + 357, y + 108, 254, 11, 254);
	gfx.PutPixel(x + 357, y + 109, 254, 40, 254);
	gfx.PutPixel(x + 357, y + 110, 254, 66, 254);
	gfx.PutPixel(x + 357, y + 111, 254, 91, 254);
	gfx.PutPixel(x + 357, y + 112, 254, 115, 254);
	gfx.PutPixel(x + 357, y + 113, 254, 137, 254);
	gfx.PutPixel(x + 357, y + 114, 254, 156, 254);
	gfx.PutPixel(x + 357, y + 115, 254, 176, 254);
	gfx.PutPixel(x + 357, y + 116, 254, 195, 254);
	gfx.PutPixel(x + 357, y + 117, 254, 211, 254);
	gfx.PutPixel(x + 357, y + 118, 254, 228, 254);
	gfx.PutPixel(x + 357, y + 119, 254, 244, 254);
	gfx.PutPixel(x + 357, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 161, 254, 233, 254);
	gfx.PutPixel(x + 357, y + 162, 254, 141, 254);
	gfx.PutPixel(x + 357, y + 163, 254, 39, 254);
	gfx.PutPixel(x + 357, y + 254, 254, 110, 254);
	gfx.PutPixel(x + 357, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 262, 254, 127, 254);
	gfx.PutPixel(x + 357, y + 275, 254, 50, 254);
	gfx.PutPixel(x + 357, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 285, 254, 50, 254);
	gfx.PutPixel(x + 357, y + 306, 254, 88, 254);
	gfx.PutPixel(x + 357, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 357, y + 316, 254, 234, 254);
	gfx.PutPixel(x + 357, y + 317, 254, 11, 254);
	gfx.PutPixel(x + 358, y + 118, 254, 27, 254);
	gfx.PutPixel(x + 358, y + 119, 254, 166, 254);
	gfx.PutPixel(x + 358, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 156, 254, 253, 254);
	gfx.PutPixel(x + 358, y + 157, 254, 221, 254);
	gfx.PutPixel(x + 358, y + 158, 254, 176, 254);
	gfx.PutPixel(x + 358, y + 159, 254, 119, 254);
	gfx.PutPixel(x + 358, y + 160, 254, 53, 254);
	gfx.PutPixel(x + 358, y + 161, 254, 2, 254);
	gfx.PutPixel(x + 358, y + 254, 254, 191, 254);
	gfx.PutPixel(x + 358, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 262, 254, 35, 254);
	gfx.PutPixel(x + 358, y + 275, 254, 162, 254);
	gfx.PutPixel(x + 358, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 284, 254, 185, 254);
	gfx.PutPixel(x + 358, y + 306, 254, 188, 254);
	gfx.PutPixel(x + 358, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 358, y + 316, 254, 114, 254);
	gfx.PutPixel(x + 359, y + 116, 254, 7, 254);
	gfx.PutPixel(x + 359, y + 117, 254, 124, 254);
	gfx.PutPixel(x + 359, y + 118, 254, 244, 254);
	gfx.PutPixel(x + 359, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 137, 254, 188, 254);
	gfx.PutPixel(x + 359, y + 138, 254, 73, 254);
	gfx.PutPixel(x + 359, y + 139, 254, 6, 254);
	gfx.PutPixel(x + 359, y + 140, 254, 23, 254);
	gfx.PutPixel(x + 359, y + 141, 254, 41, 254);
	gfx.PutPixel(x + 359, y + 142, 254, 59, 254);
	gfx.PutPixel(x + 359, y + 143, 254, 72, 254);
	gfx.PutPixel(x + 359, y + 144, 254, 84, 254);
	gfx.PutPixel(x + 359, y + 145, 254, 95, 254);
	gfx.PutPixel(x + 359, y + 146, 254, 105, 254);
	gfx.PutPixel(x + 359, y + 147, 254, 113, 254);
	gfx.PutPixel(x + 359, y + 148, 254, 120, 254);
	gfx.PutPixel(x + 359, y + 149, 254, 124, 254);
	gfx.PutPixel(x + 359, y + 150, 254, 124, 254);
	gfx.PutPixel(x + 359, y + 151, 254, 116, 254);
	gfx.PutPixel(x + 359, y + 152, 254, 107, 254);
	gfx.PutPixel(x + 359, y + 153, 254, 94, 254);
	gfx.PutPixel(x + 359, y + 154, 254, 67, 254);
	gfx.PutPixel(x + 359, y + 155, 254, 41, 254);
	gfx.PutPixel(x + 359, y + 156, 254, 10, 254);
	gfx.PutPixel(x + 359, y + 253, 254, 19, 254);
	gfx.PutPixel(x + 359, y + 254, 254, 251, 254);
	gfx.PutPixel(x + 359, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 261, 254, 199, 254);
	gfx.PutPixel(x + 359, y + 274, 254, 25, 254);
	gfx.PutPixel(x + 359, y + 275, 254, 249, 254);
	gfx.PutPixel(x + 359, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 284, 254, 64, 254);
	gfx.PutPixel(x + 359, y + 305, 254, 40, 254);
	gfx.PutPixel(x + 359, y + 306, 254, 253, 254);
	gfx.PutPixel(x + 359, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 359, y + 315, 254, 228, 254);
	gfx.PutPixel(x + 359, y + 316, 254, 7, 254);
	gfx.PutPixel(x + 360, y + 115, 254, 71, 254);
	gfx.PutPixel(x + 360, y + 116, 254, 221, 254);
	gfx.PutPixel(x + 360, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 134, 254, 252, 254);
	gfx.PutPixel(x + 360, y + 135, 254, 163, 254);
	gfx.PutPixel(x + 360, y + 136, 254, 47, 254);
	gfx.PutPixel(x + 360, y + 253, 254, 87, 254);
	gfx.PutPixel(x + 360, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 261, 254, 108, 254);
	gfx.PutPixel(x + 360, y + 274, 254, 132, 254);
	gfx.PutPixel(x + 360, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 283, 254, 199, 254);
	gfx.PutPixel(x + 360, y + 305, 254, 150, 254);
	gfx.PutPixel(x + 360, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 360, y + 315, 254, 104, 254);
	gfx.PutPixel(x + 361, y + 113, 254, 7, 254);
	gfx.PutPixel(x + 361, y + 114, 254, 147, 254);
	gfx.PutPixel(x + 361, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 133, 254, 184, 254);
	gfx.PutPixel(x + 361, y + 134, 254, 44, 254);
	gfx.PutPixel(x + 361, y + 253, 254, 153, 254);
	gfx.PutPixel(x + 361, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 260, 254, 252, 254);
	gfx.PutPixel(x + 361, y + 261, 254, 20, 254);
	gfx.PutPixel(x + 361, y + 273, 254, 9, 254);
	gfx.PutPixel(x + 361, y + 274, 254, 235, 254);
	gfx.PutPixel(x + 361, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 283, 254, 78, 254);
	gfx.PutPixel(x + 361, y + 304, 254, 17, 254);
	gfx.PutPixel(x + 361, y + 305, 254, 244, 254);
	gfx.PutPixel(x + 361, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 361, y + 314, 254, 220, 254);
	gfx.PutPixel(x + 361, y + 315, 254, 4, 254);
	gfx.PutPixel(x + 362, y + 112, 254, 46, 254);
	gfx.PutPixel(x + 362, y + 113, 254, 212, 254);
	gfx.PutPixel(x + 362, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 131, 254, 211, 254);
	gfx.PutPixel(x + 362, y + 132, 254, 71, 254);
	gfx.PutPixel(x + 362, y + 253, 254, 220, 254);
	gfx.PutPixel(x + 362, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 260, 254, 199, 254);
	gfx.PutPixel(x + 362, y + 273, 254, 109, 254);
	gfx.PutPixel(x + 362, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 282, 254, 209, 254);
	gfx.PutPixel(x + 362, y + 283, 254, 0, 254);
	gfx.PutPixel(x + 362, y + 304, 254, 128, 254);
	gfx.PutPixel(x + 362, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 362, y + 314, 254, 74, 254);
	gfx.PutPixel(x + 363, y + 111, 254, 110, 254);
	gfx.PutPixel(x + 363, y + 112, 254, 248, 254);
	gfx.PutPixel(x + 363, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 129, 254, 248, 254);
	gfx.PutPixel(x + 363, y + 130, 254, 123, 254);
	gfx.PutPixel(x + 363, y + 131, 254, 4, 254);
	gfx.PutPixel(x + 363, y + 252, 254, 20, 254);
	gfx.PutPixel(x + 363, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 260, 254, 128, 254);
	gfx.PutPixel(x + 363, y + 272, 254, 4, 254);
	gfx.PutPixel(x + 363, y + 273, 254, 225, 254);
	gfx.PutPixel(x + 363, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 282, 254, 79, 254);
	gfx.PutPixel(x + 363, y + 303, 254, 17, 254);
	gfx.PutPixel(x + 363, y + 304, 254, 240, 254);
	gfx.PutPixel(x + 363, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 363, y + 313, 254, 173, 254);
	gfx.PutPixel(x + 364, y + 109, 254, 5, 254);
	gfx.PutPixel(x + 364, y + 110, 254, 157, 254);
	gfx.PutPixel(x + 364, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 128, 254, 198, 254);
	gfx.PutPixel(x + 364, y + 129, 254, 41, 254);
	gfx.PutPixel(x + 364, y + 252, 254, 70, 254);
	gfx.PutPixel(x + 364, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 260, 254, 57, 254);
	gfx.PutPixel(x + 364, y + 272, 254, 99, 254);
	gfx.PutPixel(x + 364, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 281, 254, 199, 254);
	gfx.PutPixel(x + 364, y + 303, 254, 133, 254);
	gfx.PutPixel(x + 364, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 364, y + 312, 254, 243, 254);
	gfx.PutPixel(x + 364, y + 313, 254, 28, 254);
	gfx.PutPixel(x + 365, y + 108, 254, 21, 254);
	gfx.PutPixel(x + 365, y + 109, 254, 195, 254);
	gfx.PutPixel(x + 365, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 126, 254, 249, 254);
	gfx.PutPixel(x + 365, y + 127, 254, 118, 254);
	gfx.PutPixel(x + 365, y + 128, 254, 1, 254);
	gfx.PutPixel(x + 365, y + 252, 254, 120, 254);
	gfx.PutPixel(x + 365, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 259, 254, 240, 254);
	gfx.PutPixel(x + 365, y + 260, 254, 2, 254);
	gfx.PutPixel(x + 365, y + 271, 254, 4, 254);
	gfx.PutPixel(x + 365, y + 272, 254, 222, 254);
	gfx.PutPixel(x + 365, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 281, 254, 65, 254);
	gfx.PutPixel(x + 365, y + 302, 254, 20, 254);
	gfx.PutPixel(x + 365, y + 303, 254, 242, 254);
	gfx.PutPixel(x + 365, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 365, y + 312, 254, 116, 254);
	gfx.PutPixel(x + 366, y + 107, 254, 44, 254);
	gfx.PutPixel(x + 366, y + 108, 254, 223, 254);
	gfx.PutPixel(x + 366, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 125, 254, 221, 254);
	gfx.PutPixel(x + 366, y + 126, 254, 53, 254);
	gfx.PutPixel(x + 366, y + 252, 254, 158, 254);
	gfx.PutPixel(x + 366, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 259, 254, 192, 254);
	gfx.PutPixel(x + 366, y + 271, 254, 101, 254);
	gfx.PutPixel(x + 366, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 280, 254, 177, 254);
	gfx.PutPixel(x + 366, y + 302, 254, 138, 254);
	gfx.PutPixel(x + 366, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 366, y + 311, 254, 200, 254);
	gfx.PutPixel(x + 366, y + 312, 254, 3, 254);
	gfx.PutPixel(x + 367, y + 106, 254, 55, 254);
	gfx.PutPixel(x + 367, y + 107, 254, 238, 254);
	gfx.PutPixel(x + 367, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 124, 254, 167, 254);
	gfx.PutPixel(x + 367, y + 125, 254, 13, 254);
	gfx.PutPixel(x + 367, y + 252, 254, 189, 254);
	gfx.PutPixel(x + 367, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 259, 254, 145, 254);
	gfx.PutPixel(x + 367, y + 270, 254, 8, 254);
	gfx.PutPixel(x + 367, y + 271, 254, 227, 254);
	gfx.PutPixel(x + 367, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 279, 254, 249, 254);
	gfx.PutPixel(x + 367, y + 280, 254, 36, 254);
	gfx.PutPixel(x + 367, y + 301, 254, 23, 254);
	gfx.PutPixel(x + 367, y + 302, 254, 245, 254);
	gfx.PutPixel(x + 367, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 367, y + 310, 254, 244, 254);
	gfx.PutPixel(x + 367, y + 311, 254, 37, 254);
	gfx.PutPixel(x + 368, y + 105, 254, 63, 254);
	gfx.PutPixel(x + 368, y + 106, 254, 242, 254);
	gfx.PutPixel(x + 368, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 122, 254, 251, 254);
	gfx.PutPixel(x + 368, y + 123, 254, 114, 254);
	gfx.PutPixel(x + 368, y + 252, 254, 219, 254);
	gfx.PutPixel(x + 368, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 259, 254, 97, 254);
	gfx.PutPixel(x + 368, y + 270, 254, 118, 254);
	gfx.PutPixel(x + 368, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 279, 254, 139, 254);
	gfx.PutPixel(x + 368, y + 301, 254, 157, 254);
	gfx.PutPixel(x + 368, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 368, y + 310, 254, 104, 254);
	gfx.PutPixel(x + 369, y + 104, 254, 72, 254);
	gfx.PutPixel(x + 369, y + 105, 254, 245, 254);
	gfx.PutPixel(x + 369, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 121, 254, 237, 254);
	gfx.PutPixel(x + 369, y + 122, 254, 69, 254);
	gfx.PutPixel(x + 369, y + 252, 254, 234, 254);
	gfx.PutPixel(x + 369, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 259, 254, 56, 254);
	gfx.PutPixel(x + 369, y + 269, 254, 20, 254);
	gfx.PutPixel(x + 369, y + 270, 254, 240, 254);
	gfx.PutPixel(x + 369, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 278, 254, 225, 254);
	gfx.PutPixel(x + 369, y + 279, 254, 11, 254);
	gfx.PutPixel(x + 369, y + 300, 254, 59, 254);
	gfx.PutPixel(x + 369, y + 301, 254, 253, 254);
	gfx.PutPixel(x + 369, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 369, y + 309, 254, 181, 254);
	gfx.PutPixel(x + 370, y + 103, 254, 70, 254);
	gfx.PutPixel(x + 370, y + 104, 254, 248, 254);
	gfx.PutPixel(x + 370, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 120, 254, 217, 254);
	gfx.PutPixel(x + 370, y + 121, 254, 35, 254);
	gfx.PutPixel(x + 370, y + 252, 254, 246, 254);
	gfx.PutPixel(x + 370, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 259, 254, 37, 254);
	gfx.PutPixel(x + 370, y + 269, 254, 153, 254);
	gfx.PutPixel(x + 370, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 278, 254, 80, 254);
	gfx.PutPixel(x + 370, y + 299, 254, 3, 254);
	gfx.PutPixel(x + 370, y + 300, 254, 210, 254);
	gfx.PutPixel(x + 370, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 370, y + 308, 254, 229, 254);
	gfx.PutPixel(x + 370, y + 309, 254, 22, 254);
	gfx.PutPixel(x + 371, y + 102, 254, 58, 254);
	gfx.PutPixel(x + 371, y + 103, 254, 244, 254);
	gfx.PutPixel(x + 371, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 119, 254, 201, 254);
	gfx.PutPixel(x + 371, y + 120, 254, 20, 254);
	gfx.PutPixel(x + 371, y + 252, 254, 252, 254);
	gfx.PutPixel(x + 371, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 259, 254, 19, 254);
	gfx.PutPixel(x + 371, y + 268, 254, 55, 254);
	gfx.PutPixel(x + 371, y + 269, 254, 253, 254);
	gfx.PutPixel(x + 371, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 277, 254, 172, 254);
	gfx.PutPixel(x + 371, y + 299, 254, 117, 254);
	gfx.PutPixel(x + 371, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 371, y + 307, 254, 250, 254);
	gfx.PutPixel(x + 371, y + 308, 254, 59, 254);
	gfx.PutPixel(x + 372, y + 101, 254, 37, 254);
	gfx.PutPixel(x + 372, y + 102, 254, 236, 254);
	gfx.PutPixel(x + 372, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 118, 254, 182, 254);
	gfx.PutPixel(x + 372, y + 119, 254, 10, 254);
	gfx.PutPixel(x + 372, y + 252, 254, 246, 254);
	gfx.PutPixel(x + 372, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 259, 254, 3, 254);
	gfx.PutPixel(x + 372, y + 267, 254, 5, 254);
	gfx.PutPixel(x + 372, y + 268, 254, 209, 254);
	gfx.PutPixel(x + 372, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 276, 254, 233, 254);
	gfx.PutPixel(x + 372, y + 277, 254, 20, 254);
	gfx.PutPixel(x + 372, y + 298, 254, 29, 254);
	gfx.PutPixel(x + 372, y + 299, 254, 243, 254);
	gfx.PutPixel(x + 372, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 372, y + 307, 254, 111, 254);
	gfx.PutPixel(x + 373, y + 100, 254, 16, 254);
	gfx.PutPixel(x + 373, y + 101, 254, 218, 254);
	gfx.PutPixel(x + 373, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 117, 254, 172, 254);
	gfx.PutPixel(x + 373, y + 118, 254, 4, 254);
	gfx.PutPixel(x + 373, y + 252, 254, 232, 254);
	gfx.PutPixel(x + 373, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 259, 254, 12, 254);
	gfx.PutPixel(x + 373, y + 267, 254, 132, 254);
	gfx.PutPixel(x + 373, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 276, 254, 83, 254);
	gfx.PutPixel(x + 373, y + 298, 254, 176, 254);
	gfx.PutPixel(x + 373, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 373, y + 306, 254, 168, 254);
	gfx.PutPixel(x + 374, y + 99, 254, 1, 254);
	gfx.PutPixel(x + 374, y + 100, 254, 181, 254);
	gfx.PutPixel(x + 374, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 116, 254, 170, 254);
	gfx.PutPixel(x + 374, y + 117, 254, 3, 254);
	gfx.PutPixel(x + 374, y + 252, 254, 209, 254);
	gfx.PutPixel(x + 374, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 259, 254, 51, 254);
	gfx.PutPixel(x + 374, y + 266, 254, 67, 254);
	gfx.PutPixel(x + 374, y + 267, 254, 252, 254);
	gfx.PutPixel(x + 374, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 275, 254, 153, 254);
	gfx.PutPixel(x + 374, y + 297, 254, 97, 254);
	gfx.PutPixel(x + 374, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 374, y + 305, 254, 204, 254);
	gfx.PutPixel(x + 374, y + 306, 254, 10, 254);
	gfx.PutPixel(x + 375, y + 99, 254, 117, 254);
	gfx.PutPixel(x + 375, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 115, 254, 171, 254);
	gfx.PutPixel(x + 375, y + 116, 254, 3, 254);
	gfx.PutPixel(x + 375, y + 252, 254, 173, 254);
	gfx.PutPixel(x + 375, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 259, 254, 130, 254);
	gfx.PutPixel(x + 375, y + 265, 254, 35, 254);
	gfx.PutPixel(x + 375, y + 266, 254, 235, 254);
	gfx.PutPixel(x + 375, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 274, 254, 209, 254);
	gfx.PutPixel(x + 375, y + 275, 254, 7, 254);
	gfx.PutPixel(x + 375, y + 296, 254, 37, 254);
	gfx.PutPixel(x + 375, y + 297, 254, 243, 254);
	gfx.PutPixel(x + 375, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 375, y + 304, 254, 225, 254);
	gfx.PutPixel(x + 375, y + 305, 254, 24, 254);
	gfx.PutPixel(x + 376, y + 98, 254, 43, 254);
	gfx.PutPixel(x + 376, y + 99, 254, 249, 254);
	gfx.PutPixel(x + 376, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 114, 254, 185, 254);
	gfx.PutPixel(x + 376, y + 115, 254, 5, 254);
	gfx.PutPixel(x + 376, y + 252, 254, 127, 254);
	gfx.PutPixel(x + 376, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 259, 254, 240, 254);
	gfx.PutPixel(x + 376, y + 260, 254, 29, 254);
	gfx.PutPixel(x + 376, y + 264, 254, 32, 254);
	gfx.PutPixel(x + 376, y + 265, 254, 223, 254);
	gfx.PutPixel(x + 376, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 273, 254, 236, 254);
	gfx.PutPixel(x + 376, y + 274, 254, 36, 254);
	gfx.PutPixel(x + 376, y + 295, 254, 4, 254);
	gfx.PutPixel(x + 376, y + 296, 254, 203, 254);
	gfx.PutPixel(x + 376, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 376, y + 303, 254, 240, 254);
	gfx.PutPixel(x + 376, y + 304, 254, 44, 254);
	gfx.PutPixel(x + 377, y + 98, 254, 195, 254);
	gfx.PutPixel(x + 377, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 113, 254, 200, 254);
	gfx.PutPixel(x + 377, y + 114, 254, 10, 254);
	gfx.PutPixel(x + 377, y + 252, 254, 63, 254);
	gfx.PutPixel(x + 377, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 260, 254, 216, 254);
	gfx.PutPixel(x + 377, y + 261, 254, 55, 254);
	gfx.PutPixel(x + 377, y + 262, 254, 12, 254);
	gfx.PutPixel(x + 377, y + 263, 254, 92, 254);
	gfx.PutPixel(x + 377, y + 264, 254, 235, 254);
	gfx.PutPixel(x + 377, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 272, 254, 249, 254);
	gfx.PutPixel(x + 377, y + 273, 254, 64, 254);
	gfx.PutPixel(x + 377, y + 295, 254, 137, 254);
	gfx.PutPixel(x + 377, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 377, y + 302, 254, 248, 254);
	gfx.PutPixel(x + 377, y + 303, 254, 69, 254);
	gfx.PutPixel(x + 378, y + 97, 254, 75, 254);
	gfx.PutPixel(x + 378, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 112, 254, 222, 254);
	gfx.PutPixel(x + 378, y + 113, 254, 19, 254);
	gfx.PutPixel(x + 378, y + 252, 254, 1, 254);
	gfx.PutPixel(x + 378, y + 253, 254, 227, 254);
	gfx.PutPixel(x + 378, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 271, 254, 253, 254);
	gfx.PutPixel(x + 378, y + 272, 254, 94, 254);
	gfx.PutPixel(x + 378, y + 294, 254, 66, 254);
	gfx.PutPixel(x + 378, y + 295, 254, 253, 254);
	gfx.PutPixel(x + 378, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 378, y + 301, 254, 251, 254);
	gfx.PutPixel(x + 378, y + 302, 254, 82, 254);
	gfx.PutPixel(x + 379, y + 97, 254, 179, 254);
	gfx.PutPixel(x + 379, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 111, 254, 241, 254);
	gfx.PutPixel(x + 379, y + 112, 254, 43, 254);
	gfx.PutPixel(x + 379, y + 253, 254, 134, 254);
	gfx.PutPixel(x + 379, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 271, 254, 105, 254);
	gfx.PutPixel(x + 379, y + 293, 254, 28, 254);
	gfx.PutPixel(x + 379, y + 294, 254, 231, 254);
	gfx.PutPixel(x + 379, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 379, y + 300, 254, 253, 254);
	gfx.PutPixel(x + 379, y + 301, 254, 92, 254);
	gfx.PutPixel(x + 380, y + 97, 254, 240, 254);
	gfx.PutPixel(x + 380, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 111, 254, 83, 254);
	gfx.PutPixel(x + 380, y + 253, 254, 21, 254);
	gfx.PutPixel(x + 380, y + 254, 254, 237, 254);
	gfx.PutPixel(x + 380, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 269, 254, 252, 254);
	gfx.PutPixel(x + 380, y + 270, 254, 106, 254);
	gfx.PutPixel(x + 380, y + 292, 254, 9, 254);
	gfx.PutPixel(x + 380, y + 293, 254, 205, 254);
	gfx.PutPixel(x + 380, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 380, y + 300, 254, 103, 254);
	gfx.PutPixel(x + 381, y + 97, 254, 246, 254);
	gfx.PutPixel(x + 381, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 110, 254, 159, 254);
	gfx.PutPixel(x + 381, y + 254, 254, 98, 254);
	gfx.PutPixel(x + 381, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 268, 254, 247, 254);
	gfx.PutPixel(x + 381, y + 269, 254, 84, 254);
	gfx.PutPixel(x + 381, y + 291, 254, 0, 254);
	gfx.PutPixel(x + 381, y + 292, 254, 171, 254);
	gfx.PutPixel(x + 381, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 381, y + 298, 254, 252, 254);
	gfx.PutPixel(x + 381, y + 299, 254, 99, 254);
	gfx.PutPixel(x + 382, y + 97, 254, 229, 254);
	gfx.PutPixel(x + 382, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 109, 254, 230, 254);
	gfx.PutPixel(x + 382, y + 110, 254, 13, 254);
	gfx.PutPixel(x + 382, y + 255, 254, 130, 254);
	gfx.PutPixel(x + 382, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 267, 254, 217, 254);
	gfx.PutPixel(x + 382, y + 268, 254, 48, 254);
	gfx.PutPixel(x + 382, y + 291, 254, 129, 254);
	gfx.PutPixel(x + 382, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 382, y + 297, 254, 250, 254);
	gfx.PutPixel(x + 382, y + 298, 254, 88, 254);
	gfx.PutPixel(x + 383, y + 97, 254, 191, 254);
	gfx.PutPixel(x + 383, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 109, 254, 105, 254);
	gfx.PutPixel(x + 383, y + 256, 254, 99, 254);
	gfx.PutPixel(x + 383, y + 257, 254, 235, 254);
	gfx.PutPixel(x + 383, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 265, 254, 245, 254);
	gfx.PutPixel(x + 383, y + 266, 254, 134, 254);
	gfx.PutPixel(x + 383, y + 267, 254, 11, 254);
	gfx.PutPixel(x + 383, y + 290, 254, 86, 254);
	gfx.PutPixel(x + 383, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 383, y + 296, 254, 245, 254);
	gfx.PutPixel(x + 383, y + 297, 254, 77, 254);
	gfx.PutPixel(x + 384, y + 97, 254, 135, 254);
	gfx.PutPixel(x + 384, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 109, 254, 16, 254);
	gfx.PutPixel(x + 384, y + 257, 254, 17, 254);
	gfx.PutPixel(x + 384, y + 258, 254, 106, 254);
	gfx.PutPixel(x + 384, y + 259, 254, 185, 254);
	gfx.PutPixel(x + 384, y + 260, 254, 224, 254);
	gfx.PutPixel(x + 384, y + 261, 254, 247, 254);
	gfx.PutPixel(x + 384, y + 262, 254, 231, 254);
	gfx.PutPixel(x + 384, y + 263, 254, 194, 254);
	gfx.PutPixel(x + 384, y + 264, 254, 115, 254);
	gfx.PutPixel(x + 384, y + 265, 254, 24, 254);
	gfx.PutPixel(x + 384, y + 289, 254, 23, 254);
	gfx.PutPixel(x + 384, y + 290, 254, 241, 254);
	gfx.PutPixel(x + 384, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 384, y + 295, 254, 231, 254);
	gfx.PutPixel(x + 384, y + 296, 254, 53, 254);
	gfx.PutPixel(x + 385, y + 97, 254, 59, 254);
	gfx.PutPixel(x + 385, y + 98, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 109, 254, 21, 254);
	gfx.PutPixel(x + 385, y + 289, 254, 152, 254);
	gfx.PutPixel(x + 385, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 385, y + 294, 254, 210, 254);
	gfx.PutPixel(x + 385, y + 295, 254, 30, 254);
	gfx.PutPixel(x + 386, y + 98, 254, 215, 254);
	gfx.PutPixel(x + 386, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 109, 254, 53, 254);
	gfx.PutPixel(x + 386, y + 288, 254, 20, 254);
	gfx.PutPixel(x + 386, y + 289, 254, 249, 254);
	gfx.PutPixel(x + 386, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 386, y + 293, 254, 152, 254);
	gfx.PutPixel(x + 386, y + 294, 254, 10, 254);
	gfx.PutPixel(x + 387, y + 98, 254, 97, 254);
	gfx.PutPixel(x + 387, y + 99, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 109, 254, 131, 254);
	gfx.PutPixel(x + 387, y + 288, 254, 55, 254);
	gfx.PutPixel(x + 387, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 387, y + 290, 254, 243, 254);
	gfx.PutPixel(x + 387, y + 291, 254, 166, 254);
	gfx.PutPixel(x + 387, y + 292, 254, 48, 254);
	gfx.PutPixel(x + 388, y + 98, 254, 2, 254);
	gfx.PutPixel(x + 388, y + 99, 254, 209, 254);
	gfx.PutPixel(x + 388, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 388, y + 109, 254, 228, 254);
	gfx.PutPixel(x + 388, y + 110, 254, 4, 254);
	gfx.PutPixel(x + 388, y + 289, 254, 16, 254);
	gfx.PutPixel(x + 388, y + 290, 254, 2, 254);
	gfx.PutPixel(x + 389, y + 99, 254, 52, 254);
	gfx.PutPixel(x + 389, y + 100, 254, 250, 254);
	gfx.PutPixel(x + 389, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 389, y + 110, 254, 123, 254);
	gfx.PutPixel(x + 390, y + 100, 254, 113, 254);
	gfx.PutPixel(x + 390, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 390, y + 110, 254, 251, 254);
	gfx.PutPixel(x + 390, y + 111, 254, 77, 254);
	gfx.PutPixel(x + 391, y + 101, 254, 154, 254);
	gfx.PutPixel(x + 391, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 391, y + 111, 254, 249, 254);
	gfx.PutPixel(x + 391, y + 112, 254, 104, 254);
	gfx.PutPixel(x + 392, y + 101, 254, 1, 254);
	gfx.PutPixel(x + 392, y + 102, 254, 158, 254);
	gfx.PutPixel(x + 392, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 392, y + 113, 254, 192, 254);
	gfx.PutPixel(x + 392, y + 114, 254, 53, 254);
	gfx.PutPixel(x + 393, y + 103, 254, 124, 254);
	gfx.PutPixel(x + 393, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 393, y + 115, 254, 182, 254);
	gfx.PutPixel(x + 393, y + 116, 254, 91, 254);
	gfx.PutPixel(x + 393, y + 117, 254, 11, 254);
	gfx.PutPixel(x + 393, y + 160, 254, 0, 254);
	gfx.PutPixel(x + 393, y + 161, 254, 8, 254);
	gfx.PutPixel(x + 393, y + 162, 254, 15, 254);
	gfx.PutPixel(x + 393, y + 163, 254, 21, 254);
	gfx.PutPixel(x + 393, y + 164, 254, 25, 254);
	gfx.PutPixel(x + 393, y + 165, 254, 29, 254);
	gfx.PutPixel(x + 393, y + 166, 254, 30, 254);
	gfx.PutPixel(x + 393, y + 167, 254, 24, 254);
	gfx.PutPixel(x + 393, y + 168, 254, 11, 254);
	gfx.PutPixel(x + 394, y + 104, 254, 90, 254);
	gfx.PutPixel(x + 394, y + 105, 254, 244, 254);
	gfx.PutPixel(x + 394, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 117, 254, 244, 254);
	gfx.PutPixel(x + 394, y + 118, 254, 186, 254);
	gfx.PutPixel(x + 394, y + 119, 254, 137, 254);
	gfx.PutPixel(x + 394, y + 120, 254, 87, 254);
	gfx.PutPixel(x + 394, y + 121, 254, 48, 254);
	gfx.PutPixel(x + 394, y + 122, 254, 33, 254);
	gfx.PutPixel(x + 394, y + 123, 254, 19, 254);
	gfx.PutPixel(x + 394, y + 124, 254, 4, 254);
	gfx.PutPixel(x + 394, y + 125, 254, 0, 254);
	gfx.PutPixel(x + 394, y + 126, 254, 1, 254);
	gfx.PutPixel(x + 394, y + 127, 254, 3, 254);
	gfx.PutPixel(x + 394, y + 128, 254, 7, 254);
	gfx.PutPixel(x + 394, y + 129, 254, 11, 254);
	gfx.PutPixel(x + 394, y + 130, 254, 16, 254);
	gfx.PutPixel(x + 394, y + 131, 254, 20, 254);
	gfx.PutPixel(x + 394, y + 132, 254, 26, 254);
	gfx.PutPixel(x + 394, y + 133, 254, 32, 254);
	gfx.PutPixel(x + 394, y + 134, 254, 37, 254);
	gfx.PutPixel(x + 394, y + 135, 254, 43, 254);
	gfx.PutPixel(x + 394, y + 136, 254, 51, 254);
	gfx.PutPixel(x + 394, y + 137, 254, 58, 254);
	gfx.PutPixel(x + 394, y + 138, 254, 66, 254);
	gfx.PutPixel(x + 394, y + 139, 254, 73, 254);
	gfx.PutPixel(x + 394, y + 140, 254, 82, 254);
	gfx.PutPixel(x + 394, y + 141, 254, 91, 254);
	gfx.PutPixel(x + 394, y + 142, 254, 99, 254);
	gfx.PutPixel(x + 394, y + 143, 254, 107, 254);
	gfx.PutPixel(x + 394, y + 144, 254, 116, 254);
	gfx.PutPixel(x + 394, y + 145, 254, 124, 254);
	gfx.PutPixel(x + 394, y + 146, 254, 133, 254);
	gfx.PutPixel(x + 394, y + 147, 254, 141, 254);
	gfx.PutPixel(x + 394, y + 148, 254, 150, 254);
	gfx.PutPixel(x + 394, y + 149, 254, 158, 254);
	gfx.PutPixel(x + 394, y + 150, 254, 167, 254);
	gfx.PutPixel(x + 394, y + 151, 254, 175, 254);
	gfx.PutPixel(x + 394, y + 152, 254, 184, 254);
	gfx.PutPixel(x + 394, y + 153, 254, 193, 254);
	gfx.PutPixel(x + 394, y + 154, 254, 202, 254);
	gfx.PutPixel(x + 394, y + 155, 254, 211, 254);
	gfx.PutPixel(x + 394, y + 156, 254, 221, 254);
	gfx.PutPixel(x + 394, y + 157, 254, 230, 254);
	gfx.PutPixel(x + 394, y + 158, 254, 239, 254);
	gfx.PutPixel(x + 394, y + 159, 254, 247, 254);
	gfx.PutPixel(x + 394, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 394, y + 169, 254, 242, 254);
	gfx.PutPixel(x + 394, y + 170, 254, 202, 254);
	gfx.PutPixel(x + 394, y + 171, 254, 142, 254);
	gfx.PutPixel(x + 394, y + 172, 254, 43, 254);
	gfx.PutPixel(x + 395, y + 105, 254, 34, 254);
	gfx.PutPixel(x + 395, y + 106, 254, 191, 254);
	gfx.PutPixel(x + 395, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 395, y + 172, 254, 252, 254);
	gfx.PutPixel(x + 395, y + 173, 254, 96, 254);
	gfx.PutPixel(x + 396, y + 106, 254, 0, 254);
	gfx.PutPixel(x + 396, y + 107, 254, 107, 254);
	gfx.PutPixel(x + 396, y + 108, 254, 241, 254);
	gfx.PutPixel(x + 396, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 396, y + 173, 254, 231, 254);
	gfx.PutPixel(x + 397, y + 108, 254, 18, 254);
	gfx.PutPixel(x + 397, y + 109, 254, 135, 254);
	gfx.PutPixel(x + 397, y + 110, 254, 243, 254);
	gfx.PutPixel(x + 397, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 172, 255, 255, 255);
	gfx.PutPixel(x + 397, y + 173, 254, 190, 254);
	gfx.PutPixel(x + 398, y + 110, 254, 22, 254);
	gfx.PutPixel(x + 398, y + 111, 254, 140, 254);
	gfx.PutPixel(x + 398, y + 112, 254, 237, 254);
	gfx.PutPixel(x + 398, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 398, y + 171, 254, 230, 254);
	gfx.PutPixel(x + 398, y + 172, 254, 115, 254);
	gfx.PutPixel(x + 398, y + 173, 254, 5, 254);
	gfx.PutPixel(x + 399, y + 112, 254, 6, 254);
	gfx.PutPixel(x + 399, y + 113, 254, 87, 254);
	gfx.PutPixel(x + 399, y + 114, 254, 186, 254);
	gfx.PutPixel(x + 399, y + 115, 254, 252, 254);
	gfx.PutPixel(x + 399, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 399, y + 168, 254, 231, 254);
	gfx.PutPixel(x + 399, y + 169, 254, 159, 254);
	gfx.PutPixel(x + 399, y + 170, 254, 75, 254);
	gfx.PutPixel(x + 399, y + 171, 254, 4, 254);
	gfx.PutPixel(x + 400, y + 114, 254, 3, 254);
	gfx.PutPixel(x + 400, y + 115, 254, 188, 254);
	gfx.PutPixel(x + 400, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 400, y + 163, 254, 247, 254);
	gfx.PutPixel(x + 400, y + 164, 254, 203, 254);
	gfx.PutPixel(x + 400, y + 165, 254, 153, 254);
	gfx.PutPixel(x + 400, y + 166, 254, 100, 254);
	gfx.PutPixel(x + 400, y + 167, 254, 40, 254);
	gfx.PutPixel(x + 400, y + 168, 254, 0, 254);
	gfx.PutPixel(x + 401, y + 113, 254, 0, 254);
	gfx.PutPixel(x + 401, y + 114, 254, 163, 254);
	gfx.PutPixel(x + 401, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 401, y + 156, 254, 239, 254);
	gfx.PutPixel(x + 401, y + 157, 254, 211, 254);
	gfx.PutPixel(x + 401, y + 158, 254, 181, 254);
	gfx.PutPixel(x + 401, y + 159, 254, 147, 254);
	gfx.PutPixel(x + 401, y + 160, 254, 113, 254);
	gfx.PutPixel(x + 401, y + 161, 254, 77, 254);
	gfx.PutPixel(x + 401, y + 162, 254, 35, 254);
	gfx.PutPixel(x + 401, y + 163, 254, 2, 254);
	gfx.PutPixel(x + 402, y + 113, 254, 149, 254);
	gfx.PutPixel(x + 402, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 402, y + 135, 254, 253, 254);
	gfx.PutPixel(x + 402, y + 136, 254, 251, 254);
	gfx.PutPixel(x + 402, y + 137, 254, 249, 254);
	gfx.PutPixel(x + 402, y + 138, 254, 247, 254);
	gfx.PutPixel(x + 402, y + 139, 254, 244, 254);
	gfx.PutPixel(x + 402, y + 140, 254, 238, 254);
	gfx.PutPixel(x + 402, y + 141, 254, 231, 254);
	gfx.PutPixel(x + 402, y + 142, 254, 225, 254);
	gfx.PutPixel(x + 402, y + 143, 254, 218, 254);
	gfx.PutPixel(x + 402, y + 144, 254, 209, 254);
	gfx.PutPixel(x + 402, y + 145, 254, 197, 254);
	gfx.PutPixel(x + 402, y + 146, 254, 185, 254);
	gfx.PutPixel(x + 402, y + 147, 254, 173, 254);
	gfx.PutPixel(x + 402, y + 148, 254, 157, 254);
	gfx.PutPixel(x + 402, y + 149, 254, 140, 254);
	gfx.PutPixel(x + 402, y + 150, 254, 122, 254);
	gfx.PutPixel(x + 402, y + 151, 254, 104, 254);
	gfx.PutPixel(x + 402, y + 152, 254, 82, 254);
	gfx.PutPixel(x + 402, y + 153, 254, 59, 254);
	gfx.PutPixel(x + 402, y + 154, 254, 36, 254);
	gfx.PutPixel(x + 402, y + 155, 254, 11, 254);
	gfx.PutPixel(x + 403, y + 112, 254, 134, 254);
	gfx.PutPixel(x + 403, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 403, y + 127, 254, 239, 254);
	gfx.PutPixel(x + 403, y + 128, 254, 74, 254);
	gfx.PutPixel(x + 404, y + 111, 254, 99, 254);
	gfx.PutPixel(x + 404, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 404, y + 126, 254, 211, 254);
	gfx.PutPixel(x + 404, y + 127, 254, 36, 254);
	gfx.PutPixel(x + 405, y + 110, 254, 55, 254);
	gfx.PutPixel(x + 405, y + 111, 254, 248, 254);
	gfx.PutPixel(x + 405, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 405, y + 125, 254, 170, 254);
	gfx.PutPixel(x + 405, y + 126, 254, 11, 254);
	gfx.PutPixel(x + 406, y + 109, 254, 24, 254);
	gfx.PutPixel(x + 406, y + 110, 254, 228, 254);
	gfx.PutPixel(x + 406, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 406, y + 124, 254, 150, 254);
	gfx.PutPixel(x + 406, y + 125, 254, 0, 254);
	gfx.PutPixel(x + 407, y + 108, 254, 5, 254);
	gfx.PutPixel(x + 407, y + 109, 254, 196, 254);
	gfx.PutPixel(x + 407, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 407, y + 123, 254, 146, 254);
	gfx.PutPixel(x + 408, y + 108, 254, 139, 254);
	gfx.PutPixel(x + 408, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 408, y + 122, 254, 142, 254);
	gfx.PutPixel(x + 409, y + 107, 254, 64, 254);
	gfx.PutPixel(x + 409, y + 108, 254, 253, 254);
	gfx.PutPixel(x + 409, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 409, y + 121, 254, 147, 254);
	gfx.PutPixel(x + 410, y + 106, 254, 14, 254);
	gfx.PutPixel(x + 410, y + 107, 254, 224, 254);
	gfx.PutPixel(x + 410, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 410, y + 120, 254, 176, 254);
	gfx.PutPixel(x + 410, y + 121, 254, 1, 254);
	gfx.PutPixel(x + 411, y + 106, 254, 163, 254);
	gfx.PutPixel(x + 411, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 411, y + 119, 254, 202, 254);
	gfx.PutPixel(x + 411, y + 120, 254, 9, 254);
	gfx.PutPixel(x + 412, y + 105, 254, 76, 254);
	gfx.PutPixel(x + 412, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 412, y + 118, 254, 223, 254);
	gfx.PutPixel(x + 412, y + 119, 254, 22, 254);
	gfx.PutPixel(x + 413, y + 104, 254, 9, 254);
	gfx.PutPixel(x + 413, y + 105, 254, 223, 254);
	gfx.PutPixel(x + 413, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 413, y + 117, 254, 248, 254);
	gfx.PutPixel(x + 413, y + 118, 254, 51, 254);
	gfx.PutPixel(x + 414, y + 104, 254, 134, 254);
	gfx.PutPixel(x + 414, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 414, y + 117, 254, 106, 254);
	gfx.PutPixel(x + 415, y + 103, 254, 41, 254);
	gfx.PutPixel(x + 415, y + 104, 254, 249, 254);
	gfx.PutPixel(x + 415, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 415, y + 116, 254, 168, 254);
	gfx.PutPixel(x + 416, y + 103, 254, 185, 254);
	gfx.PutPixel(x + 416, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 416, y + 115, 254, 224, 254);
	gfx.PutPixel(x + 416, y + 116, 254, 13, 254);
	gfx.PutPixel(x + 417, y + 102, 254, 68, 254);
	gfx.PutPixel(x + 417, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 417, y + 115, 254, 70, 254);
	gfx.PutPixel(x + 418, y + 102, 254, 201, 254);
	gfx.PutPixel(x + 418, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 418, y + 114, 254, 157, 254);
	gfx.PutPixel(x + 419, y + 101, 254, 75, 254);
	gfx.PutPixel(x + 419, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 419, y + 113, 254, 232, 254);
	gfx.PutPixel(x + 419, y + 114, 254, 15, 254);
	gfx.PutPixel(x + 420, y + 101, 254, 196, 254);
	gfx.PutPixel(x + 420, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 420, y + 113, 254, 103, 254);
	gfx.PutPixel(x + 421, y + 100, 254, 55, 254);
	gfx.PutPixel(x + 421, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 421, y + 112, 254, 211, 254);
	gfx.PutPixel(x + 421, y + 113, 254, 2, 254);
	gfx.PutPixel(x + 422, y + 100, 254, 160, 254);
	gfx.PutPixel(x + 422, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 422, y + 112, 254, 77, 254);
	gfx.PutPixel(x + 423, y + 99, 254, 12, 254);
	gfx.PutPixel(x + 423, y + 100, 254, 245, 254);
	gfx.PutPixel(x + 423, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 423, y + 111, 254, 211, 254);
	gfx.PutPixel(x + 423, y + 112, 254, 0, 254);
	gfx.PutPixel(x + 423, y + 301, 254, 5, 254);
	gfx.PutPixel(x + 423, y + 302, 254, 13, 254);
	gfx.PutPixel(x + 423, y + 303, 254, 22, 254);
	gfx.PutPixel(x + 423, y + 304, 254, 31, 254);
	gfx.PutPixel(x + 423, y + 305, 254, 39, 254);
	gfx.PutPixel(x + 423, y + 306, 254, 48, 254);
	gfx.PutPixel(x + 423, y + 307, 254, 56, 254);
	gfx.PutPixel(x + 423, y + 308, 254, 66, 254);
	gfx.PutPixel(x + 423, y + 309, 254, 75, 254);
	gfx.PutPixel(x + 423, y + 310, 254, 84, 254);
	gfx.PutPixel(x + 423, y + 311, 254, 95, 254);
	gfx.PutPixel(x + 423, y + 312, 254, 104, 254);
	gfx.PutPixel(x + 423, y + 313, 254, 115, 254);
	gfx.PutPixel(x + 423, y + 314, 254, 126, 254);
	gfx.PutPixel(x + 423, y + 315, 254, 138, 254);
	gfx.PutPixel(x + 423, y + 316, 254, 150, 254);
	gfx.PutPixel(x + 423, y + 317, 254, 161, 254);
	gfx.PutPixel(x + 423, y + 318, 254, 171, 254);
	gfx.PutPixel(x + 423, y + 319, 254, 175, 254);
	gfx.PutPixel(x + 423, y + 320, 254, 164, 254);
	gfx.PutPixel(x + 423, y + 321, 254, 128, 254);
	gfx.PutPixel(x + 423, y + 322, 254, 39, 254);
	gfx.PutPixel(x + 424, y + 99, 254, 91, 254);
	gfx.PutPixel(x + 424, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 111, 254, 107, 254);
	gfx.PutPixel(x + 424, y + 263, 254, 1, 254);
	gfx.PutPixel(x + 424, y + 264, 254, 2, 254);
	gfx.PutPixel(x + 424, y + 265, 254, 5, 254);
	gfx.PutPixel(x + 424, y + 266, 254, 7, 254);
	gfx.PutPixel(x + 424, y + 267, 254, 12, 254);
	gfx.PutPixel(x + 424, y + 268, 254, 17, 254);
	gfx.PutPixel(x + 424, y + 269, 254, 21, 254);
	gfx.PutPixel(x + 424, y + 270, 254, 27, 254);
	gfx.PutPixel(x + 424, y + 271, 254, 32, 254);
	gfx.PutPixel(x + 424, y + 272, 254, 37, 254);
	gfx.PutPixel(x + 424, y + 273, 254, 44, 254);
	gfx.PutPixel(x + 424, y + 274, 254, 51, 254);
	gfx.PutPixel(x + 424, y + 275, 254, 58, 254);
	gfx.PutPixel(x + 424, y + 276, 254, 64, 254);
	gfx.PutPixel(x + 424, y + 277, 254, 71, 254);
	gfx.PutPixel(x + 424, y + 278, 254, 77, 254);
	gfx.PutPixel(x + 424, y + 279, 254, 85, 254);
	gfx.PutPixel(x + 424, y + 280, 254, 92, 254);
	gfx.PutPixel(x + 424, y + 281, 254, 100, 254);
	gfx.PutPixel(x + 424, y + 282, 254, 107, 254);
	gfx.PutPixel(x + 424, y + 283, 254, 114, 254);
	gfx.PutPixel(x + 424, y + 284, 254, 121, 254);
	gfx.PutPixel(x + 424, y + 285, 254, 130, 254);
	gfx.PutPixel(x + 424, y + 286, 254, 138, 254);
	gfx.PutPixel(x + 424, y + 287, 254, 146, 254);
	gfx.PutPixel(x + 424, y + 288, 254, 154, 254);
	gfx.PutPixel(x + 424, y + 289, 254, 162, 254);
	gfx.PutPixel(x + 424, y + 290, 254, 170, 254);
	gfx.PutPixel(x + 424, y + 291, 254, 178, 254);
	gfx.PutPixel(x + 424, y + 292, 254, 186, 254);
	gfx.PutPixel(x + 424, y + 293, 254, 195, 254);
	gfx.PutPixel(x + 424, y + 294, 254, 203, 254);
	gfx.PutPixel(x + 424, y + 295, 254, 211, 254);
	gfx.PutPixel(x + 424, y + 296, 254, 218, 254);
	gfx.PutPixel(x + 424, y + 297, 254, 227, 254);
	gfx.PutPixel(x + 424, y + 298, 254, 235, 254);
	gfx.PutPixel(x + 424, y + 299, 254, 244, 254);
	gfx.PutPixel(x + 424, y + 300, 254, 252, 254);
	gfx.PutPixel(x + 424, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 424, y + 322, 254, 235, 254);
	gfx.PutPixel(x + 425, y + 99, 254, 166, 254);
	gfx.PutPixel(x + 425, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 111, 254, 31, 254);
	gfx.PutPixel(x + 425, y + 251, 254, 31, 254);
	gfx.PutPixel(x + 425, y + 252, 254, 118, 254);
	gfx.PutPixel(x + 425, y + 253, 254, 169, 254);
	gfx.PutPixel(x + 425, y + 254, 254, 203, 254);
	gfx.PutPixel(x + 425, y + 255, 254, 221, 254);
	gfx.PutPixel(x + 425, y + 256, 254, 235, 254);
	gfx.PutPixel(x + 425, y + 257, 254, 243, 254);
	gfx.PutPixel(x + 425, y + 258, 254, 248, 254);
	gfx.PutPixel(x + 425, y + 259, 254, 252, 254);
	gfx.PutPixel(x + 425, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 425, y + 322, 254, 232, 254);
	gfx.PutPixel(x + 426, y + 99, 254, 223, 254);
	gfx.PutPixel(x + 426, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 111, 254, 10, 254);
	gfx.PutPixel(x + 426, y + 250, 254, 87, 254);
	gfx.PutPixel(x + 426, y + 251, 254, 248, 254);
	gfx.PutPixel(x + 426, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 426, y + 322, 254, 188, 254);
	gfx.PutPixel(x + 427, y + 99, 254, 247, 254);
	gfx.PutPixel(x + 427, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 111, 254, 47, 254);
	gfx.PutPixel(x + 427, y + 250, 254, 222, 254);
	gfx.PutPixel(x + 427, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 321, 255, 255, 255);
	gfx.PutPixel(x + 427, y + 322, 254, 107, 254);
	gfx.PutPixel(x + 428, y + 99, 254, 212, 254);
	gfx.PutPixel(x + 428, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 111, 254, 137, 254);
	gfx.PutPixel(x + 428, y + 250, 254, 239, 254);
	gfx.PutPixel(x + 428, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 320, 255, 255, 255);
	gfx.PutPixel(x + 428, y + 321, 254, 221, 254);
	gfx.PutPixel(x + 428, y + 322, 254, 7, 254);
	gfx.PutPixel(x + 429, y + 99, 254, 117, 254);
	gfx.PutPixel(x + 429, y + 100, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 111, 254, 238, 254);
	gfx.PutPixel(x + 429, y + 112, 254, 27, 254);
	gfx.PutPixel(x + 429, y + 250, 254, 123, 254);
	gfx.PutPixel(x + 429, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 319, 255, 255, 255);
	gfx.PutPixel(x + 429, y + 320, 254, 239, 254);
	gfx.PutPixel(x + 429, y + 321, 254, 54, 254);
	gfx.PutPixel(x + 430, y + 99, 254, 8, 254);
	gfx.PutPixel(x + 430, y + 100, 254, 219, 254);
	gfx.PutPixel(x + 430, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 112, 254, 191, 254);
	gfx.PutPixel(x + 430, y + 113, 254, 6, 254);
	gfx.PutPixel(x + 430, y + 251, 254, 101, 254);
	gfx.PutPixel(x + 430, y + 252, 254, 235, 254);
	gfx.PutPixel(x + 430, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 317, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 318, 255, 255, 255);
	gfx.PutPixel(x + 430, y + 319, 254, 208, 254);
	gfx.PutPixel(x + 430, y + 320, 254, 47, 254);
	gfx.PutPixel(x + 431, y + 100, 254, 53, 254);
	gfx.PutPixel(x + 431, y + 101, 254, 246, 254);
	gfx.PutPixel(x + 431, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 113, 254, 192, 254);
	gfx.PutPixel(x + 431, y + 114, 254, 16, 254);
	gfx.PutPixel(x + 431, y + 252, 254, 12, 254);
	gfx.PutPixel(x + 431, y + 253, 254, 106, 254);
	gfx.PutPixel(x + 431, y + 254, 254, 199, 254);
	gfx.PutPixel(x + 431, y + 255, 254, 253, 254);
	gfx.PutPixel(x + 431, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 310, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 311, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 312, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 313, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 314, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 315, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 316, 255, 255, 255);
	gfx.PutPixel(x + 431, y + 317, 254, 203, 254);
	gfx.PutPixel(x + 431, y + 318, 254, 101, 254);
	gfx.PutPixel(x + 431, y + 319, 254, 2, 254);
	gfx.PutPixel(x + 432, y + 101, 254, 83, 254);
	gfx.PutPixel(x + 432, y + 102, 254, 251, 254);
	gfx.PutPixel(x + 432, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 114, 254, 223, 254);
	gfx.PutPixel(x + 432, y + 115, 254, 77, 254);
	gfx.PutPixel(x + 432, y + 255, 254, 23, 254);
	gfx.PutPixel(x + 432, y + 256, 254, 89, 254);
	gfx.PutPixel(x + 432, y + 257, 254, 147, 254);
	gfx.PutPixel(x + 432, y + 258, 254, 201, 254);
	gfx.PutPixel(x + 432, y + 259, 254, 244, 254);
	gfx.PutPixel(x + 432, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 299, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 300, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 301, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 302, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 303, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 304, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 305, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 306, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 307, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 308, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 309, 255, 255, 255);
	gfx.PutPixel(x + 432, y + 310, 254, 250, 254);
	gfx.PutPixel(x + 432, y + 311, 254, 243, 254);
	gfx.PutPixel(x + 432, y + 312, 254, 226, 254);
	gfx.PutPixel(x + 432, y + 313, 254, 201, 254);
	gfx.PutPixel(x + 432, y + 314, 254, 163, 254);
	gfx.PutPixel(x + 432, y + 315, 254, 110, 254);
	gfx.PutPixel(x + 432, y + 316, 254, 39, 254);
	gfx.PutPixel(x + 433, y + 102, 254, 81, 254);
	gfx.PutPixel(x + 433, y + 103, 254, 246, 254);
	gfx.PutPixel(x + 433, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 116, 254, 201, 254);
	gfx.PutPixel(x + 433, y + 117, 254, 94, 254);
	gfx.PutPixel(x + 433, y + 118, 254, 12, 254);
	gfx.PutPixel(x + 433, y + 259, 254, 1, 254);
	gfx.PutPixel(x + 433, y + 260, 254, 32, 254);
	gfx.PutPixel(x + 433, y + 261, 254, 67, 254);
	gfx.PutPixel(x + 433, y + 262, 254, 99, 254);
	gfx.PutPixel(x + 433, y + 263, 254, 127, 254);
	gfx.PutPixel(x + 433, y + 264, 254, 151, 254);
	gfx.PutPixel(x + 433, y + 265, 254, 169, 254);
	gfx.PutPixel(x + 433, y + 266, 254, 187, 254);
	gfx.PutPixel(x + 433, y + 267, 254, 205, 254);
	gfx.PutPixel(x + 433, y + 268, 254, 222, 254);
	gfx.PutPixel(x + 433, y + 269, 254, 231, 254);
	gfx.PutPixel(x + 433, y + 270, 254, 237, 254);
	gfx.PutPixel(x + 433, y + 271, 254, 242, 254);
	gfx.PutPixel(x + 433, y + 272, 254, 247, 254);
	gfx.PutPixel(x + 433, y + 273, 254, 253, 254);
	gfx.PutPixel(x + 433, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 295, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 296, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 297, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 298, 255, 255, 255);
	gfx.PutPixel(x + 433, y + 299, 254, 221, 254);
	gfx.PutPixel(x + 433, y + 300, 254, 170, 254);
	gfx.PutPixel(x + 433, y + 301, 254, 132, 254);
	gfx.PutPixel(x + 433, y + 302, 254, 99, 254);
	gfx.PutPixel(x + 433, y + 303, 254, 73, 254);
	gfx.PutPixel(x + 433, y + 304, 254, 51, 254);
	gfx.PutPixel(x + 433, y + 305, 254, 33, 254);
	gfx.PutPixel(x + 433, y + 306, 254, 20, 254);
	gfx.PutPixel(x + 433, y + 307, 254, 9, 254);
	gfx.PutPixel(x + 433, y + 308, 254, 4, 254);
	gfx.PutPixel(x + 433, y + 309, 254, 0, 254);
	gfx.PutPixel(x + 434, y + 103, 254, 51, 254);
	gfx.PutPixel(x + 434, y + 104, 254, 224, 254);
	gfx.PutPixel(x + 434, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 118, 254, 246, 254);
	gfx.PutPixel(x + 434, y + 119, 254, 185, 254);
	gfx.PutPixel(x + 434, y + 120, 254, 125, 254);
	gfx.PutPixel(x + 434, y + 121, 254, 81, 254);
	gfx.PutPixel(x + 434, y + 122, 254, 46, 254);
	gfx.PutPixel(x + 434, y + 123, 254, 24, 254);
	gfx.PutPixel(x + 434, y + 124, 254, 9, 254);
	gfx.PutPixel(x + 434, y + 125, 254, 2, 254);
	gfx.PutPixel(x + 434, y + 276, 254, 1, 254);
	gfx.PutPixel(x + 434, y + 277, 254, 120, 254);
	gfx.PutPixel(x + 434, y + 278, 254, 249, 254);
	gfx.PutPixel(x + 434, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 292, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 293, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 294, 255, 255, 255);
	gfx.PutPixel(x + 434, y + 295, 254, 213, 254);
	gfx.PutPixel(x + 434, y + 296, 254, 131, 254);
	gfx.PutPixel(x + 434, y + 297, 254, 70, 254);
	gfx.PutPixel(x + 434, y + 298, 254, 18, 254);
	gfx.PutPixel(x + 435, y + 104, 254, 15, 254);
	gfx.PutPixel(x + 435, y + 105, 254, 163, 254);
	gfx.PutPixel(x + 435, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 165, 254, 251, 254);
	gfx.PutPixel(x + 435, y + 166, 254, 241, 254);
	gfx.PutPixel(x + 435, y + 167, 254, 224, 254);
	gfx.PutPixel(x + 435, y + 168, 254, 192, 254);
	gfx.PutPixel(x + 435, y + 169, 254, 136, 254);
	gfx.PutPixel(x + 435, y + 170, 254, 36, 254);
	gfx.PutPixel(x + 435, y + 275, 254, 19, 254);
	gfx.PutPixel(x + 435, y + 276, 254, 187, 254);
	gfx.PutPixel(x + 435, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 290, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 291, 255, 255, 255);
	gfx.PutPixel(x + 435, y + 292, 254, 220, 254);
	gfx.PutPixel(x + 435, y + 293, 254, 123, 254);
	gfx.PutPixel(x + 435, y + 294, 254, 40, 254);
	gfx.PutPixel(x + 436, y + 106, 254, 69, 254);
	gfx.PutPixel(x + 436, y + 107, 254, 215, 254);
	gfx.PutPixel(x + 436, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 170, 254, 247, 254);
	gfx.PutPixel(x + 436, y + 171, 254, 64, 254);
	gfx.PutPixel(x + 436, y + 274, 254, 46, 254);
	gfx.PutPixel(x + 436, y + 275, 254, 224, 254);
	gfx.PutPixel(x + 436, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 288, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 289, 255, 255, 255);
	gfx.PutPixel(x + 436, y + 290, 254, 200, 254);
	gfx.PutPixel(x + 436, y + 291, 254, 84, 254);
	gfx.PutPixel(x + 436, y + 292, 254, 2, 254);
	gfx.PutPixel(x + 437, y + 107, 254, 4, 254);
	gfx.PutPixel(x + 437, y + 108, 254, 100, 254);
	gfx.PutPixel(x + 437, y + 109, 254, 224, 254);
	gfx.PutPixel(x + 437, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 171, 254, 188, 254);
	gfx.PutPixel(x + 437, y + 273, 254, 71, 254);
	gfx.PutPixel(x + 437, y + 274, 254, 241, 254);
	gfx.PutPixel(x + 437, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 287, 255, 255, 255);
	gfx.PutPixel(x + 437, y + 288, 254, 210, 254);
	gfx.PutPixel(x + 437, y + 289, 254, 66, 254);
	gfx.PutPixel(x + 438, y + 109, 254, 4, 254);
	gfx.PutPixel(x + 438, y + 110, 254, 90, 254);
	gfx.PutPixel(x + 438, y + 111, 254, 202, 254);
	gfx.PutPixel(x + 438, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 171, 254, 219, 254);
	gfx.PutPixel(x + 438, y + 272, 254, 86, 254);
	gfx.PutPixel(x + 438, y + 273, 254, 249, 254);
	gfx.PutPixel(x + 438, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 438, y + 286, 254, 239, 254);
	gfx.PutPixel(x + 438, y + 287, 254, 110, 254);
	gfx.PutPixel(x + 438, y + 288, 254, 3, 254);
	gfx.PutPixel(x + 439, y + 112, 254, 47, 254);
	gfx.PutPixel(x + 439, y + 113, 254, 144, 254);
	gfx.PutPixel(x + 439, y + 114, 254, 231, 254);
	gfx.PutPixel(x + 439, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 171, 254, 184, 254);
	gfx.PutPixel(x + 439, y + 271, 254, 94, 254);
	gfx.PutPixel(x + 439, y + 272, 254, 252, 254);
	gfx.PutPixel(x + 439, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 439, y + 285, 254, 187, 254);
	gfx.PutPixel(x + 439, y + 286, 254, 23, 254);
	gfx.PutPixel(x + 440, y + 114, 254, 1, 254);
	gfx.PutPixel(x + 440, y + 115, 254, 56, 254);
	gfx.PutPixel(x + 440, y + 116, 254, 132, 254);
	gfx.PutPixel(x + 440, y + 117, 254, 194, 254);
	gfx.PutPixel(x + 440, y + 118, 254, 247, 254);
	gfx.PutPixel(x + 440, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 170, 254, 248, 254);
	gfx.PutPixel(x + 440, y + 171, 254, 63, 254);
	gfx.PutPixel(x + 440, y + 270, 254, 85, 254);
	gfx.PutPixel(x + 440, y + 271, 254, 251, 254);
	gfx.PutPixel(x + 440, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 440, y + 283, 254, 251, 254);
	gfx.PutPixel(x + 440, y + 284, 254, 123, 254);
	gfx.PutPixel(x + 440, y + 285, 254, 1, 254);
	gfx.PutPixel(x + 441, y + 118, 254, 6, 254);
	gfx.PutPixel(x + 441, y + 119, 254, 49, 254);
	gfx.PutPixel(x + 441, y + 120, 254, 90, 254);
	gfx.PutPixel(x + 441, y + 121, 254, 131, 254);
	gfx.PutPixel(x + 441, y + 122, 254, 161, 254);
	gfx.PutPixel(x + 441, y + 123, 254, 188, 254);
	gfx.PutPixel(x + 441, y + 124, 254, 213, 254);
	gfx.PutPixel(x + 441, y + 125, 254, 234, 254);
	gfx.PutPixel(x + 441, y + 126, 254, 251, 254);
	gfx.PutPixel(x + 441, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 169, 254, 193, 254);
	gfx.PutPixel(x + 441, y + 170, 254, 57, 254);
	gfx.PutPixel(x + 441, y + 269, 254, 76, 254);
	gfx.PutPixel(x + 441, y + 270, 254, 249, 254);
	gfx.PutPixel(x + 441, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 441, y + 282, 254, 234, 254);
	gfx.PutPixel(x + 441, y + 283, 254, 62, 254);
	gfx.PutPixel(x + 442, y + 126, 254, 0, 254);
	gfx.PutPixel(x + 442, y + 127, 254, 12, 254);
	gfx.PutPixel(x + 442, y + 128, 254, 26, 254);
	gfx.PutPixel(x + 442, y + 129, 254, 38, 254);
	gfx.PutPixel(x + 442, y + 130, 254, 50, 254);
	gfx.PutPixel(x + 442, y + 131, 254, 61, 254);
	gfx.PutPixel(x + 442, y + 132, 254, 71, 254);
	gfx.PutPixel(x + 442, y + 133, 254, 82, 254);
	gfx.PutPixel(x + 442, y + 134, 254, 92, 254);
	gfx.PutPixel(x + 442, y + 135, 254, 103, 254);
	gfx.PutPixel(x + 442, y + 136, 254, 112, 254);
	gfx.PutPixel(x + 442, y + 137, 254, 121, 254);
	gfx.PutPixel(x + 442, y + 138, 254, 131, 254);
	gfx.PutPixel(x + 442, y + 139, 254, 140, 254);
	gfx.PutPixel(x + 442, y + 140, 254, 149, 254);
	gfx.PutPixel(x + 442, y + 141, 254, 157, 254);
	gfx.PutPixel(x + 442, y + 142, 254, 165, 254);
	gfx.PutPixel(x + 442, y + 143, 254, 173, 254);
	gfx.PutPixel(x + 442, y + 144, 254, 180, 254);
	gfx.PutPixel(x + 442, y + 145, 254, 188, 254);
	gfx.PutPixel(x + 442, y + 146, 254, 195, 254);
	gfx.PutPixel(x + 442, y + 147, 254, 203, 254);
	gfx.PutPixel(x + 442, y + 148, 254, 211, 254);
	gfx.PutPixel(x + 442, y + 149, 254, 218, 254);
	gfx.PutPixel(x + 442, y + 150, 254, 226, 254);
	gfx.PutPixel(x + 442, y + 151, 254, 231, 254);
	gfx.PutPixel(x + 442, y + 152, 254, 236, 254);
	gfx.PutPixel(x + 442, y + 153, 254, 241, 254);
	gfx.PutPixel(x + 442, y + 154, 254, 245, 254);
	gfx.PutPixel(x + 442, y + 155, 254, 248, 254);
	gfx.PutPixel(x + 442, y + 156, 254, 250, 254);
	gfx.PutPixel(x + 442, y + 157, 254, 252, 254);
	gfx.PutPixel(x + 442, y + 158, 254, 253, 254);
	gfx.PutPixel(x + 442, y + 159, 254, 253, 254);
	gfx.PutPixel(x + 442, y + 160, 254, 250, 254);
	gfx.PutPixel(x + 442, y + 161, 254, 245, 254);
	gfx.PutPixel(x + 442, y + 162, 254, 236, 254);
	gfx.PutPixel(x + 442, y + 163, 254, 221, 254);
	gfx.PutPixel(x + 442, y + 164, 254, 202, 254);
	gfx.PutPixel(x + 442, y + 165, 254, 175, 254);
	gfx.PutPixel(x + 442, y + 166, 254, 137, 254);
	gfx.PutPixel(x + 442, y + 167, 254, 94, 254);
	gfx.PutPixel(x + 442, y + 168, 254, 27, 254);
	gfx.PutPixel(x + 442, y + 268, 254, 66, 254);
	gfx.PutPixel(x + 442, y + 269, 254, 247, 254);
	gfx.PutPixel(x + 442, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 442, y + 281, 254, 219, 254);
	gfx.PutPixel(x + 442, y + 282, 254, 36, 254);
	gfx.PutPixel(x + 443, y + 267, 254, 42, 254);
	gfx.PutPixel(x + 443, y + 268, 254, 240, 254);
	gfx.PutPixel(x + 443, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 443, y + 280, 254, 200, 254);
	gfx.PutPixel(x + 443, y + 281, 254, 20, 254);
	gfx.PutPixel(x + 444, y + 266, 254, 22, 254);
	gfx.PutPixel(x + 444, y + 267, 254, 223, 254);
	gfx.PutPixel(x + 444, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 444, y + 279, 254, 185, 254);
	gfx.PutPixel(x + 444, y + 280, 254, 9, 254);
	gfx.PutPixel(x + 445, y + 265, 254, 7, 254);
	gfx.PutPixel(x + 445, y + 266, 254, 198, 254);
	gfx.PutPixel(x + 445, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 445, y + 278, 254, 182, 254);
	gfx.PutPixel(x + 445, y + 279, 254, 7, 254);
	gfx.PutPixel(x + 446, y + 265, 254, 166, 254);
	gfx.PutPixel(x + 446, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 446, y + 277, 254, 180, 254);
	gfx.PutPixel(x + 446, y + 278, 254, 6, 254);
	gfx.PutPixel(x + 447, y + 264, 254, 109, 254);
	gfx.PutPixel(x + 447, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 447, y + 276, 254, 188, 254);
	gfx.PutPixel(x + 447, y + 277, 254, 6, 254);
	gfx.PutPixel(x + 448, y + 263, 254, 54, 254);
	gfx.PutPixel(x + 448, y + 264, 254, 249, 254);
	gfx.PutPixel(x + 448, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 448, y + 275, 254, 198, 254);
	gfx.PutPixel(x + 448, y + 276, 254, 11, 254);
	gfx.PutPixel(x + 449, y + 262, 254, 17, 254);
	gfx.PutPixel(x + 449, y + 263, 254, 224, 254);
	gfx.PutPixel(x + 449, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 449, y + 274, 254, 211, 254);
	gfx.PutPixel(x + 449, y + 275, 254, 16, 254);
	gfx.PutPixel(x + 450, y + 262, 254, 179, 254);
	gfx.PutPixel(x + 450, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 450, y + 273, 254, 229, 254);
	gfx.PutPixel(x + 450, y + 274, 254, 29, 254);
	gfx.PutPixel(x + 451, y + 261, 254, 103, 254);
	gfx.PutPixel(x + 451, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 451, y + 272, 254, 243, 254);
	gfx.PutPixel(x + 451, y + 273, 254, 49, 254);
	gfx.PutPixel(x + 452, y + 260, 254, 33, 254);
	gfx.PutPixel(x + 452, y + 261, 254, 243, 254);
	gfx.PutPixel(x + 452, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 452, y + 271, 254, 253, 254);
	gfx.PutPixel(x + 452, y + 272, 254, 81, 254);
	gfx.PutPixel(x + 453, y + 259, 254, 1, 254);
	gfx.PutPixel(x + 453, y + 260, 254, 193, 254);
	gfx.PutPixel(x + 453, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 453, y + 271, 254, 131, 254);
	gfx.PutPixel(x + 454, y + 259, 254, 114, 254);
	gfx.PutPixel(x + 454, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 454, y + 270, 254, 181, 254);
	gfx.PutPixel(x + 454, y + 271, 254, 1, 254);
	gfx.PutPixel(x + 455, y + 258, 254, 31, 254);
	gfx.PutPixel(x + 455, y + 259, 254, 245, 254);
	gfx.PutPixel(x + 455, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 455, y + 269, 254, 227, 254);
	gfx.PutPixel(x + 455, y + 270, 254, 18, 254);
	gfx.PutPixel(x + 456, y + 258, 254, 175, 254);
	gfx.PutPixel(x + 456, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 456, y + 268, 254, 251, 254);
	gfx.PutPixel(x + 456, y + 269, 254, 61, 254);
	gfx.PutPixel(x + 457, y + 257, 254, 74, 254);
	gfx.PutPixel(x + 457, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 457, y + 268, 254, 130, 254);
	gfx.PutPixel(x + 458, y + 256, 254, 7, 254);
	gfx.PutPixel(x + 458, y + 257, 254, 220, 254);
	gfx.PutPixel(x + 458, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 458, y + 267, 254, 204, 254);
	gfx.PutPixel(x + 458, y + 268, 254, 4, 254);
	gfx.PutPixel(x + 459, y + 256, 254, 122, 254);
	gfx.PutPixel(x + 459, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 459, y + 266, 254, 248, 254);
	gfx.PutPixel(x + 459, y + 267, 254, 42, 254);
	gfx.PutPixel(x + 460, y + 255, 254, 15, 254);
	gfx.PutPixel(x + 460, y + 256, 254, 239, 254);
	gfx.PutPixel(x + 460, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 460, y + 266, 254, 127, 254);
	gfx.PutPixel(x + 461, y + 255, 254, 131, 254);
	gfx.PutPixel(x + 461, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 461, y + 265, 254, 214, 254);
	gfx.PutPixel(x + 461, y + 266, 254, 5, 254);
	gfx.PutPixel(x + 462, y + 254, 254, 19, 254);
	gfx.PutPixel(x + 462, y + 255, 254, 242, 254);
	gfx.PutPixel(x + 462, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 462, y + 265, 254, 66, 254);
	gfx.PutPixel(x + 463, y + 254, 254, 137, 254);
	gfx.PutPixel(x + 463, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 463, y + 264, 254, 175, 254);
	gfx.PutPixel(x + 464, y + 253, 254, 14, 254);
	gfx.PutPixel(x + 464, y + 254, 254, 242, 254);
	gfx.PutPixel(x + 464, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 464, y + 263, 254, 250, 254);
	gfx.PutPixel(x + 464, y + 264, 254, 35, 254);
	gfx.PutPixel(x + 465, y + 253, 254, 114, 254);
	gfx.PutPixel(x + 465, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 465, y + 263, 254, 152, 254);
	gfx.PutPixel(x + 466, y + 252, 254, 0, 254);
	gfx.PutPixel(x + 466, y + 253, 254, 221, 254);
	gfx.PutPixel(x + 466, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 466, y + 262, 254, 247, 254);
	gfx.PutPixel(x + 466, y + 263, 254, 27, 254);
	gfx.PutPixel(x + 467, y + 252, 254, 65, 254);
	gfx.PutPixel(x + 467, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 467, y + 262, 254, 152, 254);
	gfx.PutPixel(x + 468, y + 252, 254, 161, 254);
	gfx.PutPixel(x + 468, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 468, y + 261, 254, 252, 254);
	gfx.PutPixel(x + 468, y + 262, 254, 36, 254);
	gfx.PutPixel(x + 469, y + 251, 254, 5, 254);
	gfx.PutPixel(x + 469, y + 252, 254, 240, 254);
	gfx.PutPixel(x + 469, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 469, y + 261, 254, 177, 254);
	gfx.PutPixel(x + 470, y + 251, 254, 74, 254);
	gfx.PutPixel(x + 470, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 470, y + 261, 254, 74, 254);
	gfx.PutPixel(x + 471, y + 251, 254, 147, 254);
	gfx.PutPixel(x + 471, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 471, y + 260, 254, 228, 254);
	gfx.PutPixel(x + 471, y + 261, 254, 1, 254);
	gfx.PutPixel(x + 472, y + 251, 254, 216, 254);
	gfx.PutPixel(x + 472, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 472, y + 260, 254, 143, 254);
	gfx.PutPixel(x + 473, y + 250, 254, 21, 254);
	gfx.PutPixel(x + 473, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 473, y + 260, 254, 58, 254);
	gfx.PutPixel(x + 474, y + 250, 254, 75, 254);
	gfx.PutPixel(x + 474, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 474, y + 259, 254, 238, 254);
	gfx.PutPixel(x + 474, y + 260, 254, 2, 254);
	gfx.PutPixel(x + 475, y + 250, 254, 123, 254);
	gfx.PutPixel(x + 475, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 475, y + 259, 254, 174, 254);
	gfx.PutPixel(x + 476, y + 250, 254, 164, 254);
	gfx.PutPixel(x + 476, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 476, y + 259, 254, 118, 254);
	gfx.PutPixel(x + 477, y + 250, 254, 198, 254);
	gfx.PutPixel(x + 477, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 477, y + 259, 254, 69, 254);
	gfx.PutPixel(x + 478, y + 250, 254, 223, 254);
	gfx.PutPixel(x + 478, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 478, y + 259, 254, 34, 254);
	gfx.PutPixel(x + 479, y + 250, 254, 241, 254);
	gfx.PutPixel(x + 479, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 479, y + 259, 254, 12, 254);
	gfx.PutPixel(x + 480, y + 250, 254, 250, 254);
	gfx.PutPixel(x + 480, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 480, y + 259, 254, 4, 254);
	gfx.PutPixel(x + 481, y + 250, 254, 247, 254);
	gfx.PutPixel(x + 481, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 481, y + 259, 254, 27, 254);
	gfx.PutPixel(x + 482, y + 250, 254, 229, 254);
	gfx.PutPixel(x + 482, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 482, y + 259, 254, 81, 254);
	gfx.PutPixel(x + 483, y + 250, 254, 185, 254);
	gfx.PutPixel(x + 483, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 483, y + 259, 254, 182, 254);
	gfx.PutPixel(x + 484, y + 142, 254, 13, 254);
	gfx.PutPixel(x + 484, y + 143, 254, 73, 254);
	gfx.PutPixel(x + 484, y + 144, 254, 119, 254);
	gfx.PutPixel(x + 484, y + 145, 254, 164, 254);
	gfx.PutPixel(x + 484, y + 146, 254, 204, 254);
	gfx.PutPixel(x + 484, y + 147, 254, 223, 254);
	gfx.PutPixel(x + 484, y + 148, 254, 237, 254);
	gfx.PutPixel(x + 484, y + 149, 254, 250, 254);
	gfx.PutPixel(x + 484, y + 150, 254, 246, 254);
	gfx.PutPixel(x + 484, y + 151, 254, 232, 254);
	gfx.PutPixel(x + 484, y + 152, 254, 213, 254);
	gfx.PutPixel(x + 484, y + 153, 254, 172, 254);
	gfx.PutPixel(x + 484, y + 154, 254, 125, 254);
	gfx.PutPixel(x + 484, y + 155, 254, 60, 254);
	gfx.PutPixel(x + 484, y + 156, 254, 1, 254);
	gfx.PutPixel(x + 484, y + 250, 254, 123, 254);
	gfx.PutPixel(x + 484, y + 251, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 484, y + 260, 254, 89, 254);
	gfx.PutPixel(x + 485, y + 139, 254, 26, 254);
	gfx.PutPixel(x + 485, y + 140, 254, 114, 254);
	gfx.PutPixel(x + 485, y + 141, 254, 189, 254);
	gfx.PutPixel(x + 485, y + 142, 254, 249, 254);
	gfx.PutPixel(x + 485, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 156, 254, 231, 254);
	gfx.PutPixel(x + 485, y + 157, 254, 137, 254);
	gfx.PutPixel(x + 485, y + 158, 254, 20, 254);
	gfx.PutPixel(x + 485, y + 250, 254, 34, 254);
	gfx.PutPixel(x + 485, y + 251, 254, 252, 254);
	gfx.PutPixel(x + 485, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 485, y + 260, 254, 248, 254);
	gfx.PutPixel(x + 485, y + 261, 254, 83, 254);
	gfx.PutPixel(x + 486, y + 137, 254, 65, 254);
	gfx.PutPixel(x + 486, y + 138, 254, 171, 254);
	gfx.PutPixel(x + 486, y + 139, 254, 250, 254);
	gfx.PutPixel(x + 486, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 158, 254, 244, 254);
	gfx.PutPixel(x + 486, y + 159, 254, 124, 254);
	gfx.PutPixel(x + 486, y + 160, 254, 3, 254);
	gfx.PutPixel(x + 486, y + 251, 254, 163, 254);
	gfx.PutPixel(x + 486, y + 252, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 486, y + 262, 254, 159, 254);
	gfx.PutPixel(x + 486, y + 263, 254, 24, 254);
	gfx.PutPixel(x + 486, y + 279, 254, 13, 254);
	gfx.PutPixel(x + 486, y + 280, 254, 75, 254);
	gfx.PutPixel(x + 486, y + 281, 254, 147, 254);
	gfx.PutPixel(x + 486, y + 282, 254, 213, 254);
	gfx.PutPixel(x + 486, y + 283, 254, 248, 254);
	gfx.PutPixel(x + 486, y + 284, 254, 237, 254);
	gfx.PutPixel(x + 486, y + 285, 254, 189, 254);
	gfx.PutPixel(x + 486, y + 286, 254, 77, 254);
	gfx.PutPixel(x + 487, y + 135, 254, 69, 254);
	gfx.PutPixel(x + 487, y + 136, 254, 201, 254);
	gfx.PutPixel(x + 487, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 160, 254, 202, 254);
	gfx.PutPixel(x + 487, y + 161, 254, 33, 254);
	gfx.PutPixel(x + 487, y + 251, 254, 26, 254);
	gfx.PutPixel(x + 487, y + 252, 254, 237, 254);
	gfx.PutPixel(x + 487, y + 253, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 263, 254, 250, 254);
	gfx.PutPixel(x + 487, y + 264, 254, 178, 254);
	gfx.PutPixel(x + 487, y + 265, 254, 107, 254);
	gfx.PutPixel(x + 487, y + 266, 254, 57, 254);
	gfx.PutPixel(x + 487, y + 267, 254, 25, 254);
	gfx.PutPixel(x + 487, y + 268, 254, 10, 254);
	gfx.PutPixel(x + 487, y + 269, 254, 0, 254);
	gfx.PutPixel(x + 487, y + 270, 254, 3, 254);
	gfx.PutPixel(x + 487, y + 271, 254, 8, 254);
	gfx.PutPixel(x + 487, y + 272, 254, 17, 254);
	gfx.PutPixel(x + 487, y + 273, 254, 33, 254);
	gfx.PutPixel(x + 487, y + 274, 254, 54, 254);
	gfx.PutPixel(x + 487, y + 275, 254, 84, 254);
	gfx.PutPixel(x + 487, y + 276, 254, 122, 254);
	gfx.PutPixel(x + 487, y + 277, 254, 162, 254);
	gfx.PutPixel(x + 487, y + 278, 254, 212, 254);
	gfx.PutPixel(x + 487, y + 279, 254, 252, 254);
	gfx.PutPixel(x + 487, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 487, y + 286, 254, 253, 254);
	gfx.PutPixel(x + 487, y + 287, 254, 52, 254);
	gfx.PutPixel(x + 488, y + 133, 254, 51, 254);
	gfx.PutPixel(x + 488, y + 134, 254, 191, 254);
	gfx.PutPixel(x + 488, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 161, 254, 233, 254);
	gfx.PutPixel(x + 488, y + 162, 254, 55, 254);
	gfx.PutPixel(x + 488, y + 252, 254, 77, 254);
	gfx.PutPixel(x + 488, y + 253, 254, 252, 254);
	gfx.PutPixel(x + 488, y + 254, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 286, 255, 255, 255);
	gfx.PutPixel(x + 488, y + 287, 254, 89, 254);
	gfx.PutPixel(x + 489, y + 131, 254, 14, 254);
	gfx.PutPixel(x + 489, y + 132, 254, 147, 254);
	gfx.PutPixel(x + 489, y + 133, 254, 252, 254);
	gfx.PutPixel(x + 489, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 162, 254, 243, 254);
	gfx.PutPixel(x + 489, y + 163, 254, 56, 254);
	gfx.PutPixel(x + 489, y + 253, 254, 98, 254);
	gfx.PutPixel(x + 489, y + 254, 254, 252, 254);
	gfx.PutPixel(x + 489, y + 255, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 489, y + 286, 254, 251, 254);
	gfx.PutPixel(x + 489, y + 287, 254, 31, 254);
	gfx.PutPixel(x + 490, y + 130, 254, 78, 254);
	gfx.PutPixel(x + 490, y + 131, 254, 230, 254);
	gfx.PutPixel(x + 490, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 163, 254, 237, 254);
	gfx.PutPixel(x + 490, y + 164, 254, 39, 254);
	gfx.PutPixel(x + 490, y + 254, 254, 77, 254);
	gfx.PutPixel(x + 490, y + 255, 254, 238, 254);
	gfx.PutPixel(x + 490, y + 256, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 284, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 285, 255, 255, 255);
	gfx.PutPixel(x + 490, y + 286, 254, 124, 254);
	gfx.PutPixel(x + 491, y + 128, 254, 9, 254);
	gfx.PutPixel(x + 491, y + 129, 254, 151, 254);
	gfx.PutPixel(x + 491, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 164, 254, 208, 254);
	gfx.PutPixel(x + 491, y + 165, 254, 6, 254);
	gfx.PutPixel(x + 491, y + 255, 254, 28, 254);
	gfx.PutPixel(x + 491, y + 256, 254, 177, 254);
	gfx.PutPixel(x + 491, y + 257, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 258, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 282, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 283, 255, 255, 255);
	gfx.PutPixel(x + 491, y + 284, 254, 245, 254);
	gfx.PutPixel(x + 491, y + 285, 254, 101, 254);
	gfx.PutPixel(x + 492, y + 127, 254, 42, 254);
	gfx.PutPixel(x + 492, y + 128, 254, 214, 254);
	gfx.PutPixel(x + 492, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 165, 254, 138, 254);
	gfx.PutPixel(x + 492, y + 257, 254, 63, 254);
	gfx.PutPixel(x + 492, y + 258, 254, 184, 254);
	gfx.PutPixel(x + 492, y + 259, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 260, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 261, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 262, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 279, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 280, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 281, 255, 255, 255);
	gfx.PutPixel(x + 492, y + 282, 254, 250, 254);
	gfx.PutPixel(x + 492, y + 283, 254, 159, 254);
	gfx.PutPixel(x + 492, y + 284, 254, 36, 254);
	gfx.PutPixel(x + 493, y + 126, 254, 76, 254);
	gfx.PutPixel(x + 493, y + 127, 254, 241, 254);
	gfx.PutPixel(x + 493, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 141, 254, 216, 254);
	gfx.PutPixel(x + 493, y + 142, 254, 143, 254);
	gfx.PutPixel(x + 493, y + 143, 254, 91, 254);
	gfx.PutPixel(x + 493, y + 144, 254, 43, 254);
	gfx.PutPixel(x + 493, y + 145, 254, 21, 254);
	gfx.PutPixel(x + 493, y + 146, 254, 5, 254);
	gfx.PutPixel(x + 493, y + 147, 254, 12, 254);
	gfx.PutPixel(x + 493, y + 148, 254, 34, 254);
	gfx.PutPixel(x + 493, y + 149, 254, 68, 254);
	gfx.PutPixel(x + 493, y + 150, 254, 135, 254);
	gfx.PutPixel(x + 493, y + 151, 254, 207, 254);
	gfx.PutPixel(x + 493, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 165, 254, 249, 254);
	gfx.PutPixel(x + 493, y + 166, 254, 39, 254);
	gfx.PutPixel(x + 493, y + 259, 254, 34, 254);
	gfx.PutPixel(x + 493, y + 260, 254, 124, 254);
	gfx.PutPixel(x + 493, y + 261, 254, 200, 254);
	gfx.PutPixel(x + 493, y + 262, 254, 252, 254);
	gfx.PutPixel(x + 493, y + 263, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 264, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 265, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 266, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 267, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 268, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 269, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 270, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 271, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 272, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 273, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 274, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 275, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 276, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 277, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 278, 255, 255, 255);
	gfx.PutPixel(x + 493, y + 279, 254, 252, 254);
	gfx.PutPixel(x + 493, y + 280, 254, 196, 254);
	gfx.PutPixel(x + 493, y + 281, 254, 112, 254);
	gfx.PutPixel(x + 493, y + 282, 254, 27, 254);
	gfx.PutPixel(x + 494, y + 125, 254, 120, 254);
	gfx.PutPixel(x + 494, y + 126, 254, 252, 254);
	gfx.PutPixel(x + 494, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 138, 254, 239, 254);
	gfx.PutPixel(x + 494, y + 139, 254, 138, 254);
	gfx.PutPixel(x + 494, y + 140, 254, 44, 254);
	gfx.PutPixel(x + 494, y + 152, 254, 59, 254);
	gfx.PutPixel(x + 494, y + 153, 254, 202, 254);
	gfx.PutPixel(x + 494, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 494, y + 166, 254, 175, 254);
	gfx.PutPixel(x + 494, y + 262, 254, 15, 254);
	gfx.PutPixel(x + 494, y + 263, 254, 61, 254);
	gfx.PutPixel(x + 494, y + 264, 254, 105, 254);
	gfx.PutPixel(x + 494, y + 265, 254, 149, 254);
	gfx.PutPixel(x + 494, y + 266, 254, 191, 254);
	gfx.PutPixel(x + 494, y + 267, 254, 209, 254);
	gfx.PutPixel(x + 494, y + 268, 254, 223, 254);
	gfx.PutPixel(x + 494, y + 269, 254, 237, 254);
	gfx.PutPixel(x + 494, y + 270, 254, 250, 254);
	gfx.PutPixel(x + 494, y + 271, 254, 247, 254);
	gfx.PutPixel(x + 494, y + 272, 254, 238, 254);
	gfx.PutPixel(x + 494, y + 273, 254, 228, 254);
	gfx.PutPixel(x + 494, y + 274, 254, 210, 254);
	gfx.PutPixel(x + 494, y + 275, 254, 180, 254);
	gfx.PutPixel(x + 494, y + 276, 254, 151, 254);
	gfx.PutPixel(x + 494, y + 277, 254, 113, 254);
	gfx.PutPixel(x + 494, y + 278, 254, 60, 254);
	gfx.PutPixel(x + 494, y + 279, 254, 10, 254);
	gfx.PutPixel(x + 495, y + 123, 254, 0, 254);
	gfx.PutPixel(x + 495, y + 124, 254, 147, 254);
	gfx.PutPixel(x + 495, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 137, 254, 201, 254);
	gfx.PutPixel(x + 495, y + 138, 254, 21, 254);
	gfx.PutPixel(x + 495, y + 153, 254, 3, 254);
	gfx.PutPixel(x + 495, y + 154, 254, 143, 254);
	gfx.PutPixel(x + 495, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 495, y + 166, 254, 253, 254);
	gfx.PutPixel(x + 495, y + 167, 254, 41, 254);
	gfx.PutPixel(x + 496, y + 122, 254, 3, 254);
	gfx.PutPixel(x + 496, y + 123, 254, 164, 254);
	gfx.PutPixel(x + 496, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 138, 254, 234, 254);
	gfx.PutPixel(x + 496, y + 139, 254, 71, 254);
	gfx.PutPixel(x + 496, y + 155, 254, 139, 254);
	gfx.PutPixel(x + 496, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 496, y + 167, 254, 153, 254);
	gfx.PutPixel(x + 497, y + 121, 254, 4, 254);
	gfx.PutPixel(x + 497, y + 122, 254, 175, 254);
	gfx.PutPixel(x + 497, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 139, 254, 251, 254);
	gfx.PutPixel(x + 497, y + 140, 254, 85, 254);
	gfx.PutPixel(x + 497, y + 155, 254, 1, 254);
	gfx.PutPixel(x + 497, y + 156, 254, 191, 254);
	gfx.PutPixel(x + 497, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 497, y + 167, 254, 239, 254);
	gfx.PutPixel(x + 497, y + 168, 254, 7, 254);
	gfx.PutPixel(x + 498, y + 120, 254, 2, 254);
	gfx.PutPixel(x + 498, y + 121, 254, 169, 254);
	gfx.PutPixel(x + 498, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 132, 254, 239, 254);
	gfx.PutPixel(x + 498, y + 133, 254, 165, 254);
	gfx.PutPixel(x + 498, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 140, 254, 237, 254);
	gfx.PutPixel(x + 498, y + 141, 254, 12, 254);
	gfx.PutPixel(x + 498, y + 156, 254, 39, 254);
	gfx.PutPixel(x + 498, y + 157, 254, 251, 254);
	gfx.PutPixel(x + 498, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 498, y + 168, 254, 82, 254);
	gfx.PutPixel(x + 499, y + 119, 254, 0, 254);
	gfx.PutPixel(x + 499, y + 120, 254, 161, 254);
	gfx.PutPixel(x + 499, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 131, 254, 198, 254);
	gfx.PutPixel(x + 499, y + 132, 254, 30, 254);
	gfx.PutPixel(x + 499, y + 133, 254, 138, 254);
	gfx.PutPixel(x + 499, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 141, 254, 82, 254);
	gfx.PutPixel(x + 499, y + 157, 254, 164, 254);
	gfx.PutPixel(x + 499, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 499, y + 168, 254, 155, 254);
	gfx.PutPixel(x + 500, y + 119, 254, 140, 254);
	gfx.PutPixel(x + 500, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 130, 254, 151, 254);
	gfx.PutPixel(x + 500, y + 131, 254, 5, 254);
	gfx.PutPixel(x + 500, y + 133, 254, 199, 254);
	gfx.PutPixel(x + 500, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 141, 254, 117, 254);
	gfx.PutPixel(x + 500, y + 157, 254, 60, 254);
	gfx.PutPixel(x + 500, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 500, y + 168, 254, 224, 254);
	gfx.PutPixel(x + 501, y + 118, 254, 105, 254);
	gfx.PutPixel(x + 501, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 128, 254, 252, 254);
	gfx.PutPixel(x + 501, y + 129, 254, 111, 254);
	gfx.PutPixel(x + 501, y + 132, 254, 8, 254);
	gfx.PutPixel(x + 501, y + 133, 254, 249, 254);
	gfx.PutPixel(x + 501, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 141, 254, 115, 254);
	gfx.PutPixel(x + 501, y + 157, 254, 0, 254);
	gfx.PutPixel(x + 501, y + 158, 254, 228, 254);
	gfx.PutPixel(x + 501, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 501, y + 169, 254, 24, 254);
	gfx.PutPixel(x + 502, y + 117, 254, 74, 254);
	gfx.PutPixel(x + 502, y + 118, 254, 251, 254);
	gfx.PutPixel(x + 502, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 127, 254, 248, 254);
	gfx.PutPixel(x + 502, y + 128, 254, 83, 254);
	gfx.PutPixel(x + 502, y + 132, 254, 63, 254);
	gfx.PutPixel(x + 502, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 141, 254, 103, 254);
	gfx.PutPixel(x + 502, y + 158, 254, 162, 254);
	gfx.PutPixel(x + 502, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 502, y + 169, 254, 76, 254);
	gfx.PutPixel(x + 503, y + 116, 254, 36, 254);
	gfx.PutPixel(x + 503, y + 117, 254, 240, 254);
	gfx.PutPixel(x + 503, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 126, 254, 247, 254);
	gfx.PutPixel(x + 503, y + 127, 254, 74, 254);
	gfx.PutPixel(x + 503, y + 132, 254, 124, 254);
	gfx.PutPixel(x + 503, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 141, 254, 79, 254);
	gfx.PutPixel(x + 503, y + 158, 254, 109, 254);
	gfx.PutPixel(x + 503, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 503, y + 169, 254, 121, 254);
	gfx.PutPixel(x + 504, y + 115, 254, 9, 254);
	gfx.PutPixel(x + 504, y + 116, 254, 209, 254);
	gfx.PutPixel(x + 504, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 125, 254, 250, 254);
	gfx.PutPixel(x + 504, y + 126, 254, 74, 254);
	gfx.PutPixel(x + 504, y + 132, 254, 185, 254);
	gfx.PutPixel(x + 504, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 141, 254, 49, 254);
	gfx.PutPixel(x + 504, y + 158, 254, 67, 254);
	gfx.PutPixel(x + 504, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 504, y + 169, 254, 159, 254);
	gfx.PutPixel(x + 505, y + 115, 254, 163, 254);
	gfx.PutPixel(x + 505, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 125, 254, 95, 254);
	gfx.PutPixel(x + 505, y + 131, 254, 8, 254);
	gfx.PutPixel(x + 505, y + 132, 254, 246, 254);
	gfx.PutPixel(x + 505, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 141, 254, 13, 254);
	gfx.PutPixel(x + 505, y + 158, 254, 39, 254);
	gfx.PutPixel(x + 505, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 505, y + 169, 254, 192, 254);
	gfx.PutPixel(x + 506, y + 114, 254, 95, 254);
	gfx.PutPixel(x + 506, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 124, 254, 120, 254);
	gfx.PutPixel(x + 506, y + 131, 254, 71, 254);
	gfx.PutPixel(x + 506, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 140, 254, 223, 254);
	gfx.PutPixel(x + 506, y + 158, 254, 17, 254);
	gfx.PutPixel(x + 506, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 506, y + 169, 254, 215, 254);
	gfx.PutPixel(x + 507, y + 113, 254, 33, 254);
	gfx.PutPixel(x + 507, y + 114, 254, 241, 254);
	gfx.PutPixel(x + 507, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 123, 254, 145, 254);
	gfx.PutPixel(x + 507, y + 131, 254, 142, 254);
	gfx.PutPixel(x + 507, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 140, 254, 176, 254);
	gfx.PutPixel(x + 507, y + 158, 254, 7, 254);
	gfx.PutPixel(x + 507, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 507, y + 169, 254, 236, 254);
	gfx.PutPixel(x + 508, y + 112, 254, 2, 254);
	gfx.PutPixel(x + 508, y + 113, 254, 196, 254);
	gfx.PutPixel(x + 508, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 122, 254, 184, 254);
	gfx.PutPixel(x + 508, y + 123, 254, 1, 254);
	gfx.PutPixel(x + 508, y + 131, 254, 214, 254);
	gfx.PutPixel(x + 508, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 140, 254, 120, 254);
	gfx.PutPixel(x + 508, y + 158, 254, 1, 254);
	gfx.PutPixel(x + 508, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 508, y + 169, 254, 246, 254);
	gfx.PutPixel(x + 509, y + 112, 254, 118, 254);
	gfx.PutPixel(x + 509, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 121, 254, 224, 254);
	gfx.PutPixel(x + 509, y + 122, 254, 18, 254);
	gfx.PutPixel(x + 509, y + 130, 254, 29, 254);
	gfx.PutPixel(x + 509, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 140, 254, 63, 254);
	gfx.PutPixel(x + 509, y + 158, 254, 5, 254);
	gfx.PutPixel(x + 509, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 509, y + 169, 254, 252, 254);
	gfx.PutPixel(x + 510, y + 111, 254, 36, 254);
	gfx.PutPixel(x + 510, y + 112, 254, 246, 254);
	gfx.PutPixel(x + 510, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 120, 254, 248, 254);
	gfx.PutPixel(x + 510, y + 121, 254, 52, 254);
	gfx.PutPixel(x + 510, y + 130, 254, 101, 254);
	gfx.PutPixel(x + 510, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 139, 254, 246, 254);
	gfx.PutPixel(x + 510, y + 140, 254, 8, 254);
	gfx.PutPixel(x + 510, y + 158, 254, 14, 254);
	gfx.PutPixel(x + 510, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 510, y + 169, 254, 249, 254);
	gfx.PutPixel(x + 511, y + 110, 254, 0, 254);
	gfx.PutPixel(x + 511, y + 111, 254, 192, 254);
	gfx.PutPixel(x + 511, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 120, 254, 103, 254);
	gfx.PutPixel(x + 511, y + 130, 254, 178, 254);
	gfx.PutPixel(x + 511, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 139, 254, 183, 254);
	gfx.PutPixel(x + 511, y + 158, 254, 22, 254);
	gfx.PutPixel(x + 511, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 511, y + 169, 254, 241, 254);
	gfx.PutPixel(x + 512, y + 110, 254, 99, 254);
	gfx.PutPixel(x + 512, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 119, 254, 173, 254);
	gfx.PutPixel(x + 512, y + 129, 254, 12, 254);
	gfx.PutPixel(x + 512, y + 130, 254, 247, 254);
	gfx.PutPixel(x + 512, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 139, 254, 111, 254);
	gfx.PutPixel(x + 512, y + 158, 254, 42, 254);
	gfx.PutPixel(x + 512, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 512, y + 169, 254, 226, 254);
	gfx.PutPixel(x + 513, y + 109, 254, 16, 254);
	gfx.PutPixel(x + 513, y + 110, 254, 234, 254);
	gfx.PutPixel(x + 513, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 118, 254, 234, 254);
	gfx.PutPixel(x + 513, y + 119, 254, 21, 254);
	gfx.PutPixel(x + 513, y + 129, 254, 86, 254);
	gfx.PutPixel(x + 513, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 139, 254, 38, 254);
	gfx.PutPixel(x + 513, y + 158, 254, 68, 254);
	gfx.PutPixel(x + 513, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 513, y + 169, 254, 201, 254);
	gfx.PutPixel(x + 514, y + 109, 254, 146, 254);
	gfx.PutPixel(x + 514, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 118, 254, 85, 254);
	gfx.PutPixel(x + 514, y + 129, 254, 167, 254);
	gfx.PutPixel(x + 514, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 138, 254, 221, 254);
	gfx.PutPixel(x + 514, y + 158, 254, 94, 254);
	gfx.PutPixel(x + 514, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 514, y + 169, 254, 174, 254);
	gfx.PutPixel(x + 515, y + 108, 254, 43, 254);
	gfx.PutPixel(x + 515, y + 109, 254, 251, 254);
	gfx.PutPixel(x + 515, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 117, 254, 169, 254);
	gfx.PutPixel(x + 515, y + 128, 254, 6, 254);
	gfx.PutPixel(x + 515, y + 129, 254, 241, 254);
	gfx.PutPixel(x + 515, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 138, 254, 149, 254);
	gfx.PutPixel(x + 515, y + 158, 254, 128, 254);
	gfx.PutPixel(x + 515, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 515, y + 169, 254, 136, 254);
	gfx.PutPixel(x + 516, y + 108, 254, 179, 254);
	gfx.PutPixel(x + 516, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 116, 254, 238, 254);
	gfx.PutPixel(x + 516, y + 117, 254, 20, 254);
	gfx.PutPixel(x + 516, y + 128, 254, 74, 254);
	gfx.PutPixel(x + 516, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 138, 254, 66, 254);
	gfx.PutPixel(x + 516, y + 158, 254, 170, 254);
	gfx.PutPixel(x + 516, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 516, y + 169, 254, 94, 254);
	gfx.PutPixel(x + 517, y + 107, 254, 60, 254);
	gfx.PutPixel(x + 517, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 116, 254, 111, 254);
	gfx.PutPixel(x + 517, y + 128, 254, 159, 254);
	gfx.PutPixel(x + 517, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 137, 254, 230, 254);
	gfx.PutPixel(x + 517, y + 138, 254, 1, 254);
	gfx.PutPixel(x + 517, y + 158, 254, 213, 254);
	gfx.PutPixel(x + 517, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 517, y + 169, 254, 44, 254);
	gfx.PutPixel(x + 518, y + 107, 254, 197, 254);
	gfx.PutPixel(x + 518, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 115, 254, 212, 254);
	gfx.PutPixel(x + 518, y + 116, 254, 3, 254);
	gfx.PutPixel(x + 518, y + 127, 254, 8, 254);
	gfx.PutPixel(x + 518, y + 128, 254, 241, 254);
	gfx.PutPixel(x + 518, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 137, 254, 145, 254);
	gfx.PutPixel(x + 518, y + 157, 254, 9, 254);
	gfx.PutPixel(x + 518, y + 158, 254, 250, 254);
	gfx.PutPixel(x + 518, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 518, y + 168, 254, 238, 254);
	gfx.PutPixel(x + 518, y + 169, 254, 1, 254);
	gfx.PutPixel(x + 519, y + 106, 254, 70, 254);
	gfx.PutPixel(x + 519, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 115, 254, 68, 254);
	gfx.PutPixel(x + 519, y + 127, 254, 85, 254);
	gfx.PutPixel(x + 519, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 137, 254, 56, 254);
	gfx.PutPixel(x + 519, y + 157, 254, 64, 254);
	gfx.PutPixel(x + 519, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 519, y + 168, 254, 178, 254);
	gfx.PutPixel(x + 520, y + 106, 254, 194, 254);
	gfx.PutPixel(x + 520, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 114, 254, 176, 254);
	gfx.PutPixel(x + 520, y + 127, 254, 176, 254);
	gfx.PutPixel(x + 520, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 136, 254, 222, 254);
	gfx.PutPixel(x + 520, y + 157, 254, 123, 254);
	gfx.PutPixel(x + 520, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 520, y + 168, 254, 100, 254);
	gfx.PutPixel(x + 521, y + 105, 254, 62, 254);
	gfx.PutPixel(x + 521, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 113, 254, 253, 254);
	gfx.PutPixel(x + 521, y + 114, 254, 45, 254);
	gfx.PutPixel(x + 521, y + 126, 254, 17, 254);
	gfx.PutPixel(x + 521, y + 127, 254, 249, 254);
	gfx.PutPixel(x + 521, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 136, 254, 134, 254);
	gfx.PutPixel(x + 521, y + 157, 254, 185, 254);
	gfx.PutPixel(x + 521, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 521, y + 167, 254, 252, 254);
	gfx.PutPixel(x + 521, y + 168, 254, 21, 254);
	gfx.PutPixel(x + 522, y + 105, 254, 180, 254);
	gfx.PutPixel(x + 522, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 113, 254, 172, 254);
	gfx.PutPixel(x + 522, y + 126, 254, 101, 254);
	gfx.PutPixel(x + 522, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 136, 254, 38, 254);
	gfx.PutPixel(x + 522, y + 156, 254, 10, 254);
	gfx.PutPixel(x + 522, y + 157, 254, 247, 254);
	gfx.PutPixel(x + 522, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 522, y + 167, 254, 194, 254);
	gfx.PutPixel(x + 523, y + 104, 254, 36, 254);
	gfx.PutPixel(x + 523, y + 105, 254, 253, 254);
	gfx.PutPixel(x + 523, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 112, 254, 253, 254);
	gfx.PutPixel(x + 523, y + 113, 254, 44, 254);
	gfx.PutPixel(x + 523, y + 126, 254, 198, 254);
	gfx.PutPixel(x + 523, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 135, 254, 188, 254);
	gfx.PutPixel(x + 523, y + 156, 254, 76, 254);
	gfx.PutPixel(x + 523, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 523, y + 167, 254, 113, 254);
	gfx.PutPixel(x + 524, y + 104, 254, 144, 254);
	gfx.PutPixel(x + 524, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 112, 254, 171, 254);
	gfx.PutPixel(x + 524, y + 125, 254, 43, 254);
	gfx.PutPixel(x + 524, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 135, 254, 83, 254);
	gfx.PutPixel(x + 524, y + 156, 254, 151, 254);
	gfx.PutPixel(x + 524, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 524, y + 166, 254, 252, 254);
	gfx.PutPixel(x + 524, y + 167, 254, 29, 254);
	gfx.PutPixel(x + 525, y + 103, 254, 9, 254);
	gfx.PutPixel(x + 525, y + 104, 254, 240, 254);
	gfx.PutPixel(x + 525, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 112, 254, 50, 254);
	gfx.PutPixel(x + 525, y + 125, 254, 145, 254);
	gfx.PutPixel(x + 525, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 134, 254, 229, 254);
	gfx.PutPixel(x + 525, y + 135, 254, 3, 254);
	gfx.PutPixel(x + 525, y + 155, 254, 2, 254);
	gfx.PutPixel(x + 525, y + 156, 254, 231, 254);
	gfx.PutPixel(x + 525, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 525, y + 166, 254, 177, 254);
	gfx.PutPixel(x + 526, y + 103, 254, 92, 254);
	gfx.PutPixel(x + 526, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 111, 254, 197, 254);
	gfx.PutPixel(x + 526, y + 124, 254, 7, 254);
	gfx.PutPixel(x + 526, y + 125, 254, 237, 254);
	gfx.PutPixel(x + 526, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 134, 254, 129, 254);
	gfx.PutPixel(x + 526, y + 155, 254, 65, 254);
	gfx.PutPixel(x + 526, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 526, y + 166, 254, 69, 254);
	gfx.PutPixel(x + 527, y + 103, 254, 188, 254);
	gfx.PutPixel(x + 527, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 111, 254, 88, 254);
	gfx.PutPixel(x + 527, y + 124, 254, 92, 254);
	gfx.PutPixel(x + 527, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 133, 254, 251, 254);
	gfx.PutPixel(x + 527, y + 134, 254, 27, 254);
	gfx.PutPixel(x + 527, y + 155, 254, 151, 254);
	gfx.PutPixel(x + 527, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 527, y + 165, 254, 216, 254);
	gfx.PutPixel(x + 527, y + 166, 254, 0, 254);
	gfx.PutPixel(x + 528, y + 102, 254, 28, 254);
	gfx.PutPixel(x + 528, y + 103, 254, 253, 254);
	gfx.PutPixel(x + 528, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 110, 254, 228, 254);
	gfx.PutPixel(x + 528, y + 111, 254, 4, 254);
	gfx.PutPixel(x + 528, y + 124, 254, 195, 254);
	gfx.PutPixel(x + 528, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 133, 254, 170, 254);
	gfx.PutPixel(x + 528, y + 154, 254, 7, 254);
	gfx.PutPixel(x + 528, y + 155, 254, 238, 254);
	gfx.PutPixel(x + 528, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 528, y + 165, 254, 110, 254);
	gfx.PutPixel(x + 529, y + 102, 254, 110, 254);
	gfx.PutPixel(x + 529, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 110, 254, 127, 254);
	gfx.PutPixel(x + 529, y + 123, 254, 50, 254);
	gfx.PutPixel(x + 529, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 133, 254, 50, 254);
	gfx.PutPixel(x + 529, y + 154, 254, 88, 254);
	gfx.PutPixel(x + 529, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 529, y + 164, 254, 234, 254);
	gfx.PutPixel(x + 529, y + 165, 254, 11, 254);
	gfx.PutPixel(x + 530, y + 102, 254, 191, 254);
	gfx.PutPixel(x + 530, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 110, 254, 35, 254);
	gfx.PutPixel(x + 530, y + 123, 254, 162, 254);
	gfx.PutPixel(x + 530, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 132, 254, 185, 254);
	gfx.PutPixel(x + 530, y + 154, 254, 188, 254);
	gfx.PutPixel(x + 530, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 530, y + 164, 254, 114, 254);
	gfx.PutPixel(x + 531, y + 101, 254, 19, 254);
	gfx.PutPixel(x + 531, y + 102, 254, 251, 254);
	gfx.PutPixel(x + 531, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 109, 254, 199, 254);
	gfx.PutPixel(x + 531, y + 122, 254, 25, 254);
	gfx.PutPixel(x + 531, y + 123, 254, 249, 254);
	gfx.PutPixel(x + 531, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 132, 254, 64, 254);
	gfx.PutPixel(x + 531, y + 153, 254, 40, 254);
	gfx.PutPixel(x + 531, y + 154, 254, 253, 254);
	gfx.PutPixel(x + 531, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 531, y + 163, 254, 228, 254);
	gfx.PutPixel(x + 531, y + 164, 254, 7, 254);
	gfx.PutPixel(x + 532, y + 101, 254, 87, 254);
	gfx.PutPixel(x + 532, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 109, 254, 108, 254);
	gfx.PutPixel(x + 532, y + 122, 254, 132, 254);
	gfx.PutPixel(x + 532, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 131, 254, 199, 254);
	gfx.PutPixel(x + 532, y + 153, 254, 150, 254);
	gfx.PutPixel(x + 532, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 532, y + 163, 254, 104, 254);
	gfx.PutPixel(x + 533, y + 101, 254, 153, 254);
	gfx.PutPixel(x + 533, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 108, 254, 252, 254);
	gfx.PutPixel(x + 533, y + 109, 254, 20, 254);
	gfx.PutPixel(x + 533, y + 121, 254, 9, 254);
	gfx.PutPixel(x + 533, y + 122, 254, 235, 254);
	gfx.PutPixel(x + 533, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 131, 254, 78, 254);
	gfx.PutPixel(x + 533, y + 152, 254, 17, 254);
	gfx.PutPixel(x + 533, y + 153, 254, 244, 254);
	gfx.PutPixel(x + 533, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 533, y + 162, 254, 220, 254);
	gfx.PutPixel(x + 533, y + 163, 254, 4, 254);
	gfx.PutPixel(x + 534, y + 101, 254, 220, 254);
	gfx.PutPixel(x + 534, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 108, 254, 199, 254);
	gfx.PutPixel(x + 534, y + 121, 254, 109, 254);
	gfx.PutPixel(x + 534, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 130, 254, 209, 254);
	gfx.PutPixel(x + 534, y + 131, 254, 0, 254);
	gfx.PutPixel(x + 534, y + 152, 254, 128, 254);
	gfx.PutPixel(x + 534, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 534, y + 162, 254, 74, 254);
	gfx.PutPixel(x + 535, y + 100, 254, 20, 254);
	gfx.PutPixel(x + 535, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 108, 254, 128, 254);
	gfx.PutPixel(x + 535, y + 120, 254, 4, 254);
	gfx.PutPixel(x + 535, y + 121, 254, 225, 254);
	gfx.PutPixel(x + 535, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 129, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 130, 254, 79, 254);
	gfx.PutPixel(x + 535, y + 151, 254, 17, 254);
	gfx.PutPixel(x + 535, y + 152, 254, 240, 254);
	gfx.PutPixel(x + 535, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 535, y + 161, 254, 173, 254);
	gfx.PutPixel(x + 536, y + 100, 254, 70, 254);
	gfx.PutPixel(x + 536, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 108, 254, 57, 254);
	gfx.PutPixel(x + 536, y + 120, 254, 99, 254);
	gfx.PutPixel(x + 536, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 129, 254, 199, 254);
	gfx.PutPixel(x + 536, y + 151, 254, 133, 254);
	gfx.PutPixel(x + 536, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 536, y + 160, 254, 243, 254);
	gfx.PutPixel(x + 536, y + 161, 254, 28, 254);
	gfx.PutPixel(x + 537, y + 100, 254, 120, 254);
	gfx.PutPixel(x + 537, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 107, 254, 240, 254);
	gfx.PutPixel(x + 537, y + 108, 254, 2, 254);
	gfx.PutPixel(x + 537, y + 119, 254, 4, 254);
	gfx.PutPixel(x + 537, y + 120, 254, 222, 254);
	gfx.PutPixel(x + 537, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 128, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 129, 254, 65, 254);
	gfx.PutPixel(x + 537, y + 150, 254, 20, 254);
	gfx.PutPixel(x + 537, y + 151, 254, 242, 254);
	gfx.PutPixel(x + 537, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 537, y + 160, 254, 116, 254);
	gfx.PutPixel(x + 538, y + 100, 254, 158, 254);
	gfx.PutPixel(x + 538, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 107, 254, 192, 254);
	gfx.PutPixel(x + 538, y + 119, 254, 101, 254);
	gfx.PutPixel(x + 538, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 127, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 128, 254, 177, 254);
	gfx.PutPixel(x + 538, y + 150, 254, 138, 254);
	gfx.PutPixel(x + 538, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 538, y + 159, 254, 200, 254);
	gfx.PutPixel(x + 538, y + 160, 254, 3, 254);
	gfx.PutPixel(x + 539, y + 100, 254, 189, 254);
	gfx.PutPixel(x + 539, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 107, 254, 145, 254);
	gfx.PutPixel(x + 539, y + 118, 254, 8, 254);
	gfx.PutPixel(x + 539, y + 119, 254, 227, 254);
	gfx.PutPixel(x + 539, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 127, 254, 249, 254);
	gfx.PutPixel(x + 539, y + 128, 254, 36, 254);
	gfx.PutPixel(x + 539, y + 149, 254, 23, 254);
	gfx.PutPixel(x + 539, y + 150, 254, 245, 254);
	gfx.PutPixel(x + 539, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 539, y + 158, 254, 244, 254);
	gfx.PutPixel(x + 539, y + 159, 254, 37, 254);
	gfx.PutPixel(x + 540, y + 100, 254, 219, 254);
	gfx.PutPixel(x + 540, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 107, 254, 97, 254);
	gfx.PutPixel(x + 540, y + 118, 254, 118, 254);
	gfx.PutPixel(x + 540, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 126, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 127, 254, 139, 254);
	gfx.PutPixel(x + 540, y + 149, 254, 157, 254);
	gfx.PutPixel(x + 540, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 540, y + 158, 254, 104, 254);
	gfx.PutPixel(x + 541, y + 100, 254, 234, 254);
	gfx.PutPixel(x + 541, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 107, 254, 56, 254);
	gfx.PutPixel(x + 541, y + 117, 254, 20, 254);
	gfx.PutPixel(x + 541, y + 118, 254, 240, 254);
	gfx.PutPixel(x + 541, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 126, 254, 225, 254);
	gfx.PutPixel(x + 541, y + 127, 254, 11, 254);
	gfx.PutPixel(x + 541, y + 148, 254, 59, 254);
	gfx.PutPixel(x + 541, y + 149, 254, 253, 254);
	gfx.PutPixel(x + 541, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 541, y + 157, 254, 181, 254);
	gfx.PutPixel(x + 542, y + 100, 254, 246, 254);
	gfx.PutPixel(x + 542, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 107, 254, 37, 254);
	gfx.PutPixel(x + 542, y + 117, 254, 153, 254);
	gfx.PutPixel(x + 542, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 125, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 126, 254, 80, 254);
	gfx.PutPixel(x + 542, y + 147, 254, 3, 254);
	gfx.PutPixel(x + 542, y + 148, 254, 210, 254);
	gfx.PutPixel(x + 542, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 542, y + 156, 254, 229, 254);
	gfx.PutPixel(x + 542, y + 157, 254, 22, 254);
	gfx.PutPixel(x + 543, y + 100, 254, 252, 254);
	gfx.PutPixel(x + 543, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 107, 254, 19, 254);
	gfx.PutPixel(x + 543, y + 116, 254, 55, 254);
	gfx.PutPixel(x + 543, y + 117, 254, 253, 254);
	gfx.PutPixel(x + 543, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 124, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 125, 254, 172, 254);
	gfx.PutPixel(x + 543, y + 147, 254, 117, 254);
	gfx.PutPixel(x + 543, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 543, y + 155, 254, 250, 254);
	gfx.PutPixel(x + 543, y + 156, 254, 59, 254);
	gfx.PutPixel(x + 544, y + 100, 254, 246, 254);
	gfx.PutPixel(x + 544, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 107, 254, 3, 254);
	gfx.PutPixel(x + 544, y + 115, 254, 5, 254);
	gfx.PutPixel(x + 544, y + 116, 254, 209, 254);
	gfx.PutPixel(x + 544, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 124, 254, 233, 254);
	gfx.PutPixel(x + 544, y + 125, 254, 20, 254);
	gfx.PutPixel(x + 544, y + 146, 254, 29, 254);
	gfx.PutPixel(x + 544, y + 147, 254, 243, 254);
	gfx.PutPixel(x + 544, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 544, y + 155, 254, 111, 254);
	gfx.PutPixel(x + 545, y + 100, 254, 232, 254);
	gfx.PutPixel(x + 545, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 107, 254, 12, 254);
	gfx.PutPixel(x + 545, y + 115, 254, 132, 254);
	gfx.PutPixel(x + 545, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 123, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 124, 254, 83, 254);
	gfx.PutPixel(x + 545, y + 146, 254, 176, 254);
	gfx.PutPixel(x + 545, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 545, y + 154, 254, 168, 254);
	gfx.PutPixel(x + 546, y + 100, 254, 209, 254);
	gfx.PutPixel(x + 546, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 107, 254, 51, 254);
	gfx.PutPixel(x + 546, y + 114, 254, 67, 254);
	gfx.PutPixel(x + 546, y + 115, 254, 252, 254);
	gfx.PutPixel(x + 546, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 122, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 123, 254, 153, 254);
	gfx.PutPixel(x + 546, y + 145, 254, 97, 254);
	gfx.PutPixel(x + 546, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 546, y + 153, 254, 204, 254);
	gfx.PutPixel(x + 546, y + 154, 254, 10, 254);
	gfx.PutPixel(x + 547, y + 100, 254, 173, 254);
	gfx.PutPixel(x + 547, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 107, 254, 130, 254);
	gfx.PutPixel(x + 547, y + 113, 254, 35, 254);
	gfx.PutPixel(x + 547, y + 114, 254, 235, 254);
	gfx.PutPixel(x + 547, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 121, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 122, 254, 209, 254);
	gfx.PutPixel(x + 547, y + 123, 254, 7, 254);
	gfx.PutPixel(x + 547, y + 144, 254, 37, 254);
	gfx.PutPixel(x + 547, y + 145, 254, 243, 254);
	gfx.PutPixel(x + 547, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 547, y + 152, 254, 225, 254);
	gfx.PutPixel(x + 547, y + 153, 254, 24, 254);
	gfx.PutPixel(x + 548, y + 100, 254, 127, 254);
	gfx.PutPixel(x + 548, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 107, 254, 240, 254);
	gfx.PutPixel(x + 548, y + 108, 254, 29, 254);
	gfx.PutPixel(x + 548, y + 112, 254, 32, 254);
	gfx.PutPixel(x + 548, y + 113, 254, 223, 254);
	gfx.PutPixel(x + 548, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 120, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 121, 254, 236, 254);
	gfx.PutPixel(x + 548, y + 122, 254, 36, 254);
	gfx.PutPixel(x + 548, y + 143, 254, 4, 254);
	gfx.PutPixel(x + 548, y + 144, 254, 203, 254);
	gfx.PutPixel(x + 548, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 548, y + 151, 254, 240, 254);
	gfx.PutPixel(x + 548, y + 152, 254, 44, 254);
	gfx.PutPixel(x + 549, y + 100, 254, 63, 254);
	gfx.PutPixel(x + 549, y + 101, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 108, 254, 216, 254);
	gfx.PutPixel(x + 549, y + 109, 254, 55, 254);
	gfx.PutPixel(x + 549, y + 110, 254, 12, 254);
	gfx.PutPixel(x + 549, y + 111, 254, 92, 254);
	gfx.PutPixel(x + 549, y + 112, 254, 235, 254);
	gfx.PutPixel(x + 549, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 119, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 120, 254, 249, 254);
	gfx.PutPixel(x + 549, y + 121, 254, 64, 254);
	gfx.PutPixel(x + 549, y + 143, 254, 137, 254);
	gfx.PutPixel(x + 549, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 549, y + 150, 254, 248, 254);
	gfx.PutPixel(x + 549, y + 151, 254, 69, 254);
	gfx.PutPixel(x + 550, y + 100, 254, 1, 254);
	gfx.PutPixel(x + 550, y + 101, 254, 227, 254);
	gfx.PutPixel(x + 550, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 119, 254, 253, 254);
	gfx.PutPixel(x + 550, y + 120, 254, 94, 254);
	gfx.PutPixel(x + 550, y + 142, 254, 66, 254);
	gfx.PutPixel(x + 550, y + 143, 254, 253, 254);
	gfx.PutPixel(x + 550, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 550, y + 149, 254, 251, 254);
	gfx.PutPixel(x + 550, y + 150, 254, 82, 254);
	gfx.PutPixel(x + 551, y + 101, 254, 134, 254);
	gfx.PutPixel(x + 551, y + 102, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 117, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 118, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 119, 254, 105, 254);
	gfx.PutPixel(x + 551, y + 141, 254, 28, 254);
	gfx.PutPixel(x + 551, y + 142, 254, 231, 254);
	gfx.PutPixel(x + 551, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 551, y + 148, 254, 253, 254);
	gfx.PutPixel(x + 551, y + 149, 254, 92, 254);
	gfx.PutPixel(x + 552, y + 101, 254, 21, 254);
	gfx.PutPixel(x + 552, y + 102, 254, 237, 254);
	gfx.PutPixel(x + 552, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 116, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 117, 254, 252, 254);
	gfx.PutPixel(x + 552, y + 118, 254, 106, 254);
	gfx.PutPixel(x + 552, y + 140, 254, 9, 254);
	gfx.PutPixel(x + 552, y + 141, 254, 205, 254);
	gfx.PutPixel(x + 552, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 552, y + 148, 254, 103, 254);
	gfx.PutPixel(x + 553, y + 102, 254, 98, 254);
	gfx.PutPixel(x + 553, y + 103, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 115, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 116, 254, 247, 254);
	gfx.PutPixel(x + 553, y + 117, 254, 84, 254);
	gfx.PutPixel(x + 553, y + 139, 254, 0, 254);
	gfx.PutPixel(x + 553, y + 140, 254, 171, 254);
	gfx.PutPixel(x + 553, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 553, y + 146, 254, 252, 254);
	gfx.PutPixel(x + 553, y + 147, 254, 99, 254);
	gfx.PutPixel(x + 554, y + 103, 254, 130, 254);
	gfx.PutPixel(x + 554, y + 104, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 105, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 113, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 114, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 115, 254, 217, 254);
	gfx.PutPixel(x + 554, y + 116, 254, 48, 254);
	gfx.PutPixel(x + 554, y + 139, 254, 129, 254);
	gfx.PutPixel(x + 554, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 554, y + 145, 254, 250, 254);
	gfx.PutPixel(x + 554, y + 146, 254, 88, 254);
	gfx.PutPixel(x + 555, y + 104, 254, 99, 254);
	gfx.PutPixel(x + 555, y + 105, 254, 235, 254);
	gfx.PutPixel(x + 555, y + 106, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 107, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 108, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 109, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 110, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 111, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 112, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 113, 254, 245, 254);
	gfx.PutPixel(x + 555, y + 114, 254, 134, 254);
	gfx.PutPixel(x + 555, y + 115, 254, 11, 254);
	gfx.PutPixel(x + 555, y + 138, 254, 86, 254);
	gfx.PutPixel(x + 555, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 555, y + 144, 254, 245, 254);
	gfx.PutPixel(x + 555, y + 145, 254, 77, 254);
	gfx.PutPixel(x + 556, y + 105, 254, 17, 254);
	gfx.PutPixel(x + 556, y + 106, 254, 106, 254);
	gfx.PutPixel(x + 556, y + 107, 254, 185, 254);
	gfx.PutPixel(x + 556, y + 108, 254, 224, 254);
	gfx.PutPixel(x + 556, y + 109, 254, 247, 254);
	gfx.PutPixel(x + 556, y + 110, 254, 231, 254);
	gfx.PutPixel(x + 556, y + 111, 254, 194, 254);
	gfx.PutPixel(x + 556, y + 112, 254, 115, 254);
	gfx.PutPixel(x + 556, y + 113, 254, 24, 254);
	gfx.PutPixel(x + 556, y + 137, 254, 23, 254);
	gfx.PutPixel(x + 556, y + 138, 254, 241, 254);
	gfx.PutPixel(x + 556, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 556, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 556, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 556, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 556, y + 143, 254, 231, 254);
	gfx.PutPixel(x + 556, y + 144, 254, 53, 254);
	gfx.PutPixel(x + 557, y + 137, 254, 152, 254);
	gfx.PutPixel(x + 557, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 557, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 557, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 557, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 557, y + 142, 254, 210, 254);
	gfx.PutPixel(x + 557, y + 143, 254, 30, 254);
	gfx.PutPixel(x + 558, y + 136, 254, 20, 254);
	gfx.PutPixel(x + 558, y + 137, 254, 249, 254);
	gfx.PutPixel(x + 558, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 558, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 558, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 558, y + 141, 254, 152, 254);
	gfx.PutPixel(x + 558, y + 142, 254, 10, 254);
	gfx.PutPixel(x + 559, y + 136, 254, 55, 254);
	gfx.PutPixel(x + 559, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 559, y + 138, 254, 243, 254);
	gfx.PutPixel(x + 559, y + 139, 254, 166, 254);
	gfx.PutPixel(x + 559, y + 140, 254, 48, 254);
	gfx.PutPixel(x + 560, y + 137, 254, 16, 254);
	gfx.PutPixel(x + 560, y + 138, 254, 2, 254);
}
void Game::DrawStartScreen(int x, int y)
{
	gfx.PutPixel(x + 16, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 16, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 17, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 18, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 19, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 20, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 21, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 22, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 23, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 24, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 25, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 26, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 130, 254, 253, 254);
	gfx.PutPixel(x + 27, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 135, 254, 2, 254);
	gfx.PutPixel(x + 27, y + 150, 254, 5, 254);
	gfx.PutPixel(x + 27, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 27, y + 155, 254, 253, 254);
	gfx.PutPixel(x + 28, y + 130, 254, 247, 254);
	gfx.PutPixel(x + 28, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 135, 254, 13, 254);
	gfx.PutPixel(x + 28, y + 150, 254, 20, 254);
	gfx.PutPixel(x + 28, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 28, y + 155, 254, 246, 254);
	gfx.PutPixel(x + 29, y + 130, 254, 235, 254);
	gfx.PutPixel(x + 29, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 135, 254, 27, 254);
	gfx.PutPixel(x + 29, y + 150, 254, 45, 254);
	gfx.PutPixel(x + 29, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 29, y + 155, 254, 234, 254);
	gfx.PutPixel(x + 30, y + 130, 254, 216, 254);
	gfx.PutPixel(x + 30, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 135, 254, 63, 254);
	gfx.PutPixel(x + 30, y + 150, 254, 86, 254);
	gfx.PutPixel(x + 30, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 30, y + 155, 254, 209, 254);
	gfx.PutPixel(x + 31, y + 130, 254, 185, 254);
	gfx.PutPixel(x + 31, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 135, 254, 112, 254);
	gfx.PutPixel(x + 31, y + 150, 254, 151, 254);
	gfx.PutPixel(x + 31, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 31, y + 155, 254, 176, 254);
	gfx.PutPixel(x + 32, y + 130, 254, 145, 254);
	gfx.PutPixel(x + 32, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 135, 254, 193, 254);
	gfx.PutPixel(x + 32, y + 149, 254, 10, 254);
	gfx.PutPixel(x + 32, y + 150, 254, 240, 254);
	gfx.PutPixel(x + 32, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 32, y + 155, 254, 128, 254);
	gfx.PutPixel(x + 33, y + 130, 254, 89, 254);
	gfx.PutPixel(x + 33, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 136, 254, 57, 254);
	gfx.PutPixel(x + 33, y + 149, 254, 124, 254);
	gfx.PutPixel(x + 33, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 33, y + 155, 254, 64, 254);
	gfx.PutPixel(x + 34, y + 130, 254, 15, 254);
	gfx.PutPixel(x + 34, y + 131, 254, 248, 254);
	gfx.PutPixel(x + 34, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 136, 254, 212, 254);
	gfx.PutPixel(x + 34, y + 137, 254, 15, 254);
	gfx.PutPixel(x + 34, y + 148, 254, 68, 254);
	gfx.PutPixel(x + 34, y + 149, 254, 251, 254);
	gfx.PutPixel(x + 34, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 34, y + 154, 254, 233, 254);
	gfx.PutPixel(x + 34, y + 155, 254, 3, 254);
	gfx.PutPixel(x + 35, y + 131, 254, 179, 254);
	gfx.PutPixel(x + 35, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 137, 254, 196, 254);
	gfx.PutPixel(x + 35, y + 138, 254, 31, 254);
	gfx.PutPixel(x + 35, y + 146, 254, 0, 254);
	gfx.PutPixel(x + 35, y + 147, 254, 104, 254);
	gfx.PutPixel(x + 35, y + 148, 254, 248, 254);
	gfx.PutPixel(x + 35, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 35, y + 154, 254, 129, 254);
	gfx.PutPixel(x + 36, y + 131, 254, 63, 254);
	gfx.PutPixel(x + 36, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 138, 254, 243, 254);
	gfx.PutPixel(x + 36, y + 139, 254, 147, 254);
	gfx.PutPixel(x + 36, y + 140, 254, 66, 254);
	gfx.PutPixel(x + 36, y + 141, 254, 27, 254);
	gfx.PutPixel(x + 36, y + 142, 254, 5, 254);
	gfx.PutPixel(x + 36, y + 143, 254, 16, 254);
	gfx.PutPixel(x + 36, y + 144, 254, 44, 254);
	gfx.PutPixel(x + 36, y + 145, 254, 107, 254);
	gfx.PutPixel(x + 36, y + 146, 254, 210, 254);
	gfx.PutPixel(x + 36, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 36, y + 153, 254, 235, 254);
	gfx.PutPixel(x + 36, y + 154, 254, 15, 254);
	gfx.PutPixel(x + 37, y + 132, 254, 180, 254);
	gfx.PutPixel(x + 37, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 37, y + 153, 254, 90, 254);
	gfx.PutPixel(x + 38, y + 132, 254, 33, 254);
	gfx.PutPixel(x + 38, y + 133, 254, 239, 254);
	gfx.PutPixel(x + 38, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 38, y + 152, 254, 148, 254);
	gfx.PutPixel(x + 39, y + 133, 254, 69, 254);
	gfx.PutPixel(x + 39, y + 134, 254, 248, 254);
	gfx.PutPixel(x + 39, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 39, y + 151, 254, 156, 254);
	gfx.PutPixel(x + 39, y + 152, 254, 1, 254);
	gfx.PutPixel(x + 40, y + 134, 254, 67, 254);
	gfx.PutPixel(x + 40, y + 135, 254, 236, 254);
	gfx.PutPixel(x + 40, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 40, y + 149, 254, 248, 254);
	gfx.PutPixel(x + 40, y + 150, 254, 105, 254);
	gfx.PutPixel(x + 41, y + 135, 254, 25, 254);
	gfx.PutPixel(x + 41, y + 136, 254, 166, 254);
	gfx.PutPixel(x + 41, y + 137, 254, 252, 254);
	gfx.PutPixel(x + 41, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 41, y + 147, 254, 251, 254);
	gfx.PutPixel(x + 41, y + 148, 254, 157, 254);
	gfx.PutPixel(x + 41, y + 149, 254, 32, 254);
	gfx.PutPixel(x + 42, y + 137, 254, 34, 254);
	gfx.PutPixel(x + 42, y + 138, 254, 118, 254);
	gfx.PutPixel(x + 42, y + 139, 254, 181, 254);
	gfx.PutPixel(x + 42, y + 140, 254, 221, 254);
	gfx.PutPixel(x + 42, y + 141, 254, 242, 254);
	gfx.PutPixel(x + 42, y + 142, 254, 250, 254);
	gfx.PutPixel(x + 42, y + 143, 254, 232, 254);
	gfx.PutPixel(x + 42, y + 144, 254, 210, 254);
	gfx.PutPixel(x + 42, y + 145, 254, 170, 254);
	gfx.PutPixel(x + 42, y + 146, 254, 98, 254);
	gfx.PutPixel(x + 42, y + 147, 254, 24, 254);
	gfx.PutPixel(x + 50, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 50, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 51, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 52, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 53, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 54, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 144, 254, 162, 254);
	gfx.PutPixel(x + 55, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 55, y + 148, 254, 164, 254);
	gfx.PutPixel(x + 56, y + 143, 254, 96, 254);
	gfx.PutPixel(x + 56, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 56, y + 147, 254, 222, 254);
	gfx.PutPixel(x + 56, y + 148, 254, 13, 254);
	gfx.PutPixel(x + 57, y + 142, 254, 29, 254);
	gfx.PutPixel(x + 57, y + 143, 254, 241, 254);
	gfx.PutPixel(x + 57, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 57, y + 147, 254, 67, 254);
	gfx.PutPixel(x + 58, y + 142, 254, 178, 254);
	gfx.PutPixel(x + 58, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 58, y + 146, 254, 183, 254);
	gfx.PutPixel(x + 59, y + 141, 254, 69, 254);
	gfx.PutPixel(x + 59, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 59, y + 146, 254, 47, 254);
	gfx.PutPixel(x + 60, y + 141, 254, 195, 254);
	gfx.PutPixel(x + 60, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 60, y + 145, 254, 193, 254);
	gfx.PutPixel(x + 61, y + 140, 254, 43, 254);
	gfx.PutPixel(x + 61, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 61, y + 145, 254, 110, 254);
	gfx.PutPixel(x + 62, y + 140, 254, 127, 254);
	gfx.PutPixel(x + 62, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 62, y + 145, 254, 50, 254);
	gfx.PutPixel(x + 63, y + 140, 254, 189, 254);
	gfx.PutPixel(x + 63, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 63, y + 145, 254, 15, 254);
	gfx.PutPixel(x + 64, y + 140, 254, 228, 254);
	gfx.PutPixel(x + 64, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 64, y + 145, 254, 2, 254);
	gfx.PutPixel(x + 65, y + 140, 254, 248, 254);
	gfx.PutPixel(x + 65, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 65, y + 145, 254, 9, 254);
	gfx.PutPixel(x + 66, y + 140, 254, 252, 254);
	gfx.PutPixel(x + 66, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 66, y + 145, 254, 30, 254);
	gfx.PutPixel(x + 67, y + 140, 254, 244, 254);
	gfx.PutPixel(x + 67, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 67, y + 145, 254, 74, 254);
	gfx.PutPixel(x + 68, y + 140, 254, 220, 254);
	gfx.PutPixel(x + 68, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 68, y + 145, 254, 143, 254);
	gfx.PutPixel(x + 69, y + 140, 254, 185, 254);
	gfx.PutPixel(x + 69, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 69, y + 145, 254, 231, 254);
	gfx.PutPixel(x + 73, y + 149, 254, 13, 254);
	gfx.PutPixel(x + 73, y + 150, 254, 81, 254);
	gfx.PutPixel(x + 73, y + 151, 254, 138, 254);
	gfx.PutPixel(x + 73, y + 152, 254, 184, 254);
	gfx.PutPixel(x + 73, y + 153, 254, 215, 254);
	gfx.PutPixel(x + 73, y + 154, 254, 238, 254);
	gfx.PutPixel(x + 73, y + 155, 254, 248, 254);
	gfx.PutPixel(x + 73, y + 156, 254, 252, 254);
	gfx.PutPixel(x + 73, y + 157, 254, 244, 254);
	gfx.PutPixel(x + 73, y + 158, 254, 229, 254);
	gfx.PutPixel(x + 73, y + 159, 254, 201, 254);
	gfx.PutPixel(x + 73, y + 160, 254, 162, 254);
	gfx.PutPixel(x + 73, y + 161, 254, 110, 254);
	gfx.PutPixel(x + 73, y + 162, 254, 40, 254);
	gfx.PutPixel(x + 74, y + 147, 254, 65, 254);
	gfx.PutPixel(x + 74, y + 148, 254, 173, 254);
	gfx.PutPixel(x + 74, y + 149, 254, 249, 254);
	gfx.PutPixel(x + 74, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 74, y + 163, 254, 213, 254);
	gfx.PutPixel(x + 74, y + 164, 254, 108, 254);
	gfx.PutPixel(x + 74, y + 165, 254, 9, 254);
	gfx.PutPixel(x + 75, y + 145, 254, 41, 254);
	gfx.PutPixel(x + 75, y + 146, 254, 190, 254);
	gfx.PutPixel(x + 75, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 75, y + 165, 254, 227, 254);
	gfx.PutPixel(x + 75, y + 166, 254, 84, 254);
	gfx.PutPixel(x + 76, y + 144, 254, 105, 254);
	gfx.PutPixel(x + 76, y + 145, 254, 248, 254);
	gfx.PutPixel(x + 76, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 76, y + 167, 254, 153, 254);
	gfx.PutPixel(x + 76, y + 168, 254, 3, 254);
	gfx.PutPixel(x + 77, y + 143, 254, 139, 254);
	gfx.PutPixel(x + 77, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 77, y + 168, 254, 175, 254);
	gfx.PutPixel(x + 77, y + 169, 254, 3, 254);
	gfx.PutPixel(x + 78, y + 142, 254, 128, 254);
	gfx.PutPixel(x + 78, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 148, 254, 251, 254);
	gfx.PutPixel(x + 78, y + 149, 254, 171, 254);
	gfx.PutPixel(x + 78, y + 150, 254, 77, 254);
	gfx.PutPixel(x + 78, y + 151, 254, 24, 254);
	gfx.PutPixel(x + 78, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 156, 254, 8, 254);
	gfx.PutPixel(x + 78, y + 157, 254, 28, 254);
	gfx.PutPixel(x + 78, y + 158, 254, 48, 254);
	gfx.PutPixel(x + 78, y + 159, 254, 98, 254);
	gfx.PutPixel(x + 78, y + 160, 254, 166, 254);
	gfx.PutPixel(x + 78, y + 161, 254, 236, 254);
	gfx.PutPixel(x + 78, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 78, y + 169, 254, 151, 254);
	gfx.PutPixel(x + 79, y + 141, 254, 76, 254);
	gfx.PutPixel(x + 79, y + 142, 254, 253, 254);
	gfx.PutPixel(x + 79, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 147, 254, 209, 254);
	gfx.PutPixel(x + 79, y + 148, 254, 49, 254);
	gfx.PutPixel(x + 79, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 161, 254, 9, 254);
	gfx.PutPixel(x + 79, y + 162, 254, 117, 254);
	gfx.PutPixel(x + 79, y + 163, 254, 234, 254);
	gfx.PutPixel(x + 79, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 79, y + 170, 254, 81, 254);
	gfx.PutPixel(x + 80, y + 140, 254, 12, 254);
	gfx.PutPixel(x + 80, y + 141, 254, 228, 254);
	gfx.PutPixel(x + 80, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 146, 254, 196, 254);
	gfx.PutPixel(x + 80, y + 147, 254, 13, 254);
	gfx.PutPixel(x + 80, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 163, 254, 23, 254);
	gfx.PutPixel(x + 80, y + 164, 254, 198, 254);
	gfx.PutPixel(x + 80, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 80, y + 170, 254, 226, 254);
	gfx.PutPixel(x + 80, y + 171, 254, 8, 254);
	gfx.PutPixel(x + 81, y + 140, 254, 122, 254);
	gfx.PutPixel(x + 81, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 145, 254, 227, 254);
	gfx.PutPixel(x + 81, y + 146, 254, 16, 254);
	gfx.PutPixel(x + 81, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 164, 254, 6, 254);
	gfx.PutPixel(x + 81, y + 165, 254, 187, 254);
	gfx.PutPixel(x + 81, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 81, y + 171, 254, 108, 254);
	gfx.PutPixel(x + 82, y + 139, 254, 2, 254);
	gfx.PutPixel(x + 82, y + 140, 254, 232, 254);
	gfx.PutPixel(x + 82, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 145, 254, 76, 254);
	gfx.PutPixel(x + 82, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 165, 254, 20, 254);
	gfx.PutPixel(x + 82, y + 166, 254, 226, 254);
	gfx.PutPixel(x + 82, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 82, y + 171, 254, 212, 254);
	gfx.PutPixel(x + 83, y + 139, 254, 68, 254);
	gfx.PutPixel(x + 83, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 144, 254, 214, 254);
	gfx.PutPixel(x + 83, y + 145, 254, 0, 254);
	gfx.PutPixel(x + 83, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 166, 254, 92, 254);
	gfx.PutPixel(x + 83, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 83, y + 172, 254, 41, 254);
	gfx.PutPixel(x + 84, y + 139, 254, 136, 254);
	gfx.PutPixel(x + 84, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 144, 254, 125, 254);
	gfx.PutPixel(x + 84, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 166, 254, 3, 254);
	gfx.PutPixel(x + 84, y + 167, 254, 223, 254);
	gfx.PutPixel(x + 84, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 84, y + 172, 254, 111, 254);
	gfx.PutPixel(x + 85, y + 139, 254, 189, 254);
	gfx.PutPixel(x + 85, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 144, 254, 67, 254);
	gfx.PutPixel(x + 85, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 167, 254, 143, 254);
	gfx.PutPixel(x + 85, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 85, y + 172, 254, 163, 254);
	gfx.PutPixel(x + 86, y + 139, 254, 224, 254);
	gfx.PutPixel(x + 86, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 144, 254, 28, 254);
	gfx.PutPixel(x + 86, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 167, 254, 79, 254);
	gfx.PutPixel(x + 86, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 86, y + 172, 254, 202, 254);
	gfx.PutPixel(x + 87, y + 139, 254, 243, 254);
	gfx.PutPixel(x + 87, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 144, 254, 11, 254);
	gfx.PutPixel(x + 87, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 167, 254, 35, 254);
	gfx.PutPixel(x + 87, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 87, y + 172, 254, 230, 254);
	gfx.PutPixel(x + 88, y + 139, 254, 251, 254);
	gfx.PutPixel(x + 88, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 144, 254, 5, 254);
	gfx.PutPixel(x + 88, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 167, 254, 16, 254);
	gfx.PutPixel(x + 88, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 88, y + 172, 254, 245, 254);
	gfx.PutPixel(x + 89, y + 139, 254, 235, 254);
	gfx.PutPixel(x + 89, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 144, 254, 21, 254);
	gfx.PutPixel(x + 89, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 167, 254, 3, 254);
	gfx.PutPixel(x + 89, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 89, y + 172, 254, 252, 254);
	gfx.PutPixel(x + 90, y + 139, 254, 216, 254);
	gfx.PutPixel(x + 90, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 144, 254, 49, 254);
	gfx.PutPixel(x + 90, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 167, 254, 20, 254);
	gfx.PutPixel(x + 90, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 90, y + 172, 254, 242, 254);
	gfx.PutPixel(x + 91, y + 139, 254, 193, 254);
	gfx.PutPixel(x + 91, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 144, 254, 103, 254);
	gfx.PutPixel(x + 91, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 167, 254, 39, 254);
	gfx.PutPixel(x + 91, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 91, y + 172, 254, 227, 254);
	gfx.PutPixel(x + 92, y + 139, 254, 135, 254);
	gfx.PutPixel(x + 92, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 144, 254, 184, 254);
	gfx.PutPixel(x + 92, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 167, 254, 61, 254);
	gfx.PutPixel(x + 92, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 92, y + 172, 254, 209, 254);
	gfx.PutPixel(x + 93, y + 139, 254, 69, 254);
	gfx.PutPixel(x + 93, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 144, 254, 253, 254);
	gfx.PutPixel(x + 93, y + 145, 254, 48, 254);
	gfx.PutPixel(x + 93, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 167, 254, 110, 254);
	gfx.PutPixel(x + 93, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 93, y + 172, 254, 169, 254);
	gfx.PutPixel(x + 94, y + 139, 254, 7, 254);
	gfx.PutPixel(x + 94, y + 140, 254, 233, 254);
	gfx.PutPixel(x + 94, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 145, 254, 214, 254);
	gfx.PutPixel(x + 94, y + 146, 254, 13, 254);
	gfx.PutPixel(x + 94, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 167, 254, 168, 254);
	gfx.PutPixel(x + 94, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 94, y + 172, 254, 120, 254);
	gfx.PutPixel(x + 95, y + 140, 254, 116, 254);
	gfx.PutPixel(x + 95, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 146, 254, 211, 254);
	gfx.PutPixel(x + 95, y + 147, 254, 39, 254);
	gfx.PutPixel(x + 95, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 167, 254, 226, 254);
	gfx.PutPixel(x + 95, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 95, y + 172, 254, 68, 254);
	gfx.PutPixel(x + 96, y + 140, 254, 9, 254);
	gfx.PutPixel(x + 96, y + 141, 254, 224, 254);
	gfx.PutPixel(x + 96, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 147, 254, 251, 254);
	gfx.PutPixel(x + 96, y + 148, 254, 164, 254);
	gfx.PutPixel(x + 96, y + 149, 254, 86, 254);
	gfx.PutPixel(x + 96, y + 150, 254, 32, 254);
	gfx.PutPixel(x + 96, y + 151, 254, 10, 254);
	gfx.PutPixel(x + 96, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 166, 254, 41, 254);
	gfx.PutPixel(x + 96, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 96, y + 171, 254, 244, 254);
	gfx.PutPixel(x + 96, y + 172, 254, 7, 254);
	gfx.PutPixel(x + 97, y + 141, 254, 47, 254);
	gfx.PutPixel(x + 97, y + 142, 254, 243, 254);
	gfx.PutPixel(x + 97, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 166, 254, 127, 254);
	gfx.PutPixel(x + 97, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 97, y + 171, 254, 179, 254);
	gfx.PutPixel(x + 98, y + 142, 254, 74, 254);
	gfx.PutPixel(x + 98, y + 143, 254, 242, 254);
	gfx.PutPixel(x + 98, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 165, 254, 1, 254);
	gfx.PutPixel(x + 98, y + 166, 254, 222, 254);
	gfx.PutPixel(x + 98, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 98, y + 171, 254, 99, 254);
	gfx.PutPixel(x + 99, y + 143, 254, 38, 254);
	gfx.PutPixel(x + 99, y + 144, 254, 209, 254);
	gfx.PutPixel(x + 99, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 165, 254, 80, 254);
	gfx.PutPixel(x + 99, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 99, y + 170, 254, 247, 254);
	gfx.PutPixel(x + 99, y + 171, 254, 15, 254);
	gfx.PutPixel(x + 100, y + 144, 254, 5, 254);
	gfx.PutPixel(x + 100, y + 145, 254, 96, 254);
	gfx.PutPixel(x + 100, y + 146, 254, 210, 254);
	gfx.PutPixel(x + 100, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 165, 254, 215, 254);
	gfx.PutPixel(x + 100, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 100, y + 170, 254, 170, 254);
	gfx.PutPixel(x + 101, y + 147, 254, 50, 254);
	gfx.PutPixel(x + 101, y + 148, 254, 111, 254);
	gfx.PutPixel(x + 101, y + 149, 254, 171, 254);
	gfx.PutPixel(x + 101, y + 150, 254, 208, 254);
	gfx.PutPixel(x + 101, y + 151, 254, 225, 254);
	gfx.PutPixel(x + 101, y + 152, 254, 243, 254);
	gfx.PutPixel(x + 101, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 101, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 144, 254, 0, 254);
	gfx.PutPixel(x + 108, y + 145, 254, 73, 254);
	gfx.PutPixel(x + 108, y + 146, 254, 161, 254);
	gfx.PutPixel(x + 108, y + 147, 254, 217, 254);
	gfx.PutPixel(x + 108, y + 148, 254, 244, 254);
	gfx.PutPixel(x + 108, y + 149, 254, 244, 254);
	gfx.PutPixel(x + 108, y + 150, 254, 218, 254);
	gfx.PutPixel(x + 108, y + 151, 254, 160, 254);
	gfx.PutPixel(x + 108, y + 152, 254, 66, 254);
	gfx.PutPixel(x + 108, y + 164, 254, 207, 254);
	gfx.PutPixel(x + 108, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 108, y + 170, 254, 112, 254);
	gfx.PutPixel(x + 109, y + 143, 254, 42, 254);
	gfx.PutPixel(x + 109, y + 144, 254, 196, 254);
	gfx.PutPixel(x + 109, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 153, 254, 185, 254);
	gfx.PutPixel(x + 109, y + 154, 254, 25, 254);
	gfx.PutPixel(x + 109, y + 164, 254, 37, 254);
	gfx.PutPixel(x + 109, y + 165, 254, 241, 254);
	gfx.PutPixel(x + 109, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 109, y + 170, 254, 223, 254);
	gfx.PutPixel(x + 109, y + 171, 254, 2, 254);
	gfx.PutPixel(x + 110, y + 142, 254, 58, 254);
	gfx.PutPixel(x + 110, y + 143, 254, 241, 254);
	gfx.PutPixel(x + 110, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 154, 254, 218, 254);
	gfx.PutPixel(x + 110, y + 155, 254, 24, 254);
	gfx.PutPixel(x + 110, y + 165, 254, 88, 254);
	gfx.PutPixel(x + 110, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 110, y + 171, 254, 77, 254);
	gfx.PutPixel(x + 111, y + 141, 254, 40, 254);
	gfx.PutPixel(x + 111, y + 142, 254, 242, 254);
	gfx.PutPixel(x + 111, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 155, 254, 183, 254);
	gfx.PutPixel(x + 111, y + 166, 254, 179, 254);
	gfx.PutPixel(x + 111, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 111, y + 171, 254, 165, 254);
	gfx.PutPixel(x + 112, y + 140, 254, 2, 254);
	gfx.PutPixel(x + 112, y + 141, 254, 199, 254);
	gfx.PutPixel(x + 112, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 156, 254, 79, 254);
	gfx.PutPixel(x + 112, y + 166, 254, 47, 254);
	gfx.PutPixel(x + 112, y + 167, 254, 253, 254);
	gfx.PutPixel(x + 112, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 112, y + 171, 254, 242, 254);
	gfx.PutPixel(x + 112, y + 172, 254, 7, 254);
	gfx.PutPixel(x + 113, y + 140, 254, 95, 254);
	gfx.PutPixel(x + 113, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 145, 254, 208, 254);
	gfx.PutPixel(x + 113, y + 146, 254, 78, 254);
	gfx.PutPixel(x + 113, y + 147, 254, 17, 254);
	gfx.PutPixel(x + 113, y + 148, 254, 6, 254);
	gfx.PutPixel(x + 113, y + 149, 254, 48, 254);
	gfx.PutPixel(x + 113, y + 150, 254, 169, 254);
	gfx.PutPixel(x + 113, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 156, 254, 196, 254);
	gfx.PutPixel(x + 113, y + 167, 254, 168, 254);
	gfx.PutPixel(x + 113, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 113, y + 172, 254, 63, 254);
	gfx.PutPixel(x + 114, y + 140, 254, 201, 254);
	gfx.PutPixel(x + 114, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 144, 254, 198, 254);
	gfx.PutPixel(x + 114, y + 145, 254, 7, 254);
	gfx.PutPixel(x + 114, y + 151, 254, 168, 254);
	gfx.PutPixel(x + 114, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 157, 254, 36, 254);
	gfx.PutPixel(x + 114, y + 167, 254, 49, 254);
	gfx.PutPixel(x + 114, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 114, y + 172, 254, 116, 254);
	gfx.PutPixel(x + 115, y + 139, 254, 44, 254);
	gfx.PutPixel(x + 115, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 143, 254, 250, 254);
	gfx.PutPixel(x + 115, y + 144, 254, 35, 254);
	gfx.PutPixel(x + 115, y + 151, 254, 26, 254);
	gfx.PutPixel(x + 115, y + 152, 254, 248, 254);
	gfx.PutPixel(x + 115, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 157, 254, 116, 254);
	gfx.PutPixel(x + 115, y + 168, 254, 211, 254);
	gfx.PutPixel(x + 115, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 115, y + 172, 254, 169, 254);
	gfx.PutPixel(x + 116, y + 139, 254, 108, 254);
	gfx.PutPixel(x + 116, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 143, 254, 169, 254);
	gfx.PutPixel(x + 116, y + 152, 254, 172, 254);
	gfx.PutPixel(x + 116, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 157, 254, 186, 254);
	gfx.PutPixel(x + 116, y + 168, 254, 135, 254);
	gfx.PutPixel(x + 116, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 116, y + 172, 254, 206, 254);
	gfx.PutPixel(x + 117, y + 139, 254, 165, 254);
	gfx.PutPixel(x + 117, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 143, 254, 88, 254);
	gfx.PutPixel(x + 117, y + 152, 254, 93, 254);
	gfx.PutPixel(x + 117, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 157, 254, 243, 254);
	gfx.PutPixel(x + 117, y + 158, 254, 2, 254);
	gfx.PutPixel(x + 117, y + 168, 254, 74, 254);
	gfx.PutPixel(x + 117, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 117, y + 172, 254, 223, 254);
	gfx.PutPixel(x + 118, y + 139, 254, 208, 254);
	gfx.PutPixel(x + 118, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 143, 254, 37, 254);
	gfx.PutPixel(x + 118, y + 152, 254, 28, 254);
	gfx.PutPixel(x + 118, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 158, 254, 45, 254);
	gfx.PutPixel(x + 118, y + 168, 254, 31, 254);
	gfx.PutPixel(x + 118, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 118, y + 172, 254, 240, 254);
	gfx.PutPixel(x + 119, y + 139, 254, 226, 254);
	gfx.PutPixel(x + 119, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 143, 254, 12, 254);
	gfx.PutPixel(x + 119, y + 153, 254, 229, 254);
	gfx.PutPixel(x + 119, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 158, 254, 96, 254);
	gfx.PutPixel(x + 119, y + 168, 254, 9, 254);
	gfx.PutPixel(x + 119, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 119, y + 172, 254, 251, 254);
	gfx.PutPixel(x + 120, y + 139, 254, 243, 254);
	gfx.PutPixel(x + 120, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 143, 254, 2, 254);
	gfx.PutPixel(x + 120, y + 153, 254, 176, 254);
	gfx.PutPixel(x + 120, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 158, 254, 145, 254);
	gfx.PutPixel(x + 120, y + 168, 254, 2, 254);
	gfx.PutPixel(x + 120, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 120, y + 172, 254, 240, 254);
	gfx.PutPixel(x + 121, y + 139, 254, 249, 254);
	gfx.PutPixel(x + 121, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 143, 254, 14, 254);
	gfx.PutPixel(x + 121, y + 153, 254, 126, 254);
	gfx.PutPixel(x + 121, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 158, 254, 194, 254);
	gfx.PutPixel(x + 121, y + 168, 254, 14, 254);
	gfx.PutPixel(x + 121, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 121, y + 172, 254, 224, 254);
	gfx.PutPixel(x + 122, y + 139, 254, 234, 254);
	gfx.PutPixel(x + 122, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 143, 254, 42, 254);
	gfx.PutPixel(x + 122, y + 153, 254, 77, 254);
	gfx.PutPixel(x + 122, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 158, 254, 244, 254);
	gfx.PutPixel(x + 122, y + 159, 254, 3, 254);
	gfx.PutPixel(x + 122, y + 168, 254, 41, 254);
	gfx.PutPixel(x + 122, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 122, y + 172, 254, 203, 254);
	gfx.PutPixel(x + 123, y + 139, 254, 219, 254);
	gfx.PutPixel(x + 123, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 143, 254, 82, 254);
	gfx.PutPixel(x + 123, y + 153, 254, 28, 254);
	gfx.PutPixel(x + 123, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 159, 254, 55, 254);
	gfx.PutPixel(x + 123, y + 168, 254, 87, 254);
	gfx.PutPixel(x + 123, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 123, y + 172, 254, 156, 254);
	gfx.PutPixel(x + 124, y + 139, 254, 191, 254);
	gfx.PutPixel(x + 124, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 143, 254, 142, 254);
	gfx.PutPixel(x + 124, y + 154, 254, 230, 254);
	gfx.PutPixel(x + 124, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 159, 254, 128, 254);
	gfx.PutPixel(x + 124, y + 168, 254, 163, 254);
	gfx.PutPixel(x + 124, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 124, y + 172, 254, 104, 254);
	gfx.PutPixel(x + 125, y + 139, 254, 146, 254);
	gfx.PutPixel(x + 125, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 143, 254, 216, 254);
	gfx.PutPixel(x + 125, y + 154, 254, 165, 254);
	gfx.PutPixel(x + 125, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 159, 254, 219, 254);
	gfx.PutPixel(x + 125, y + 160, 254, 1, 254);
	gfx.PutPixel(x + 125, y + 167, 254, 28, 254);
	gfx.PutPixel(x + 125, y + 168, 254, 248, 254);
	gfx.PutPixel(x + 125, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 125, y + 171, 254, 253, 254);
	gfx.PutPixel(x + 125, y + 172, 254, 33, 254);
	gfx.PutPixel(x + 126, y + 139, 254, 102, 254);
	gfx.PutPixel(x + 126, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 144, 254, 49, 254);
	gfx.PutPixel(x + 126, y + 154, 254, 92, 254);
	gfx.PutPixel(x + 126, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 160, 254, 103, 254);
	gfx.PutPixel(x + 126, y + 166, 254, 6, 254);
	gfx.PutPixel(x + 126, y + 167, 254, 192, 254);
	gfx.PutPixel(x + 126, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 126, y + 171, 254, 190, 254);
	gfx.PutPixel(x + 127, y + 139, 254, 45, 254);
	gfx.PutPixel(x + 127, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 144, 254, 162, 254);
	gfx.PutPixel(x + 127, y + 154, 254, 10, 254);
	gfx.PutPixel(x + 127, y + 155, 254, 240, 254);
	gfx.PutPixel(x + 127, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 160, 254, 250, 254);
	gfx.PutPixel(x + 127, y + 161, 254, 124, 254);
	gfx.PutPixel(x + 127, y + 162, 254, 31, 254);
	gfx.PutPixel(x + 127, y + 163, 254, 4, 254);
	gfx.PutPixel(x + 127, y + 164, 254, 23, 254);
	gfx.PutPixel(x + 127, y + 165, 254, 87, 254);
	gfx.PutPixel(x + 127, y + 166, 254, 211, 254);
	gfx.PutPixel(x + 127, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 127, y + 171, 254, 80, 254);
	gfx.PutPixel(x + 128, y + 140, 254, 229, 254);
	gfx.PutPixel(x + 128, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 144, 254, 249, 254);
	gfx.PutPixel(x + 128, y + 145, 254, 26, 254);
	gfx.PutPixel(x + 128, y + 155, 254, 136, 254);
	gfx.PutPixel(x + 128, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 128, y + 170, 254, 179, 254);
	gfx.PutPixel(x + 129, y + 140, 254, 157, 254);
	gfx.PutPixel(x + 129, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 145, 254, 148, 254);
	gfx.PutPixel(x + 129, y + 155, 254, 12, 254);
	gfx.PutPixel(x + 129, y + 156, 254, 226, 254);
	gfx.PutPixel(x + 129, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 129, y + 169, 254, 228, 254);
	gfx.PutPixel(x + 129, y + 170, 254, 28, 254);
	gfx.PutPixel(x + 130, y + 140, 254, 63, 254);
	gfx.PutPixel(x + 130, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 145, 254, 253, 254);
	gfx.PutPixel(x + 130, y + 146, 254, 58, 254);
	gfx.PutPixel(x + 130, y + 156, 254, 51, 254);
	gfx.PutPixel(x + 130, y + 157, 254, 244, 254);
	gfx.PutPixel(x + 130, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 130, y + 168, 254, 233, 254);
	gfx.PutPixel(x + 130, y + 169, 254, 40, 254);
	gfx.PutPixel(x + 131, y + 141, 254, 214, 254);
	gfx.PutPixel(x + 131, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 146, 254, 214, 254);
	gfx.PutPixel(x + 131, y + 157, 254, 49, 254);
	gfx.PutPixel(x + 131, y + 158, 254, 220, 254);
	gfx.PutPixel(x + 131, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 131, y + 167, 254, 179, 254);
	gfx.PutPixel(x + 131, y + 168, 254, 26, 254);
	gfx.PutPixel(x + 132, y + 158, 254, 4, 254);
	gfx.PutPixel(x + 132, y + 159, 254, 96, 254);
	gfx.PutPixel(x + 132, y + 160, 254, 176, 254);
	gfx.PutPixel(x + 132, y + 161, 254, 226, 254);
	gfx.PutPixel(x + 132, y + 162, 254, 246, 254);
	gfx.PutPixel(x + 132, y + 163, 254, 237, 254);
	gfx.PutPixel(x + 132, y + 164, 254, 209, 254);
	gfx.PutPixel(x + 132, y + 165, 254, 150, 254);
	gfx.PutPixel(x + 132, y + 166, 254, 59, 254);
	gfx.PutPixel(x + 138, y + 144, 254, 0, 254);
	gfx.PutPixel(x + 138, y + 145, 254, 73, 254);
	gfx.PutPixel(x + 138, y + 146, 254, 161, 254);
	gfx.PutPixel(x + 138, y + 147, 254, 217, 254);
	gfx.PutPixel(x + 138, y + 148, 254, 244, 254);
	gfx.PutPixel(x + 138, y + 149, 254, 244, 254);
	gfx.PutPixel(x + 138, y + 150, 254, 218, 254);
	gfx.PutPixel(x + 138, y + 151, 254, 160, 254);
	gfx.PutPixel(x + 138, y + 152, 254, 66, 254);
	gfx.PutPixel(x + 138, y + 164, 254, 207, 254);
	gfx.PutPixel(x + 138, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 138, y + 170, 254, 112, 254);
	gfx.PutPixel(x + 139, y + 143, 254, 42, 254);
	gfx.PutPixel(x + 139, y + 144, 254, 196, 254);
	gfx.PutPixel(x + 139, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 153, 254, 185, 254);
	gfx.PutPixel(x + 139, y + 154, 254, 25, 254);
	gfx.PutPixel(x + 139, y + 164, 254, 37, 254);
	gfx.PutPixel(x + 139, y + 165, 254, 241, 254);
	gfx.PutPixel(x + 139, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 139, y + 170, 254, 223, 254);
	gfx.PutPixel(x + 139, y + 171, 254, 2, 254);
	gfx.PutPixel(x + 140, y + 142, 254, 58, 254);
	gfx.PutPixel(x + 140, y + 143, 254, 241, 254);
	gfx.PutPixel(x + 140, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 154, 254, 218, 254);
	gfx.PutPixel(x + 140, y + 155, 254, 24, 254);
	gfx.PutPixel(x + 140, y + 165, 254, 88, 254);
	gfx.PutPixel(x + 140, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 140, y + 171, 254, 77, 254);
	gfx.PutPixel(x + 141, y + 141, 254, 40, 254);
	gfx.PutPixel(x + 141, y + 142, 254, 242, 254);
	gfx.PutPixel(x + 141, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 155, 254, 183, 254);
	gfx.PutPixel(x + 141, y + 166, 254, 179, 254);
	gfx.PutPixel(x + 141, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 141, y + 171, 254, 165, 254);
	gfx.PutPixel(x + 142, y + 140, 254, 2, 254);
	gfx.PutPixel(x + 142, y + 141, 254, 199, 254);
	gfx.PutPixel(x + 142, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 156, 254, 79, 254);
	gfx.PutPixel(x + 142, y + 166, 254, 47, 254);
	gfx.PutPixel(x + 142, y + 167, 254, 253, 254);
	gfx.PutPixel(x + 142, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 142, y + 171, 254, 242, 254);
	gfx.PutPixel(x + 142, y + 172, 254, 7, 254);
	gfx.PutPixel(x + 143, y + 140, 254, 95, 254);
	gfx.PutPixel(x + 143, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 145, 254, 208, 254);
	gfx.PutPixel(x + 143, y + 146, 254, 78, 254);
	gfx.PutPixel(x + 143, y + 147, 254, 17, 254);
	gfx.PutPixel(x + 143, y + 148, 254, 6, 254);
	gfx.PutPixel(x + 143, y + 149, 254, 48, 254);
	gfx.PutPixel(x + 143, y + 150, 254, 169, 254);
	gfx.PutPixel(x + 143, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 156, 254, 196, 254);
	gfx.PutPixel(x + 143, y + 167, 254, 168, 254);
	gfx.PutPixel(x + 143, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 143, y + 172, 254, 63, 254);
	gfx.PutPixel(x + 144, y + 140, 254, 201, 254);
	gfx.PutPixel(x + 144, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 144, 254, 198, 254);
	gfx.PutPixel(x + 144, y + 145, 254, 7, 254);
	gfx.PutPixel(x + 144, y + 151, 254, 168, 254);
	gfx.PutPixel(x + 144, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 157, 254, 36, 254);
	gfx.PutPixel(x + 144, y + 167, 254, 49, 254);
	gfx.PutPixel(x + 144, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 144, y + 172, 254, 116, 254);
	gfx.PutPixel(x + 145, y + 139, 254, 44, 254);
	gfx.PutPixel(x + 145, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 143, 254, 250, 254);
	gfx.PutPixel(x + 145, y + 144, 254, 35, 254);
	gfx.PutPixel(x + 145, y + 151, 254, 26, 254);
	gfx.PutPixel(x + 145, y + 152, 254, 248, 254);
	gfx.PutPixel(x + 145, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 157, 254, 116, 254);
	gfx.PutPixel(x + 145, y + 168, 254, 211, 254);
	gfx.PutPixel(x + 145, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 145, y + 172, 254, 169, 254);
	gfx.PutPixel(x + 146, y + 139, 254, 108, 254);
	gfx.PutPixel(x + 146, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 143, 254, 169, 254);
	gfx.PutPixel(x + 146, y + 152, 254, 172, 254);
	gfx.PutPixel(x + 146, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 157, 254, 186, 254);
	gfx.PutPixel(x + 146, y + 168, 254, 135, 254);
	gfx.PutPixel(x + 146, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 146, y + 172, 254, 206, 254);
	gfx.PutPixel(x + 147, y + 139, 254, 165, 254);
	gfx.PutPixel(x + 147, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 143, 254, 88, 254);
	gfx.PutPixel(x + 147, y + 152, 254, 93, 254);
	gfx.PutPixel(x + 147, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 157, 254, 243, 254);
	gfx.PutPixel(x + 147, y + 158, 254, 2, 254);
	gfx.PutPixel(x + 147, y + 168, 254, 74, 254);
	gfx.PutPixel(x + 147, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 147, y + 172, 254, 223, 254);
	gfx.PutPixel(x + 148, y + 139, 254, 208, 254);
	gfx.PutPixel(x + 148, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 143, 254, 37, 254);
	gfx.PutPixel(x + 148, y + 152, 254, 28, 254);
	gfx.PutPixel(x + 148, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 158, 254, 45, 254);
	gfx.PutPixel(x + 148, y + 168, 254, 31, 254);
	gfx.PutPixel(x + 148, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 148, y + 172, 254, 240, 254);
	gfx.PutPixel(x + 149, y + 139, 254, 226, 254);
	gfx.PutPixel(x + 149, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 143, 254, 12, 254);
	gfx.PutPixel(x + 149, y + 153, 254, 229, 254);
	gfx.PutPixel(x + 149, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 158, 254, 96, 254);
	gfx.PutPixel(x + 149, y + 168, 254, 9, 254);
	gfx.PutPixel(x + 149, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 149, y + 172, 254, 251, 254);
	gfx.PutPixel(x + 150, y + 139, 254, 243, 254);
	gfx.PutPixel(x + 150, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 143, 254, 2, 254);
	gfx.PutPixel(x + 150, y + 153, 254, 176, 254);
	gfx.PutPixel(x + 150, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 158, 254, 145, 254);
	gfx.PutPixel(x + 150, y + 168, 254, 2, 254);
	gfx.PutPixel(x + 150, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 150, y + 172, 254, 240, 254);
	gfx.PutPixel(x + 151, y + 139, 254, 249, 254);
	gfx.PutPixel(x + 151, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 143, 254, 14, 254);
	gfx.PutPixel(x + 151, y + 153, 254, 126, 254);
	gfx.PutPixel(x + 151, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 158, 254, 194, 254);
	gfx.PutPixel(x + 151, y + 168, 254, 14, 254);
	gfx.PutPixel(x + 151, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 151, y + 172, 254, 224, 254);
	gfx.PutPixel(x + 152, y + 139, 254, 234, 254);
	gfx.PutPixel(x + 152, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 143, 254, 42, 254);
	gfx.PutPixel(x + 152, y + 153, 254, 77, 254);
	gfx.PutPixel(x + 152, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 158, 254, 244, 254);
	gfx.PutPixel(x + 152, y + 159, 254, 3, 254);
	gfx.PutPixel(x + 152, y + 168, 254, 41, 254);
	gfx.PutPixel(x + 152, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 152, y + 172, 254, 203, 254);
	gfx.PutPixel(x + 153, y + 139, 254, 219, 254);
	gfx.PutPixel(x + 153, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 143, 254, 82, 254);
	gfx.PutPixel(x + 153, y + 153, 254, 28, 254);
	gfx.PutPixel(x + 153, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 159, 254, 55, 254);
	gfx.PutPixel(x + 153, y + 168, 254, 87, 254);
	gfx.PutPixel(x + 153, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 153, y + 172, 254, 156, 254);
	gfx.PutPixel(x + 154, y + 139, 254, 191, 254);
	gfx.PutPixel(x + 154, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 143, 254, 142, 254);
	gfx.PutPixel(x + 154, y + 154, 254, 230, 254);
	gfx.PutPixel(x + 154, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 159, 254, 128, 254);
	gfx.PutPixel(x + 154, y + 168, 254, 163, 254);
	gfx.PutPixel(x + 154, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 154, y + 172, 254, 104, 254);
	gfx.PutPixel(x + 155, y + 139, 254, 146, 254);
	gfx.PutPixel(x + 155, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 143, 254, 216, 254);
	gfx.PutPixel(x + 155, y + 154, 254, 165, 254);
	gfx.PutPixel(x + 155, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 159, 254, 219, 254);
	gfx.PutPixel(x + 155, y + 160, 254, 1, 254);
	gfx.PutPixel(x + 155, y + 167, 254, 28, 254);
	gfx.PutPixel(x + 155, y + 168, 254, 248, 254);
	gfx.PutPixel(x + 155, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 155, y + 171, 254, 253, 254);
	gfx.PutPixel(x + 155, y + 172, 254, 33, 254);
	gfx.PutPixel(x + 156, y + 139, 254, 102, 254);
	gfx.PutPixel(x + 156, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 144, 254, 49, 254);
	gfx.PutPixel(x + 156, y + 154, 254, 92, 254);
	gfx.PutPixel(x + 156, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 160, 254, 103, 254);
	gfx.PutPixel(x + 156, y + 166, 254, 6, 254);
	gfx.PutPixel(x + 156, y + 167, 254, 192, 254);
	gfx.PutPixel(x + 156, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 156, y + 171, 254, 190, 254);
	gfx.PutPixel(x + 157, y + 139, 254, 45, 254);
	gfx.PutPixel(x + 157, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 144, 254, 162, 254);
	gfx.PutPixel(x + 157, y + 154, 254, 10, 254);
	gfx.PutPixel(x + 157, y + 155, 254, 240, 254);
	gfx.PutPixel(x + 157, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 160, 254, 250, 254);
	gfx.PutPixel(x + 157, y + 161, 254, 124, 254);
	gfx.PutPixel(x + 157, y + 162, 254, 31, 254);
	gfx.PutPixel(x + 157, y + 163, 254, 4, 254);
	gfx.PutPixel(x + 157, y + 164, 254, 23, 254);
	gfx.PutPixel(x + 157, y + 165, 254, 87, 254);
	gfx.PutPixel(x + 157, y + 166, 254, 211, 254);
	gfx.PutPixel(x + 157, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 157, y + 171, 254, 80, 254);
	gfx.PutPixel(x + 158, y + 140, 254, 229, 254);
	gfx.PutPixel(x + 158, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 144, 254, 249, 254);
	gfx.PutPixel(x + 158, y + 145, 254, 26, 254);
	gfx.PutPixel(x + 158, y + 155, 254, 136, 254);
	gfx.PutPixel(x + 158, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 158, y + 170, 254, 179, 254);
	gfx.PutPixel(x + 159, y + 140, 254, 157, 254);
	gfx.PutPixel(x + 159, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 145, 254, 148, 254);
	gfx.PutPixel(x + 159, y + 155, 254, 12, 254);
	gfx.PutPixel(x + 159, y + 156, 254, 226, 254);
	gfx.PutPixel(x + 159, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 159, y + 169, 254, 228, 254);
	gfx.PutPixel(x + 159, y + 170, 254, 28, 254);
	gfx.PutPixel(x + 160, y + 140, 254, 63, 254);
	gfx.PutPixel(x + 160, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 145, 254, 253, 254);
	gfx.PutPixel(x + 160, y + 146, 254, 58, 254);
	gfx.PutPixel(x + 160, y + 156, 254, 51, 254);
	gfx.PutPixel(x + 160, y + 157, 254, 244, 254);
	gfx.PutPixel(x + 160, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 160, y + 168, 254, 233, 254);
	gfx.PutPixel(x + 160, y + 169, 254, 40, 254);
	gfx.PutPixel(x + 161, y + 141, 254, 214, 254);
	gfx.PutPixel(x + 161, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 146, 254, 214, 254);
	gfx.PutPixel(x + 161, y + 157, 254, 49, 254);
	gfx.PutPixel(x + 161, y + 158, 254, 220, 254);
	gfx.PutPixel(x + 161, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 161, y + 167, 254, 179, 254);
	gfx.PutPixel(x + 161, y + 168, 254, 26, 254);
	gfx.PutPixel(x + 162, y + 158, 254, 4, 254);
	gfx.PutPixel(x + 162, y + 159, 254, 96, 254);
	gfx.PutPixel(x + 162, y + 160, 254, 176, 254);
	gfx.PutPixel(x + 162, y + 161, 254, 226, 254);
	gfx.PutPixel(x + 162, y + 162, 254, 246, 254);
	gfx.PutPixel(x + 162, y + 163, 254, 237, 254);
	gfx.PutPixel(x + 162, y + 164, 254, 209, 254);
	gfx.PutPixel(x + 162, y + 165, 254, 150, 254);
	gfx.PutPixel(x + 162, y + 166, 254, 59, 254);
	gfx.PutPixel(x + 191, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 191, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 192, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 193, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 194, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 195, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 196, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 197, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 198, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 199, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 200, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 201, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 202, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 203, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 204, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 205, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 206, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 207, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 208, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 209, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 210, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 211, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 212, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 213, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 214, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 215, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 216, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 217, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 218, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 227, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 228, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 229, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 230, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 231, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 142, 254, 10, 254);
	gfx.PutPixel(x + 232, y + 143, 254, 217, 254);
	gfx.PutPixel(x + 232, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 232, y + 147, 254, 148, 254);
	gfx.PutPixel(x + 233, y + 142, 254, 154, 254);
	gfx.PutPixel(x + 233, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 233, y + 146, 254, 225, 254);
	gfx.PutPixel(x + 233, y + 147, 254, 12, 254);
	gfx.PutPixel(x + 234, y + 141, 254, 76, 254);
	gfx.PutPixel(x + 234, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 234, y + 146, 254, 80, 254);
	gfx.PutPixel(x + 235, y + 140, 254, 2, 254);
	gfx.PutPixel(x + 235, y + 141, 254, 214, 254);
	gfx.PutPixel(x + 235, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 235, y + 145, 254, 205, 254);
	gfx.PutPixel(x + 236, y + 140, 254, 93, 254);
	gfx.PutPixel(x + 236, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 236, y + 145, 254, 79, 254);
	gfx.PutPixel(x + 237, y + 140, 254, 217, 254);
	gfx.PutPixel(x + 237, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 237, y + 144, 254, 223, 254);
	gfx.PutPixel(x + 237, y + 145, 254, 0, 254);
	gfx.PutPixel(x + 238, y + 139, 254, 58, 254);
	gfx.PutPixel(x + 238, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 238, y + 144, 254, 136, 254);
	gfx.PutPixel(x + 239, y + 139, 254, 133, 254);
	gfx.PutPixel(x + 239, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 239, y + 144, 254, 68, 254);
	gfx.PutPixel(x + 240, y + 139, 254, 191, 254);
	gfx.PutPixel(x + 240, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 240, y + 144, 254, 24, 254);
	gfx.PutPixel(x + 241, y + 139, 254, 229, 254);
	gfx.PutPixel(x + 241, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 241, y + 144, 254, 4, 254);
	gfx.PutPixel(x + 242, y + 139, 254, 247, 254);
	gfx.PutPixel(x + 242, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 242, y + 144, 254, 12, 254);
	gfx.PutPixel(x + 243, y + 139, 254, 245, 254);
	gfx.PutPixel(x + 243, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 243, y + 144, 254, 30, 254);
	gfx.PutPixel(x + 244, y + 139, 254, 225, 254);
	gfx.PutPixel(x + 244, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 244, y + 144, 254, 81, 254);
	gfx.PutPixel(x + 245, y + 139, 254, 200, 254);
	gfx.PutPixel(x + 245, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 245, y + 144, 254, 159, 254);
	gfx.PutPixel(x + 246, y + 139, 254, 139, 254);
	gfx.PutPixel(x + 246, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 246, y + 144, 254, 251, 254);
	gfx.PutPixel(x + 246, y + 145, 254, 49, 254);
	gfx.PutPixel(x + 247, y + 139, 254, 69, 254);
	gfx.PutPixel(x + 247, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 247, y + 145, 254, 235, 254);
	gfx.PutPixel(x + 247, y + 146, 254, 66, 254);
	gfx.PutPixel(x + 248, y + 139, 254, 0, 254);
	gfx.PutPixel(x + 248, y + 140, 254, 207, 254);
	gfx.PutPixel(x + 248, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 248, y + 147, 254, 197, 254);
	gfx.PutPixel(x + 248, y + 148, 254, 125, 254);
	gfx.PutPixel(x + 248, y + 149, 254, 73, 254);
	gfx.PutPixel(x + 248, y + 150, 254, 43, 254);
	gfx.PutPixel(x + 248, y + 151, 254, 18, 254);
	gfx.PutPixel(x + 248, y + 152, 254, 9, 254);
	gfx.PutPixel(x + 248, y + 153, 254, 0, 254);
	gfx.PutPixel(x + 249, y + 140, 254, 70, 254);
	gfx.PutPixel(x + 249, y + 141, 254, 253, 254);
	gfx.PutPixel(x + 249, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 249, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 141, 254, 117, 254);
	gfx.PutPixel(x + 250, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 250, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 142, 254, 110, 254);
	gfx.PutPixel(x + 251, y + 143, 254, 247, 254);
	gfx.PutPixel(x + 251, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 251, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 143, 254, 44, 254);
	gfx.PutPixel(x + 252, y + 144, 254, 173, 254);
	gfx.PutPixel(x + 252, y + 145, 254, 251, 254);
	gfx.PutPixel(x + 252, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 252, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 145, 254, 28, 254);
	gfx.PutPixel(x + 253, y + 146, 254, 104, 254);
	gfx.PutPixel(x + 253, y + 147, 254, 160, 254);
	gfx.PutPixel(x + 253, y + 148, 254, 208, 254);
	gfx.PutPixel(x + 253, y + 149, 254, 227, 254);
	gfx.PutPixel(x + 253, y + 150, 254, 244, 254);
	gfx.PutPixel(x + 253, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 253, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 260, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 261, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 262, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 263, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 264, y + 161, 254, 253, 254);
	gfx.PutPixel(x + 264, y + 162, 254, 241, 254);
	gfx.PutPixel(x + 264, y + 163, 254, 226, 254);
	gfx.PutPixel(x + 264, y + 164, 254, 203, 254);
	gfx.PutPixel(x + 264, y + 165, 254, 153, 254);
	gfx.PutPixel(x + 264, y + 166, 254, 98, 254);
	gfx.PutPixel(x + 264, y + 167, 254, 16, 254);
	gfx.PutPixel(x + 265, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 265, y + 167, 254, 245, 254);
	gfx.PutPixel(x + 265, y + 168, 254, 153, 254);
	gfx.PutPixel(x + 265, y + 169, 254, 16, 254);
	gfx.PutPixel(x + 266, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 266, y + 169, 254, 224, 254);
	gfx.PutPixel(x + 266, y + 170, 254, 48, 254);
	gfx.PutPixel(x + 267, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 267, y + 170, 254, 228, 254);
	gfx.PutPixel(x + 267, y + 171, 254, 24, 254);
	gfx.PutPixel(x + 268, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 268, y + 171, 254, 170, 254);
	gfx.PutPixel(x + 269, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 160, 254, 2, 254);
	gfx.PutPixel(x + 269, y + 161, 254, 9, 254);
	gfx.PutPixel(x + 269, y + 162, 254, 21, 254);
	gfx.PutPixel(x + 269, y + 163, 254, 55, 254);
	gfx.PutPixel(x + 269, y + 164, 254, 116, 254);
	gfx.PutPixel(x + 269, y + 165, 254, 221, 254);
	gfx.PutPixel(x + 269, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 269, y + 171, 254, 252, 254);
	gfx.PutPixel(x + 269, y + 172, 254, 39, 254);
	gfx.PutPixel(x + 270, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 165, 254, 5, 254);
	gfx.PutPixel(x + 270, y + 166, 254, 154, 254);
	gfx.PutPixel(x + 270, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 270, y + 172, 254, 122, 254);
	gfx.PutPixel(x + 271, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 166, 254, 1, 254);
	gfx.PutPixel(x + 271, y + 167, 254, 197, 254);
	gfx.PutPixel(x + 271, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 271, y + 172, 254, 185, 254);
	gfx.PutPixel(x + 272, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 167, 254, 84, 254);
	gfx.PutPixel(x + 272, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 272, y + 172, 254, 223, 254);
	gfx.PutPixel(x + 273, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 167, 254, 31, 254);
	gfx.PutPixel(x + 273, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 273, y + 172, 254, 241, 254);
	gfx.PutPixel(x + 274, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 167, 254, 6, 254);
	gfx.PutPixel(x + 274, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 274, y + 172, 254, 251, 254);
	gfx.PutPixel(x + 275, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 167, 254, 1, 254);
	gfx.PutPixel(x + 275, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 275, y + 172, 254, 241, 254);
	gfx.PutPixel(x + 276, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 167, 254, 15, 254);
	gfx.PutPixel(x + 276, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 276, y + 172, 254, 219, 254);
	gfx.PutPixel(x + 277, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 167, 254, 48, 254);
	gfx.PutPixel(x + 277, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 277, y + 172, 254, 188, 254);
	gfx.PutPixel(x + 278, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 167, 254, 98, 254);
	gfx.PutPixel(x + 278, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 278, y + 172, 254, 148, 254);
	gfx.PutPixel(x + 279, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 167, 254, 163, 254);
	gfx.PutPixel(x + 279, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 279, y + 172, 254, 100, 254);
	gfx.PutPixel(x + 280, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 167, 254, 235, 254);
	gfx.PutPixel(x + 280, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 280, y + 172, 254, 41, 254);
	gfx.PutPixel(x + 285, y + 149, 254, 13, 254);
	gfx.PutPixel(x + 285, y + 150, 254, 81, 254);
	gfx.PutPixel(x + 285, y + 151, 254, 138, 254);
	gfx.PutPixel(x + 285, y + 152, 254, 184, 254);
	gfx.PutPixel(x + 285, y + 153, 254, 215, 254);
	gfx.PutPixel(x + 285, y + 154, 254, 238, 254);
	gfx.PutPixel(x + 285, y + 155, 254, 248, 254);
	gfx.PutPixel(x + 285, y + 156, 254, 252, 254);
	gfx.PutPixel(x + 285, y + 157, 254, 244, 254);
	gfx.PutPixel(x + 285, y + 158, 254, 229, 254);
	gfx.PutPixel(x + 285, y + 159, 254, 201, 254);
	gfx.PutPixel(x + 285, y + 160, 254, 162, 254);
	gfx.PutPixel(x + 285, y + 161, 254, 110, 254);
	gfx.PutPixel(x + 285, y + 162, 254, 40, 254);
	gfx.PutPixel(x + 286, y + 147, 254, 65, 254);
	gfx.PutPixel(x + 286, y + 148, 254, 173, 254);
	gfx.PutPixel(x + 286, y + 149, 254, 249, 254);
	gfx.PutPixel(x + 286, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 286, y + 163, 254, 213, 254);
	gfx.PutPixel(x + 286, y + 164, 254, 108, 254);
	gfx.PutPixel(x + 286, y + 165, 254, 9, 254);
	gfx.PutPixel(x + 287, y + 145, 254, 41, 254);
	gfx.PutPixel(x + 287, y + 146, 254, 190, 254);
	gfx.PutPixel(x + 287, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 287, y + 165, 254, 227, 254);
	gfx.PutPixel(x + 287, y + 166, 254, 84, 254);
	gfx.PutPixel(x + 288, y + 144, 254, 105, 254);
	gfx.PutPixel(x + 288, y + 145, 254, 248, 254);
	gfx.PutPixel(x + 288, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 288, y + 167, 254, 153, 254);
	gfx.PutPixel(x + 288, y + 168, 254, 3, 254);
	gfx.PutPixel(x + 289, y + 143, 254, 139, 254);
	gfx.PutPixel(x + 289, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 289, y + 168, 254, 175, 254);
	gfx.PutPixel(x + 289, y + 169, 254, 3, 254);
	gfx.PutPixel(x + 290, y + 142, 254, 128, 254);
	gfx.PutPixel(x + 290, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 148, 254, 251, 254);
	gfx.PutPixel(x + 290, y + 149, 254, 171, 254);
	gfx.PutPixel(x + 290, y + 150, 254, 77, 254);
	gfx.PutPixel(x + 290, y + 151, 254, 24, 254);
	gfx.PutPixel(x + 290, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 156, 254, 8, 254);
	gfx.PutPixel(x + 290, y + 157, 254, 28, 254);
	gfx.PutPixel(x + 290, y + 158, 254, 48, 254);
	gfx.PutPixel(x + 290, y + 159, 254, 98, 254);
	gfx.PutPixel(x + 290, y + 160, 254, 166, 254);
	gfx.PutPixel(x + 290, y + 161, 254, 236, 254);
	gfx.PutPixel(x + 290, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 290, y + 169, 254, 151, 254);
	gfx.PutPixel(x + 291, y + 141, 254, 76, 254);
	gfx.PutPixel(x + 291, y + 142, 254, 253, 254);
	gfx.PutPixel(x + 291, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 147, 254, 209, 254);
	gfx.PutPixel(x + 291, y + 148, 254, 49, 254);
	gfx.PutPixel(x + 291, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 161, 254, 9, 254);
	gfx.PutPixel(x + 291, y + 162, 254, 117, 254);
	gfx.PutPixel(x + 291, y + 163, 254, 234, 254);
	gfx.PutPixel(x + 291, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 291, y + 170, 254, 81, 254);
	gfx.PutPixel(x + 292, y + 140, 254, 12, 254);
	gfx.PutPixel(x + 292, y + 141, 254, 228, 254);
	gfx.PutPixel(x + 292, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 146, 254, 196, 254);
	gfx.PutPixel(x + 292, y + 147, 254, 13, 254);
	gfx.PutPixel(x + 292, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 163, 254, 23, 254);
	gfx.PutPixel(x + 292, y + 164, 254, 198, 254);
	gfx.PutPixel(x + 292, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 292, y + 170, 254, 226, 254);
	gfx.PutPixel(x + 292, y + 171, 254, 8, 254);
	gfx.PutPixel(x + 293, y + 140, 254, 122, 254);
	gfx.PutPixel(x + 293, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 145, 254, 227, 254);
	gfx.PutPixel(x + 293, y + 146, 254, 16, 254);
	gfx.PutPixel(x + 293, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 164, 254, 6, 254);
	gfx.PutPixel(x + 293, y + 165, 254, 187, 254);
	gfx.PutPixel(x + 293, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 293, y + 171, 254, 108, 254);
	gfx.PutPixel(x + 294, y + 139, 254, 2, 254);
	gfx.PutPixel(x + 294, y + 140, 254, 232, 254);
	gfx.PutPixel(x + 294, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 145, 254, 76, 254);
	gfx.PutPixel(x + 294, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 165, 254, 20, 254);
	gfx.PutPixel(x + 294, y + 166, 254, 226, 254);
	gfx.PutPixel(x + 294, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 294, y + 171, 254, 212, 254);
	gfx.PutPixel(x + 295, y + 139, 254, 68, 254);
	gfx.PutPixel(x + 295, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 144, 254, 214, 254);
	gfx.PutPixel(x + 295, y + 145, 254, 0, 254);
	gfx.PutPixel(x + 295, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 166, 254, 92, 254);
	gfx.PutPixel(x + 295, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 295, y + 172, 254, 41, 254);
	gfx.PutPixel(x + 296, y + 139, 254, 136, 254);
	gfx.PutPixel(x + 296, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 144, 254, 125, 254);
	gfx.PutPixel(x + 296, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 166, 254, 3, 254);
	gfx.PutPixel(x + 296, y + 167, 254, 223, 254);
	gfx.PutPixel(x + 296, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 296, y + 172, 254, 111, 254);
	gfx.PutPixel(x + 297, y + 139, 254, 189, 254);
	gfx.PutPixel(x + 297, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 144, 254, 67, 254);
	gfx.PutPixel(x + 297, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 167, 254, 143, 254);
	gfx.PutPixel(x + 297, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 297, y + 172, 254, 163, 254);
	gfx.PutPixel(x + 298, y + 139, 254, 224, 254);
	gfx.PutPixel(x + 298, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 144, 254, 28, 254);
	gfx.PutPixel(x + 298, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 167, 254, 79, 254);
	gfx.PutPixel(x + 298, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 298, y + 172, 254, 202, 254);
	gfx.PutPixel(x + 299, y + 139, 254, 243, 254);
	gfx.PutPixel(x + 299, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 144, 254, 11, 254);
	gfx.PutPixel(x + 299, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 167, 254, 35, 254);
	gfx.PutPixel(x + 299, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 299, y + 172, 254, 230, 254);
	gfx.PutPixel(x + 300, y + 139, 254, 251, 254);
	gfx.PutPixel(x + 300, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 144, 254, 5, 254);
	gfx.PutPixel(x + 300, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 167, 254, 16, 254);
	gfx.PutPixel(x + 300, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 300, y + 172, 254, 245, 254);
	gfx.PutPixel(x + 301, y + 139, 254, 235, 254);
	gfx.PutPixel(x + 301, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 144, 254, 21, 254);
	gfx.PutPixel(x + 301, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 167, 254, 3, 254);
	gfx.PutPixel(x + 301, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 301, y + 172, 254, 252, 254);
	gfx.PutPixel(x + 302, y + 139, 254, 216, 254);
	gfx.PutPixel(x + 302, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 144, 254, 49, 254);
	gfx.PutPixel(x + 302, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 167, 254, 20, 254);
	gfx.PutPixel(x + 302, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 302, y + 172, 254, 242, 254);
	gfx.PutPixel(x + 303, y + 139, 254, 193, 254);
	gfx.PutPixel(x + 303, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 144, 254, 103, 254);
	gfx.PutPixel(x + 303, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 167, 254, 39, 254);
	gfx.PutPixel(x + 303, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 303, y + 172, 254, 227, 254);
	gfx.PutPixel(x + 304, y + 139, 254, 135, 254);
	gfx.PutPixel(x + 304, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 144, 254, 184, 254);
	gfx.PutPixel(x + 304, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 167, 254, 61, 254);
	gfx.PutPixel(x + 304, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 304, y + 172, 254, 209, 254);
	gfx.PutPixel(x + 305, y + 139, 254, 69, 254);
	gfx.PutPixel(x + 305, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 144, 254, 253, 254);
	gfx.PutPixel(x + 305, y + 145, 254, 48, 254);
	gfx.PutPixel(x + 305, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 167, 254, 110, 254);
	gfx.PutPixel(x + 305, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 305, y + 172, 254, 169, 254);
	gfx.PutPixel(x + 306, y + 139, 254, 7, 254);
	gfx.PutPixel(x + 306, y + 140, 254, 233, 254);
	gfx.PutPixel(x + 306, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 145, 254, 214, 254);
	gfx.PutPixel(x + 306, y + 146, 254, 13, 254);
	gfx.PutPixel(x + 306, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 167, 254, 168, 254);
	gfx.PutPixel(x + 306, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 306, y + 172, 254, 120, 254);
	gfx.PutPixel(x + 307, y + 140, 254, 116, 254);
	gfx.PutPixel(x + 307, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 146, 254, 211, 254);
	gfx.PutPixel(x + 307, y + 147, 254, 39, 254);
	gfx.PutPixel(x + 307, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 167, 254, 226, 254);
	gfx.PutPixel(x + 307, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 307, y + 172, 254, 68, 254);
	gfx.PutPixel(x + 308, y + 140, 254, 9, 254);
	gfx.PutPixel(x + 308, y + 141, 254, 224, 254);
	gfx.PutPixel(x + 308, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 147, 254, 251, 254);
	gfx.PutPixel(x + 308, y + 148, 254, 164, 254);
	gfx.PutPixel(x + 308, y + 149, 254, 86, 254);
	gfx.PutPixel(x + 308, y + 150, 254, 32, 254);
	gfx.PutPixel(x + 308, y + 151, 254, 10, 254);
	gfx.PutPixel(x + 308, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 166, 254, 41, 254);
	gfx.PutPixel(x + 308, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 308, y + 171, 254, 244, 254);
	gfx.PutPixel(x + 308, y + 172, 254, 7, 254);
	gfx.PutPixel(x + 309, y + 141, 254, 47, 254);
	gfx.PutPixel(x + 309, y + 142, 254, 243, 254);
	gfx.PutPixel(x + 309, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 166, 254, 127, 254);
	gfx.PutPixel(x + 309, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 309, y + 171, 254, 179, 254);
	gfx.PutPixel(x + 310, y + 142, 254, 74, 254);
	gfx.PutPixel(x + 310, y + 143, 254, 242, 254);
	gfx.PutPixel(x + 310, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 165, 254, 1, 254);
	gfx.PutPixel(x + 310, y + 166, 254, 222, 254);
	gfx.PutPixel(x + 310, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 310, y + 171, 254, 99, 254);
	gfx.PutPixel(x + 311, y + 143, 254, 38, 254);
	gfx.PutPixel(x + 311, y + 144, 254, 209, 254);
	gfx.PutPixel(x + 311, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 165, 254, 80, 254);
	gfx.PutPixel(x + 311, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 311, y + 170, 254, 247, 254);
	gfx.PutPixel(x + 311, y + 171, 254, 15, 254);
	gfx.PutPixel(x + 312, y + 144, 254, 5, 254);
	gfx.PutPixel(x + 312, y + 145, 254, 96, 254);
	gfx.PutPixel(x + 312, y + 146, 254, 210, 254);
	gfx.PutPixel(x + 312, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 165, 254, 215, 254);
	gfx.PutPixel(x + 312, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 312, y + 170, 254, 170, 254);
	gfx.PutPixel(x + 313, y + 147, 254, 50, 254);
	gfx.PutPixel(x + 313, y + 148, 254, 111, 254);
	gfx.PutPixel(x + 313, y + 149, 254, 171, 254);
	gfx.PutPixel(x + 313, y + 150, 254, 208, 254);
	gfx.PutPixel(x + 313, y + 151, 254, 225, 254);
	gfx.PutPixel(x + 313, y + 152, 254, 243, 254);
	gfx.PutPixel(x + 313, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 313, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 322, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 323, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 324, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 325, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 160, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 161, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 162, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 163, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 164, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 165, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 326, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 144, 254, 162, 254);
	gfx.PutPixel(x + 327, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 327, y + 148, 254, 164, 254);
	gfx.PutPixel(x + 328, y + 143, 254, 96, 254);
	gfx.PutPixel(x + 328, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 328, y + 147, 254, 222, 254);
	gfx.PutPixel(x + 328, y + 148, 254, 13, 254);
	gfx.PutPixel(x + 329, y + 142, 254, 29, 254);
	gfx.PutPixel(x + 329, y + 143, 254, 241, 254);
	gfx.PutPixel(x + 329, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 329, y + 147, 254, 67, 254);
	gfx.PutPixel(x + 330, y + 142, 254, 178, 254);
	gfx.PutPixel(x + 330, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 330, y + 146, 254, 183, 254);
	gfx.PutPixel(x + 331, y + 141, 254, 69, 254);
	gfx.PutPixel(x + 331, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 331, y + 146, 254, 47, 254);
	gfx.PutPixel(x + 332, y + 141, 254, 195, 254);
	gfx.PutPixel(x + 332, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 332, y + 145, 254, 193, 254);
	gfx.PutPixel(x + 333, y + 140, 254, 43, 254);
	gfx.PutPixel(x + 333, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 333, y + 145, 254, 110, 254);
	gfx.PutPixel(x + 334, y + 140, 254, 127, 254);
	gfx.PutPixel(x + 334, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 334, y + 145, 254, 50, 254);
	gfx.PutPixel(x + 335, y + 140, 254, 189, 254);
	gfx.PutPixel(x + 335, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 335, y + 145, 254, 15, 254);
	gfx.PutPixel(x + 336, y + 140, 254, 228, 254);
	gfx.PutPixel(x + 336, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 336, y + 145, 254, 2, 254);
	gfx.PutPixel(x + 337, y + 140, 254, 248, 254);
	gfx.PutPixel(x + 337, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 337, y + 145, 254, 9, 254);
	gfx.PutPixel(x + 338, y + 140, 254, 252, 254);
	gfx.PutPixel(x + 338, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 338, y + 145, 254, 30, 254);
	gfx.PutPixel(x + 339, y + 140, 254, 244, 254);
	gfx.PutPixel(x + 339, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 339, y + 145, 254, 74, 254);
	gfx.PutPixel(x + 340, y + 140, 254, 220, 254);
	gfx.PutPixel(x + 340, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 340, y + 145, 254, 143, 254);
	gfx.PutPixel(x + 341, y + 140, 254, 185, 254);
	gfx.PutPixel(x + 341, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 341, y + 145, 254, 231, 254);
	gfx.PutPixel(x + 351, y + 130, 254, 252, 254);
	gfx.PutPixel(x + 351, y + 131, 254, 246, 254);
	gfx.PutPixel(x + 351, y + 132, 254, 239, 254);
	gfx.PutPixel(x + 351, y + 133, 254, 233, 254);
	gfx.PutPixel(x + 351, y + 134, 254, 226, 254);
	gfx.PutPixel(x + 351, y + 135, 254, 220, 254);
	gfx.PutPixel(x + 351, y + 136, 254, 214, 254);
	gfx.PutPixel(x + 351, y + 137, 254, 207, 254);
	gfx.PutPixel(x + 351, y + 138, 254, 201, 254);
	gfx.PutPixel(x + 351, y + 139, 254, 194, 254);
	gfx.PutPixel(x + 351, y + 140, 254, 188, 254);
	gfx.PutPixel(x + 351, y + 141, 254, 182, 254);
	gfx.PutPixel(x + 351, y + 142, 254, 175, 254);
	gfx.PutPixel(x + 351, y + 143, 254, 169, 254);
	gfx.PutPixel(x + 351, y + 144, 254, 162, 254);
	gfx.PutPixel(x + 351, y + 145, 254, 156, 254);
	gfx.PutPixel(x + 351, y + 146, 254, 150, 254);
	gfx.PutPixel(x + 351, y + 147, 254, 143, 254);
	gfx.PutPixel(x + 351, y + 148, 254, 137, 254);
	gfx.PutPixel(x + 351, y + 149, 254, 130, 254);
	gfx.PutPixel(x + 351, y + 150, 254, 124, 254);
	gfx.PutPixel(x + 351, y + 151, 254, 118, 254);
	gfx.PutPixel(x + 351, y + 152, 254, 111, 254);
	gfx.PutPixel(x + 351, y + 153, 254, 105, 254);
	gfx.PutPixel(x + 351, y + 154, 254, 98, 254);
	gfx.PutPixel(x + 351, y + 155, 254, 92, 254);
	gfx.PutPixel(x + 351, y + 156, 254, 86, 254);
	gfx.PutPixel(x + 351, y + 157, 254, 79, 254);
	gfx.PutPixel(x + 351, y + 158, 254, 73, 254);
	gfx.PutPixel(x + 351, y + 159, 254, 66, 254);
	gfx.PutPixel(x + 351, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 351, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 352, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 353, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 354, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 130, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 131, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 132, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 133, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 134, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 135, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 136, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 137, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 138, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 139, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 140, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 141, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 142, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 143, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 144, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 145, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 146, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 147, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 148, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 149, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 150, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 151, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 152, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 153, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 154, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 155, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 156, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 157, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 158, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 159, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 355, y + 171, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 130, 254, 252, 254);
	gfx.PutPixel(x + 356, y + 131, 254, 246, 254);
	gfx.PutPixel(x + 356, y + 132, 254, 240, 254);
	gfx.PutPixel(x + 356, y + 133, 254, 234, 254);
	gfx.PutPixel(x + 356, y + 134, 254, 228, 254);
	gfx.PutPixel(x + 356, y + 135, 254, 222, 254);
	gfx.PutPixel(x + 356, y + 136, 254, 216, 254);
	gfx.PutPixel(x + 356, y + 137, 254, 210, 254);
	gfx.PutPixel(x + 356, y + 138, 254, 204, 254);
	gfx.PutPixel(x + 356, y + 139, 254, 198, 254);
	gfx.PutPixel(x + 356, y + 140, 254, 192, 254);
	gfx.PutPixel(x + 356, y + 141, 254, 186, 254);
	gfx.PutPixel(x + 356, y + 142, 254, 180, 254);
	gfx.PutPixel(x + 356, y + 143, 254, 174, 254);
	gfx.PutPixel(x + 356, y + 144, 254, 168, 254);
	gfx.PutPixel(x + 356, y + 145, 254, 162, 254);
	gfx.PutPixel(x + 356, y + 146, 254, 156, 254);
	gfx.PutPixel(x + 356, y + 147, 254, 150, 254);
	gfx.PutPixel(x + 356, y + 148, 254, 144, 254);
	gfx.PutPixel(x + 356, y + 149, 254, 138, 254);
	gfx.PutPixel(x + 356, y + 150, 254, 132, 254);
	gfx.PutPixel(x + 356, y + 151, 254, 126, 254);
	gfx.PutPixel(x + 356, y + 152, 254, 120, 254);
	gfx.PutPixel(x + 356, y + 153, 254, 114, 254);
	gfx.PutPixel(x + 356, y + 154, 254, 108, 254);
	gfx.PutPixel(x + 356, y + 155, 254, 102, 254);
	gfx.PutPixel(x + 356, y + 156, 254, 96, 254);
	gfx.PutPixel(x + 356, y + 157, 254, 90, 254);
	gfx.PutPixel(x + 356, y + 158, 254, 84, 254);
	gfx.PutPixel(x + 356, y + 159, 254, 78, 254);
	gfx.PutPixel(x + 356, y + 166, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 167, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 168, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 169, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 170, 255, 255, 255);
	gfx.PutPixel(x + 356, y + 171, 255, 255, 255);

}
